package com.mxnavi.tspv2.car;

import android.util.Log;

import com.mxnavi.tspv2.car.model.BindingMyCarInfo;
import com.mxnavi.tspv2.car.model.QueryMyCarInfo;
import com.mxnavi.tspv2.car.model.QueryMyCarInfoResult;
import com.mxnavi.tspv2.car.model.UpdateMyCarInfo;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2017/11/23.
 */

public class CarManager extends BaseTsp {
    private static final String TAG = "CarManager";
    // 请求服务类型为用户管理
    final int thisServiceType = Config.MxNGTPServiceType.MX_USER_MANAGE.getServiceType();
    //  用户管理的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_USER_MANAGE.getServiceVer();

    public CarManager() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int BINDINGMYCAR = 1;//绑定我的爱车
    final int QUERYMYCAR = 2;//查询我的爱车信息
    final int UNBINDINGMYCAR = 3;//解绑我的爱车
    final int UPDATEMYCAR = 4;//修改我的爱车信息

    /**
     * 绑定我的爱车
     *
     * @param bindingMyCarInfo 绑定我的爱车相关信息
     * @return
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo) {
        return bindingMyCar(bindingMyCarInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定我的爱车
     *
     * @param bindingMyCarInfo 绑定我的爱车相关信息
     * @param timeOutSecs
     * @return
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo, int timeOutSecs) {
        return bindingMyCar(bindingMyCarInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定我的爱车
     *
     * @param bindingMyCarInfo 绑定我的爱车相关信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @param priorityLevel    接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int bindingMyCar(BindingMyCarInfo bindingMyCarInfo, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(bindingMyCarInfo.getToken())
                || Utils.strIsEmpty(bindingMyCarInfo.getCarPlateNo())) {
            if (mBindingMyCarListener != null) {
                mBindingMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/user/car/bindingMyCar";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("device_no", Config.VIN);
        params.put("device_type", Config.DEVICE_TYPE);
        params.put("token", Utils.converNullToEmpty(bindingMyCarInfo.getToken()));
        params.put("user_id", bindingMyCarInfo.getUserId());
        params.put("car_plate_no", bindingMyCarInfo.getCarPlateNo());
        params.put("is_major", bindingMyCarInfo.getIsMajor());
        params.put("car_brand", bindingMyCarInfo.getCarBrand());
        params.put("car_type", bindingMyCarInfo.getCarType());
        params.put("fuel_type", bindingMyCarInfo.getFuelType());
        params.put("engine_no", bindingMyCarInfo.getEngineNo());
        params.put("car_vin", bindingMyCarInfo.getCarVin());
        params.put("buy_time", bindingMyCarInfo.getBuyTime());
        params.put("insurance_time", bindingMyCarInfo.getInsuranceTime());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, BINDINGMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @return
     */
    public int queryMyCar(String token, int userId) {
        return queryMyCar(token, userId, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询我的爱车信息
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @param timeOutSecs   接口超时时间，传0为不关注
     * @return
     */
    public int queryMyCar(String token, int userId, int timeOutSecs) {
        return queryMyCar(token, userId, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询我的爱车信息
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @param timeOutSecs   接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int queryMyCar(String token, int userId, int timeOutSecs, int priorityLevel) {
        if ( Utils.strIsEmpty(token)) {
            if (mQueryMyCarListener != null) {
                mQueryMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/user/car/queryMyCar";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("device_no", Config.VIN);
        params.put("device_type", Config.DEVICE_TYPE);
        params.put("token", token);
        params.put("user_id", userId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, QUERYMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @param carPlateNo    车牌号码
     * @return
     */
    public int unBindingMyCar(String token, int userId, String carPlateNo) {
        return unBindingMyCar(token, userId, carPlateNo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @param carPlateNo    车牌号码
     * @param timeOutSecs   接口超时时间，传0为不关注
     * @return
     */
    public int unBindingMyCar(String token, int userId, String carPlateNo, int timeOutSecs) {
        return unBindingMyCar(token, userId, carPlateNo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 解绑我的爱车
     *
     * @param token          登录标识
     * @param userId         用户ID
     * @param carPlateNo    车牌号码
     * @param timeOutSecs   接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int unBindingMyCar(String token, int userId, String carPlateNo, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(token) || Utils.strIsEmpty(carPlateNo)) {
            if (mUnBindingMyCarListener != null) {
                mUnBindingMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/user/car/unBindingMyCar";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("device_no", Config.VIN);
        params.put("device_type", Config.DEVICE_TYPE);
        params.put("token", token);
        params.put("user_id", userId);
        params.put("car_plate_no", carPlateNo);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UNBINDINGMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改我的爱车信息
     *
     * @param updateMyCarInfo 修改我的爱车相关信息
     * @return
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo) {
        return updateMyCar(updateMyCarInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改我的爱车信息
     *
     * @param updateMyCarInfo 修改我的爱车相关信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @return
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo, int timeOutSecs) {
        return updateMyCar(updateMyCarInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改我的爱车信息
     *
     * @param updateMyCarInfo 修改我的爱车相关信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @param priorityLevel    接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int updateMyCar(UpdateMyCarInfo updateMyCarInfo, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(updateMyCarInfo.getToken())
                || Utils.strIsEmpty(updateMyCarInfo.getCarPlateNoOld())) {
            if (mUpdateMyCarListener != null) {
                mUpdateMyCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/user/car/updateMyCar";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("device_no", Config.VIN);
        params.put("device_type", Config.DEVICE_TYPE);
        params.put("token", updateMyCarInfo.getToken());
        params.put("user_id", updateMyCarInfo.getUserId());
        params.put("car_plate_no", updateMyCarInfo.getCarPlateNoOld());
        params.put("car_plate_no_new", updateMyCarInfo.getCarPlateNoNew());
        params.put("car_brand", updateMyCarInfo.getCarBrand());
        params.put("car_type", updateMyCarInfo.getCarType());
        params.put("fuel_type", updateMyCarInfo.getFuelType());
        params.put("engine_no", updateMyCarInfo.getEngineNo());
        params.put("car_vin", updateMyCarInfo.getCarVin());
        params.put("buy_time", updateMyCarInfo.getBuyTime());
        params.put("insurance_time", updateMyCarInfo.getInsuranceTime());
        params.put("is_major", updateMyCarInfo.getIsMajor());
        String requestData = Utils.makeUrl(url, params);
        Log.e(TAG, "requestData: " +requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEMYCAR);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    public interface BindingMyCarListener extends CallBackListener {
        /**
         * 绑定我的爱车成功
         */
        public void onBindingMyCarSuccess();
    }
    public interface QueryMyCarListener extends CallBackListener {
        /**
         * 查询我的爱车
         * @param list 爱车信息列表
         */
        public void onQueryMyCar(List<QueryMyCarInfoResult> list);
    }
    public interface UnBindingMyCarListener extends CallBackListener {
        /**
         * 解绑我的爱车成功
         */
        public void onUnBindingMyCarSuccess();
    }
    public interface UpdateMyCarListener extends CallBackListener {
        /**
         * 修改我的爱车成功
         */
        public void onUpdateMyCarSuccess();
    }

    BindingMyCarListener mBindingMyCarListener;//绑定我的爱车
    QueryMyCarListener mQueryMyCarListener;//查询我的爱车信息
    UnBindingMyCarListener mUnBindingMyCarListener;//解绑我的爱车
    UpdateMyCarListener mUpdateMyCarListener;//修改我的爱车信息

    public void setBindingMyCarListener(BindingMyCarListener bindingMyCarListener) {
        this.mBindingMyCarListener = bindingMyCarListener;
    }
    public void setQueryMyCarListener(QueryMyCarListener queryMyCarListener) {
        this.mQueryMyCarListener = queryMyCarListener;
    }
    public void setUnBindingMyCarListener(UnBindingMyCarListener unBindingMyCarListener) {
        this.mUnBindingMyCarListener = unBindingMyCarListener;
    }
    public void setUpdateMyCarListener(UpdateMyCarListener updateMyCarListener) {
        this.mUpdateMyCarListener = updateMyCarListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, CarManager.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case BINDINGMYCAR://1
                if (mBindingMyCarListener != null) {
                    mBindingMyCarListener.onTimeOut();
                }
                break;
            case QUERYMYCAR://2
                if (mQueryMyCarListener != null) {
                    mQueryMyCarListener.onTimeOut();
                }
                break;
            case UNBINDINGMYCAR://3
                if (mUnBindingMyCarListener != null) {
                    mUnBindingMyCarListener.onTimeOut();
                }
                break;
            case UPDATEMYCAR://4
                if (mUpdateMyCarListener != null) {
                    mUpdateMyCarListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case BINDINGMYCAR://1
                if (mBindingMyCarListener != null) {
                    if (hasDSPTError) {
                        mBindingMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mBindingMyCarListener.onBindingMyCarSuccess();
                        } else {
                            String msg = jsonObject.getString("msg");
                            mBindingMyCarListener.onFail(respCode, msg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case QUERYMYCAR://2
                if (mQueryMyCarListener != null) {
                    if (hasDSPTError) {
                        mQueryMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        List<QueryMyCarInfoResult> list = null;
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray jsonArray = jsonObject.getJSONArray("data");
                            int size = jsonArray.length();
                            if (size>0){
                                list = new ArrayList<QueryMyCarInfoResult>(size);
                            }
                            for (int i= 0; i < size; i++){
                                JSONObject dataObject = jsonArray.getJSONObject(i);
                                QueryMyCarInfoResult info = new QueryMyCarInfoResult();
                                if (dataObject.has("carPlateNo")) {
                                    info.setCarPlateNo(dataObject.getString("carPlateNo"));
                                }
                                if (dataObject.has("carBrand")) {
                                    info.setCarBrand(dataObject.getString("carBrand"));
                                }
                                if (dataObject.has("carType")) {
                                    info.setCarType(dataObject.getString("carType"));
                                }
                                if (dataObject.has("fuelType")) {
                                    info.setFuelType(dataObject.getString("fuelType"));
                                }
                                if (dataObject.has("engineNo")) {
                                    info.setEngineNo(dataObject.getString("engineNo"));
                                }
                                if (dataObject.has("carVin")) {
                                    info.setCarVin(dataObject.getString("carVin"));
                                }
                                if (dataObject.has("buyTime")) {
                                    info.setBuyTime(dataObject.getString("buyTime"));
                                }
                                if (dataObject.has("insuranceTime")) {
                                    info.setInsuranceTime(dataObject.getString("insuranceTime"));
                                }
                                if (dataObject.has("isMajor")) {
                                    info.setIsMajor(dataObject.getInt("isMajor"));
                                }
                                list.add(info);
                            }
                            mQueryMyCarListener.onQueryMyCar(list);
                        } else {
                            String msg = jsonObject.getString("msg");
                            mQueryMyCarListener.onFail(respCode, msg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UNBINDINGMYCAR://3
                if (mUnBindingMyCarListener != null) {
                    if (hasDSPTError) {
                        mUnBindingMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUnBindingMyCarListener.onUnBindingMyCarSuccess();
                        } else {
                            String msg = jsonObject.getString("msg");
                            mUnBindingMyCarListener.onFail(respCode, msg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATEMYCAR:
                if (mUpdateMyCarListener != null) {
                    if (hasDSPTError) {
                        mUpdateMyCarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateMyCarListener.onUpdateMyCarSuccess();
                        } else {
                            String msg = jsonObject.getString("msg");
                            mUpdateMyCarListener.onFail(respCode, msg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mUpdateMyCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
