package com.mxnavi.tspv2.catering;

import com.google.gson.Gson;
import com.mxnavi.tspv2.catering.model.GetDetailResult;
import com.mxnavi.tspv2.catering.model.QueryBusinessesInfo;
import com.mxnavi.tspv2.catering.model.QueryBusinessesResult;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/5/23.
 */

public class Catering extends BaseTsp {
    // 请求服务类型为餐饮服务
    final int thisServiceType = Config.MxNGTPServiceType.CATERING.getServiceType();
    // 餐饮服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.CATERING.getServiceVer();

    public Catering() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int QUERYBUSINESSES = 1;//店铺查询列表
    final int GETDETAIL = 2;//店铺详细查询

    /**
     * 店铺查询列表
     * @param queryBusinessesInfo
     * @return
     */
    public int queryBusinesses(QueryBusinessesInfo queryBusinessesInfo) {
        return queryBusinesses(queryBusinessesInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 店铺查询列表
     * @param queryBusinessesInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @return
     */
    public int queryBusinesses(QueryBusinessesInfo queryBusinessesInfo, int timeOutSecs) {
        return queryBusinesses(queryBusinessesInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 店铺查询列表
     * @param queryBusinessesInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int queryBusinesses(QueryBusinessesInfo queryBusinessesInfo, int timeOutSecs, int priorityLevel) {
        if (queryBusinessesInfo.getDeviceNo()==null
                ||queryBusinessesInfo.getLatLng()==null) {
            if (mQueryBusinessesListener != null) {
                mQueryBusinessesListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "queryBusinesses";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("deviceNo", queryBusinessesInfo.getRadius());
        params.put("latitude", queryBusinessesInfo.getLatLng().getLatitude());
        params.put("longitude", queryBusinessesInfo.getLatLng().getLongitude());
        if (queryBusinessesInfo.getRadius()!=null){
            params.put("radius", queryBusinessesInfo.getRadius());
        }
        if (queryBusinessesInfo.getKeyword()!=null){
            params.put("keyword",queryBusinessesInfo.getKeyword());
        }
        if (queryBusinessesInfo.getLimit()!=null){
            params.put("limit",queryBusinessesInfo.getLimit());
        }
        if (queryBusinessesInfo.getPage()!=null){
            params.put("page",queryBusinessesInfo.getPage());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, QUERYBUSINESSES);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 店铺详细查询
     * @param deviceNo 设备号
     * @param businessId 店铺ID
     * @return
     */
    public int getDetail(String deviceNo, String businessId) {
        return getDetail(deviceNo, businessId, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 店铺详细查询
     * @param deviceNo 设备号
     * @param businessId 店铺ID
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @return
     */
    public int getDetail(String deviceNo, String businessId, int timeOutSecs) {
        return getDetail(deviceNo, businessId, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 店铺详细查询
     * @param deviceNo 设备号
     * @param businessId 店铺ID
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getDetail(String deviceNo, String businessId, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(deviceNo) || Utils.strIsEmpty(businessId)) {
            if (mGetDetailListener != null) {
                mGetDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getDetail";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("deviceNo", deviceNo);
        params.put("businessId", businessId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETDETAIL);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, Catering.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    public interface QueryBusinessesListener extends CallBackListener {
        public void onQueryBusinesses(int requestId, QueryBusinessesResult queryBusinessesResult);
    }
    public interface GetDetailListener extends CallBackListener {
        public void onGetDetail(int requestId, GetDetailResult record);
    }

    QueryBusinessesListener mQueryBusinessesListener;
    GetDetailListener mGetDetailListener;


    public void setQueryBusinessesListener(QueryBusinessesListener queryBusinessesListener) {
        this.mQueryBusinessesListener = queryBusinessesListener;
    }
    public void setGetDetailListener(GetDetailListener getDetailListener) {
        this.mGetDetailListener = getDetailListener;
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case QUERYBUSINESSES://1
                if (mQueryBusinessesListener != null) {
                    mQueryBusinessesListener.onTimeOut();
                }
                break;
            case GETDETAIL://2
                if (mGetDetailListener != null) {
                    mGetDetailListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case QUERYBUSINESSES:
                if (mQueryBusinessesListener != null) {
                    if (hasDSPTError) {
                        mQueryBusinessesListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response.replaceAll("\n", ""));
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            QueryBusinessesResult queryBusinessesResult = gson.fromJson(response, QueryBusinessesResult.class);
                            mQueryBusinessesListener.onQueryBusinesses(requestHandleId,queryBusinessesResult);
                        } else {
                            mQueryBusinessesListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryBusinessesListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETDETAIL:
                if (mGetDetailListener != null) {
                    if (hasDSPTError) {
                        mGetDetailListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response.replaceAll("\n", ""));
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            GetDetailResult record = gson.fromJson(jsonObject.getString("record"), GetDetailResult.class);
                            mGetDetailListener.onGetDetail(requestHandleId,record);
                        } else {
                            mGetDetailListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
