package com.mxnavi.tspv2.core;

import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;

import com.google.protobuf.ByteString;
import com.mxnavi.tspv2.utils.Utils;

import java.util.concurrent.ConcurrentHashMap;

/**
 * Created by Administrator on 2016/3/30.
 */
abstract public  class BaseTsp {
    protected static final int ERROR_MSG_TYPE =1;
    protected  static final int STRING_MSG_TYPE =2;
    protected static final int BYTE_MSG_TYPE =3;
    protected static final int NOTIFY_MSG_TYPE =4;
    protected static final int MESSAGE_TIMEOUT =0;
    protected static final int HTTPERROR_MSG_TYPE = 5;
    protected static final int ONPROGRESS_MSG_TYPE =6;
    protected static ConcurrentHashMap<Integer,BaseTsp> requestMap= new ConcurrentHashMap<Integer,BaseTsp>();
    protected  void addCallbackMap(int eventId, BaseTsp baseTsp){
        synchronized (requestMap){
                requestMap.put(eventId, baseTsp);
                setUseSynchronousMode(false);
        }
    }
    protected  void removeCallbackMap(int eventId){
        synchronized (requestMap) {
                requestMap.remove(eventId);
        }
    }
    static public void cancelRequest(int requestId){
        RequestDispathUtil.cancelRequest(requestId);
    }
    abstract protected void callTimeOut(int requestHandleId);
    abstract protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errorMsg);
    protected void onProgress(int requestHandleId,long bytesWritten, long totalSize) {

    }
    protected void onCallBack(int requestHandleId, ByteString response,boolean hasDSPTError,int errorCode,String errorMsg){

    }

    /**
     * 发送返回消息
     * @param eventId
     * @param msgType
     * @param responseObj
     */
    public void sendResponseMsg(int eventId,int msgType,Object responseObj ){
        Message msg = this.obtainMessage(eventId,msgType,responseObj);
        if(!synchronousMode&& this.handler != null) {
            if(!Thread.currentThread().isInterrupted()) {
                this.handler.sendMessage(msg);
            }
        } else {
            this.thisHandleMessage(msg);
        }
    }
    private Message obtainMessage(int eventId,int msgType, Object responseMessageData) {
        Message msg;
        if(this.handler == null) {
            msg = Message.obtain();
            if(msg != null) {
                msg.arg1 = eventId;
                msg.what = msgType;
                msg.obj = responseMessageData;
            }
        } else {
            msg = Message.obtain(this.handler, msgType, responseMessageData);
            msg.arg1 = eventId;
        }

        return msg;
    }
    private  class TSPHandler extends Handler {

        public void handleMessage(Message msg) {
            super.handleMessage(msg);
            thisHandleMessage(msg);
        }

    }
    private Handler handler;
    private boolean synchronousMode = false;
    // 如果在主线程中new 查询实例，则在主线程中回调，否则，在netty子线程中回调
    private void setUseSynchronousMode(boolean value) {
        if(!value && Looper.myLooper() == null) {
            value = true;
//            Utils.logd("BaseTsp Current thread has not called Looper.prepare(). so callback in netty thread ");
//            Log.w("AsyncHttpResponseHandler", "Current thread has not called Looper.prepare(). Forcing synchronous mode.");
        }

        if(!value && this.handler == null) {
            this.handler = new TSPHandler();
        } else if(value && this.handler != null) {
            this.handler = null;
        }

        this.synchronousMode = value;
    }

    private void thisHandleMessage(Message msg) {
        int requstId = msg.arg1;
        switch(msg.what){
            case MESSAGE_TIMEOUT:

//                BaseTsp baseTsp0 =  BaseTsp.requestMap.get(requstId);
//                if(baseTsp0!=null){
//                    baseTsp0.callTimeOut(requstId);
//                }
                callTimeOut(requstId);
                // 取消超时的请求
//                TcpRequestUtil.cancelRequest(requstId);
                break;
            case ERROR_MSG_TYPE:

//                BaseTsp baseTsp =  BaseTsp.requestMap.get(requstId);
//                if(baseTsp!=null){
//                    String errorMsg =(String) msg.obj;
//                    baseTsp.onCallBack(requstId,"",true,Constants.ERROR_CODE_DSPT,errorMsg);
//                }
                String errorMsg =(String) msg.obj;
                onCallBack(requstId, "", true, Constants.ERROR_CODE_DSPT, errorMsg);
                break;
            case STRING_MSG_TYPE:

//                BaseTsp baseTsp1 =  BaseTsp.requestMap.get(requstId);
//                if(baseTsp1!=null){
//                    String responseTxt =(String) msg.obj;
//                    baseTsp1.onCallBack(requstId,responseTxt,false,0,null);
//                }
                String responseTxt =(String) msg.obj;
                onCallBack(requstId, responseTxt, false, 0, null);
                break;
            case BYTE_MSG_TYPE:
//                BaseTsp baseTsp2 =  BaseTsp.requestMap.get(requstId);
//                if(baseTsp2!=null){
//                    ByteString serviceByteData =(ByteString) msg.obj;
//                    baseTsp2.onCallBack(requstId,serviceByteData,false,0,null);
//                }
                ByteString serviceByteData =(ByteString) msg.obj;
                onCallBack(requstId,serviceByteData,false,0,null);
                break;
            case HTTPERROR_MSG_TYPE:// http连接错误
//                 errorMsg =(String) msg.obj;
                 onCallBack(requstId, "", true, Constants.ERROR_CODE_NETCONNECT_ERROR,Constants.ERROR_MESSAGE_NETCONNECT_ERROR);
                break;
            case NOTIFY_MSG_TYPE:
//                    if(ResponseDispathUtil.mNoticeListener != null){
//                        String responseTxt =(String) msg.obj;
//                        ResponseDispathUtil.mNoticeListener.onReceiveNotice(responseTxt);
//                    }
                break;
            case ONPROGRESS_MSG_TYPE:
                Bundle bundle = (Bundle) msg.obj;
                onProgress(requstId,bundle.getLong("bytesWritten"),bundle.getLong("totalSize"));
                break;


        }

    }
}
