package com.mxnavi.tspv2.core;


import android.util.Log;

import com.google.protobuf.ByteString;
import com.google.protobuf.GeneratedMessage;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.DSPTMessageType;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.MXMessageType;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.MxMessage;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPDispatcherData;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPDispatcherSignature;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPMessage;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPMessageHeader;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPMessageSignature;
import com.mxnavi.tspv2.core.message.MxNGTPMessage.NGTPServiceData;
import com.mxnavi.tspv2.model.InitializeInfo;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.utils.Utils;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;

public class MessageBuilder {
//    private static final String VIN = "1234567890";
    private String VIN = Config.VIN;
    private Integer CUSTOMER_KEY = Config.CUSTOMER_KEY;
    private String SECRETKEY = Config.SECRETKEY;
    private String PURPOSE = "0";
    private String APPID = Config.APPID;

    public static final short ENCODINGTYPE_PROTOBUF =0;
    public static final short ENCODINGTYPE_JSON_UTF8 =1;
    public  static final short ENCODINGTYPE_PLAIN_TEXT =2;
    public  static final short ENCODINGTYPE_FORM_DATA =3;
    /**
     * 没有子服务类型时，传1
     */
    public static final int NON_SUB_SERVICE_TYPE = 1;

    public final int LOGIN_EVENT_ID = -100;

    private  InitializeInfo mInitializeInfo;

    final int ProtoVer = 0x09;

//    public static  String token;
//    public static  String userId;
    /**
     * 推送三方ID
     */
    public  static String PUSH_KEY = "";

    public void setInitializeInfo( InitializeInfo initializeInfo){
        mInitializeInfo = initializeInfo;
        VIN = mInitializeInfo.getVin();
        CUSTOMER_KEY = mInitializeInfo.getCustomerKey();
        SECRETKEY = mInitializeInfo.getSecretKey();
        APPID = mInitializeInfo.getAppId();
    }

    /**
     * conti项目专用
     * @param customerKey
     * @param secretKey
     * @param vin
     */
    public void reSetInitializeInfo(int customerKey,String secretKey,String vin){
        VIN = vin;
        CUSTOMER_KEY = customerKey;
        SECRETKEY = secretKey;

    }

    private NGTPMessageHeader.Builder messageHeader = NGTPMessageHeader.newBuilder();
    NGTPMessageHeader.Builder messageHeaderForPhone = NGTPMessageHeader.newBuilder();
    private MessageBuilder() {
        makeMsgHeader();
        makeMsgHeaderForPhone();
    }


    private static MessageBuilder instance = new MessageBuilder();
    public static MessageBuilder instance() {
        return instance;
    }

    ////////////////////////////////////////////////////////////////////////////////////////////

    public void init(String vin, int customerKey, String secretKey, String purpose) {
        VIN = vin;
        CUSTOMER_KEY = customerKey;
        SECRETKEY = secretKey;
        PURPOSE = purpose;
    }

    public MxMessage makeHeartbeatMessage() {
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.HEARTBEAT);
        return mxMessage.build();
    }

    public MxMessage makeLoginMessage() {
        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
        // Message Header

        // DispatcherData
        NGTPDispatcherData.Builder dispatcherData = makeDispatcherData(Config.MxNGTPServiceType.MX_ROUTE_CALC.getServiceType(), 0x0100, DSPTMessageType.LOGIN, eventCreateTime,ENCODINGTYPE_PLAIN_TEXT);
        dispatcherData.setEventId(LOGIN_EVENT_ID);//登录特有的EventId
        // Dispatcher的签名(MD5加密)
        NGTPDispatcherSignature.Builder dispatcherSig = makeDispatcherSig(dispatcherData, eventCreateTime);
        // 构造NGTP消息
        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
        ngtpMessage.setMessageHeader(messageHeader);
        ngtpMessage.setDispatcherData(dispatcherData);
        ngtpMessage.setDispatcherSignature(dispatcherSig);

        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.BUSINESS);
        mxMessage.setNgtpMessage(ngtpMessage);
        return mxMessage.build();
    }
//   public MxMessage makeRequestBusinessMsg(int eventId,String requestData,int serviceType,int serviceVer){
//       Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
//       NGTPDispatcherData.Builder dispatcherData = makeDispatcherData(serviceType, serviceVer, DSPTMessageType.UP_SERVICEDATA, eventCreateTime,ENCODINGTYPE_PLAIN_TEXT);
//       dispatcherData.setEventId(eventId);
//
//
//       // Dispatcher的签名(MD5加密)
//       NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSig(dispatcherData,eventCreateTime);
//       // ServiceData（AES加密）
//       NGTPServiceData.Builder serviceData =  makeServiceData(requestData);
//       // 整个消息的签名(MD5加密)
//       NGTPMessageSignature.Builder  serviceSig = makeServiceDataSig(requestData.getBytes());
//       // 构造NGTP消息
//       NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
//       ngtpMessage.setMessageHeader(messageHeader);
//       ngtpMessage.setDispatcherData(dispatcherData);
//       ngtpMessage.setDispatcherSignature(dispatcherSig);
//       ngtpMessage.setServiceData(serviceData);
//       ngtpMessage.setMessageSignature(serviceSig);
//       MxMessage.Builder mxMessage = MxMessage.newBuilder();
//       mxMessage.setMessageType(MXMessageType.BUSINESS);
//       mxMessage.setNgtpMessage(ngtpMessage);
//       return mxMessage.build();
//   }

    /**
     * 手机发送请求消息(未登录前调用，不需要验证token)
     * @param filePath
     * @param eventId 随机事件id
     * @param requestData 请求数据
     * @param serviceType 服务类型
     * @param serviceVer 服务版本
     * @param service_data_encoding
     * @param subServiceType
     * @return
     */
    public MxMessage makeRequestBusinessMsgBeforeLoginByPhone(String filePath, int eventId, String requestData, int serviceType, int serviceVer, short service_data_encoding, int subServiceType){
        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
        NGTPDispatcherData.Builder dispatcherData;
        dispatcherData = makeDispatcherData(serviceType, serviceVer, DSPTMessageType.LOGIN, eventCreateTime,service_data_encoding,subServiceType,null);

        dispatcherData.setEventId(eventId);

        // Dispatcher的签名(MD5加密)
        NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSig(dispatcherData,eventCreateTime);
//        NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSigForPhone(null,eventCreateTime,2);
        // ServiceData（AES加密）
        NGTPServiceData.Builder serviceData =  makeServiceData(requestData);
        // 整个消息的签名(MD5加密)
        NGTPMessageSignature.Builder  serviceSig = makeServiceDataSig(requestData.getBytes());
        // 构造NGTP消息
        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
        ngtpMessage.setMessageHeader(messageHeaderForPhone);
        ngtpMessage.setDispatcherData(dispatcherData);
        ngtpMessage.setDispatcherSignature(dispatcherSig);
        ngtpMessage.setServiceData(serviceData);
        if (service_data_encoding == ENCODINGTYPE_FORM_DATA) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] b = new byte[4096];
            int n;
            try {
                if (!Utils.strIsEmpty(filePath)){
                    File file = new File(filePath);
                    FileInputStream fis = new FileInputStream(file);
                    while ((n = fis.read(b)) != -1)
                    {
                        bos.write(b, 0, n);
                    }
                    MxNGTPMessage.NGTPFormData.Builder formData =  makeFormData("file",Utils.getUUID().replace("-",""), bos.toByteArray());
                    ngtpMessage.addFormData(formData);
                    fis.close();
                    bos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
                Utils.loge(Log.getStackTraceString(e));
            }
        }
        ngtpMessage.setMessageSignature(serviceSig);
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.BUSINESS);
        mxMessage.setNgtpMessage(ngtpMessage);

        return mxMessage.build();
    }
    /**
     * 手机发送请求消息
     * @param eventId 随机事件id
     * @param requestData 请求数据
     * @param serviceType 服务类型
     * @param serviceVer 服务版本
     * @return
     */
    public MxMessage makeRequestBusinessMsgByPhone(String filePath, int eventId,String userId,String token, String requestData, int serviceType, int serviceVer, short service_data_encoding, int subServiceType){
        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
        NGTPDispatcherData.Builder dispatcherData;
        dispatcherData = makeDispatcherData(serviceType, serviceVer, DSPTMessageType.MOBILE_REQ, eventCreateTime,service_data_encoding,subServiceType,userId);

        dispatcherData.setEventId(eventId);

        // Dispatcher的签名(MD5加密)
        NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSigForPhone(token,eventCreateTime,2);
        // ServiceData（AES加密）
        NGTPServiceData.Builder serviceData =  makeServiceData(requestData);
        // 整个消息的签名(MD5加密)
        NGTPMessageSignature.Builder  serviceSig = makeServiceDataSig(requestData.getBytes());
        // 构造NGTP消息
        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
        ngtpMessage.setMessageHeader(messageHeaderForPhone);
        ngtpMessage.setDispatcherData(dispatcherData);
        ngtpMessage.setDispatcherSignature(dispatcherSig);
        ngtpMessage.setServiceData(serviceData);
        if (service_data_encoding == ENCODINGTYPE_FORM_DATA) {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            byte[] b = new byte[4096];
            int n;
            try {
                if (!Utils.strIsEmpty(filePath)){
                    File file = new File(filePath);
                    FileInputStream fis = new FileInputStream(file);
                    while ((n = fis.read(b)) != -1)
                    {
                        bos.write(b, 0, n);
                    }
                    MxNGTPMessage.NGTPFormData.Builder formData =  makeFormData("file",Utils.getUUID().replace("-",""), bos.toByteArray());
                    ngtpMessage.addFormData(formData);
                    fis.close();
                    bos.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
                Utils.loge(Log.getStackTraceString(e));
            }
        }
        ngtpMessage.setMessageSignature(serviceSig);
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.BUSINESS);
        mxMessage.setNgtpMessage(ngtpMessage);

        return mxMessage.build();
    }
    /**
     * 手机发送请求消息
     * @param eventId 随机事件id
     * @param requestData 请求数据
     * @param serviceType 服务类型
     * @param serviceVer 服务版本
     * @return
     */
    public MxMessage makeRequestBusinessMsgByPhone(int eventId,String userId,String token,String requestData,int serviceType,int serviceVer,short service_data_encoding,int subServiceType){
        return makeRequestBusinessMsgByPhone(null,eventId,userId,token,requestData,serviceType,serviceVer,service_data_encoding,subServiceType);
    }
    public MxMessage makeRequestBusinessMsgBeforeLoginByPhone(int eventId,String requestData,int serviceType,int serviceVer,short service_data_encoding,int subServiceType){
        return makeRequestBusinessMsgBeforeLoginByPhone(null,eventId, requestData,serviceType,serviceVer,service_data_encoding,subServiceType);
    }
    public MxMessage makeRequestBusinessMsg(int eventId,String requestData,int serviceType,int serviceVer){
        return makeRequestBusinessMsg(eventId,requestData,serviceType,serviceVer,ENCODINGTYPE_PLAIN_TEXT);
    }
    public MxMessage makeRequestBusinessUTF8Msg(int eventId,String requestData,int serviceType,int serviceVer){
        return makeRequestBusinessMsg(eventId,requestData,serviceType,serviceVer,ENCODINGTYPE_JSON_UTF8);
    }
    private MxMessage makeRequestBusinessMsg(int eventId,String requestData,int serviceType,int serviceVer,int encodingType){
        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
        NGTPDispatcherData.Builder dispatcherData = makeDispatcherData(serviceType, serviceVer, DSPTMessageType.UP_SERVICEDATA, eventCreateTime,encodingType);
        dispatcherData.setEventId(eventId);


        // Dispatcher的签名(MD5加密)
        NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSig(dispatcherData,eventCreateTime);
        // ServiceData（AES加密）
        NGTPServiceData.Builder serviceData =  makeServiceData(requestData);
        // 整个消息的签名(MD5加密)
        NGTPMessageSignature.Builder  serviceSig = makeServiceDataSig(requestData.getBytes());
        // 构造NGTP消息
        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
        ngtpMessage.setMessageHeader(messageHeader);
        ngtpMessage.setDispatcherData(dispatcherData);
        ngtpMessage.setDispatcherSignature(dispatcherSig);
        ngtpMessage.setServiceData(serviceData);
        ngtpMessage.setMessageSignature(serviceSig);
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.BUSINESS);
        mxMessage.setNgtpMessage(ngtpMessage);
        return mxMessage.build();
    }
    public MxMessage makeRequestBusinessProtoBufMsg(int eventId,GeneratedMessage.ExtendableMessage requestProtoBuf,int serviceType,int serviceVer,int subServiceType){
        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
        NGTPDispatcherData.Builder dispatcherData = makeDispatcherData(serviceType, serviceVer, DSPTMessageType.UP_SERVICEDATA, eventCreateTime,ENCODINGTYPE_PROTOBUF,subServiceType,null);
        dispatcherData.setEventId(eventId);
        if(serviceType == Config.MxNGTPServiceType.MX_UPLOAD.getServiceType()){
            dispatcherData.setDsptMessageType(DSPTMessageType.UPLOAD_TRACK);
        }

        // Dispatcher的签名(MD5加密)
        NGTPDispatcherSignature.Builder dispatcherSig =  makeDispatcherSig(dispatcherData,eventCreateTime);
        // ServiceData 不加密
//        NGTPServiceData.Builder serviceData =  makeServiceData(requestData);
        NGTPServiceData.Builder serviceData = NGTPServiceData.newBuilder();
        serviceData.setServiceData(requestProtoBuf.toByteString());
        // 整个消息的签名(MD5加密)
        NGTPMessageSignature.Builder  serviceSig = makeServiceDataSig(requestProtoBuf.toByteString().toByteArray());
        // 构造NGTP消息
        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
        ngtpMessage.setMessageHeader(messageHeader);
        ngtpMessage.setDispatcherData(dispatcherData);
        ngtpMessage.setDispatcherSignature(dispatcherSig);
        ngtpMessage.setServiceData(serviceData);
        ngtpMessage.setMessageSignature(serviceSig);
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        mxMessage.setMessageType(MXMessageType.BUSINESS);
        mxMessage.setNgtpMessage(ngtpMessage);
        return mxMessage.build();
    }
    private  NGTPDispatcherSignature.Builder makeDispatcherSigForPhone( String token,Integer eventCreateTime,int type) {
        NGTPDispatcherSignature.Builder dispatcherSig = NGTPDispatcherSignature.newBuilder();
        try {

            ByteString dispSig = null;
//            if (!Config.isLogin||type==1){
//                String srcDispSig = CUSTOMER_KEY.toString() + eventCreateTime.toString() + SECRETKEY;
//                dispSig = ByteString.copyFrom(Utils.getMD5Byte(srcDispSig.getBytes()));
//            }else {
//            Utils.loge(TAG,"makeDispatcherSig  token:"+token);
                if (token != null){
                    dispSig = ByteString.copyFrom(token.getBytes());
                }

//            Utils.loge(TAG,"makeDispatcherSig token:"+token);
//            }

            dispatcherSig.setNgtpDispatcherSignature(dispSig);



        }catch (Exception e){
            e.printStackTrace();


        }
        return dispatcherSig;

    }
//    public MxMessage makeBusinessMsg(TestCaseInfo testCaseInfo, DSPTMessageType dsptType) {
//        Integer eventCreateTime = (int) System.currentTimeMillis() / 1000;
//        // Message Header
//        makeMsgHeader();
//        // DispatcherData
//        makeDispatcherData(testCaseInfo.getServiceType(), testCaseInfo.getServiceVer(), dsptType, eventCreateTime);
//        // Dispatcher的签名(MD5加密)
//        makeDispatcherSig(eventCreateTime);
//        // ServiceData（AES加密）
//        makeServiceData(testCaseInfo.getReqMethod());
//        // 整个消息的签名(MD5加密)
//        makeServiceDataSig(testCaseInfo.getReqMethod().getBytes());
//        // 构造NGTP消息
//        NGTPMessage.Builder ngtpMessage = NGTPMessage.newBuilder();
//        ngtpMessage.setMessageHeader(messageHeader);
//        ngtpMessage.setDispatcherData(dispatcherData);
//        ngtpMessage.setDispatcherSignature(dispatcherSig);
//        ngtpMessage.setServiceData(serviceData);
//        ngtpMessage.setMessageSignature(serviceSig);
//        MxMessage.Builder mxMessage = MxMessage.newBuilder();
//        mxMessage.setMessageType(MXMessageType.BUSINESS);
//        mxMessage.setNgtpMessage(ngtpMessage);
//        return mxMessage.build();
//    }

    // ///////////////////////////////////////////////////////////////////////////////////////////////////////
    private  NGTPDispatcherSignature.Builder makeDispatcherSig( NGTPDispatcherData.Builder dispatcherData,Integer eventCreateTime) {
        String srcDispSig = CUSTOMER_KEY.toString() + eventCreateTime.toString() + SECRETKEY;
//        System.out.println("srcDispSig=" + srcDispSig);
        ByteString dispSig = ByteString.copyFrom(Utils.getMD5Byte(srcDispSig));
        NGTPDispatcherSignature.Builder dispatcherSig = NGTPDispatcherSignature.newBuilder();
        dispatcherSig.setNgtpDispatcherSignature(dispSig);
        return dispatcherSig;
    }
    public  NGTPDispatcherData.Builder makeDispatcherData( int serviceType,
                                                            int servicever,
                                                            DSPTMessageType dsptMessageType,
                                                            Integer eventCreateTime,int encodingType,int subServiceType,String userId){
        NGTPDispatcherData.Builder dispatcherData = NGTPDispatcherData.newBuilder();
//        MxNGTPMessage.ServiceType serviceType = MxNGTPMessage.ServiceType.valueOf(serviceTypeNum);
//        if (serviceType != null) {
            dispatcherData.setServiceType(serviceType); // 服务类型
//        }
        dispatcherData.setSubServiceType(subServiceType);//子服务类型
        dispatcherData.setVin(VIN); // 车辆唯一标识号
        dispatcherData.setEventCreationTime(eventCreateTime); // Event创建时间(从1970年开始经过的秒)
        if (dsptMessageType != null) { // DSPT消息类型
            dispatcherData.setDsptMessageType(dsptMessageType);
        } else {
            dispatcherData.setDsptMessageType(DSPTMessageType.UP_SERVICEDATA);
        }
        dispatcherData.setDownlinkCounter(0); // 下行消息条数
        dispatcherData.setUplinkCounter(1); // 上行消息条数
        // 当前位置
        long lon = 0;
        long lat = 0;
        if(mInitializeInfo!=null&&mInitializeInfo.getPositionListener()!=null){
            LatLng latLng = mInitializeInfo.getPositionListener().geLatLng();
            lon = Utils.converDoubleCoordinate2MX(latLng.getLongitude());
            lat = Utils.converDoubleCoordinate2MX(latLng.getLatitude());
        }
        long lonlat = (lon << 32) | lat;
//        System.out.println("lonlat=" + lonlat);
        dispatcherData.setBasicPosition(lonlat);
        dispatcherData.setServiceDataEncoding(encodingType); // ServiceData编码(plaintext)
        dispatcherData.setServiceDataProtocolVersion(servicever); // ServiceData协议版本
        dispatcherData.setCustomerKey(CUSTOMER_KEY); // 客户标识
        dispatcherData.setPurpose(Integer.parseInt(PURPOSE));
        if (!Utils.strIsEmpty(PUSH_KEY)){
            dispatcherData.setPushKey(PUSH_KEY);
        }
        if (APPID != null){
            dispatcherData.setAppId(APPID);
        }
        
//        if (Config.isLogin){
            if (userId!=null){
                dispatcherData.setUserId(userId);
            }
//        }
        return dispatcherData;
    }
    /**
     *
     * @param serviceType
     * @param servicever
     * @param dsptMessageType
     * @param eventCreateTime
     * @param encodingType      * ServiceData编码(0) 0=protobuf 1=JSON UTF-8 2=plain text
     * @return
     */
   public  NGTPDispatcherData.Builder makeDispatcherData(  int serviceType,
                                                  int servicever,
                                                  DSPTMessageType dsptMessageType,
                                                  Integer eventCreateTime,int encodingType){
            return makeDispatcherData(serviceType, servicever, dsptMessageType, eventCreateTime, encodingType,1,null);
    }
//    private void makeDispatcherData(int serviceTypeNum,
//            int servicever,
//            DSPTMessageType dsptMessageType,
//            Integer eventCreateTime) {
//        MxNGTPMessage.ServiceType serviceType = MxNGTPMessage.ServiceType.valueOf(serviceTypeNum);
//        if (serviceType != null) {
//            dispatcherData.setServiceType(serviceType); // 服务类型
//        }
//        dispatcherData.setVin(VIN); // 车辆唯一标识号
//        dispatcherData.setEventCreationTime(eventCreateTime); // Event创建时间(从1970年开始经过的秒)
//        if (dsptMessageType != null) { // DSPT消息类型
//            dispatcherData.setDsptMessageType(dsptMessageType);
//        } else {
//            dispatcherData.setDsptMessageType(DSPTMessageType.UP_SERVICEDATA);
//        }
//        dispatcherData.setDownlinkCounter(0); // 下行消息条数
//        dispatcherData.setUplinkCounter(1); // 上行消息条数
//        // 当前位置
//        long lon = 1137220209;
//        long lat = 385105582;
//        long lonlat = (lon << 32) | lat;
////        System.out.println("lonlat=" + lonlat);
//        dispatcherData.setBasicPosition(lonlat);
//        dispatcherData.setServiceDataEncoding(2); // ServiceData编码(plaintext)
//        dispatcherData.setServiceDataProtocolVersion(servicever); // ServiceData协议版本
//        dispatcherData.setCustomerKey(CUSTOMER_KEY); // 客户标识
//        dispatcherData.setPurpose(Integer.parseInt(PURPOSE));
//    }

    private void makeMsgHeader() {
        // MessageHeader
        messageHeader.setProtoVer(ProtoVer); // 0.9
        messageHeader.setSecurityContext(0x1212); // 00010010,00010010
        messageHeader.setEncoding(0x00); // protobuf
    }
    private void makeMsgHeaderForPhone() {
        // MessageHeader

        messageHeaderForPhone.setProtoVer(256); // 0.9
        messageHeaderForPhone.setSecurityContext(0x1212); // 00010010,00010010
        messageHeaderForPhone.setEncoding(0x00); // protobuf
    }
    private NGTPMessageSignature.Builder  makeServiceDataSig(byte[] srcData) {
        NGTPMessageSignature.Builder  serviceSig = NGTPMessageSignature.newBuilder();
        ByteString sdataSig = ByteString.copyFrom(Utils.getMD5Byte(srcData));
        serviceSig.setNgtpMessageSignature(sdataSig);
        return serviceSig;
    }

    private  NGTPServiceData.Builder  makeServiceData(String data) {
        NGTPServiceData.Builder serviceData = NGTPServiceData.newBuilder();
        try {
            byte[] srcData = data.getBytes("UTF-8");
            byte[] codeData = AESCodec.encrypt4Buf(srcData, SECRETKEY);
            serviceData.setServiceData(ByteString.copyFrom(codeData));
        } catch (Exception e) {
            e.printStackTrace();
        }
        return serviceData;
    }

    public String getMxMessageData(Object obj) throws Exception {
        MxMessage mxMessage = (MxMessage)obj;
        return mxMessage.toString();
    }

    public String getServiceData(MxMessage mxMessage) throws Exception {

        NGTPServiceData serviceData = mxMessage.getNgtpMessage().getServiceData();
        byte[] respSData = serviceData.getServiceData().toByteArray();
//        MxResponseLocationMessageInfo.ResponseLocationMessage msg = MxResponseLocationMessageInfo.ResponseLocationMessage.parseFrom(respSData);
        respSData = AESCodec.decrypt2Buf(respSData, SECRETKEY);
        String respJsonContents = new String(respSData, "UTF-8");
        return respJsonContents;
    }
    public String getServiceData(NGTPMessage obj) throws Exception {
        NGTPServiceData serviceData = obj.getServiceData();
        byte[] respSData = serviceData.getServiceData().toByteArray();
        respSData = AESCodec.decrypt2Buf(respSData, SECRETKEY);
        String respJsonContents = new String(respSData, "UTF-8");
        return respJsonContents;
    }
    public ByteString getServiceByteData(NGTPMessage obj) throws Exception {
        NGTPServiceData serviceData = obj.getServiceData();
        ByteString respSData = serviceData.getServiceData();
        return respSData;
    }
    public int getRequestHandleId (NGTPDispatcherData dispatcherData){
       int event_id = dispatcherData.getEventId();
//        MxNGTPMessage.ServiceType serviceType = mxMessage.getNgtpMessage().getDispatcherData().getServiceType();
        return event_id;
    }
    public int getServiceType (NGTPDispatcherData dispatcherData){
       return dispatcherData.getServiceType();
    }
    public MxMessage decode(byte[] respData) throws Exception {
        MxMessage.Builder mxMessage = MxMessage.newBuilder();
        try {
            mxMessage.mergeFrom(new ByteArrayInputStream(respData));
            if (!mxMessage.hasNgtpMessage()) {
                throw new Exception("the NgtpMessage is not exist!");
            }


        } catch (IOException e) {

            throw new Exception(e.getMessage());
        }
        return mxMessage.build();
    }
    /**
     * 制作Form表单数据
     * @param type
     * @param name
     * @param data
     */
    private  MxNGTPMessage.NGTPFormData.Builder makeFormData(String type, String name, byte[] data) {
        MxNGTPMessage.NGTPFormData.Builder formData = MxNGTPMessage.NGTPFormData.newBuilder();
        formData.setFormType(type);
        formData.setFormName(name);
        formData.setFormData(ByteString.copyFrom(data));
        return  formData;
    }
}
