package com.mxnavi.tspv2.feichuan.client;

import android.util.Log;

import com.mxnavi.tspv2.feichuan.client.core.listener.ITcpSender;
import com.mxnavi.tspv2.feichuan.client.core.ServerInfo;
import com.mxnavi.tspv2.feichuan.client.core.TcpClient;
import com.mxnavi.tspv2.feichuan.client.core.UdpSender;
import com.mxnavi.tspv2.feichuan.client.listener.SendFileListener;
import com.mxnavi.tspv2.feichuan.client.model.FileInfo;
import com.mxnavi.tspv2.feichuan.common.Constance;
import com.mxnavi.tspv2.feichuan.common.ExecutorManager;

import java.util.List;

/**
 * Created by Administrator on 2017/12/26.
 */

public class FileSenderManager {

    String TAG = Constance.LOG_TAG;
    private static FileSenderManager serverManager;

    public static FileSenderManager getInstance(){
        if(serverManager == null){
            serverManager =new FileSenderManager();
        }
        return serverManager;
    }
    private FileSenderManager(){
    }
    UdpSender mUdpSender;
    ITcpSender mTcpSender;
    ConnectionTask connectionTask;

    SendFileListener mSendFileListener;

    /**
     * 发送文件
     * @param fileInfoList 文件信息列表
     * @param sendFileListener 文件发送回调
     */
    public void startSendFile(List<FileInfo> fileInfoList, SendFileListener sendFileListener){
        mSendFileListener = sendFileListener;
        if(connectionTask == null){
            Log.e(TAG,"first send");
            connectionTask = new ConnectionTask();
            connectionTask.setFileList(fileInfoList);
            ExecutorManager.INSTANCE.getConnectThread().submit(connectionTask);
        }else if(connectionTask.isDone){// 连接线程已完成
            if(mTcpSender==null || !mTcpSender.isConnected()){// 未连接成功，则重新连接
                Log.e(TAG,"tcp has not connected and then reconnect");
                connectionTask.setFileList(fileInfoList);
                ExecutorManager.INSTANCE.getConnectThread().submit(connectionTask);
            }else {// 已连接成功，则发送
                Log.e(TAG,"tcp has connected and then send files");
                mTcpSender.sendFile(fileInfoList);
            }
        }else{// 还没建立连接完成
            Log.e(TAG,"connect thread has not end and then reset fileInfoList");
            connectionTask.setFileList(fileInfoList);
        }
    }

    /**
     * 暂停传输
     */
    public void pauseSend(){
        if(mTcpSender!=null){
            mTcpSender.pauseSend();
        }
    }

    /**
     * 继续传输
     */
    public void continueSend(){
        if(mTcpSender!=null){
            mTcpSender.continueSend();
        }
    }
    /**
     * 取消传输
     */
    public void cancelSendFile(){
        if(mTcpSender!=null){
            mTcpSender.cancelSend();
        }

    }

    /**
     * 退出传输
     */
    public void exitSendFile(){
        if(mTcpSender!=null){
            mTcpSender.exitSend();
        }
        if(mUdpSender !=null ){
            mUdpSender.stopConnect();
        }

    }

    class ConnectionTask implements Runnable{
        public boolean isDone = false;
        List<FileInfo> fileInfoList;
        public void setFileList(List<FileInfo> fileInfoList){
            this.fileInfoList = fileInfoList;
        }
        public void run(){
            isDone = false;
            mUdpSender = new UdpSender();
            ServerInfo serverInfo = mUdpSender.startUdpScan(mSendFileListener);
            if(serverInfo !=null){
                mTcpSender = new TcpClient(serverInfo,mSendFileListener);
                mTcpSender.connect();
                mTcpSender.sendFile(this.fileInfoList);

            }else{

            }
            isDone = true;
        }

    }


}
