package com.mxnavi.tspv2.feichuan.server.core;

import android.os.Environment;
import android.util.Log;

import com.mxnavi.tspv2.feichuan.common.Constance;
import com.mxnavi.tspv2.feichuan.common.PacketEncoder;
import com.mxnavi.tspv2.feichuan.server.FileReceiveListener;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.RandomAccessFile;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

/**
 * Created by Administrator on 2017/12/26.
 */

public class TcpServer {

    String TAG = Constance.LOG_TAG;
    String completeOneFileReplyStr = "{\"msgType\":\"fileReceive\",\"result\":\""+Constance.RESULT_OK+"\"}";

    ServerSocket serverSocket;
    int port = 0;
    Socket client;
    int MAXLENGTH = 1024*40;
    int MINLENGTH = 100;
    FileReceiveListener mFileReceiveListener;
    RandomAccessFile randomAccessFile;
    public TcpServer(ServerSocket serverSocket,FileReceiveListener fileReceiveListener){
        this.serverSocket = serverSocket;
        mFileReceiveListener = fileReceiveListener;
    }
    String receiveFileName;
    long willRecevicefileSize;
    int hasReceivedFileCount;
    int totalFileCount = 0;


    String path = Environment.getExternalStorageDirectory().getAbsolutePath()+"/feichuan/";

    public void startReceive(){
         hasReceivedFileCount = 0;
         totalFileCount = 0;
        DataOutputStream dos = null;
        DataInputStream dis = null;
        try {
            client = serverSocket.accept();
            client.getInetAddress().getHostAddress();
            client.setTcpNoDelay(true);
            client.setKeepAlive(true);
            client.setSoTimeout(Constance.TCP_READ_TIME_OUT);
            InputStream mInputStream = client.getInputStream();
            dis = new DataInputStream(mInputStream);
            dos = new DataOutputStream(client.getOutputStream());
            receiveData(dis,dos);
        }catch (Exception e){
            Log.e(TAG,Log.getStackTraceString(e));
            e.printStackTrace();
            try{
                client.close();
            }catch (IOException f){
                f.printStackTrace();
            }

        }
        // 删除未接收完的文件
        delUncompleteFile();
//        try{
//            if(randomAccessFile !=null){
//                if(randomAccessFile.length()<willRecevicefileSize){
//                    new File(path+receiveFileName).delete();
//                }
//            }
//        }catch (Exception e){
//            e.printStackTrace();
//        }

    }

   private void reSetData(){
        receiveFileName = null;
        willRecevicefileSize = 0;
        hasReceivedFileCount = 0;
        totalFileCount = 0;
   }
    private void receiveData(DataInputStream inputStream,DataOutputStream outputStream)throws IOException{
        byte[] bufferBytes = new byte[MAXLENGTH];

        int readLength = 0;
        byte[] nextPackagePart = null;
        boolean isTCPStop = false;
        while ((readLength = inputStream.read(bufferBytes))!=-1){
            if(readLength < bufferBytes.length){
                byte[] temp = new  byte[readLength];
                System.arraycopy(bufferBytes,0,temp,0,readLength);
                bufferBytes = temp;
            }
            if(nextPackagePart!=null){
                bufferBytes = Utils.concat(nextPackagePart,bufferBytes);
            }
            while (bufferBytes.length>=Constance.HEADER_LEN&&!isTCPStop) {
                byte type = bufferBytes[0];
                byte[] bodyLengthBytes = new byte[4];

                System.arraycopy(bufferBytes, 1, bodyLengthBytes, 0, 4);
                int bodyLength = Utils.bytes2Int(bodyLengthBytes);
                if (bufferBytes.length >= bodyLength + Constance.HEADER_LEN) {// 包含一个全包
                    if (type == Constance.STRING_PACKET_BYTE) {
                        // 解析字符串
//                            byte[] strBytes = new byte[bodyLength];
//                            System.arraycopy(bufferBytes,5,strBytes,0,bodyLength);
                        String recvMsg = Utils.decode(new String(bufferBytes, 5, bodyLength));
                        Log.e(TAG, "接收一个文本" + recvMsg);
                        parseAndSendTCPMsg(recvMsg,outputStream);
                    }else if(type == Constance.BYTE_PACKET_BYTE){
//                        Log.e(TAG, "receive BYTE_PACKET_BYTE" );
                        int saveResult = saveFile(bufferBytes,Constance.HEADER_LEN,bodyLength,outputStream);
                        if(saveResult == 1){
                        }
                    }else if(type == Constance.HB_PACKET_BYTE){// 接收到心跳包
                        Log.e(TAG, "receive hearbeat" );
                        PacketEncoder.sendTcpHeatbeatMsg(outputStream);
                    }
                    int nextLength = bufferBytes.length-Constance.HEADER_LEN-bodyLength;
//                    Utils.loge("nextLength="+nextLength);
                    if(nextLength==0){
                        nextPackagePart = null;
                        // 完成解析bufferBytes
                        bufferBytes =  new byte[0];
                        break;
                    }
                    byte[] nextBytes = new byte[nextLength];
                    System.arraycopy(bufferBytes,Constance.HEADER_LEN+bodyLength,nextBytes,0,nextLength);
                    bufferBytes = nextBytes;
                }else {//
                    nextPackagePart = bufferBytes;
                    break;
                }

            }
            if(bufferBytes.length==0){
                nextPackagePart = null;
            }else if(bufferBytes.length<5){
                nextPackagePart = bufferBytes;
            }
            bufferBytes =  new byte[MAXLENGTH];
        }
        Log.e(TAG,"receiveData end");
    }
    private int saveFile(byte[] bytes,int byteOffset, int byteCount,DataOutputStream outputStream){
        try {
            randomAccessFile.write(bytes,byteOffset,byteCount);
//            Log.e(TAG,"willRecevicefileSize="+willRecevicefileSize+"randomAccessFile.length()="+randomAccessFile.length());
                if(willRecevicefileSize == randomAccessFile.length()){
                    Log.e(TAG,receiveFileName+"文件传输完毕");
                    randomAccessFile.close();
                    randomAccessFile = null;
                    hasReceivedFileCount++;
                    if(mFileReceiveListener != null){
                        mFileReceiveListener.onReceivingFiles(hasReceivedFileCount,
                                totalFileCount);
                    }
                    sendTcpTextMsg(outputStream,completeOneFileReplyStr);
                    return 1;
                }

        } catch (Exception e) {
            e.printStackTrace();
            Log.e(TAG,Log.getStackTraceString(e));
        }
        return 0;
    }
    private void parseAndSendTCPMsg(String recvMsg,DataOutputStream outputStream){
        JSONTokener jsonTokener = new JSONTokener(recvMsg);
        JSONObject json = null;
        String msgType = null;
        try {
            json = (JSONObject) jsonTokener.nextValue();
            msgType = json.getString("msgType");
            if("fileTotalInfo".equals(msgType)){
                totalFileCount = json.getInt("totalFileCount");
                hasReceivedFileCount = 0;
                if(mFileReceiveListener != null){
                    mFileReceiveListener.onWillReceiveFiles(totalFileCount);
                }
                sendTcpTextMsg(outputStream,"{\"msgType\":\"fileTotalInfo\",\"result\":\"200\"}" );

            }else if("fileInfo".equals(msgType)){
                delUncompleteFile();
                receiveFileName = json.getString("fileName");
                willRecevicefileSize = json.getLong("fileSize");
                int result = createFile(receiveFileName,willRecevicefileSize);
                sendTcpTextMsg(outputStream,"{\"msgType\":\"fileInfo\",\"result\":\""+result+"\"}" );
            }else if("fileInfoMD5".equals(msgType)){
                receiveFileName = json.getString("fileName");
                willRecevicefileSize = json.getLong("fileSize");
                String md5 = json.getString("MD5");
                String filePath= path+receiveFileName;
                File file = new File(filePath);
                boolean isSameFile = compareMd5(md5,file);
                int createFileResult = 0;
                if(isSameFile){// 如果文件相同，则删除(正常不用传输)
                    file.delete();
                    createFileResult = createFile(receiveFileName,willRecevicefileSize);
                }else{
                    receiveFileName = getNewFile(receiveFileName);
                    Log.e(TAG,"receiveFileName="+receiveFileName );
                    createFileResult = createFile(receiveFileName,willRecevicefileSize);
                }

                sendTcpTextMsg(outputStream,"{\"msgType\":\"fileInfoMD5\",\"result\":\""+ createFileResult+"\"}" );

            }else if("cancel".equals(msgType)){// 停止传输
                delUncompleteFile();
                reSetData();
                if(mFileReceiveListener != null){
                    mFileReceiveListener.onCancelSendFile();
                }

            }
        }catch (Exception e) {
            e.printStackTrace();
            Log.e(TAG,Log.getStackTraceString(e));
        }

    }

    /**
     * 删除没接受完全的文件
     */
    private void delUncompleteFile(){
        try{
            if(randomAccessFile!=null){
                if(randomAccessFile.length()<willRecevicefileSize){
                    new File(path+receiveFileName).delete();
                }
                randomAccessFile.close();
                randomAccessFile = null;
            }
        }catch (IOException e){
            e.printStackTrace();
        }

    }
    private void sendTcpTextMsg(DataOutputStream out, String msg) {
        String encode = Utils.encode(msg);
        byte[] bytes = encode.getBytes();
        int totalLen = bytes.length;
        try {
            out.writeByte(Constance.STRING_PACKET_BYTE);
            out.writeInt(totalLen);
            out.write(bytes);
            out.flush();
        } catch (Exception e) {
            e.printStackTrace();
            Log.e(TAG,Log.getStackTraceString(e));
        }
    }
    private boolean compareMd5(String md5, File file){

        try {
            MessageDigest msgDigest = MessageDigest.getInstance("MD5");
            String localMd5 =  Utils.getFileMD5String(file,msgDigest);
            return md5.equals(localMd5);
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalStateException(
                    "System doesn't support MD5 algorithm.");
        }catch (IOException e){

        }
        return false;

    }
    private String getNewFile(String fileName){
        Log.e(TAG,"fileName="+fileName);


        String namePre = "";
        String nameSuff = "";
        int index = fileName.lastIndexOf(".");
        if(index>-1){
            namePre = fileName.substring(0,index);
            nameSuff = fileName.substring(index,fileName.length());
        }
        Log.e(TAG,"namePre="+namePre+"nameSuff="+nameSuff);
        String nextFileName = null;
        File file = null;
        for(int i= 2;i<Integer.MAX_VALUE;i++){
            nextFileName = namePre+"("+i+")"+nameSuff;
            file = new File(path+nextFileName);
            if(!file.exists()){
                break;
            }
        }
        return nextFileName;

    }
    private int createFile(String fileName,long fileSize){
        String filePath= path+fileName;
        File file = new File(filePath);
        long size = 0L;

        if(file.exists()){
            return Constance.RESULT_NEED_MD5;
        }else{
            if(Utils.getSDAvailableSize()<fileSize){
                // 存储空间不足
                return Constance.RESULT_RECEVER_NOT_EnoughMemory;
            }
            Log.e(TAG,"剩余容量"+Utils.getSDAvailableSize());
            try {
                if (!file.getParentFile().exists()) {
                    file.getParentFile().mkdirs();
                    return Constance.RESULT_OK;
                }
                file.createNewFile();
                if(randomAccessFile!=null){
                    randomAccessFile.close();
                }
                randomAccessFile  = new RandomAccessFile(filePath, "rwd");
            } catch (Exception e) {
                Log.e(TAG,"createFile Exception"+Log.getStackTraceString(e));
                e.printStackTrace();
                return Constance.RESULT_WIRTE_FILE_FAIL;
            }
            return Constance.RESULT_OK;

        }
    }
}
