package com.mxnavi.tspv2.fm;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.fm.model.AlbumMediaQuery;
import com.mxnavi.tspv2.fm.model.LiveCityInfo;
import com.mxnavi.tspv2.fm.model.MediaResult;
import com.mxnavi.tspv2.fm.model.AlbumTagsResult;
import com.mxnavi.tspv2.fm.model.AlbumsQuery;
import com.mxnavi.tspv2.fm.model.AlbumsResult;
import com.mxnavi.tspv2.fm.model.AlbumsSearch;
import com.mxnavi.tspv2.fm.model.CategoriesResult;
import com.mxnavi.tspv2.fm.model.ProgramInfo;
import com.mxnavi.tspv2.fm.model.LiveProvincesResult;
import com.mxnavi.tspv2.fm.model.RadioCategorieInfo;
import com.mxnavi.tspv2.fm.model.Radios;
import com.mxnavi.tspv2.fm.model.RadiosByCategoryQuery;
import com.mxnavi.tspv2.fm.model.RadiosByCityQuery;
import com.mxnavi.tspv2.fm.model.RadiosQuery;
import com.mxnavi.tspv2.fm.model.RadiosByPageResult;
import com.mxnavi.tspv2.fm.model.Schedule;
import com.mxnavi.tspv2.fm.model.SearchAlbumsResult;
import com.mxnavi.tspv2.fm.model.TagHotMediaQuery;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2016/4/21.
 * FM
 */
public class FmQuery extends BaseTsp{

    final int thisServiceType =  Config.MxNGTPServiceType.ONLINE_FM.getServiceType();
    // ServiceData协议版本
    final int serviceDataVer =  Config.MxNGTPServiceType.ONLINE_FM.getServiceVer() ;
    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int FM_getCategories = 1;
    final int FM_getAlbumTags = 2;
    final int FM_getAlbums = 3;
    final int FM_getAlbumMediaList = 4;
    final int FM_getTagHotMediaList = 5;
    final int FM_searchAlbums = 6;
    final int FM_searchMedias = 7;
    final int FM_getRecommendAlbums = 8;
    final int FM_provinces = 9;//获取直播省市列表
    final int FM_radios = 10;//获取直播电台
    final int FM_schedules = 11;//获取某个直播电台某一天的节目排期的列表
    final int FM_getPlayingProgram = 12;//获取某个电台正在直播的节目
    final int FM_cities = 13;//获取某省份城市列表
    final int FM_getRadiosByCity = 14;//获取某个城市下的电台列表
    final int FM_radioCategories = 15;//获取直播电台的分类
    final int FM_getRadiosByCategory = 16;//根据电台分类获取直播电台数据
    final int FM_getRadiosByIds = 17;//根据电台ID，批量获取电台列表


    /**
     * 获取分类列表
     * @return 请求ID，0:参数错误
     */
    public int getCategories(){
        return getCategories(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取分类列表
     * @param timeOutSecs  接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getCategories(int timeOutSecs){
        return getCategories(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取分类列表
     * @param timeOutSecs  接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getCategories(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetCategories";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getCategories);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取分类下的专辑标签列表
     * @param categoryId 分类id
     * @return 请求ID，0:参数错误
     */
    public int getAlbumTags(String categoryId){
        return getAlbumTags(categoryId,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取分类下的专辑标签列表
     * @param categoryId 分类id
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 0 参数错误
     */
    public int getAlbumTags(String categoryId,int timeOutSecs){
        return getAlbumTags(categoryId,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取分类下的专辑标签列表
     * @param categoryId 分类id
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getAlbumTags(String categoryId,int timeOutSecs,int priorityLevel){
        if(categoryId == null){
            if (mOnGetAlbumTagsListener != null) {
                mOnGetAlbumTagsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetAlbumTags";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("category_id", categoryId );

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getAlbumTags);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取专辑标签下的专辑列表
     * @param albumsQuery
     * @return 请求ID，0:参数错误
     */
    public int getAlbums(AlbumsQuery albumsQuery){
        return getAlbums(albumsQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取专辑标签下的专辑列表
     * @param albumsQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getAlbums(AlbumsQuery albumsQuery,int timeOutSecs){
        return getAlbums(albumsQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取专辑标签下的专辑列表
     * @param albumsQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getAlbums(AlbumsQuery albumsQuery,int timeOutSecs,int priorityLevel){
        if(albumsQuery == null){
            if (mOnGetAlbumsListener != null) {
                mOnGetAlbumsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        int eventId = RequestDispathUtil.createEventId();
        String url = "GetAlbums";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("category_id",Utils.converNullToEmpty(albumsQuery.getCategoryId() ));
        params.put("tag_id",Utils.converNullToEmpty(albumsQuery.getTagId()));
        params.put("offset",albumsQuery.getOffset() );
        params.put("limit",albumsQuery.getLimit() );

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getAlbums);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取专辑下的声音列表
     * @param albumMediaQuery
     * @return 请求ID，0:参数错误
     */
    public int getAlbumMediaList(AlbumMediaQuery albumMediaQuery){
        return getAlbumMediaList( albumMediaQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取专辑下的声音列表
     * @param albumMediaQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getAlbumMediaList(AlbumMediaQuery albumMediaQuery,int timeOutSecs){
        return getAlbumMediaList( albumMediaQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取专辑下的声音列表
     * @param albumMediaQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getAlbumMediaList(AlbumMediaQuery albumMediaQuery,int timeOutSecs,int priorityLevel){
        if(albumMediaQuery == null){
            if (mOnGetAlbumMediaListener != null) {
                mOnGetAlbumMediaListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetAlbumMediaList";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("album_id",Utils.converNullToEmpty(albumMediaQuery.getAlbumId()));
        params.put("offset",albumMediaQuery.getOffset() );
        params.put("limit",albumMediaQuery.getLimit() );
        params.put("is_asc",albumMediaQuery.getIsAsc() );

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getAlbumMediaList);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取热门声音列表
     * @param tagHotMediaQuery
     * @return 请求ID，0:参数错误
     */
    public int getTagHotMediaList(TagHotMediaQuery tagHotMediaQuery){
        return getTagHotMediaList(tagHotMediaQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取热门声音列表
     * @param tagHotMediaQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getTagHotMediaList(TagHotMediaQuery tagHotMediaQuery,int timeOutSecs){
        return getTagHotMediaList(tagHotMediaQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取热门声音列表
     * @param tagHotMediaQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getTagHotMediaList(TagHotMediaQuery tagHotMediaQuery,int timeOutSecs,int priorityLevel){
        if(tagHotMediaQuery == null){
            if (mOnGetTagHotMediaListListener != null) {
                mOnGetTagHotMediaListListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetTagHotMediaList";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("category_id",Utils.converNullToEmpty(tagHotMediaQuery.getCategoryId()));
        params.put("tag_id",Utils.converNullToEmpty(tagHotMediaQuery.getTagId()));
        params.put("offset",tagHotMediaQuery.getOffset() );
        params.put("limit",tagHotMediaQuery.getLimit() );
        params.put("condition",Utils.converNullToEmpty(tagHotMediaQuery.getCondition()));

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getTagHotMediaList);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 专辑检索
     * @param albumsSearch
     * @return 请求ID，0:参数错误
     */
    public int searchAlbums(AlbumsSearch albumsSearch){
        return searchAlbums(albumsSearch,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 专辑检索
     * @param albumsSearch 检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchAlbums(AlbumsSearch albumsSearch,int timeOutSecs){
        return searchAlbums(albumsSearch,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 专辑检索
     * @param albumsSearch 检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchAlbums(AlbumsSearch albumsSearch,int timeOutSecs,int priorityLevel){
        if(albumsSearch == null){
            if (mOnSearchAlbumsListener != null) {
                mOnSearchAlbumsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchAlbums";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("category_id",Utils.converNullToEmpty(albumsSearch.getCategoryId()));
        params.put("keyword",Utils.converNullToEmpty(albumsSearch.getKeyword()));
        params.put("offset",albumsSearch.getOffset() );
        params.put("limit",albumsSearch.getLimit() );

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_searchAlbums);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 声音搜索
     * @param albumsSearch
     * @return 请求ID，0:参数错误
     */
    public int searchMedias(AlbumsSearch albumsSearch){
        return searchMedias(albumsSearch,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 声音搜索
     * @param albumsSearch
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchMedias(AlbumsSearch albumsSearch,int timeOutSecs){
        return searchMedias(albumsSearch,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 声音搜索
     * @param albumsSearch
     * @param timeOutSecs  接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchMedias(AlbumsSearch albumsSearch,int timeOutSecs,int priorityLevel){
        if(albumsSearch == null){
            if (mOnSearchMediasListener != null) {
                mOnSearchMediasListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchMedias";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("category_id",Utils.converNullToEmpty(albumsSearch.getCategoryId()));
        params.put("keyword",Utils.converNullToEmpty(albumsSearch.getKeyword()));
        params.put("offset",albumsSearch.getOffset() );
        params.put("limit",albumsSearch.getLimit() );

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_searchMedias);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取推荐专辑
     * @param offset 从第几条开始取，默认为1
     * @param limit 知道获取条数，默认20条
     * @return 请求ID，0:参数错误
     */
    public int getRecommendAlbums(int offset,int limit){
        return getRecommendAlbums(offset,limit,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取推荐专辑
     * @param offset 从第几条开始取，默认为1
     * @param limit 知道获取条数，默认20条
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getRecommendAlbums(int offset,int limit,int timeOutSecs){
        return getRecommendAlbums(offset,limit,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取推荐专辑
     * @param offset 从第几条开始取，默认为1
     * @param limit 知道获取条数，默认20条
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getRecommendAlbums(int offset,int limit,int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetRecommendAlbums";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("offset", offset);
        params.put("limit", limit);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getRecommendAlbums);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取直播省和直辖市列表
     * @return 请求ID，0:参数错误
     */
    public int getLiveProvinces(){
        return getLiveProvinces(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取直播省和直辖市列表
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getLiveProvinces(int timeOutSecs){
        return getLiveProvinces(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取直播省和直辖市列表
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getLiveProvinces(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/provinces";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_provinces);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 分页获取直播电台
     * @param radiosQuery
     * @return 请求ID，0:参数错误
     */
    public int getLiveRadios(RadiosQuery radiosQuery){
        return getLiveRadios(radiosQuery, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 分页获取直播电台
     * @param radiosQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getLiveRadios(RadiosQuery radiosQuery, int timeOutSecs){
        return getLiveRadios(radiosQuery, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 分页获取直播电台
     * @param radiosQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getLiveRadios(RadiosQuery radiosQuery, int timeOutSecs, int priorityLevel){
        if(radiosQuery == null){
            if (mOnGetLiveRadiosListener != null) {
                mOnGetLiveRadiosListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/radios";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("radioType", radiosQuery.getRadioType());
        if (radiosQuery.getProvinceCode() != 0) {
            params.put("provinceCode", radiosQuery.getProvinceCode());
        }
        params.put("page", radiosQuery.getPage());
        params.put("count", radiosQuery.getCount());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_radios);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取某个直播电台某一天的节目排期的列表
     * @param radioId 直播电台ID
     * @param weekday 表示星期几，不填则取今天的星期。0-星期天，1-星期一，2-星期二，3-星期三，4-星期四，5-星期五，6-星期六
     * @return 请求ID，0:参数错误
     */
    public int getSchedules(String radioId, String weekday){
        return getSchedules(radioId, weekday, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个直播电台某一天的节目排期的列表
     * @param radioId 直播电台ID
     * @param weekday 表示星期几，不填则取今天的星期。0-星期天，1-星期一，2-星期二，3-星期三，4-星期四，5-星期五，6-星期六
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getSchedules(String radioId, String weekday, int timeOutSecs){
        return getSchedules(radioId, weekday, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个直播电台某一天的节目排期的列表
     * @param radioId 直播电台ID
     * @param weekday 表示星期几，不填则取今天的星期。0-星期天，1-星期一，2-星期二，3-星期三，4-星期四，5-星期五，6-星期六
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getSchedules(String radioId, String weekday, int timeOutSecs,int priorityLevel){
        if(Utils.strIsEmpty(radioId)){
            if (mOnGetSchedulesListener != null) {
                mOnGetSchedulesListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/schedules";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("radioId", radioId);
        if (!Utils.strIsEmpty(weekday)){
            params.put("weekday", weekday);
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_schedules);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取某个电台正在直播的节目
     * @param radioId 直播电台ID
     * @return 请求ID，0:参数错误
     */
    public int getPlayingProgram(String radioId){
        return getPlayingProgram(radioId, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个电台正在直播的节目
     * @param radioId 直播电台ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getPlayingProgram(String radioId, int timeOutSecs){
        return getPlayingProgram(radioId, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个电台正在直播的节目
     * @param radioId 直播电台ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getPlayingProgram(String radioId, int timeOutSecs,int priorityLevel){
        if(Utils.strIsEmpty(radioId)){
            if (mOnGetPlayingProgramListener != null) {
                mOnGetPlayingProgramListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/getPlayingProgram";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("radioId", radioId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getPlayingProgram);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取某省份城市列表
     * @param provinceCode 省份code (国家行政规划的省代码)
     * @return 请求ID，0:参数错误
     */
    public int getCitiesByProvince(String provinceCode){
        return getCitiesByProvince(provinceCode, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某省份城市列表
     * @param provinceCode 省份code (国家行政规划的省代码)
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getCitiesByProvince(String provinceCode, int timeOutSecs){
        return getCitiesByProvince(provinceCode, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某省份城市列表
     * @param provinceCode 省份code (国家行政规划的省代码)
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getCitiesByProvince(String provinceCode, int timeOutSecs,int priorityLevel){
        if(Utils.strIsEmpty(provinceCode)){
            if (mOnGetCitiesByProvinceListener != null) {
                mOnGetCitiesByProvinceListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/cities";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("provinceCode", provinceCode);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_cities);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取某个城市下的电台列表
     * @param radiosByCityQuery
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCity(RadiosByCityQuery radiosByCityQuery){
        return getRadiosByCity(radiosByCityQuery, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个城市下的电台列表
     * @param radiosByCityQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCity(RadiosByCityQuery radiosByCityQuery, int timeOutSecs){
        return getRadiosByCity(radiosByCityQuery, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取某个城市下的电台列表
     * @param radiosByCityQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCity(RadiosByCityQuery radiosByCityQuery, int timeOutSecs, int priorityLevel){
        if(radiosByCityQuery == null
                || radiosByCityQuery.getCityCode() == 0){
            if (mOnGetRadiosByCityListener != null) {
                mOnGetRadiosByCityListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/getRadiosByCity";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("cityCode", radiosByCityQuery.getCityCode());
        params.put("page", radiosByCityQuery.getPage());
        params.put("count", radiosByCityQuery.getCount());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getRadiosByCity);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取直播电台的分类
     * @return 请求ID，0:参数错误
     */
    public int radioCategories(){
        return radioCategories(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取直播电台的分类
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int radioCategories(int timeOutSecs){
        return radioCategories(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取直播电台的分类
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int radioCategories(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/radioCategories";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin",  Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_radioCategories);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 根据电台分类获取直播电台数据
     * @param radiosByCategoryQuery
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCategory(RadiosByCategoryQuery radiosByCategoryQuery){
        return getRadiosByCategory(radiosByCategoryQuery, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据电台分类获取直播电台数据
     * @param radiosByCategoryQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCategory(RadiosByCategoryQuery radiosByCategoryQuery, int timeOutSecs){
        return getRadiosByCategory(radiosByCategoryQuery, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据电台分类获取直播电台数据
     * @param radiosByCategoryQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByCategory(RadiosByCategoryQuery radiosByCategoryQuery, int timeOutSecs, int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/getRadiosByCategory";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        if (radiosByCategoryQuery.getRadioCategoryId() != 0){
            params.put("radioCategoryId", radiosByCategoryQuery.getRadioCategoryId());
        }
        if (radiosByCategoryQuery.getPage() != 0){
            params.put("page", radiosByCategoryQuery.getPage());
        }
        if (radiosByCategoryQuery.getCount() != 0){
            params.put("count", radiosByCategoryQuery.getCount());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getRadiosByCategory);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 根据电台ID，批量获取电台列表
     * @param ids 电台ID列表，传参时用英文逗号分隔
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByIds(String ids){
        return getRadiosByIds(ids, Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据电台ID，批量获取电台列表
     * @param ids 电台ID列表，传参时用英文逗号分隔
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByIds(String ids, int timeOutSecs){
        return getRadiosByIds(ids, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据电台ID，批量获取电台列表
     * @param ids 电台ID列表，传参时用英文逗号分隔
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getRadiosByIds(String ids, int timeOutSecs,int priorityLevel){
        if(Utils.strIsEmpty(ids)){
            if (mOnGetRadiosByIdsListener != null) {
                mOnGetRadiosByIdsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/live/getRadiosByIds";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", Config.VIN);
        params.put("ids", ids);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, FM_getRadiosByIds);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }



    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, FmQuery.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    OnGetCategoriesListener mOnGetCategoriesListener;
    OnGetAlbumTagsListener mOnGetAlbumTagsListener;
    OnGetAlbumsListener mOnGetAlbumsListener;
    OnGetAlbumMediaListener mOnGetAlbumMediaListener;
    OnGetTagHotMediaListListener mOnGetTagHotMediaListListener;
    OnSearchAlbumsListener mOnSearchAlbumsListener;
    OnSearchMediasListener mOnSearchMediasListener;
    OnGetRecommendAlbumsListener mOnGetRecommendAlbumsListener;
    OnGetProvincesListener mOnGetProvincesListener;
    OnGetSchedulesListener mOnGetSchedulesListener;
    OnGetPlayingProgramListener mOnGetPlayingProgramListener;
    OnGetCitiesByProvinceListener mOnGetCitiesByProvinceListener;
    OnGetRadiosByCityListener mOnGetRadiosByCityListener;
    OnGetRadioCategoriesListener mOnGetRadioCategoriesListener;
    OnGetRadiosByCategoryListener mOnGetRadiosByCategoryListener;
    OnGetRadiosByIdsListener mOnGetRadiosByIdsListener;
    OnGetLiveRadiosListener mOnGetLiveRadiosListener;


    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case FM_getCategories:
                if (mOnGetCategoriesListener != null) {
                    mOnGetCategoriesListener.onTimeOut();
                }
                break;
            case FM_getAlbumTags:
                if (mOnGetAlbumTagsListener != null) {
                    mOnGetAlbumTagsListener.onTimeOut();
                }
                break;
            case FM_getAlbums:
                if (mOnGetAlbumsListener != null) {
                    mOnGetAlbumsListener.onTimeOut();
                }
                break;
            case FM_getAlbumMediaList:
                if (mOnGetAlbumMediaListener != null) {
                    mOnGetAlbumMediaListener.onTimeOut();
                }
                break;
            case FM_getTagHotMediaList:
                if (mOnGetTagHotMediaListListener != null) {
                    mOnGetTagHotMediaListListener.onTimeOut();
                }
                break;
            case FM_searchAlbums:
                if (mOnSearchAlbumsListener != null) {
                    mOnSearchAlbumsListener.onTimeOut();
                }
                break;
            case FM_searchMedias:
                if (mOnSearchMediasListener != null) {
                    mOnSearchMediasListener.onTimeOut();
                }
                break;
            case FM_getRecommendAlbums:
                if (mOnGetRecommendAlbumsListener != null) {
                    mOnGetRecommendAlbumsListener.onTimeOut();
                }
                break;
            case FM_provinces:
                if (mOnGetProvincesListener != null) {
                    mOnGetProvincesListener.onTimeOut();
                }
                break;
            case FM_radios:
                if (mOnGetLiveRadiosListener != null) {
                    mOnGetLiveRadiosListener.onTimeOut();
                }
                break;
            case FM_schedules:
                if (mOnGetSchedulesListener != null) {
                    mOnGetSchedulesListener.onTimeOut();
                }
                break;
            case FM_getPlayingProgram:
                if (mOnGetPlayingProgramListener != null) {
                    mOnGetPlayingProgramListener.onTimeOut();
                }
                break;
            case FM_cities:
                if (mOnGetCitiesByProvinceListener != null) {
                    mOnGetCitiesByProvinceListener.onTimeOut();
                }
                break;
            case FM_getRadiosByCity:
                if (mOnGetRadiosByCityListener != null) {
                    mOnGetRadiosByCityListener.onTimeOut();
                }
                break;
            case FM_radioCategories:
                if (mOnGetRadioCategoriesListener != null) {
                    mOnGetRadioCategoriesListener.onTimeOut();
                }
                break;
            case FM_getRadiosByCategory:
                if (mOnGetRadiosByCategoryListener != null) {
                    mOnGetRadiosByCategoryListener.onTimeOut();
                }
                break;
            case FM_getRadiosByIds:
                if (mOnGetRadiosByIdsListener != null) {
                    mOnGetRadiosByIdsListener.onTimeOut();
                }
                break;
        }
    }

    /**
     * 输出结果
     * @param requestHandleId
     * @param response
     * @param hasDSPTError
     * @param errorCode
     * @param errorMsg
     */
    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.logd(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case FM_getCategories:// 1
                if (mOnGetCategoriesListener != null) {
                    if(hasDSPTError){
                        mOnGetCategoriesListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            CategoriesResult categoriesResult = gson.fromJson(response, CategoriesResult.class);
                            mOnGetCategoriesListener.onGetCategories(categoriesResult);
                        }else{
                            mOnGetCategoriesListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetCategoriesListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
            case FM_getAlbumTags:// 2
                if(mOnGetAlbumTagsListener != null){
                    if(hasDSPTError){
                        mOnGetAlbumTagsListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            AlbumTagsResult albumTagsResult = gson.fromJson(response, AlbumTagsResult.class);
                            mOnGetAlbumTagsListener.onGetAlbumTags(albumTagsResult);
                        }else{
                            mOnGetAlbumTagsListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetAlbumTagsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getAlbums:// 3
                if(mOnGetAlbumsListener != null){
                    if(hasDSPTError){
                        mOnGetAlbumsListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            AlbumsResult albumsResult = gson.fromJson(response, AlbumsResult.class);
                            mOnGetAlbumsListener.onGetAlbums(albumsResult);
                        }else{
                            mOnGetAlbumsListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetAlbumsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getAlbumMediaList:// 4
                if(mOnGetAlbumMediaListener != null){
                    if(hasDSPTError){
                        mOnGetAlbumMediaListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            MediaResult mediaResult = gson.fromJson(response, MediaResult.class);
                            mOnGetAlbumMediaListener.onGetAlbumMediaList(mediaResult);
                        }else{
                            mOnGetAlbumMediaListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetAlbumMediaListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getTagHotMediaList:// 5
                if(mOnGetTagHotMediaListListener != null){
                    if(hasDSPTError){
                        mOnGetTagHotMediaListListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            MediaResult mediaResult = gson.fromJson(response, MediaResult.class);
                            mOnGetTagHotMediaListListener.onGetTagHotMediaList(mediaResult);
                        }else{
                            mOnGetTagHotMediaListListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetTagHotMediaListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_searchAlbums:// 6
                if(mOnSearchAlbumsListener != null){
                    if(hasDSPTError){
                        mOnSearchAlbumsListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            SearchAlbumsResult searchAlbumsResult = gson.fromJson(response, SearchAlbumsResult.class);
                            mOnSearchAlbumsListener.onSearchAlbums(searchAlbumsResult);
                        }else{
                            mOnSearchAlbumsListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchAlbumsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_searchMedias:// 7
                if(mOnSearchMediasListener != null){
                    if(hasDSPTError){
                        mOnSearchMediasListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            MediaResult mediaResult = gson.fromJson(response, MediaResult.class);
                            mOnSearchMediasListener.onSearchMedias(mediaResult);
                        }else{
                            mOnSearchMediasListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchMediasListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getRecommendAlbums:// 8
                if(mOnGetRecommendAlbumsListener != null){
                    if(hasDSPTError){
                        mOnGetRecommendAlbumsListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            AlbumsResult albumsResult = gson.fromJson(response, AlbumsResult.class);
                            mOnGetRecommendAlbumsListener.onGetRecommendAlbums(albumsResult);
                        }else{
                            mOnGetRecommendAlbumsListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetRecommendAlbumsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_provinces:// 9
                if(mOnGetProvincesListener != null){
                    if(hasDSPTError){
                        mOnGetProvincesListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray districtObj = jsonObject.getJSONArray("provinces");
                            List<LiveProvincesResult> liveProvincesResultList = new ArrayList<LiveProvincesResult>();
                            for(int i = 0; i<districtObj.length(); i++) {
                                JSONObject object = districtObj.getJSONObject(i);
                                LiveProvincesResult liveProvincesResult = new LiveProvincesResult();
                                liveProvincesResult.setId(object.getInt("id"));
                                liveProvincesResult.setKind(object.getString("kind"));
                                liveProvincesResult.setCreatedAt(object.getLong("createdAt"));
                                liveProvincesResult.setProvinceCode(object.getInt("provinceCode"));
                                liveProvincesResult.setProvinceName(object.getString("provinceName"));
                                liveProvincesResultList.add(liveProvincesResult);
                            }
                            mOnGetProvincesListener.onGetProvinces(liveProvincesResultList);
                        }else{
                            mOnGetProvincesListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetProvincesListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_radios:// 10
                if(mOnGetLiveRadiosListener != null){
                    if(hasDSPTError){
                        mOnGetLiveRadiosListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            RadiosByPageResult radiosByPageResult = gson.fromJson(
                                    jsonObject.getJSONObject("result").toString(),RadiosByPageResult.class);
                            mOnGetLiveRadiosListener.onGetLiveRadios(radiosByPageResult);
                        }else{
                            mOnGetLiveRadiosListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetLiveRadiosListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_schedules:// 11
                if(mOnGetSchedulesListener != null){
                    if(hasDSPTError){
                        mOnGetSchedulesListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            Type jsonType = new TypeToken<List<Schedule>>() {}.getType();
                            List<Schedule> radiosList = gson.fromJson(jsonObject.getJSONArray("schedules").toString(), jsonType);

                            mOnGetSchedulesListener.onGetSchedules(radiosList);
                        }else{
                            mOnGetSchedulesListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetSchedulesListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getPlayingProgram:// 12
                if(mOnGetPlayingProgramListener != null){
                    if(hasDSPTError){
                        mOnGetPlayingProgramListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            ProgramInfo programInfo = gson.fromJson(jsonObject.getJSONObject("program").toString(), ProgramInfo.class);
                            mOnGetPlayingProgramListener.onGetPlayingProgram(programInfo);
                        }else{
                            mOnGetPlayingProgramListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetPlayingProgramListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_cities:// 13
                if(mOnGetCitiesByProvinceListener != null){
                    if(hasDSPTError){
                        mOnGetCitiesByProvinceListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            Type jsonType = new TypeToken<List<LiveCityInfo>>() {}.getType();
                            List<LiveCityInfo> liveCityInfoList = gson.fromJson(jsonObject.getJSONArray("cities").toString(), jsonType);
                            mOnGetCitiesByProvinceListener.onGetCitiesByProvince(liveCityInfoList);
                        }else{
                            mOnGetCitiesByProvinceListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetCitiesByProvinceListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getRadiosByCity:// 14
                if(mOnGetRadiosByCityListener != null){
                    if(hasDSPTError){
                        mOnGetRadiosByCityListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            RadiosByPageResult radiosByPageResult = gson.fromJson(jsonObject.getJSONObject("result").toString(), RadiosByPageResult.class);
                            mOnGetRadiosByCityListener.onGetRadiosByCity(radiosByPageResult);
                        }else{
                            mOnGetRadiosByCityListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetRadiosByCityListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_radioCategories:// 15
                if(mOnGetRadioCategoriesListener != null){
                    if(hasDSPTError){
                        mOnGetRadioCategoriesListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            Type jsonType = new TypeToken<List<RadioCategorieInfo>>() {}.getType();
                            List<RadioCategorieInfo> radioCategorieInfoList =
                                    gson.fromJson(jsonObject.getJSONArray("radioCategories").toString(), jsonType);
                            mOnGetRadioCategoriesListener.onGetRadioCategories(radioCategorieInfoList);
                        }else{
                            mOnGetRadioCategoriesListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetRadioCategoriesListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getRadiosByCategory:// 16
                if(mOnGetRadiosByCategoryListener != null){
                    if(hasDSPTError){
                        mOnGetRadiosByCategoryListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            RadiosByPageResult radiosByPageResult = gson.fromJson(
                                    jsonObject.getJSONObject("result").toString(),RadiosByPageResult.class);
                            mOnGetRadiosByCategoryListener.onGetRadiosByCategory(radiosByPageResult);
                        }else{
                            mOnGetRadiosByCategoryListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetRadiosByCategoryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case FM_getRadiosByIds:// 17
                if(mOnGetRadiosByIdsListener != null){
                    if(hasDSPTError){
                        mOnGetRadiosByIdsListener.onFail(errorCode,errorMsg);
                        return;
                    }

                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();

                            Type jsonType = new TypeToken<List<Radios>>() {}.getType();
                            List<Radios> radiosList = gson.fromJson(jsonObject.getJSONArray("radios").toString(), jsonType);
                            mOnGetRadiosByIdsListener.onGetRadiosByIds(radiosList);
                        }else{
                            mOnGetRadiosByIdsListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetRadiosByIdsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }

    public interface OnGetCategoriesListener extends CallBackListener{
        public void onGetCategories(CategoriesResult categoriesResult);
    }
    public interface OnGetAlbumTagsListener extends CallBackListener{
        public void onGetAlbumTags(AlbumTagsResult albumTagsResult);
    }
    public interface OnGetAlbumsListener extends CallBackListener{
        public void onGetAlbums(AlbumsResult albumsResult);
    }
    public interface OnGetAlbumMediaListener extends CallBackListener{
        public void onGetAlbumMediaList(MediaResult mediaResult);
    }
    public interface OnGetTagHotMediaListListener extends CallBackListener{
        public void onGetTagHotMediaList(MediaResult mediaResult);
    }
    public interface OnSearchAlbumsListener extends CallBackListener{
        public void onSearchAlbums(SearchAlbumsResult searchAlbumsResult);
    }
    public interface OnSearchMediasListener extends CallBackListener{
        public void onSearchMedias(MediaResult mediaResult);
    }
    public interface OnGetRecommendAlbumsListener extends CallBackListener{
        public void onGetRecommendAlbums(AlbumsResult albumsResult);
    }
    public interface OnGetProvincesListener extends CallBackListener{
        public void onGetProvinces(List<LiveProvincesResult> liveProvincesResultList);
    }
    public interface OnGetLiveRadiosListener extends CallBackListener{
        public void onGetLiveRadios(RadiosByPageResult radiosByPageResult);
    }
    public interface OnGetSchedulesListener extends CallBackListener{
        public void onGetSchedules(List<Schedule> scheduleList);
    }
    public interface OnGetPlayingProgramListener extends CallBackListener{
        public void onGetPlayingProgram(ProgramInfo programInfo);
    }
    public interface OnGetCitiesByProvinceListener extends CallBackListener{
        public void onGetCitiesByProvince(List<LiveCityInfo> liveCityInfo);
    }
    public interface OnGetRadiosByCityListener extends CallBackListener{
        public void onGetRadiosByCity(RadiosByPageResult radiosByPageResult);
    }
    public interface OnGetRadioCategoriesListener extends CallBackListener{
        public void onGetRadioCategories(List<RadioCategorieInfo> radioCategorieInfoList);
    }
    public interface OnGetRadiosByCategoryListener extends CallBackListener{
        public void onGetRadiosByCategory(RadiosByPageResult radiosByPageResult);
    }
    public interface OnGetRadiosByIdsListener extends CallBackListener{
        public void onGetRadiosByIds(List<Radios> radiosList);
    }



    public void setOnGetCategoriesListener(OnGetCategoriesListener onGetCategoriesListener) {
        this.mOnGetCategoriesListener = onGetCategoriesListener;
    }

    public void setOnGetAlbumTagsListener(OnGetAlbumTagsListener onGetAlbumTagsListener) {
        this.mOnGetAlbumTagsListener = onGetAlbumTagsListener;
    }

    public void setOnGetAlbumsListener(OnGetAlbumsListener onGetAlbumsListener) {
        this.mOnGetAlbumsListener = onGetAlbumsListener;
    }

    public void setOnGetAlbumMediaListener(OnGetAlbumMediaListener onGetAlbumMediaListener) {
        this.mOnGetAlbumMediaListener = onGetAlbumMediaListener;
    }

    public void setOnGetTagHotMediaListListener(OnGetTagHotMediaListListener onGetTagHotMediaListListener) {
        this.mOnGetTagHotMediaListListener = onGetTagHotMediaListListener;
    }

    public void setOnSearchAlbumsListener(OnSearchAlbumsListener onSearchAlbumsListener) {
        this.mOnSearchAlbumsListener = onSearchAlbumsListener;
    }

    public void setOnSearchMediasListener(OnSearchMediasListener onSearchMediasListener) {
        this.mOnSearchMediasListener = onSearchMediasListener;
    }

    public void setOnGetRecommendAlbumsListener(OnGetRecommendAlbumsListener onGetRecommendAlbumsListener) {
        this.mOnGetRecommendAlbumsListener = onGetRecommendAlbumsListener;
    }

    public void setOnGetProvincesListener(OnGetProvincesListener onProvincesListener) {
        this.mOnGetProvincesListener = onProvincesListener;
    }

    public void setOnGetLiveRadiosListener(OnGetLiveRadiosListener onGetLiveRadiosListener) {
        this.mOnGetLiveRadiosListener = onGetLiveRadiosListener;
    }

    public void setOnGetSchedulesListener(OnGetSchedulesListener onGetSchedulesListener) {
        this.mOnGetSchedulesListener = onGetSchedulesListener;
    }
    public void setOnGetPlayingProgramListener(OnGetPlayingProgramListener onGetPlayingProgramListener){
        this.mOnGetPlayingProgramListener = onGetPlayingProgramListener;
    }
    public void setOnGetCitiesByProvinceListener(OnGetCitiesByProvinceListener onGetCitiesByProvinceListener) {
        this.mOnGetCitiesByProvinceListener = onGetCitiesByProvinceListener;
    }


    public void setOnGetRadiosByCityListener(OnGetRadiosByCityListener onGetRadiosByCityListener) {
        this.mOnGetRadiosByCityListener = onGetRadiosByCityListener;
    }

    public void setOnGetRadioCategoriesListener(OnGetRadioCategoriesListener onRadioCategoriesListener) {
        this.mOnGetRadioCategoriesListener = onRadioCategoriesListener;
    }

    public void setOnGetRadiosByCategoryListener(OnGetRadiosByCategoryListener onGetRadiosByCategoryListener) {
        this.mOnGetRadiosByCategoryListener = onGetRadiosByCategoryListener;
    }
    public void setOnGetRadiosByIdsListener(OnGetRadiosByIdsListener onGetRadiosByIdsListener){
        mOnGetRadiosByIdsListener = onGetRadiosByIdsListener;
    }
}

//class GsonUtil {
//    // 将Json数据解析成相应的映射对象
//    public static <T> T parseJsonWithGson(String jsonData, Class<T> type) {
//        Gson gson = new Gson();
//        T result = gson.fromJson(jsonData, type);
//        return result;
//    }
//
//    // 将Json数组解析成相应的映射对象列表
//    public static <T> List<T> parseJsonArrayWithGson(String jsonData, Class<T> type) {
//        Gson gson = new Gson();
//        List<T> result = gson.fromJson(jsonData, new TypeToken<List<T>>() {
//        }.getType());
//        return result;
//    }
//}
