package com.mxnavi.tspv2.gasstation;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.gasstation.model.GasBrandInfo;
import com.mxnavi.tspv2.gasstation.model.GasPreferInfo;
import com.mxnavi.tspv2.gasstation.model.GasPrice;
import com.mxnavi.tspv2.gasstation.model.GasStationInfo;
import com.mxnavi.tspv2.gasstation.model.GasStationInfoById;
import com.mxnavi.tspv2.gasstation.model.GasStationListInfo;
import com.mxnavi.tspv2.gasstation.model.GasStationResult;
import com.mxnavi.tspv2.gasstation.model.Route;
import com.mxnavi.tspv2.gasstation.model.SearchByNameQuery;
import com.mxnavi.tspv2.gasstation.model.SearchByPreferBrandResult;
import com.mxnavi.tspv2.gasstation.model.SearchByPreferQuery;
import com.mxnavi.tspv2.gasstation.model.SearchByPreferResult;
import com.mxnavi.tspv2.gasstation.model.SearchByRouteQuery;
import com.mxnavi.tspv2.gasstation.model.SearchInRectangleQuery;
import com.mxnavi.tspv2.gasstation.model.SearchInRectangleResult;
import com.mxnavi.tspv2.gasstation.model.SearchNearByQuery;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;


/**
 * Created by Administrator on 2016/4/29.
 * 动态加油站检索类
 */
public class GasStationSearch extends BaseTsp {

    //请求服务类型为动态加油站服务
    final int thisServiceType =  Config.MxNGTPServiceType.GASSTATION.getServiceType();
    //动态加油站服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.GASSTATION.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int SEARCH_GASSTATION_NEAR_BY= 1;//周边加油站检索
    final int SEARCH_GASSTATION_BY_NAME= 2;//按名称检索加油站
    final int SEARCH_GASSTATION_BY_ROUTE= 3;//沿路线检索加油站
    final int GET_ADMIN_CODE_LIST= 4;//城市列表获取
    final int SEARCH_GASSTATION_BY_CITY_LONLAT= 5;//取得城下所有加油站基本信息
    final int SEARCH_GASSTATION_BY_ID= 6;//根据ID取得加油站详细信息
    final int SEARCH_GASSTATION_IN_RECTANGLE= 7;//屏幕范围内加油站检索
    final int SEARCH_GASSTATION_BY_PREFER = 8; // 常去加油站
    final int SEARCH_GASSTATION_BY_PREFER_BRAND = 9; // 常去品牌


    OnSearchGasStationNearByListener mOnSearchGasStationNearByListener;
    OnSearchGasStationByNameListener mOnSearchGasStationByNameListener;
    OnSearchGasStationByRouteListener mOnSearchGasStationByRouteListener;
    OnGetAdminCodeListListener mOnGetAdminCodeListListener;
    OnSearchByCityLonLatListener mOnSearchByCityLonLatListener;
    OnSearchGasStationByIdListener mOnSearchGasStationByIdListener;
    OnSearchGasStationInRectangleListener mOnSearchGasStationInRectangleListener;
    OnSearchGasStationByPreferListener mOnSearchGasStationByPreferListener;
    OnSearchGasStationByPreferBrandListener mOnSearchGasStationByPreferBrandListener;


    /**
     * 输入置车点经纬度，返回检索半径范围内的加油站列表
     * @param searchNearByQuery 周边加油站检索条件
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationNearBy(SearchNearByQuery searchNearByQuery){
        return searchGasStationNearBy(searchNearByQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 输入置车点经纬度，返回检索半径范围内的加油站列表
     * @param searchNearByQuery 周边加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationNearBy(SearchNearByQuery searchNearByQuery,int timeOutSecs){
        return searchGasStationNearBy(searchNearByQuery, timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 输入置车点经纬度，返回检索半径范围内的加油站列表
     * @param searchNearByQuery 周边加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationNearBy(SearchNearByQuery searchNearByQuery,int timeOutSecs,int priorityLevel){
        if( searchNearByQuery== null||searchNearByQuery.getLocation()==null||searchNearByQuery.getGasSubtype()==null||searchNearByQuery.getGasType()==null){
            if (mOnSearchGasStationNearByListener != null) {
               mOnSearchGasStationNearByListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchNearBy";
        HashMap<String, Object> params = new HashMap<String, Object>();
        LatLng latLng =  searchNearByQuery.getLocation();
        params.put("location", latLng.getLongitude()+","+latLng.getLatitude());
        params.put("radius", searchNearByQuery.getRadius());
        params.put("offset", searchNearByQuery.getOffset());
        params.put("limit", searchNearByQuery.getLimit());
        params.put("gasType", searchNearByQuery.getGasType());
        params.put("gasSubtype", searchNearByQuery.getGasSubtype());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_NEAR_BY);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 按城市及名称做加油站的检索
     * @param searchByNameQuery 通过名称检索加油站检索条件
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByName(SearchByNameQuery searchByNameQuery){
        return searchGasStationByName(searchByNameQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 按城市及名称做加油站的检索
     * @param searchByNameQuery 通过名称检索加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByName(SearchByNameQuery searchByNameQuery,int timeOutSecs){
        return searchGasStationByName(searchByNameQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 按城市及名称做加油站的检索
     * @param searchByNameQuery 通过名称检索加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByName(SearchByNameQuery searchByNameQuery,int timeOutSecs,int priorityLevel){
        if( searchByNameQuery== null||searchByNameQuery.getLocation()==null||searchByNameQuery.getKeyword()==null||searchByNameQuery.getGasType()==null||searchByNameQuery.getGasSubtype()==null){
            if (mOnSearchGasStationByNameListener != null) {
                mOnSearchGasStationByNameListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchByName";
        HashMap<String, Object> params = new HashMap<String, Object>();
        LatLng citylatLng =  searchByNameQuery.getCityLatLng();
        params.put("city_point",citylatLng.getLongitude()+","+citylatLng.getLatitude());
        params.put("word",searchByNameQuery.getKeyword());
        LatLng latLng =  searchByNameQuery.getLocation();
        params.put("location",latLng.getLongitude()+","+latLng.getLatitude());
        params.put("offset",searchByNameQuery.getOffset());
        params.put("limit",searchByNameQuery.getLimit());
        params.put("gasType",searchByNameQuery.getGasType());
        params.put("gasSubtype",searchByNameQuery.getGasSubtype());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_BY_NAME);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 沿路线检索加油站
     * @param searchByRouteQuery 沿路加油站检索条件
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByRoute(SearchByRouteQuery searchByRouteQuery){
        return searchGasStationByRoute(searchByRouteQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 沿路线检索加油站
     * @param searchByRouteQuery 沿路加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByRoute(SearchByRouteQuery searchByRouteQuery,int timeOutSecs){
        return searchGasStationByRoute(searchByRouteQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 沿路线检索加油站
     * @param searchByRouteQuery 沿路加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationByRoute(SearchByRouteQuery searchByRouteQuery,int timeOutSecs,int priorityLevel){
        if( searchByRouteQuery== null||searchByRouteQuery.getRouteDatas()==null||searchByRouteQuery.getGasType()==null||searchByRouteQuery.getGasSubtype()==null){
            if (mOnSearchGasStationByRouteListener != null) {
                mOnSearchGasStationByRouteListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchByRoute";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<Route> loc_info_list = searchByRouteQuery.getRouteDatas();
        JSONObject jsonObject = new JSONObject();
        JSONArray routeDatas = new JSONArray();
        JSONArray locationDatas = new JSONArray();
        try {
            jsonObject.put("gasType",searchByRouteQuery.getGasType());
            jsonObject.put("offset",searchByRouteQuery.getOffset());
            jsonObject.put("limit",searchByRouteQuery.getLimit());
            jsonObject.put("gasSubtype",searchByRouteQuery.getGasSubtype());
            for (int i = 0; i < loc_info_list.size(); i++) {
                Route route = loc_info_list.get(i);
                List<LatLng> locationList = route.getLocationList();
                JSONObject object = new JSONObject();
                object.put("radius", route.getRadius());
                object.put("side", route.getSide());
                for (int j = 0; j < locationList.size(); j++) {
                    JSONObject routeObject = new JSONObject();
                    LatLng latLng = locationList.get(j);
                    routeObject.put("lon",latLng.getLongitude());
                    routeObject.put("lat", latLng.getLatitude());
                    locationDatas.put(routeObject);
                    object.put("loc_info_list",locationDatas);
                }
                routeDatas.put(object);
            }
            jsonObject.put("data", routeDatas);
        } catch (JSONException e) {
            e.printStackTrace();
        }
        params.put("data_list", jsonObject);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_BY_ROUTE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 城市列表获取
     * @return 请求ID，0:参数错误
     */
    public int getServiceSupportAdminCodeList(){
        return getServiceSupportAdminCodeList(Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 城市列表获取
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int getServiceSupportAdminCodeList(int timeOutSecs){
        return getServiceSupportAdminCodeList(timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 城市列表获取
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int getServiceSupportAdminCodeList(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "GetServiceSupportAdminCodeList";
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, url, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, GET_ADMIN_CODE_LIST);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 取得城下所有加油站基本信息
     * @param cityPoint 查询城市经纬度
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationsByCityLonLat(LatLng cityPoint){
        return searchGasStationsByCityLonLat(cityPoint, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 取得城下所有加油站基本信息
     * @param cityPoint 查询城市经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationsByCityLonLat(LatLng cityPoint, int timeOutSecs){
        return searchGasStationsByCityLonLat(cityPoint, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 取得城下所有加油站基本信息
     * @param cityPoint 查询城市经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationsByCityLonLat(LatLng cityPoint,int timeOutSecs,int priorityLevel){
        if (cityPoint==null){
            if (mOnSearchByCityLonLatListener!=null){
                mOnSearchByCityLonLatListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchByCityLonLat";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("city_point_lon", cityPoint.getLongitude());
        params.put("city_point_lat", cityPoint.getLatitude());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_BY_CITY_LONLAT);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 根据ID取得加油站详细信息
     * @param ids 查询的加油站ID
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationById(List<String> ids ){
        return searchGasStationById(ids, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据ID取得加油站详细信息
     * @param ids 查询的加油站ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationById(List<String> ids, int timeOutSecs){
        return searchGasStationById(ids, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据ID取得加油站详细信息
     * @param ids 查询的加油站ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationById(List<String> ids,int timeOutSecs,int priorityLevel){
        if (ids==null){
            if (mOnSearchGasStationByIdListener!=null){
                mOnSearchGasStationByIdListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchById";
        HashMap<String, Object> params = new HashMap<String, Object>();
        StringBuffer idSbf = new StringBuffer();
        for (int i=0;i<ids.size();i++){
            if(i == ids.size()-1){
                idSbf.append(ids.get(i));
            }else {
                idSbf.append(ids.get(i)).append(",");
            }
        }
        params.put("poiIds", idSbf);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_BY_ID);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 屏幕范围内加油站检索
     * @param searchInRectangleQuery 屏幕范围内加油站检索条件
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationInRectangle(SearchInRectangleQuery searchInRectangleQuery){
        return searchGasStationInRectangle(searchInRectangleQuery,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 屏幕范围内加油站检索
     * @param searchInRectangleQuery 屏幕范围内加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationInRectangle(SearchInRectangleQuery searchInRectangleQuery,int timeOutSecs){
        return searchGasStationInRectangle(searchInRectangleQuery,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 屏幕范围内加油站检索
     * @param searchInRectangleQuery 屏幕范围内加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchGasStationInRectangle(SearchInRectangleQuery searchInRectangleQuery, int timeOutSecs, int priorityLevel){
        if( searchInRectangleQuery== null||searchInRectangleQuery.getLeftBottom()==null||searchInRectangleQuery.getRightTop()==null
                ||searchInRectangleQuery.getGasSubtype()==null||searchInRectangleQuery.getGasType()==null || searchInRectangleQuery.getScale() == null){
            if (mOnSearchGasStationNearByListener != null) {
                mOnSearchGasStationNearByListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "SearchInRectangle";
        HashMap<String, Object> params = new HashMap<String, Object>();
        LatLng leftBottom =  searchInRectangleQuery.getLeftBottom();
        LatLng rightTop =  searchInRectangleQuery.getRightTop();
        params.put("left_bottom", leftBottom.getLongitude()+","+leftBottom.getLatitude());
        params.put("right_top", rightTop.getLongitude()+","+rightTop.getLatitude());
        params.put("gasType", searchInRectangleQuery.getGasType());
        params.put("gasSubtype", searchInRectangleQuery.getGasSubtype());
        params.put("scale", searchInRectangleQuery.getScale());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SEARCH_GASSTATION_IN_RECTANGLE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 常去加油站检索
     * @param searchByPreferQuery 常去加油站检索条件
     * @return
     */
    public int searchGasStationByPrefer(SearchByPreferQuery searchByPreferQuery){
        return searchGasStationByPrefer(searchByPreferQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 常去加油站检索
     * @param searchByPreferQuery 常去加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int searchGasStationByPrefer(SearchByPreferQuery searchByPreferQuery,int timeOutSecs){
        return searchGasStationByPrefer(searchByPreferQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 常去加油站检索
     * @param searchByPreferQuery 常去加油站检索条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel
     * @return
     */
    public int searchGasStationByPrefer(SearchByPreferQuery searchByPreferQuery,int timeOutSecs,int priorityLevel){
        if (searchByPreferQuery == null ||Config.CUSTOMER_KEY == 0 ||Config.VIN == null
                ||searchByPreferQuery.getLatLng() == null){
            if (mOnSearchGasStationByPreferListener != null){
                mOnSearchGasStationByPreferListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = Config.CUSTOMER_KEY + "/VIN/" + Config.VIN + "/gas/preferGas";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("lon",searchByPreferQuery.getLatLng().getLongitude());
        params.put("lat",searchByPreferQuery.getLatLng().getLatitude());
        params.put("radius",searchByPreferQuery.getRadius());
        //String responseData = Utils.makeRestFulUrl(url,params);
        String responseData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,responseData,thisServiceType,serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId,SEARCH_GASSTATION_BY_PREFER);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    public int searchGasStationByPreferBrand(String vin,String productId){
        return searchGasStationByPreferBrand(vin,productId,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    public int searchGasStationByPreferBrand(String vin,String productId,int timeOutSecs){
        return searchGasStationByPreferBrand(vin,productId,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    public int searchGasStationByPreferBrand(String vin,String productId,int timeOutSecs,int priorityLevel){
        if (vin == null ||productId == null){
            if (mOnSearchGasStationByPreferBrandListener != null){
                mOnSearchGasStationByPreferBrandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = Config.CUSTOMER_KEY + "/VIN/" + Config.VIN + "/gas/preferBrand";
        Utils.loge(url);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,url,thisServiceType,serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId,SEARCH_GASSTATION_BY_PREFER_BRAND);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, GasStationSearch.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCH_GASSTATION_NEAR_BY:
                if (mOnSearchGasStationNearByListener != null) {
                    mOnSearchGasStationNearByListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_NAME:
                if (mOnSearchGasStationByNameListener != null) {
                    mOnSearchGasStationByNameListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_ROUTE:
                if (mOnSearchGasStationByRouteListener != null) {
                    mOnSearchGasStationByRouteListener.onTimeOut();
                }
                break;
            case GET_ADMIN_CODE_LIST:
                if (mOnGetAdminCodeListListener != null) {
                    mOnGetAdminCodeListListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_CITY_LONLAT:
                if (mOnSearchByCityLonLatListener != null) {
                    mOnSearchByCityLonLatListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_ID:
                if (mOnSearchGasStationByIdListener != null) {
                    mOnSearchGasStationByIdListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_IN_RECTANGLE:
                if (mOnSearchGasStationInRectangleListener != null) {
                    mOnSearchGasStationInRectangleListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_PREFER:
                if (mOnSearchGasStationByPreferListener != null){
                    mOnSearchGasStationByPreferListener.onTimeOut();
                }
                break;
            case SEARCH_GASSTATION_BY_PREFER_BRAND:
                if (mOnSearchGasStationByPreferBrandListener != null){
                    mOnSearchGasStationByPreferBrandListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCH_GASSTATION_NEAR_BY://周边加油站检索
                if (mOnSearchGasStationNearByListener != null) {
                    if(hasDSPTError){
                        mOnSearchGasStationNearByListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    GasStationResult gasStationResult = new GasStationResult();
                    List<GasStationInfo> list = new ArrayList<GasStationInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = jsonObject.getInt("total");
                            JSONArray resultsarray = jsonObject.getJSONArray("results");
                            GasStationInfo gasStationInfo;
                            for (int i = 0; i < resultsarray.length(); i++) {
                                JSONObject object = resultsarray.getJSONObject(i);
                                gasStationInfo = new GasStationInfo();
                                LatLng latLng = new LatLng();
                                latLng.setLongitude(object.getDouble("lon"));
                                latLng.setLatitude(object.getDouble("lat"));
                                gasStationInfo.setLocation(latLng);
                                gasStationInfo.setId(object.getString("id"));
                                gasStationInfo.setName(object.getString("name"));
                                gasStationInfo.setAreaname(object.getString("areaname"));
                                gasStationInfo.setAddress(object.getString("address"));
                                gasStationInfo.setBrandname(object.getString("brandname"));
                                gasStationInfo.setType(object.getString("type"));
                                gasStationInfo.setDiscount(object.getString("discount"));
                                gasStationInfo.setExhaust(object.getString("exhaust"));
                                gasStationInfo.setFwlsmc(object.getString("fwlsmc"));
                                gasStationInfo.setDistance(object.getInt("distance"));
                                List<GasPrice> gasOlinePrices =  new ArrayList<GasPrice>();
                                GasPrice gasPrice;
                                if (object.has("gasolinePrice")){
                                    JSONObject gasolinePrice = object.getJSONObject("gasolinePrice");
                                    Iterator<String> it = gasolinePrice.keys();
                                    while (it.hasNext()){
                                        String key = it.next();
                                        double value = gasolinePrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        gasOlinePrices.add(gasPrice);
                                    }
                                    gasStationInfo.setGasolinePrice(gasOlinePrices);
                                }

                                if(object.has("dieselOilPrice")){
                                    List<GasPrice> dieselOilPrices =  new ArrayList<GasPrice>();
                                    JSONObject dieselOilPrice = object.getJSONObject("dieselOilPrice");
                                    Iterator<String> dieselOilit = dieselOilPrice.keys();
                                    while (dieselOilit.hasNext()){
                                        String key = dieselOilit.next();
                                        double value = dieselOilPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        dieselOilPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setDieselOilPrice(dieselOilPrices);
                                }
                                if(object.has("naturalGasPrice")){
                                    List<GasPrice> naturalGasPrices =  new ArrayList<GasPrice>();
                                    JSONObject naturalGasPrice = object.getJSONObject("naturalGasPrice");
                                    Iterator<String> naturalGasit = naturalGasPrice.keys();
                                    while (naturalGasit.hasNext()){
                                        String key = naturalGasit.next();
                                        double value = naturalGasPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        naturalGasPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setNaturalGasPrice(naturalGasPrices);
                                }
                                if(object.has("otherPrice")){
                                    List<GasPrice> otherPrices =  new ArrayList<GasPrice>();
                                    JSONObject otherPrice = object.getJSONObject("otherPrice");
                                    Iterator<String> otherPriceit = otherPrice.keys();
                                    while (otherPriceit.hasNext()){
                                        String key = otherPriceit.next();
                                        double value = otherPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        otherPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setOtherPrice(otherPrices);
                                }
                                list.add(gasStationInfo);
                                gasStationResult.setTotal(total);
                                gasStationResult.setGasStations(list);
                            }
                            mOnSearchGasStationNearByListener.onSearchGasStationNearBy(requestHandleId, gasStationResult);
                        }else{
                            mOnSearchGasStationNearByListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchGasStationNearByListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_NAME://按名称检索加油站
                if (mOnSearchGasStationByNameListener != null) {
                    if(hasDSPTError){
                        mOnSearchGasStationByNameListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    GasStationResult gasStationResult = new GasStationResult();
                    List<GasStationInfo> list = new ArrayList<GasStationInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = jsonObject.getInt("total");
                            JSONArray resultsarray = jsonObject.getJSONArray("results");
                            GasStationInfo gasStationInfo;
                            for (int j = 0; j < resultsarray.length(); j++) {
                                JSONObject object = resultsarray.getJSONObject(j);
                                gasStationInfo = new GasStationInfo();
                                LatLng latLng = new LatLng();
                                latLng.setLongitude(object.getDouble("lon"));
                                latLng.setLatitude(object.getDouble("lat"));
                                gasStationInfo.setLocation(latLng);
                                gasStationInfo.setId(object.getString("id"));
                                gasStationInfo.setName(object.getString("name"));
                                gasStationInfo.setAreaname(object.getString("areaname"));
                                gasStationInfo.setAddress(object.getString("address"));
                                gasStationInfo.setBrandname(object.getString("brandname"));
                                gasStationInfo.setType(object.getString("type"));
                                gasStationInfo.setDiscount(object.getString("discount"));
                                gasStationInfo.setExhaust(object.getString("exhaust"));
                                gasStationInfo.setFwlsmc(object.getString("fwlsmc"));
                                gasStationInfo.setDistance(object.getInt("distance"));
                                List<GasPrice> gasOlinePrices =  new ArrayList<GasPrice>();
                                GasPrice gasPrice;
                                if (object.has("gasolinePrice")){
                                    JSONObject gasolinePrice = object.getJSONObject("gasolinePrice");
                                    Iterator<String> it = gasolinePrice.keys();
                                    while (it.hasNext()){
                                        String key = it.next();
                                        double value = gasolinePrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        gasOlinePrices.add(gasPrice);
                                    }
                                    gasStationInfo.setGasolinePrice(gasOlinePrices);
                                }

                                if(object.has("dieselOilPrice")){
                                    List<GasPrice> dieselOilPrices =  new ArrayList<GasPrice>();
                                    JSONObject dieselOilPrice = object.getJSONObject("dieselOilPrice");
                                    Iterator<String> dieselOilit = dieselOilPrice.keys();
                                    while (dieselOilit.hasNext()){
                                        String key = dieselOilit.next();
                                        double value = dieselOilPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        dieselOilPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setDieselOilPrice(dieselOilPrices);
                                }
                                if(object.has("naturalGasPrice")){
                                    List<GasPrice> naturalGasPrices =  new ArrayList<GasPrice>();
                                    JSONObject naturalGasPrice = object.getJSONObject("naturalGasPrice");
                                    Iterator<String> naturalGasit = naturalGasPrice.keys();
                                    while (naturalGasit.hasNext()){
                                        String key = naturalGasit.next();
                                        double value = naturalGasPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        naturalGasPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setNaturalGasPrice(naturalGasPrices);
                                }
                                if(object.has("otherPrice")){
                                    List<GasPrice> otherPrices =  new ArrayList<GasPrice>();
                                    JSONObject otherPrice = object.getJSONObject("otherPrice");
                                    Iterator<String> otherPriceit = otherPrice.keys();
                                    while (otherPriceit.hasNext()){
                                        String key = otherPriceit.next();
                                        double value = otherPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        otherPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setOtherPrice(otherPrices);
                                }
                                list.add(gasStationInfo);
                                gasStationResult.setTotal(total);
                                gasStationResult.setGasStations(list);
                            }
                            mOnSearchGasStationByNameListener.onSearchGasStationByName(requestHandleId, gasStationResult);
                        }else{
                            mOnSearchGasStationByNameListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchGasStationByNameListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_ROUTE://沿路线检索加油站
                if (mOnSearchGasStationByRouteListener != null) {
                    if(hasDSPTError){
                        mOnSearchGasStationByRouteListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    GasStationResult gasStationResult = new GasStationResult();
                    List<GasStationInfo> list = new ArrayList<GasStationInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = jsonObject.getInt("total");
                            JSONArray resultsarray = jsonObject.getJSONArray("results");
                            GasStationInfo gasStationInfo;
                            for (int j = 0; j < resultsarray.length(); j++) {
                                JSONObject object = resultsarray.getJSONObject(j);
                                gasStationInfo = new GasStationInfo();
                                LatLng latLng = new LatLng();
                                latLng.setLongitude(object.getDouble("lon"));
                                latLng.setLatitude(object.getDouble("lat"));
                                gasStationInfo.setLocation(latLng);
                                gasStationInfo.setId(object.getString("id"));
                                gasStationInfo.setName(object.getString("name"));
                                gasStationInfo.setAreaname(object.getString("areaname"));
                                gasStationInfo.setAddress(object.getString("address"));
                                gasStationInfo.setBrandname(object.getString("brandname"));
                                gasStationInfo.setType(object.getString("type"));
                                gasStationInfo.setDiscount(object.getString("discount"));
                                gasStationInfo.setExhaust(object.getString("exhaust"));
                                gasStationInfo.setFwlsmc(object.getString("fwlsmc"));
                                gasStationInfo.setDistance(object.getInt("distance"));
                                gasStationInfo.setSide(object.getInt("side"));
                                List<GasPrice> gasOlinePrices =  new ArrayList<GasPrice>();
                                GasPrice gasPrice;
                                if (object.has("gasolinePrice")){
                                    JSONObject gasolinePrice = object.getJSONObject("gasolinePrice");
                                    Iterator<String> it = gasolinePrice.keys();
                                    while (it.hasNext()){
                                        String key = it.next();
                                        double value = gasolinePrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        gasOlinePrices.add(gasPrice);
                                    }
                                    gasStationInfo.setGasolinePrice(gasOlinePrices);
                                }

                                if(object.has("dieselOilPrice")){
                                    List<GasPrice> dieselOilPrices =  new ArrayList<GasPrice>();
                                    JSONObject dieselOilPrice = object.getJSONObject("dieselOilPrice");
                                    Iterator<String> dieselOilit = dieselOilPrice.keys();
                                    while (dieselOilit.hasNext()){
                                        String key = dieselOilit.next();
                                        double value = dieselOilPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        dieselOilPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setDieselOilPrice(dieselOilPrices);
                                }
                                if(object.has("naturalGasPrice")){
                                    List<GasPrice> naturalGasPrices =  new ArrayList<GasPrice>();
                                    JSONObject naturalGasPrice = object.getJSONObject("naturalGasPrice");
                                    Iterator<String> naturalGasit = naturalGasPrice.keys();
                                    while (naturalGasit.hasNext()){
                                        String key = naturalGasit.next();
                                        double value = naturalGasPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        naturalGasPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setNaturalGasPrice(naturalGasPrices);
                                }
                                if(object.has("otherPrice")){
                                    List<GasPrice> otherPrices =  new ArrayList<GasPrice>();
                                    JSONObject otherPrice = object.getJSONObject("otherPrice");
                                    Iterator<String> otherPriceit = otherPrice.keys();
                                    while (otherPriceit.hasNext()){
                                        String key = otherPriceit.next();
                                        double value = otherPrice.getDouble(key);
                                        gasPrice = new GasPrice();
                                        gasPrice.setGasName(key);
                                        gasPrice.setPrice(value);
                                        otherPrices.add(gasPrice);
                                    }
                                    gasStationInfo.setOtherPrice(otherPrices);
                                }
                                list.add(gasStationInfo);
                                gasStationResult.setTotal(total);
                                gasStationResult.setGasStations(list);
                            }
                            mOnSearchGasStationByRouteListener.onSearchGasStationByRoute(requestHandleId, gasStationResult);
                        }else{
                            mOnSearchGasStationByRouteListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchGasStationByRouteListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GET_ADMIN_CODE_LIST://城市列表获取
                if (mOnGetAdminCodeListListener!=null){
                    if(hasDSPTError){
                        mOnGetAdminCodeListListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String adminCodeList = jsonObject.getString("result");
                            mOnGetAdminCodeListListener.onGetAdminCodeList(requestHandleId, adminCodeList);
                        }else {
                            mOnGetAdminCodeListListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetAdminCodeListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_CITY_LONLAT://取得城下所有加油站基本信息
                if (mOnSearchByCityLonLatListener!=null){
                    if(hasDSPTError){
                        mOnSearchByCityLonLatListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = jsonObject.getInt("total");
                            JSONArray jsonArray = jsonObject.getJSONArray("results");
                            List<GasStationListInfo> gasStationListInfoList = new ArrayList<GasStationListInfo>();
                            GasStationListInfo gasStationListInfo ;
                            LatLng latLng;
                            for (int i=0;i<jsonArray.length();i++){
                                JSONObject gasStationObj = jsonArray.getJSONObject(i);
                                gasStationListInfo = new GasStationListInfo();
                                latLng = new LatLng();
                                gasStationListInfo.setId(gasStationObj.getString("id"));
                                latLng.setLatitude(gasStationObj.getDouble("lat"));
                                latLng.setLongitude(gasStationObj.getDouble("lon"));
                                gasStationListInfo.setLatLng(latLng);
                                gasStationListInfoList.add(gasStationListInfo);
                            }
                            mOnSearchByCityLonLatListener.onSearchByCityLonLat(requestHandleId, total, gasStationListInfoList);
                        }else {
                            mOnSearchByCityLonLatListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSearchByCityLonLatListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_ID://根据ID取得加油站详细信息
                if (mOnSearchGasStationByIdListener!=null){
                    if(hasDSPTError){
                        mOnSearchGasStationByIdListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = 0;
                            List<GasStationInfoById> gasStationInfoByIdList = new ArrayList<GasStationInfoById>();
                            if (jsonObject.has("total")){
                                total = jsonObject.getInt("total");
                            }
                            if (jsonObject.has("results")){
                                JSONArray jsonArray = jsonObject.getJSONArray("results");
                                GasStationInfoById gasStationInfoById ;
                                for (int j = 0; j < jsonArray.length(); j++) {
                                    JSONObject object = jsonArray.getJSONObject(j);
                                    gasStationInfoById = new GasStationInfoById();
                                    LatLng latLng = new LatLng();
                                    latLng.setLongitude(object.getDouble("lon"));
                                    latLng.setLatitude(object.getDouble("lat"));
                                    gasStationInfoById.setLocation(latLng);
                                    gasStationInfoById.setId(object.getString("id"));
                                    gasStationInfoById.setName(object.getString("name"));
                                    gasStationInfoById.setAreaname(object.getString("areaname"));
                                    gasStationInfoById.setAddress(object.getString("address"));
                                    gasStationInfoById.setBrandname(object.getString("brandname"));
                                    gasStationInfoById.setType(object.getString("type"));
                                    gasStationInfoById.setDiscount(object.getString("discount"));
                                    gasStationInfoById.setExhaust(object.getString("exhaust"));
                                    gasStationInfoById.setFwlsmc(object.getString("fwlsmc"));
                                    if (object.has("gasolinePrice")){
                                        List<GasPrice> gasOlinePrices =  new ArrayList<GasPrice>();
                                        JSONObject gasolinePrice = object.getJSONObject("gasolinePrice");
                                        Iterator<String> it = gasolinePrice.keys();
                                        GasPrice gasPrice;
                                        while (it.hasNext()){
                                            String key = it.next();
                                            double value = gasolinePrice.getDouble(key);
                                            gasPrice = new GasPrice();
                                            gasPrice.setGasName(key);
                                            gasPrice.setPrice(value);
                                            gasOlinePrices.add(gasPrice);
                                        }
                                        gasStationInfoById.setGasolinePrice(gasOlinePrices);
                                    }

                                    if(object.has("dieselOilPrice")){
                                        List<GasPrice> dieselOilPrices =  new ArrayList<GasPrice>();
                                        JSONObject dieselOilPrice = object.getJSONObject("dieselOilPrice");
                                        Iterator<String> dieselOilit = dieselOilPrice.keys();
                                        while (dieselOilit.hasNext()){
                                            String key = dieselOilit.next();
                                            double value = dieselOilPrice.getDouble(key);
                                            GasPrice gasPrice = new GasPrice();
                                            gasPrice.setGasName(key);
                                            gasPrice.setPrice(value);
                                            dieselOilPrices.add(gasPrice);
                                        }
                                        gasStationInfoById.setDieselOilPrice(dieselOilPrices);
                                    }
                                    if(object.has("naturalGasPrice")){
                                        List<GasPrice> naturalGasPrices =  new ArrayList<GasPrice>();
                                        JSONObject naturalGasPrice = object.getJSONObject("naturalGasPrice");
                                        Iterator<String> naturalGasit = naturalGasPrice.keys();
                                        while (naturalGasit.hasNext()){
                                            String key = naturalGasit.next();
                                            double value = naturalGasPrice.getDouble(key);
                                            GasPrice gasPrice = new GasPrice();
                                            gasPrice.setGasName(key);
                                            gasPrice.setPrice(value);
                                            naturalGasPrices.add(gasPrice);
                                        }
                                        gasStationInfoById.setNaturalGasPrice(naturalGasPrices);
                                    }
                                    if(object.has("otherPrice")){
                                        List<GasPrice> otherPrices =  new ArrayList<GasPrice>();
                                        JSONObject otherPrice = object.getJSONObject("otherPrice");
                                        Iterator<String> otherPriceit = otherPrice.keys();
                                        while (otherPriceit.hasNext()){
                                            String key = otherPriceit.next();
                                            double value = otherPrice.getDouble(key);
                                            GasPrice gasPrice = new GasPrice();
                                            gasPrice.setGasName(key);
                                            gasPrice.setPrice(value);
                                            otherPrices.add(gasPrice);
                                        }
                                        gasStationInfoById.setOtherPrice(otherPrices);
                                    }
                                    gasStationInfoByIdList.add(gasStationInfoById);
                                }
                            }
                            mOnSearchGasStationByIdListener.onSearchGasStationById(requestHandleId, total, gasStationInfoByIdList);
                        }else {
                            mOnSearchGasStationByIdListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSearchGasStationByIdListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_IN_RECTANGLE:  //屏幕范围内加油站搜索
                if (mOnSearchGasStationInRectangleListener != null) {
                    if (hasDSPTError) {
                        mOnSearchGasStationInRectangleListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    SearchInRectangleResult searchInRectangleResult = new SearchInRectangleResult();
                    List<GasStationListInfo> list = new ArrayList<GasStationListInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    Utils.loge(jsonParser.toString());
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            searchInRectangleResult.setTotal(jsonObject.getInt("total"));
                            searchInRectangleResult.setScale(jsonObject.getDouble("scale"));
                            JSONArray resultsarray = jsonObject.getJSONArray("results");
                            GasStationListInfo gasStationListInfo;
                            for (int i = 0; i < resultsarray.length(); i++) {
                                JSONObject object = resultsarray.getJSONObject(i);
                                gasStationListInfo = new GasStationListInfo();
                                LatLng latLng = new LatLng();
                                if (object.has("lon")){
                                    latLng.setLongitude(object.getDouble("lon"));
                                }
                                if (object.has("lat")){
                                    latLng.setLatitude(object.getDouble("lat"));
                                }
                                gasStationListInfo.setLatLng(latLng);
                                if (object.has("id")) {
                                    gasStationListInfo.setId(object.getString("id"));
                                }
                                if (object.has("name")){
                                    gasStationListInfo.setName(object.getString("name"));
                                }
                                if (object.has("type")) {
                                    gasStationListInfo.setType(object.getString("type"));
                                }
                                if (object.has("brandname")) {
                                    gasStationListInfo.setBrandname(object.getString("brandname"));
                                }
                                if (object.has("gasolinePrice")){
                                    gasStationListInfo.setGasolinePrice(object.getString("gasolinePrice"));
                                }
                                if (object.has("dieselOilPrice")){
                                    gasStationListInfo.setDieselOilPrice(object.getString("dieselOilPrice"));
                                }
                                if (object.has("otherPrice")){
                                    gasStationListInfo.setOtherPrice(object.getString("otherPrice"));
                                }
                                list.add(gasStationListInfo);
                            }
                            searchInRectangleResult.setGasStationListInfoList(list);
                            mOnSearchGasStationInRectangleListener.onSearchGasStationInRectangle(requestHandleId, searchInRectangleResult);
                        }else{
                            mOnSearchGasStationInRectangleListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchGasStationInRectangleListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_PREFER: // 常去加油站搜索
                if (mOnSearchGasStationByPreferListener != null){
                    if (hasDSPTError){
                        mOnSearchGasStationByPreferListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    SearchByPreferResult searchByPreferResult = new SearchByPreferResult();
                    List<GasPreferInfo> list = new ArrayList<GasPreferInfo>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            JSONArray jsonArray = jsonObject.getJSONArray("data");
                            GasPreferInfo gasPreferInfo;
                            for (int i = 0 ; i< jsonArray.length(); i++){
                                JSONObject object = jsonArray.getJSONObject(i);
                                gasPreferInfo = new GasPreferInfo();
                                LatLng latLng = new LatLng();
                                latLng.setLatitude(object.getDouble("lat"));
                                latLng.setLongitude(object.getDouble("lon"));
                                gasPreferInfo.setLatLng(latLng);
                                gasPreferInfo.setGasStationId(object.getString("gasStationId"));
                                gasPreferInfo.setGasStationName(object.getString("gasStationName"));
                                gasPreferInfo.setScore(object.getDouble("score"));
                                list.add(gasPreferInfo);
                            }
                            searchByPreferResult.setGasPreferInfoList(list);
                            mOnSearchGasStationByPreferListener.onSearchGasStationByPreferListener(requestHandleId, searchByPreferResult);
                        }else {
                            mOnSearchGasStationByPreferListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSearchGasStationByPreferListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCH_GASSTATION_BY_PREFER_BRAND:  // 常去品牌搜索
                if (mOnSearchGasStationByPreferBrandListener != null){
                    if (hasDSPTError){
                        mOnSearchGasStationByPreferBrandListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    SearchByPreferBrandResult searchByPreferBrandResult = new SearchByPreferBrandResult();
                    List<GasBrandInfo> list = new ArrayList<GasBrandInfo>();
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            JSONArray jsonArray =jsonObject.getJSONArray("data");
                            GasBrandInfo gasBrandInfo;
                            for (int i = 0 ;i < jsonArray.length(); i++){
                                JSONObject object = jsonArray.getJSONObject(i);
                                gasBrandInfo = new GasBrandInfo();
                                gasBrandInfo.setBrand(object.getString("brand"));
                                gasBrandInfo.setScore(object.getDouble("score"));
                                list.add(gasBrandInfo);
                            }
                            searchByPreferBrandResult.setGasBrandInfoList(list);
                            mOnSearchGasStationByPreferBrandListener.onSearchGasStationByPreferBrandListener(requestHandleId, searchByPreferBrandResult);
                        }else {
                            mOnSearchGasStationByPreferBrandListener.onFail(respCode,"");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSearchGasStationByPreferBrandListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
        }
    }

    public interface OnSearchGasStationNearByListener extends CallBackListener {
        public void onSearchGasStationNearBy(int requestId, GasStationResult gasStationResult);
    }

    public interface OnSearchGasStationByNameListener extends CallBackListener{
        public void onSearchGasStationByName(int requestId, GasStationResult gasStationResult);
    }

    public interface OnSearchGasStationByRouteListener extends CallBackListener{
        public void onSearchGasStationByRoute(int requestId, GasStationResult gasStationResult);
    }

    public interface OnGetAdminCodeListListener extends CallBackListener{
        public void onGetAdminCodeList(int requestId, String adminCodeList);
    }

    public interface OnSearchByCityLonLatListener extends CallBackListener{
        /**
         * 结果回调
         * @param total 加油站数量
         * @param gasStationListInfoList 加油站信息集合
         */
        public void onSearchByCityLonLat(int requestId, int total,List<GasStationListInfo> gasStationListInfoList);
    }

    public interface OnSearchGasStationByIdListener extends CallBackListener{
        public void onSearchGasStationById(int requestId, int total,List<GasStationInfoById> gasStationInfoByIdList);
    }

    public interface OnSearchGasStationInRectangleListener extends CallBackListener{
        public void onSearchGasStationInRectangle(int requestId, SearchInRectangleResult searchInRectangleResult);
    }

    public interface OnSearchGasStationByPreferListener extends CallBackListener{
        public void onSearchGasStationByPreferListener(int requestId, SearchByPreferResult searchByPrefer);
    }
    public interface OnSearchGasStationByPreferBrandListener extends CallBackListener{
        public void onSearchGasStationByPreferBrandListener(int requestId, SearchByPreferBrandResult searchByPreferBrandResult);
    }

    public void setOnSearchGasStationNearByListener(GasStationSearch.OnSearchGasStationNearByListener onSearchGasStationNearByListener) {
        mOnSearchGasStationNearByListener = onSearchGasStationNearByListener;
    }

    public void setOnSearchGasStationByNameListener(GasStationSearch.OnSearchGasStationByNameListener onSearchGasStationByNameListener) {
        mOnSearchGasStationByNameListener = onSearchGasStationByNameListener;
    }

    public void setOnSearchGasStationByRouteListener(GasStationSearch.OnSearchGasStationByRouteListener onSearchGasStationByRouteListener) {
        mOnSearchGasStationByRouteListener = onSearchGasStationByRouteListener;
    }

    public void setOnGetAdminCodeListListener(OnGetAdminCodeListListener onGetAdminCodeListListener) {
        this.mOnGetAdminCodeListListener = onGetAdminCodeListListener;
    }

    public void setOnSearchByCityLonLatListener(OnSearchByCityLonLatListener onSearchByCityLonLatListener) {
        this.mOnSearchByCityLonLatListener = onSearchByCityLonLatListener;
    }

    public void setOnSearchGasStationByIdListener(OnSearchGasStationByIdListener onSearchGasStationByIdListener){
        this.mOnSearchGasStationByIdListener = onSearchGasStationByIdListener;
    }

    public void setOnSearchGasStationInRectangleListener(OnSearchGasStationInRectangleListener onSearchGasStationInRectangleListener){
        this.mOnSearchGasStationInRectangleListener = onSearchGasStationInRectangleListener;
    }

    public void setOnSearchGasStationByPreferListener(OnSearchGasStationByPreferListener onSearchGasStationByPreferListener){
        this.mOnSearchGasStationByPreferListener = onSearchGasStationByPreferListener;
    }

    public void setOnSearchGasStationByPreferBrandListener(OnSearchGasStationByPreferBrandListener onSearchGasStationByPreferBrandListener){
        this.mOnSearchGasStationByPreferBrandListener = onSearchGasStationByPreferBrandListener;
    }

}
