package com.mxnavi.tspv2.hotel;

import com.google.gson.Gson;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.hotel.model.GetHotelsInfo;
import com.mxnavi.tspv2.hotel.model.GetHotelsResult;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/5/22.
 */

public class Hotel extends BaseTsp {
    // 请求服务类型为酒店查询服务
    final int thisServiceType = Config.MxNGTPServiceType.HOTEL.getServiceType();
    // 酒店查询服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.HOTEL.getServiceVer();

    public Hotel() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int GETHOTELS = 1;//酒店周边检索

    /**
     * 酒店周边检索
     * @param getHotelsInfo
     * @return
     */
    public int getHotels(GetHotelsInfo getHotelsInfo) {
        return getHotels(getHotelsInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 酒店周边检索
     * @param getHotelsInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @return
     */
    public int getHotels(GetHotelsInfo getHotelsInfo, int timeOutSecs) {
        return getHotels(getHotelsInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 酒店周边检索
     * @param getHotelsInfo
     * @param timeOutSecs 接口超时时间，单位为秒，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getHotels(GetHotelsInfo getHotelsInfo, int timeOutSecs, int priorityLevel) {
        if (String.valueOf(getHotelsInfo.getLatLng().getLongitude()).equals("")
                ||String.valueOf(getHotelsInfo.getLatLng().getLatitude()).equals("")
                ||getHotelsInfo.getRadius()==null) {
            if (mGetHotelsListener != null) {
                mGetHotelsListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getHotels";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("longitude", getHotelsInfo.getLatLng().getLongitude());
        params.put("latitude", getHotelsInfo.getLatLng().getLatitude());
        params.put("radius", getHotelsInfo.getRadius());
        if (getHotelsInfo.getPageIndex()!=null){
            params.put("pageIndex", getHotelsInfo.getPageIndex());
        }
        if (getHotelsInfo.getPageSize()!=null){
            params.put("pageSize",getHotelsInfo.getPageSize());
        }
        if (getHotelsInfo.getArrivalDate()!=null){
            params.put("arrivalDate",getHotelsInfo.getArrivalDate());
        }
        if (getHotelsInfo.getDepartureDate()!=null){
            params.put("departureDate",getHotelsInfo.getDepartureDate());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETHOTELS);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, Hotel.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    public interface GetHotelsListener extends CallBackListener {
        public void onGetHotels(int requestId, GetHotelsResult result);
    }

    GetHotelsListener mGetHotelsListener;


    public void setGetHotelsListener(GetHotelsListener getHotelsListener) {
        this.mGetHotelsListener = getHotelsListener;
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GETHOTELS://1
                if (mGetHotelsListener != null) {
                    mGetHotelsListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case GETHOTELS:
                if (mGetHotelsListener != null) {
                    if (hasDSPTError) {
                        mGetHotelsListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            Gson gson = new Gson();
                            GetHotelsResult result = gson.fromJson(jsonObject.getJSONObject("result").toString(), GetHotelsResult.class);
                            mGetHotelsListener.onGetHotels(requestHandleId, result);
                        } else {
                            mGetHotelsListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mGetHotelsListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }
}
