package com.mxnavi.tspv2.http;

import android.os.Handler;
import android.os.Message;
import android.util.Log;

import java.io.UnsupportedEncodingException;

/**
 * Created by Administrator on 2017/6/2.
 */

public abstract class AsyncHttpResponseHandler {
    static final int MSG_SUCCESS = 0;
    static final int MSG_FAIL = 1;
    static final int MSG_PROGRESS = 2;

    public abstract void onSuccess(int i, byte[] bytes) ;
    public abstract void onFailure(int i,  byte[] bytes, Throwable throwable) ;
    public void onRequestProgress(long bytesWritten, long contentLength, boolean done){

    }
    public final void sendProgressMessage(long bytesWritten,  long contentLength, boolean done) {
        if(handler == null){
            onRequestProgress(bytesWritten,contentLength,done);
        }else{
            Message msg = handler.obtainMessage();
            msg.obj = new Object[]{Long.valueOf(bytesWritten), Long.valueOf(contentLength),Boolean.valueOf(done)};
            msg.what = MSG_PROGRESS;
            handler.sendMessage(msg);
        }
    }
    public final void sendSuccessMessage(int statusCode,  byte[] responseBytes) {
        if(handler == null){
            onSuccess(statusCode,responseBytes);
        }else{
            Message msg = handler.obtainMessage();
            msg.obj = new Object[]{Integer.valueOf(statusCode), responseBytes};
            msg.what = MSG_SUCCESS;
            handler.sendMessage(msg);
        }
    }
    ResponderHandler handler;
    public final void sendFailureMessage(int statusCode, byte[] responseBody, Throwable throwable) {
          if(handler == null){
              onFailure(statusCode,responseBody,throwable);
          }else{
              Message msg = handler.obtainMessage();
              msg.obj = new Object[]{Integer.valueOf(statusCode), responseBody, throwable};
              msg.what = MSG_FAIL;
              handler.sendMessage(msg);
          }

    }
    public void setCallBackMode(boolean isMainThread){
        if(isMainThread){
            if(handler == null){
                handler = new ResponderHandler(this);
            }
        }else{
            handler = null;
        }
    }
    public static String getResponseString(byte[] stringBytes) {
        try {
            String e = stringBytes == null?null:new String(stringBytes, "UTF-8");
            return e != null && e.startsWith("\ufeff")?e.substring(1):e;
        } catch (UnsupportedEncodingException var3) {
            Log.e("TextHttpRH", "Encoding response into string failed", var3);
            return null;
        }
    }
    private static class ResponderHandler extends Handler {
        private final AsyncHttpResponseHandler mResponder;

        ResponderHandler(AsyncHttpResponseHandler mResponder) {
            this.mResponder = mResponder;
        }

        public void handleMessage(Message msg) {
            mResponder.handleMessage(msg);

        }
    }
    protected void handleMessage(Message msg) {
        Object[] arr;
        switch (msg.what){
            case MSG_SUCCESS:
                arr = (Object[])msg.obj;
                onSuccess(((Integer) arr[0]).intValue(),(byte[])arr[1]);
                break;
            case MSG_FAIL:
                arr = (Object[])msg.obj;
                onFailure(((Integer) arr[0]).intValue(),(byte[])arr[1],(Throwable)arr[2]);
                break;
            case MSG_PROGRESS:
                arr = (Object[])msg.obj;
                onRequestProgress(((Long) arr[0]).longValue(),((Long) arr[1]).longValue(),((Boolean) arr[2]).booleanValue());
                break;
        }
    }
}
