package com.mxnavi.tspv2.messagepush;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.messagepush.model.QueryHistoryInfo;
import com.mxnavi.tspv2.messagepush.model.QueryHistoryResult;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import okhttp3.Call;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/3/26.
 */

public class MessagePush extends BaseTsp {

    // 请求服务类型为用户鉴权
    final int thisServiceType = Config.MxNGTPServiceType.MESSAGE_PUSH.getServiceType();
    //  用户鉴权的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MESSAGE_PUSH.getServiceVer();

    public MessagePush() {

    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int BIND_USER_PUSH_RELATION = 1;//绑定用户和个推关系
    final int QUERY_HISTORY = 2;//查询车机推送历史
    final int UPDATE_HISTORY = 3;//更新历史信息状态
    final int DELETE_HISTORY = 4; // 删除记录

    /**
     * 绑定用户和个推关系
     *
     * @param userId
     * @return 请求ID，0:参数错误
     */
    public int bindUserPushRelation(String userId, String pushKey, String token) {
        return bindUserPushRelation(userId, pushKey, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定用户和个推关系
     *
     * @param userId
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int bindUserPushRelation(String userId, String pushKey, String token, int timeOutSecs) {
        return bindUserPushRelation(userId, pushKey, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 绑定用户和个推关系
     *
     * @param userId        用户id
     * @param timeOutSecs   接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int bindUserPushRelation(String userId, String pushKey, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(pushKey)
                || Utils.strIsEmpty(token)) {
            if (mBindUserPushRelationListener != null) {
                mBindUserPushRelationListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/relations/v1/modify";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("userId", userId);
        params.put("clientId", pushKey);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, BIND_USER_PUSH_RELATION);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询车机推送历史
     * @param queryHistoryInfo
     * @param token
     * @return
     */
    public int queryHistory(QueryHistoryInfo queryHistoryInfo, String token) {
        return queryHistory(queryHistoryInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车机推送历史
     * @param queryHistoryInfo
     * @param token
     * @param timeOutSecs
     * @return
     */
    public int queryHistory(QueryHistoryInfo queryHistoryInfo, String token, int timeOutSecs) {
        return queryHistory(queryHistoryInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车机推送历史
     * @param queryHistoryInfo
     * @param token
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int queryHistory(QueryHistoryInfo queryHistoryInfo, String token, int timeOutSecs, int priorityLevel) {
        if (queryHistoryInfo==null
                || queryHistoryInfo.getProductId()==null || queryHistoryInfo.getUserId() == null
                || queryHistoryInfo.getPage() == null || queryHistoryInfo.getRows() == null
                ||Utils.strIsEmpty(token)) {
            if (mQueryHistoryListener != null) {
                mQueryHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/history/v1/query";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if (queryHistoryInfo.getId()!=null){
            params.put("id", queryHistoryInfo.getId());
        }
        if (queryHistoryInfo.getDeviceNo()!=null){
            params.put("deviceNo", queryHistoryInfo.getDeviceNo());
        }
        params.put("userId",queryHistoryInfo.getUserId());
        params.put("productId", Config.CUSTOMER_KEY);
        if (queryHistoryInfo.getReadState()!=null){
            params.put("readState", queryHistoryInfo.getReadState());
        }
        if (queryHistoryInfo.getDelState() != null ){
            params.put("delState", queryHistoryInfo.getDelState());
        }
        if (queryHistoryInfo.getInfoType()!=null){
            params.put("infoType", queryHistoryInfo.getInfoType());
        }
        params.put("page",queryHistoryInfo.getPage());
        params.put("rows",queryHistoryInfo.getRows());

        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                queryHistoryInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, QUERY_HISTORY);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 更新历史信息状态
     * @param ids
     * @param userId
     * @param token
     * @return
     */
    public int updateHistory(String ids, String userId, String token) {
        return updateHistory(ids,userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 更新历史信息状态
     * @param ids
     * @param userId
     * @param token
     * @param timeOutSecs
     * @return
     */
    public int updateHistory(String ids,String userId,String token, int timeOutSecs) {
        return updateHistory(ids, userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 更新历史信息状态
     * @param ids
     * @param userId
     * @param token
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int updateHistory(String ids, String userId, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(ids)
                ||Utils.strIsEmpty(userId)
                ||Utils.strIsEmpty(token)) {
            if (mUpdateHistoryListener != null) {
                mUpdateHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/history/v1/update";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("ids", ids);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATE_HISTORY);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    public int deleteHistory(String ids, String userId, String token){
        return deleteHistory(ids, userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    public int deleteHistory(String ids, String userId, String token, int timeOutSecs){
        return deleteHistory(ids, userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    public int deleteHistory(String ids, String userId, String token, int timeOutSecs, int priorityLevel){
        if (Utils.strIsEmpty(ids) || Utils.strIsEmpty(userId) || Utils.strIsEmpty(token)){
            if (mOnDeleteHistoryListener != null){
                mOnDeleteHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "history/v1/del";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("ids", ids);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId, userId, token, requestData,
                thisServiceType, serviceDataVer, MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE );
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, DELETE_HISTORY);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    public static interface BindUserPushRelationListener extends CallBackListener {
        public void onBindUserSuccessPushRelation();
    }
    public static interface QueryHistoryListener extends CallBackListener {
        public void onQueryHistory(int total, List<QueryHistoryResult> queryHistoryResultList);
    }
    public static interface UpdateHistoryListener extends CallBackListener {
        public void onUpdateHistorySuccess();
    }

    public static interface OnDeleteHistoryListener extends CallBackListener {
        public void onDeleteHistorySuccess();
    }

    BindUserPushRelationListener mBindUserPushRelationListener;//获取验证码
    QueryHistoryListener mQueryHistoryListener;
    UpdateHistoryListener mUpdateHistoryListener;
    OnDeleteHistoryListener mOnDeleteHistoryListener; // 删除记录


    public void setBindUserPushRelationListener(BindUserPushRelationListener bindUserPushRelationListener) {
        this.mBindUserPushRelationListener = bindUserPushRelationListener;
    }
    public void setQueryHistoryListener(QueryHistoryListener queryHistoryListener) {
        this.mQueryHistoryListener = queryHistoryListener;
    }
    public void setUpdateHistoryListener(UpdateHistoryListener updateHistoryListener) {
        this.mUpdateHistoryListener = updateHistoryListener;
    }
    public void setOnDeleteHistoryListener(OnDeleteHistoryListener onDeleteHistoryListener){
        this.mOnDeleteHistoryListener = onDeleteHistoryListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, MessagePush.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case BIND_USER_PUSH_RELATION://1
                if (mBindUserPushRelationListener != null) {
                    mBindUserPushRelationListener.onTimeOut();
                }
                break;
            case QUERY_HISTORY://1
                if (mQueryHistoryListener != null) {
                    mQueryHistoryListener.onTimeOut();
                }
                break;
            case UPDATE_HISTORY://1
                if (mUpdateHistoryListener != null) {
                    mUpdateHistoryListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case BIND_USER_PUSH_RELATION://1
                if (mBindUserPushRelationListener != null) {
                    if (hasDSPTError) {
                        mBindUserPushRelationListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mBindUserPushRelationListener.onBindUserSuccessPushRelation();
                        }else {
                            mBindUserPushRelationListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mBindUserPushRelationListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case QUERY_HISTORY://2
                if (mQueryHistoryListener != null) {
                    if (hasDSPTError) {
                        mQueryHistoryListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        List<QueryHistoryResult> list = null;
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int total = jsonObject.getInt("total");
                            JSONArray jsonArray = jsonObject.getJSONArray("data");
                            int size = jsonArray.length();
                            if (size>0) {
                                list = new ArrayList<QueryHistoryResult>(size);
                                for (int i = 0; i < size; i++) {
                                    JSONObject dataObject = jsonArray.getJSONObject(i);
                                    QueryHistoryResult info = new QueryHistoryResult();
                                    info.setId(dataObject.getString("id"));
                                    info.setWarningId(dataObject.getString("warning_id"));
                                    info.setProductId(dataObject.getString("product_id"));
                                    if (dataObject.has("device_no")){
                                        info.setDeviceNo(dataObject.getString("device_no"));
                                    }
                                    info.setInfoType(dataObject.getString("info_type"));
                                    info.setInfoTime(dataObject.getString("info_time"));
                                    info.setInfoTitle(dataObject.getString("info_title"));
                                    info.setInfoContent(dataObject.getString("info_content"));
                                    info.setReadState(dataObject.getString("read_state"));
                                    info.setDelState(dataObject.getString("del_state"));
                                    if (dataObject.has("user_id")){
                                        info.setUserId(dataObject.getString("user_id"));
                                    }
                                    list.add(info);
                                }
                            }
                            mQueryHistoryListener.onQueryHistory(total, list);
                        }else {
                            mQueryHistoryListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mQueryHistoryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATE_HISTORY://3
                if (mUpdateHistoryListener != null) {
                    if (hasDSPTError) {
                        mUpdateHistoryListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateHistoryListener.onUpdateHistorySuccess();
                        }else {
                            mUpdateHistoryListener.onFail(respCode, "出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mUpdateHistoryListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case DELETE_HISTORY:
                if (mOnDeleteHistoryListener != null){
                    if (hasDSPTError) {
                        mOnDeleteHistoryListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS ){
                            mOnDeleteHistoryListener.onDeleteHistorySuccess();
                        }else {
                            mOnDeleteHistoryListener.onFail(respCode, errorMsg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDeleteHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            default:
                break;
        }
    }
}
