package com.mxnavi.tspv2.music;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.music.model.MusicTypeInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2017/4/12.
 */

public class MusicService extends BaseTsp {
    final int thisServiceType =  Config.MxNGTPServiceType.MX_MUSIC.getServiceType();
    final int serviceDataVer =  Config.MxNGTPServiceType.MX_MUSIC.getServiceVer();
    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int SEARCHMUSICTYPE = 1;//音乐类型查询

    public interface OnSearchMusicTypeListener extends CallBackListener {
        public void onSearchMusicType(List<MusicTypeInfo> types);
    }

    OnSearchMusicTypeListener mOnSearchMusicTypeListener;

    /**
     * 音乐类型查询
     * @param productId 产品系列
     * @param typeName 类型名称
     * @param remark 备注
     * @param onSearchMusicTypeListener 回调监听
     * @return 请求ID，0:参数错误
     */
    public int searchMusicType(String productId,String typeName,String remark,OnSearchMusicTypeListener onSearchMusicTypeListener){
        return searchMusicType(productId,typeName,remark,onSearchMusicTypeListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 音乐类型查询
     * @param productId 产品系列
     * @param typeName 类型名称
     * @param remark 备注
     * @param onSearchMusicTypeListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int searchMusicType(String productId,String typeName,String remark,OnSearchMusicTypeListener onSearchMusicTypeListener
            ,int timeOutSecs){
        return searchMusicType(productId,typeName,remark,onSearchMusicTypeListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 音乐类型查询
     * @param productId 产品系列
     * @param typeName 类型名称
     * @param remark 备注
     * @param onSearchMusicTypeListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int searchMusicType(String productId,String typeName,String remark,OnSearchMusicTypeListener onSearchMusicTypeListener
            ,int timeOutSecs, int priorityLevel){
        mOnSearchMusicTypeListener = onSearchMusicTypeListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "xiami/searchMusicType";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if (productId!=null&&productId.trim().length()>0){
            params.put("productId", productId);
        }
        if (typeName!=null&&typeName.trim().length()>0){
            params.put("typeName", typeName);
        }
        if (remark!=null&&remark.trim().length()>0){
            params.put("remark", remark);
        }
        String requestData = Utils.makeUrl(url, params);
        Utils.loge("requestData==="+requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, SEARCHMUSICTYPE);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, MusicService.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHMUSICTYPE:
                if (mOnSearchMusicTypeListener != null) {
                    mOnSearchMusicTypeListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHMUSICTYPE:
                if (mOnSearchMusicTypeListener!=null){
                    if(hasDSPTError){
                        mOnSearchMusicTypeListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            List<MusicTypeInfo> types = new ArrayList<MusicTypeInfo>();
                            JSONArray typeArr = jsonObject.getJSONArray("data");
                            MusicTypeInfo musicTypeInfo;
                            for (int i = 0;i<typeArr.length();i++){
                                JSONObject object = typeArr.getJSONObject(i);
                                musicTypeInfo = new MusicTypeInfo();
                                musicTypeInfo.setId(object.getString("id"));
                                musicTypeInfo.setTypeName(object.getString("typeName"));
                                musicTypeInfo.setProductId(object.getString("productId"));
                                musicTypeInfo.setCreateTime(object.getString("createTime"));
                                musicTypeInfo.setRemark(object.getString("remark"));
                                types.add(musicTypeInfo);
                            }
                            mOnSearchMusicTypeListener.onSearchMusicType(types);
                        } else{
                            mOnSearchMusicTypeListener.onFail(respCode,"查询失败");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnSearchMusicTypeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            default:
                break;
        }
    }
}
