package com.mxnavi.tspv2.ndsupgrade;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.ndsupgrade.model.BaseDataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.CurverAndRegion;
import com.mxnavi.tspv2.ndsupgrade.model.DataHeadResult;
import com.mxnavi.tspv2.ndsupgrade.model.DataKindInfo;
import com.mxnavi.tspv2.ndsupgrade.model.DataVerInfoCodeResult;
import com.mxnavi.tspv2.ndsupgrade.model.DataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.DiffData;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiBaseDataAllResult;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiBaseDataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiBaseDataResult;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiDiffDataAllRequest;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiDiffDataAllResult;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadMutiDiffDataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.GetDataHeadRequest;
import com.mxnavi.tspv2.ndsupgrade.model.GetUpdateLimitResult;
import com.mxnavi.tspv2.ndsupgrade.model.GetUpgradeListInfo;
import com.mxnavi.tspv2.ndsupgrade.model.GetUpgradeListResult;
import com.mxnavi.tspv2.ndsupgrade.model.GetShortestPathResult;
import com.mxnavi.tspv2.ndsupgrade.model.DownloadDataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.MutiDiffDataInfo;
import com.mxnavi.tspv2.ndsupgrade.model.MutiDiffDataUpgradeRelation;
import com.mxnavi.tspv2.ndsupgrade.model.ReginIdAndDestVer;
import com.mxnavi.tspv2.ndsupgrade.model.RegionInfo;
import com.mxnavi.tspv2.ndsupgrade.model.RegionShortestPathResult;
import com.mxnavi.tspv2.ndsupgrade.model.GetShortestPathRequest;
import com.mxnavi.tspv2.ndsupgrade.model.GetUpgradeInfoRequest;
import com.mxnavi.tspv2.ndsupgrade.model.GetUpgradeInfoResult;
import com.mxnavi.tspv2.ndsupgrade.model.UpgradeListBaseData;
import com.mxnavi.tspv2.ndsupgrade.model.UpgradeListBaseInfo;
import com.mxnavi.tspv2.ndsupgrade.model.UpgradeListDiffData;
import com.mxnavi.tspv2.ndsupgrade.model.UpgradeListDiffInfo;
import com.mxnavi.tspv2.ndsupgrade.model.VersionInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/6/29.
 */
public class NDSUpdateGrade extends BaseTsp{
    //请求服务类型为NDS升级(增量更新)服务
    final int thisServiceType = Config.MxNGTPServiceType.MX_NDS_UPGRADE.getServiceType();
    //NDS升级(增量更新)服务的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_NDS_UPGRADE.getServiceVer();

    final int NDS_getShortestPathTwo= 10;//获取最短数据升级路径2（加入审图号和出版号）
    final int NDS_getUpgradeList = 1;//获取可升级列表
    final int NDS_downloadMutiBaseData= 2;//批量下载基础数据
    final int NDS_downloadMutiDiffData=3;//批量下载差分数据
//    final int NDS_judgeBigger = 4;//比较两个数据版本号大小
    final int NDS_getShortestPath = 5;//获取最短数据升级路径
    final int NDS_getUpdateLimit = 6;//获取单个商品的数据更新上限
    final int NDS_getUpgradeInfo = 7;//获取升级描述信息
//    final int NDS_getDataVers = 8;//获取数据版本
    final int NDS_getDataHead = 9;//获取数据头接口
    final int NDS_downloadMutiBaseDataAll = 12;//批量下载基础数据(用于一次获取所有基础数据包)
    final int NDS_downloadMutiDiffDataAll = 13;//批量下载差分数据(用于一次获取所有差分数据包)

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    public interface OnDownloadMutiDiffDataAllListener extends CallBackListener{
        public void onDownloadMutiDiffDataAll(List<DownloadMutiDiffDataAllResult> downloadMutiDiffDataAllResult);
    }

    public interface OnDownloadMutiBaseDataAllListener extends CallBackListener{
        /**
         * 批量下载基础数据回调函数
         * @param downloadMutiBaseDataAllResult 批量下载基础数据结果
         */
        public void onDownloadMutiBaseDataAll(List<DownloadMutiBaseDataAllResult> downloadMutiBaseDataAllResult);
    }


    public interface OnGetDataHeadListener extends CallBackListener{
        /**
         * 批量下载差分数据回调函数
         * @param dataHeadResults 批量下载差分数据结果
         */
        public void onGetDataHead(List<DataHeadResult> dataHeadResults);
    }

//    public interface OnGetDataVersListener extends CallBackListener{
//        public void onGetDataVers(DataVersResult dataVersResult);
//    }

    public interface OnGetUpgradeInfoListener extends CallBackListener{
        public void onGetUpgradeInfo(GetUpgradeInfoResult getUpgradeInfoResult);
    }

    public interface OnGetUpdateLimitListener extends CallBackListener{
        public void onGetUpdateLimit(GetUpdateLimitResult getUpdateLimitResult);
    }

//    public interface OnGetShortestPathListener extends CallBackListener{
//        public void onGetShortestPath(List<RegionShortestPathResult> regionShortestPathResults);
//    }

    public interface OnGetShortestPathListener extends CallBackListener {
        public void onGetShortestPath(GetShortestPathResult getShortestPathResult);
    }

    public interface OnGetUpgradeListListener extends CallBackListener{
        public void onGetUpgradeList(List<GetUpgradeListResult> getUpgradeListResult);
    }

    public interface OndownloadMutiBaseDataListener extends CallBackListener{
        public void ondownloadMutiBaseData(DownloadMutiBaseDataResult downloadMutiBaseDataResult);
    }

    public interface OnDownloadMutiDiffDataListener extends CallBackListener{
        public void onDownloadMutiDiffData(List<DownloadDataInfo> regions);
    }


    OnGetDataHeadListener mOnGetDataHeadListener;
//    OnGetDataVersListener mOnGetDataVersListener;
    OnGetUpgradeInfoListener mOnGetUpgradeInfoListener;

    OnGetUpgradeListListener mOnGetUpgradeListListener;
//    OnGetShortestPathTwoListener mOnGetShortestPathTwoListener;
    OndownloadMutiBaseDataListener mOndownloadMutiBaseDataListener;
    OnDownloadMutiDiffDataListener mOnDownloadMutiDiffDataListener;
    OnGetShortestPathListener mOnGetShortestPathListener;
    OnGetUpdateLimitListener mOnGetUpdateLimitListener;
    OnDownloadMutiBaseDataAllListener mOnDownloadMutiBaseDataAllListener;
    OnDownloadMutiDiffDataAllListener mOnDownloadMutiDiffDataAllListener;

    public void setOnGetDataHeadListener(OnGetDataHeadListener onGetDataHeadListener){
        this.mOnGetDataHeadListener = onGetDataHeadListener;
    }

//    public void setOnGetDataVersListener(OnGetDataVersListener OnGetDataVersListener){
//        this.mOnGetDataVersListener = OnGetDataVersListener;
//    }

    public void setOnGetUpgradeInfoListener(OnGetUpgradeInfoListener onGetUpgradeInfoListener){
        this.mOnGetUpgradeInfoListener = onGetUpgradeInfoListener;
    }

    public void setOnGetUpdateLimitListener(OnGetUpdateLimitListener onGetUpdateLimitListener){
        this.mOnGetUpdateLimitListener = onGetUpdateLimitListener;
    }

    public void setOnGetShortestPathListener(OnGetShortestPathListener onGetShortestPathListener){
        this.mOnGetShortestPathListener = onGetShortestPathListener;
    }



    public void setOnGetUpgradeListListener(OnGetUpgradeListListener onGetUpgradeListListener){
        this.mOnGetUpgradeListListener = onGetUpgradeListListener;
    }

//    public void setOnGetShortestPathTwoListener(OnGetShortestPathTwoListener onGetShortestPathTwoListener) {
//        this.mOnGetShortestPathTwoListener = onGetShortestPathTwoListener;
//    }

    public void setOndownloadMutiBaseDataListener(OndownloadMutiBaseDataListener ondownloadMutiBaseDataListener){
        this.mOndownloadMutiBaseDataListener = ondownloadMutiBaseDataListener;
    }

    public void setOnDownloadMutiDiffDataListener(OnDownloadMutiDiffDataListener onDownlooadMutiDiffDataListener){
        this.mOnDownloadMutiDiffDataListener = onDownlooadMutiDiffDataListener;
    }

    /**
     * 批量下载差分数据(用于一次获取所有差分数据包)
     * @param downloadMutiDiffDataAllRequests 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiDiffDataAllListener 回调接口
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiDiffDataAll(List<DownloadMutiDiffDataAllRequest> downloadMutiDiffDataAllRequests,String productId,
                                       OnDownloadMutiDiffDataAllListener onDownloadMutiDiffDataAllListener){
        return downloadMutiDiffDataAll(downloadMutiDiffDataAllRequests,productId,onDownloadMutiDiffDataAllListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 批量下载差分数据(用于一次获取所有差分数据包)
     * @param downloadMutiDiffDataAllRequests 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiDiffDataAllListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiDiffDataAll(List<DownloadMutiDiffDataAllRequest> downloadMutiDiffDataAllRequests,String productId,
                                       OnDownloadMutiDiffDataAllListener onDownloadMutiDiffDataAllListener,int timeOutSecs){
        return downloadMutiDiffDataAll(downloadMutiDiffDataAllRequests,productId,onDownloadMutiDiffDataAllListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 批量下载差分数据(用于一次获取所有差分数据包)
     * @param downloadMutiDiffDataAllRequests 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiDiffDataAllListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiDiffDataAll(List<DownloadMutiDiffDataAllRequest> downloadMutiDiffDataAllRequests,String productId,
                                       OnDownloadMutiDiffDataAllListener onDownloadMutiDiffDataAllListener,int timeOutSecs,int priorityLevel){
        if (downloadMutiDiffDataAllRequests==null||productId==null){
            if (onDownloadMutiDiffDataAllListener!=null){
                onDownloadMutiDiffDataAllListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnDownloadMutiDiffDataAllListener = onDownloadMutiDiffDataAllListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "downloadMutiDiffDataAll";
        HashMap<String, Object> params = new HashMap<String, Object>();
        JSONArray paramArr = new JSONArray();
        JSONObject object;
        DownloadMutiDiffDataAllRequest downloadMutiDiffDataAllRequest;
        for (int i=0;i<downloadMutiDiffDataAllRequests.size();i++){
            downloadMutiDiffDataAllRequest = downloadMutiDiffDataAllRequests.get(i);
            object = new JSONObject();
            try {
                object.put("regionId",downloadMutiDiffDataAllRequest.getRegionId());
                List<MutiDiffDataUpgradeRelation> mutiDiffDataUpgradeRelations = downloadMutiDiffDataAllRequest.getMutiDiffDataUpgradeRelations();
                JSONArray jsonArray = new JSONArray();

                for (int j=0;j<mutiDiffDataUpgradeRelations.size();j++){
                    MutiDiffDataUpgradeRelation mutiDiffDataUpgradeRelation = mutiDiffDataUpgradeRelations.get(j);
                    JSONObject jsonObject = new JSONObject();
                    jsonObject.put("curVer",mutiDiffDataUpgradeRelation.getCurVer());
                    jsonObject.put("destVer",mutiDiffDataUpgradeRelation.getDestVer());
                    int[] dataKinds = mutiDiffDataUpgradeRelation.getDataKinds();
                    JSONArray dataKindsArr = new JSONArray();
                    for (int k=0;k<dataKinds.length;k++){
                        dataKindsArr.put(dataKinds[k]);
                    }
                    jsonObject.put("dataKinds",dataKindsArr);
                    jsonArray.put(jsonObject);
                }
                object.put("ver",jsonArray);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            paramArr.put(object);
        }
        params.put("param",paramArr);
        params.put("product_id",productId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, NDS_downloadMutiDiffDataAll);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    /**
     * 批量下载基础数据(用于一次获取所有基础数据包)
     * @param regionInfos 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiBaseDataAllListener 回调接口
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiBaseDataAll(List<RegionInfo> regionInfos,String productId,OnDownloadMutiBaseDataAllListener onDownloadMutiBaseDataAllListener){
        return downloadMutiBaseDataAll(regionInfos,productId,onDownloadMutiBaseDataAllListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *批量下载基础数据(用于一次获取所有基础数据包)
     * @param regionInfos 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiBaseDataAllListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiBaseDataAll(List<RegionInfo> regionInfos,String productId,OnDownloadMutiBaseDataAllListener onDownloadMutiBaseDataAllListener,int timeOutSecs){
        return downloadMutiBaseDataAll(regionInfos,productId,onDownloadMutiBaseDataAllListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *批量下载基础数据(用于一次获取所有基础数据包)
     * @param regionInfos 请求信息集合
     * @param productId 产品系列
     * @param onDownloadMutiBaseDataAllListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int downloadMutiBaseDataAll(List<RegionInfo> regionInfos,String productId,OnDownloadMutiBaseDataAllListener onDownloadMutiBaseDataAllListener,
                                       int timeOutSecs,int priorityLevel){
        if (regionInfos==null||productId==null){
            if (onDownloadMutiBaseDataAllListener!=null){
                onDownloadMutiBaseDataAllListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnDownloadMutiBaseDataAllListener = onDownloadMutiBaseDataAllListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "downloadMutiBaseDataAll";
        HashMap<String, Object> params = new HashMap<String, Object>();
        JSONArray paramsArr = new JSONArray();
        JSONObject regionInfoObj ;
        for (int i = 0;i<regionInfos.size();i++){
            RegionInfo regionInfo = regionInfos.get(i);
            regionInfoObj = new JSONObject();
            ReginIdAndDestVer reginIdAndDestVer = regionInfo.getReginIdAndDestVer();
            try {
                regionInfoObj.put("regionId",reginIdAndDestVer.getRegionId());
                regionInfoObj.put("destVer",reginIdAndDestVer.getDestVer());
                int[] dataKinds = regionInfo.getDataKinds();
                JSONArray dataKindsArr = new JSONArray();
                for (int j =0;j<dataKinds.length;j++){
                    dataKindsArr.put(dataKinds[j]);
                }
                regionInfoObj.put("dataKinds",dataKindsArr);
            } catch (JSONException e) {
                e.printStackTrace();
            }
            paramsArr.put(regionInfoObj);
        }
        params.put("param",paramsArr);
        params.put("product_id",productId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, NDS_downloadMutiBaseDataAll);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    /**
     * 获取数据头接口
     * @param getDataHeadRequest 获取数据头请求条件
     * @return
     */
    public int getDataHead(GetDataHeadRequest getDataHeadRequest){
        return getDataHead(getDataHeadRequest,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取数据头接口
     * @param getDataHeadRequest 获取数据头请求条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getDataHead(GetDataHeadRequest getDataHeadRequest,int timeOutSecs){
        return getDataHead(getDataHeadRequest,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取数据头接口
     * @param getDataHeadRequest 获取数据头请求条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getDataHead(GetDataHeadRequest getDataHeadRequest,int timeOutSecs,int priorityLevel){
        if (getDataHeadRequest ==null|| getDataHeadRequest.getProductId()==null|| getDataHeadRequest.getDestVer()==null){
            if (mOnGetDataHeadListener != null) {
                mOnGetDataHeadListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getDataHead";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", getDataHeadRequest.getProductId());
        params.put("dest_ver", getDataHeadRequest.getDestVer());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_getDataHead);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

//    /**
//     * 获取数据版本
//     * @param getDataVersRequest 获取数据版本请求条件
//     * @return
//     */
//    public int getDataVers(GetDataVersRequest getDataVersRequest){
//        return getDataVers(getDataVersRequest,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
//    }
//
//    /**
//     *获取数据版本
//     * @param getDataVersRequest 获取数据版本请求条件
//     * @param timeOutSecs 接口超时时间，传0为不关注
//     * @return
//     */
//    public int getDataVers(GetDataVersRequest getDataVersRequest,int timeOutSecs){
//        return getDataVers(getDataVersRequest,timeOutSecs , Config.DEFAULT_PRIORITY);
//    }
//
//    /**
//     * 获取数据版本
//     * @param getDataVersRequest 获取数据版本请求条件
//     * @param timeOutSecs 接口超时时间，传0为不关注
//     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
//     * @return
//     */
//    public int getDataVers(GetDataVersRequest getDataVersRequest,int timeOutSecs,int priorityLevel){
//        if (getDataVersRequest==null||getDataVersRequest.getProductId()==null){
//            if (mOnGetDataVersListener != null) {
//                mOnGetDataVersListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
//            }
//            return 0;
//        }
//        int eventId = TcpRequestUtil.createEventId();
//        String url = "getDataVers";
//        HashMap<String, Object> params = new HashMap<String, Object>();
//        params.put("product_id",getDataVersRequest.getProductId());
//        if (getDataVersRequest.getDataType()!=null){
//            params.put("data_type",getDataVersRequest.getDataType());
//        }
//        String requestData = Utils.makeUrl(url, params);
//        Utils.loge(requestData);
//        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
//        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
//        requestMessage.timeOutSeconds = timeOutSecs;
//        requestMessage.priority = priorityLevel;
//        // 添加对应关系;
//        addCallbackMap(eventId, NDS_getDataVers);
//        TcpRequestUtil.submitRequest(requestMessage);
//        return eventId;
//    }

    /**
     * 获取升级描述信息
     * @param getUpgradeInfoRequest 获取升级描述信息请求条件
     * @return
     */
    public int getUpgradeInfo(GetUpgradeInfoRequest getUpgradeInfoRequest){
        return getUpgradeInfo(getUpgradeInfoRequest,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取升级描述信息
     * @param getUpgradeInfoRequest 获取升级描述信息请求条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getUpgradeInfo(GetUpgradeInfoRequest getUpgradeInfoRequest,int timeOutSecs){
        return getUpgradeInfo(getUpgradeInfoRequest,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取升级描述信息
     * @param getUpgradeInfoRequest 获取升级描述信息请求条件
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getUpgradeInfo(GetUpgradeInfoRequest getUpgradeInfoRequest,int timeOutSecs,int priorityLevel){
        if (getUpgradeInfoRequest ==null|| getUpgradeInfoRequest.getProductId()==null|| getUpgradeInfoRequest.getDestVer()==null|| getUpgradeInfoRequest.getRegionId()==null){
            if (mOnGetUpgradeInfoListener != null) {
                mOnGetUpgradeInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getUpgradeInfo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", getUpgradeInfoRequest.getProductId());
        params.put("dest_ver", getUpgradeInfoRequest.getDestVer());
        params.put("region", getUpgradeInfoRequest.getRegionId());
        if(getUpgradeInfoRequest.getOriginVer()!=null){
            params.put("origin_ver", getUpgradeInfoRequest.getOriginVer());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_getUpgradeInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取单个商品的数据更新上限
     * @param productId 产品系列id
     * @return
     */
    public int getUpdateLimit(String productId){
        return getUpdateLimit(productId,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *获取单个商品的数据更新上限
     * @param productId 产品系列id
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getUpdateLimit(String productId,int timeOutSecs){
        return getUpdateLimit(productId,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取单个商品的数据更新上限
     * @param productId 产品系列id
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getUpdateLimit(String productId,int timeOutSecs,int priorityLevel){
        if (productId==null){
            if (mOnGetUpdateLimitListener != null) {
                mOnGetUpdateLimitListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getUpdateLimit";
        String requestData = url + "?productId="+productId;
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_getUpdateLimit);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

//    /**
//     *获取最短数据升级路径
//     * @param getShortestPathRequest 获取最短数据升级路径请求条件
//     * @return
//     */
//    public int getShortestPath(GetShortestPathorjRequest getShortestPathRequest){
//        return getShortestPath(getShortestPathRequest,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
//    }

//    /**
//     *获取最短数据升级路径
//     * @param getShortestPathRequest 获取最短数据升级路径请求条件
//     * @param timeOutSecs 接口超时时间，传0为不关注
//     * @return
//     */
//    public int getShortestPath(GetShortestPathorjRequest getShortestPathRequest,int timeOutSecs){
//        return getShortestPath(getShortestPathRequest,timeOutSecs,Config.DEFAULT_PRIORITY);
//    }
//
//    /**
//     * 获取最短数据升级路径
//     * @param getShortestPathRequest 获取最短数据升级路径请求条件
//     * @param timeOutSecs 接口超时时间，传0为不关注
//     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
//     * @return
//     */
//    public int getShortestPath(GetShortestPathorjRequest getShortestPathRequest,int timeOutSecs,int priorityLevel){
//        if(getShortestPathRequest == null|| getShortestPathRequest.getProductId()==null|| getShortestPathRequest.getSeries()==null|| getShortestPathRequest.getVersionInfoOri()==null){
//            if (mOnGetShortestPathListener != null) {
//                mOnGetShortestPathListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
//            }
//            return 0;
//        }
//        int eventId = TcpRequestUtil.createEventId();
//        String url = "getShortestPath";
//        HashMap<String, Object> params = new HashMap<String, Object>();
//        List<VersionInfoOri> versionInfoOris = getShortestPathRequest.getVersionInfoOri();
//        StringBuffer currVer = new StringBuffer();
//        StringBuffer destVer = new StringBuffer();
//        StringBuffer region = new StringBuffer();
//        for (int i=0;i<versionInfoOris.size();i++){
//            VersionInfoOri versionInfoOri = versionInfoOris.get(i);
//            if(i == versionInfoOris.size()-1){
//                currVer.append(versionInfoOri.getCurVer());
//                destVer.append(versionInfoOri.getDestVer());
//                region.append(versionInfoOri.getRegion());
//            }else {
//                currVer.append(versionInfoOri.getCurVer()).append(",");
//                destVer.append(versionInfoOri.getDestVer()).append(",");
//                region.append(versionInfoOri.getRegion()).append(",");
//            }
//        }
//        params.put("cur_ver",currVer.toString());
//        params.put("dest_ver",destVer.toString());
//        params.put("regions",region.toString());
//        params.put("series", getShortestPathRequest.getSeries());
//        params.put("product_id", getShortestPathRequest.getProductId());
//        String requestData = Utils.makeUrl(url, params);
//        Utils.loge(requestData);
//        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
//        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
//        requestMessage.timeOutSeconds = timeOutSecs;
//        requestMessage.priority = priorityLevel;
//        // 添加对应关系;
//        addCallbackMap(eventId, NDS_getShortestPath);
//        TcpRequestUtil.submitRequest(requestMessage);
//        return eventId;
//    }

    /**
     * 获取可升级列表
     * @param getUpgradeListInfo 获取可升级列表请求数据
     * @return
     */
    public int getUpgradeList(GetUpgradeListInfo getUpgradeListInfo){
        return getUpgradeList(getUpgradeListInfo,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *获取可升级列表
     * @param getUpgradeListInfo 获取可升级列表请求数据
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getUpgradeList(GetUpgradeListInfo getUpgradeListInfo,int timeOutSecs){
        return getUpgradeList(getUpgradeListInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *获取可升级列表
     * @param getUpgradeListInfo 获取可升级列表请求数据
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getUpgradeList(GetUpgradeListInfo getUpgradeListInfo,int timeOutSecs,int priorityLevel){
        if (getUpgradeListInfo==null||getUpgradeListInfo.getCurverAndRegions()==null||getUpgradeListInfo.getProductId()==null
                ||getUpgradeListInfo.getSeries()==null){
            if (mOnGetUpgradeListListener != null) {
                mOnGetUpgradeListListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getUpgradeList";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<CurverAndRegion> curverAndRegions = getUpgradeListInfo.getCurverAndRegions();
        StringBuffer currVer = new StringBuffer();
        StringBuffer regionId = new StringBuffer();
        for (int i=0;i<curverAndRegions.size();i++){
            CurverAndRegion curverAndRegion = curverAndRegions.get(i);
            if(i == curverAndRegions.size()-1){
                currVer.append(curverAndRegion.getCurVer());
                regionId.append(curverAndRegion.getRegionId());
            }else {
                currVer.append(curverAndRegion.getCurVer()).append(",");
                regionId.append(curverAndRegion.getRegionId()).append(",");
            }
        }
        params.put("curVer",currVer.toString());
        params.put("regions",regionId.toString());
        params.put("series",getUpgradeListInfo.getSeries());
        params.put("product_id",getUpgradeListInfo.getProductId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_getUpgradeList);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 批量下载差分数据
     * @param info 批量下载差分数据请求信息
     * @return
     */
    public int downloadMutiDiffData(DownloadMutiDiffDataInfo info){
        return downloadMutiDiffData(info,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 批量下载差分数据
     * @param info 批量下载差分数据请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int downloadMutiDiffData(DownloadMutiDiffDataInfo info,int timeOutSecs){
        return downloadMutiDiffData(info,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *批量下载差分数据
     * @param info 批量下载差分数据请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int downloadMutiDiffData(DownloadMutiDiffDataInfo info,int timeOutSecs,int priorityLevel){
        if (info==null||info.getProductId()==null||info.getCurrVer()==null||info.getDestVer()==null||info.getDataKinds()==null||info.getRegionIds()==null){
            if (mOnDownloadMutiDiffDataListener != null) {
                mOnDownloadMutiDiffDataListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "downloadMutiDiffData";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<String> regionIds = info.getRegionIds();
        String regions = Utils.join(regionIds, ',');
        params.put("regions",regions);
        List<Integer> dataKinds = info.getDataKinds();
        String dataKind = Utils.join(dataKinds,'_');
        params.put("data_kind",dataKind);
        params.put("curr_ver",info.getCurrVer());
        params.put("dest_ver",info.getDestVer());
        params.put("product_id",info.getProductId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_downloadMutiDiffData);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 批量下载基础数据
     * @param downloadMutiBaseDataInfo 批量下载基础数据请求信息
     * @return
     */
    public int downloadMutiBaseData(DownloadMutiBaseDataInfo downloadMutiBaseDataInfo){
        return downloadMutiBaseData(downloadMutiBaseDataInfo,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *批量下载基础数据
     * @param downloadMutiBaseDataInfo 批量下载基础数据请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int downloadMutiBaseData(DownloadMutiBaseDataInfo downloadMutiBaseDataInfo,int timeOutSecs){
        return downloadMutiBaseData(downloadMutiBaseDataInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *批量下载基础数据
     * @param downloadMutiBaseDataInfo 批量下载基础数据请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int downloadMutiBaseData(DownloadMutiBaseDataInfo downloadMutiBaseDataInfo,int timeOutSecs,int priorityLevel){
        if (downloadMutiBaseDataInfo==null||downloadMutiBaseDataInfo.getDestVer()==null||downloadMutiBaseDataInfo.getProductId()==null
                ||downloadMutiBaseDataInfo.getRegionIds()==null){
            if (mOndownloadMutiBaseDataListener != null) {
                mOndownloadMutiBaseDataListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "downloadMutiBaseData";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<String> regionIds = downloadMutiBaseDataInfo.getRegionIds();
        String regions = Utils.join(regionIds,',');
        params.put("regions",regions);
        params.put("dest_ver",downloadMutiBaseDataInfo.getDestVer());
        String dataKind = Utils.join(downloadMutiBaseDataInfo.getDataKinds(),'_');
        params.put("data_kind",dataKind);
        params.put("product_id",downloadMutiBaseDataInfo.getProductId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_downloadMutiBaseData);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }


    /**
     * 获取最短数据升级路径2（加入审图号和出版号）
     * @param info 获取最短数据升级路径2 请求信息
     * @return
     */
    public int getShortestPath(GetShortestPathRequest info){
        return getShortestPath(info, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     *获取最短数据升级路径2（加入审图号和出版号）
     * @param info 获取最短数据升级路径2 请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getShortestPath(GetShortestPathRequest info,int timeOutSecs){
        return getShortestPath(info, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     *获取最短数据升级路径2（加入审图号和出版号）
     * @param info 获取最短数据升级路径2 请求信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getShortestPath(GetShortestPathRequest info,int timeOutSecs,int priorityLevel){
        if(info == null||info.getProductId()==null||info.getSeries()==null||info.getVersionInfos()==null){
            if (mOnGetShortestPathListener != null) {
                mOnGetShortestPathListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "getShortestPathTwo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        List<VersionInfo> versionInfos = info.getVersionInfos();
        StringBuffer currVer = new StringBuffer();
        StringBuffer destVer = new StringBuffer();
        StringBuffer region = new StringBuffer();
        StringBuffer getDiffType = new StringBuffer();
        for (int i=0;i<versionInfos.size();i++){
            VersionInfo versionInfo = versionInfos.get(i);
            if(i == versionInfos.size()-1){
                currVer.append(versionInfo.getCurVer());
                destVer.append(versionInfo.getDestVer());
                region.append(versionInfo.getRegion());
                getDiffType.append(versionInfo.getGetDiffType());
            }else {
                currVer.append(versionInfo.getCurVer()).append(",");
                destVer.append(versionInfo.getDestVer()).append(",");
                region.append(versionInfo.getRegion()).append(",");
                getDiffType.append(versionInfo.getGetDiffType()).append(",");
            }
        }
        params.put("cur_ver",currVer);
        params.put("dest_ver",destVer);
        params.put("regions",region);
        params.put("series",info.getSeries());
        params.put("product_id",info.getProductId());
        params.put("get_diff_type",getDiffType);
        params.put("soft_ver",info.getSoftVer());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, NDS_getShortestPath);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, NDSUpdateGrade.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case NDS_getUpgradeList:
                if (mOnGetUpgradeListListener != null) {
                    mOnGetUpgradeListListener.onTimeOut();
                }
                break;
//            case NDS_getShortestPathTwo:
//                if (mOnGetShortestPathListener != null) {
//                    mOnGetShortestPathListener.onTimeOut();
//                }
//                break;
            case NDS_downloadMutiBaseData:
                if (mOndownloadMutiBaseDataListener != null) {
                    mOndownloadMutiBaseDataListener.onTimeOut();
                }
                break;
            case NDS_downloadMutiDiffData:
                if (mOnDownloadMutiDiffDataListener != null) {
                    mOnDownloadMutiDiffDataListener.onTimeOut();
                }
                break;

            case NDS_getShortestPath:
                if (mOnGetShortestPathListener != null) {
                    mOnGetShortestPathListener.onTimeOut();
                }
                break;
            case NDS_getUpdateLimit:
                if (mOnGetUpdateLimitListener != null) {
                    mOnGetUpdateLimitListener.onTimeOut();
                }
                break;
            case NDS_getUpgradeInfo:
                if (mOnGetUpgradeInfoListener != null) {
                    mOnGetUpgradeInfoListener.onTimeOut();
                }
                break;
//            case NDS_getDataVers:
//                if (mOnGetDataVersListener != null) {
//                    mOnGetDataVersListener.onTimeOut();
//                }
//                break;
            case NDS_getDataHead:
                if (mOnGetDataHeadListener != null) {
                    mOnGetDataHeadListener.onTimeOut();
                }
                break;
            case NDS_downloadMutiBaseDataAll:
                if (mOnDownloadMutiBaseDataAllListener != null) {
                    mOnDownloadMutiBaseDataAllListener.onTimeOut();
                }
                break;
            case NDS_downloadMutiDiffDataAll:
                if (mOnDownloadMutiDiffDataAllListener != null) {
                    mOnDownloadMutiDiffDataAllListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case NDS_downloadMutiDiffDataAll:
                if (mOnDownloadMutiDiffDataAllListener!=null){
                    if(hasDSPTError){
                        mOnDownloadMutiDiffDataAllListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray regionsArr = jsonObject.getJSONArray("regions");
                            List<DownloadMutiDiffDataAllResult> downloadMutiDiffDataAllResults = new ArrayList<DownloadMutiDiffDataAllResult>();
                            DownloadMutiDiffDataAllResult downloadMutiDiffDataAllResult;
                            for (int i=0;i<regionsArr.length();i++){
                                JSONObject regionObj = regionsArr.getJSONObject(i);
                                downloadMutiDiffDataAllResult = new DownloadMutiDiffDataAllResult();
                                downloadMutiDiffDataAllResult.setRegionId(regionObj.getString("regionId"));
                                JSONArray jsonArray = regionObj.getJSONArray("ver");
                                List<MutiDiffDataInfo> mutiDiffDataInfos = new ArrayList<MutiDiffDataInfo>();
                                MutiDiffDataInfo mutiDiffDataInfo;
                                for (int j=0;j<jsonArray.length();j++){
                                    JSONObject dataInfoObj = jsonArray.getJSONObject(j);
                                    mutiDiffDataInfo = new MutiDiffDataInfo();
                                    mutiDiffDataInfo.setDestVer(dataInfoObj.getString("destVer"));
                                    mutiDiffDataInfo.setCurVer(dataInfoObj.getString("curVer"));
                                    JSONArray dataKindInfosArr = dataInfoObj.getJSONArray("dataKind");
                                    List<DataKindInfo> dataKindInfos = new ArrayList<DataKindInfo>();
                                    DataKindInfo dataKindInfo;
                                    for (int k=0;k<dataKindInfosArr.length();k++){
                                        JSONObject dataKindInfoObj = dataKindInfosArr.getJSONObject(k);
                                        dataKindInfo = new DataKindInfo();
                                        dataKindInfo.setDataKind(dataKindInfoObj.getInt("id"));
                                        dataKindInfo.setUrl(dataKindInfoObj.getString("url"));
                                        dataKindInfos.add(dataKindInfo);
                                    }
                                    mutiDiffDataInfo.setDataKindInfos(dataKindInfos);
                                    mutiDiffDataInfos.add(mutiDiffDataInfo);
                                }
                                downloadMutiDiffDataAllResult.setMutiDiffDataInfos(mutiDiffDataInfos);
                                downloadMutiDiffDataAllResults.add(downloadMutiDiffDataAllResult);
                            }
                            mOnDownloadMutiDiffDataAllListener.onDownloadMutiDiffDataAll(downloadMutiDiffDataAllResults);
                        }else {
                            mOnDownloadMutiDiffDataAllListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDownloadMutiDiffDataAllListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case NDS_downloadMutiBaseDataAll:
                if (mOnDownloadMutiBaseDataAllListener!=null){
                    if(hasDSPTError){
                        mOnDownloadMutiBaseDataAllListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray regionsArr = jsonObject.getJSONArray("regions");
                            List<DownloadMutiBaseDataAllResult> downloadMutiBaseDataAllResults = new ArrayList<DownloadMutiBaseDataAllResult>();
                            DownloadMutiBaseDataAllResult downloadMutiBaseDataAllResult;
                            for (int i = 0;i<regionsArr.length();i++){
                                downloadMutiBaseDataAllResult = new DownloadMutiBaseDataAllResult();
                                JSONObject regionObj = regionsArr.getJSONObject(i);
                                downloadMutiBaseDataAllResult.setRegionId(regionObj.getString("regionId"));
                                downloadMutiBaseDataAllResult.setDestVer(regionObj.getString("destVer"));
                                JSONArray dataKindArr = regionObj.getJSONArray("dataKind");
                                DataKindInfo dataKindInfo;
                                List<DataKindInfo> DataKindInfos = new ArrayList<DataKindInfo>();
                                for (int j =0;j<dataKindArr.length();j++){
                                    JSONObject dataKindInfoObj = dataKindArr.getJSONObject(j);
                                    dataKindInfo = new DataKindInfo();
                                    dataKindInfo.setUrl(dataKindInfoObj.getString("url"));
                                    dataKindInfo.setDataKind(dataKindInfoObj.getInt("id"));
                                    DataKindInfos.add(dataKindInfo);
                                }
                                downloadMutiBaseDataAllResult.setDataKindInfo(DataKindInfos);
                                downloadMutiBaseDataAllResults.add(downloadMutiBaseDataAllResult);
                            }
                            mOnDownloadMutiBaseDataAllListener.onDownloadMutiBaseDataAll(downloadMutiBaseDataAllResults);
                        }else {
                            mOnDownloadMutiBaseDataAllListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDownloadMutiBaseDataAllListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case NDS_getDataHead:
                if (mOnGetDataHeadListener!=null){
                    if(hasDSPTError){
                        mOnGetDataHeadListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray datInfoResultsArr = jsonObject.getJSONArray("datInfo");
                            List<DataHeadResult> dataHeadResults = new ArrayList<DataHeadResult>();
                            DataHeadResult dataHeadResult;
                            for (int i=0;i<datInfoResultsArr.length();i++){
                                JSONObject datInfoResultObj = datInfoResultsArr.getJSONObject(i);
                                dataHeadResult = new DataHeadResult();
                                dataHeadResult.setUrl(datInfoResultObj.getString("addr"));
                                dataHeadResult.setSize(datInfoResultObj.getInt("size"));
                                dataHeadResult.setInfo(datInfoResultObj.getString("info"));
                                dataHeadResult.setDataKind(datInfoResultObj.getInt("datakind"));
                                dataHeadResult.setPublicTime(datInfoResultObj.getString("public_time"));
                                dataHeadResult.setDataName(datInfoResultObj.getString("datname"));
                                dataHeadResult.setMd5(datInfoResultObj.getString("md5"));
                                dataHeadResult.setUncompressSize(datInfoResultObj.getInt("uncompressSize"));
                                dataHeadResult.setUpdateSize(datInfoResultObj.getInt("updateSize"));
                                dataHeadResults.add(dataHeadResult);
                            }
                            mOnGetDataHeadListener.onGetDataHead(dataHeadResults);
                        }else {
                            mOnGetDataHeadListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetDataHeadListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
//            case NDS_getDataVers:
//                if (mOnGetDataVersListener!=null){
//                    if(hasDSPTError){
//                        mOnGetDataVersListener.onFail(errorCode,errorMsg);
//                        return;
//                    }
//                    JSONTokener jsonParser = new JSONTokener(response);
//                    try {
//                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
//                        int respCode = jsonObject.getInt("respCode");
//                        if (respCode == Constants.RESPCODE_SUCCESS) {
//                            DataVersResult dataVersResult = new DataVersResult();
//                            if (jsonObject.has("putAwayData")){
//                                JSONArray putAwayDataArr = jsonObject.getJSONArray("putAwayData");
//                                List<DataVersInfo> putAwayDatas = new ArrayList<DataVersInfo>();
//                                DataVersInfo dataVersInfo;
//                                for (int i=0;i<putAwayDataArr.length();i++){
//                                    JSONObject dataVersInfoObj = putAwayDataArr.getJSONObject(i);
//                                    dataVersInfo = new DataVersInfo();
//                                    dataVersInfo.setInfo(dataVersInfoObj.getString(""));
//                                    dataVersInfo.setPublicTime(dataVersInfoObj.getString("publicTime"));
//                                    dataVersInfo.setVer(dataVersInfoObj.getString("ver"));
//                                    dataVersInfo.setVerName(dataVersInfoObj.getString("verName"));
//                                    putAwayDatas.add(dataVersInfo);
//                                }
//                                dataVersResult.setPutAwayDatas(putAwayDatas);
//                            }
//                            if (jsonObject.has("unPutAwayData")){
//                                JSONArray unPutAwayDataArr = jsonObject.getJSONArray("unPutAwayData");
//                                List<DataVersInfo> unPutAwayDatas = new ArrayList<DataVersInfo>();
//                                DataVersInfo dataVersInfo;
//                                for (int i =0;i<unPutAwayDataArr.length();i++){
//                                    JSONObject dataVersInfoObj = unPutAwayDataArr.getJSONObject(i);
//                                    dataVersInfo = new DataVersInfo();
//                                    dataVersInfo.setVerName(dataVersInfoObj.getString("verName"));
//                                    dataVersInfo.setVer(dataVersInfoObj.getString("ver"));
//                                    dataVersInfo.setPublicTime(dataVersInfoObj.getString("publicTime"));
//                                    dataVersInfo.setInfo(dataVersInfoObj.getString("info"));
//                                    unPutAwayDatas.add(dataVersInfo);
//                                }
//                                dataVersResult.setUnPutAwayDatas(unPutAwayDatas);
//                            }
//                            mOnGetDataVersListener.onGetDataVers(dataVersResult);
//                        }else {
//                            mOnGetDataVersListener.onFail(respCode,"出错了");
//                        }
//                    } catch (JSONException e) {
//                        e.printStackTrace();
//                        mOnGetDataVersListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
//                    }
//                }
//                break;
            case NDS_getUpgradeInfo:
                if (mOnGetUpgradeInfoListener != null){
                    if(hasDSPTError){
                        mOnGetUpgradeInfoListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String baseInfo = jsonObject.getString("baseInfo");
                            String deltaInfo = jsonObject.getString("deltaInfo");
                            GetUpgradeInfoResult getUpgradeInfoResult = new GetUpgradeInfoResult();
                            getUpgradeInfoResult.setBaseInfo(baseInfo);
                            getUpgradeInfoResult.setDeltaInfo(deltaInfo);
                            mOnGetUpgradeInfoListener.onGetUpgradeInfo(getUpgradeInfoResult);
                        }else {
                            mOnGetUpgradeInfoListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetUpgradeInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case NDS_getUpdateLimit:
                if (mOnGetUpdateLimitListener != null) {
                    if(hasDSPTError){
                        mOnGetUpdateLimitListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            GetUpdateLimitResult getUpdateLimitResult = new GetUpdateLimitResult();
                            getUpdateLimitResult.setProductId(jsonObject.getString("productId"));
                            getUpdateLimitResult.setCount(jsonObject.getInt("count"));
                            mOnGetUpdateLimitListener.onGetUpdateLimit(getUpdateLimitResult);
                        }else {
                            mOnGetUpdateLimitListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetUpdateLimitListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case NDS_getUpgradeList:
                if (mOnGetUpgradeListListener != null) {
                    if(hasDSPTError){
                        mOnGetUpgradeListListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<GetUpgradeListResult> getUpgradeListResults = new ArrayList<GetUpgradeListResult>();
                            JSONArray regionResultsArr = jsonObject.getJSONArray("regions");
                            GetUpgradeListResult getUpgradeListResult ;
                            for (int i=0;i<regionResultsArr.length();i++){
                                JSONObject regionResultObj = regionResultsArr.getJSONObject(i);
                                getUpgradeListResult = new GetUpgradeListResult();
                                getUpgradeListResult.setRegionId(regionResultObj.getString("regionId"));
                                getUpgradeListResult.setCurVer(regionResultObj.getString("curVer"));
                                List<UpgradeListBaseData> upGradeListBaseDatas = new ArrayList<UpgradeListBaseData>();
                                UpgradeListBaseData upgradeListBaseData ;
                                JSONArray baseDatasArr = regionResultObj.getJSONArray("base");
                                for (int j=0;j<baseDatasArr.length();j++){
                                    JSONObject baseDataObj = baseDatasArr.getJSONObject(j);
                                    upgradeListBaseData = new UpgradeListBaseData();
                                    upgradeListBaseData.setVerCode(baseDataObj.getString("verCode"));
                                    upgradeListBaseData.setPublicTime(baseDataObj.getString("publicTime"));
                                    JSONArray dataKindArr = baseDataObj.getJSONArray("dataKind");
                                    JSONArray sizeArr = baseDataObj.getJSONArray("size");
                                    UpgradeListBaseInfo dataInfo ;
                                    List<UpgradeListBaseInfo> dataInfos = new ArrayList<UpgradeListBaseInfo>();
                                    for (int k=0;k<dataKindArr.length();k++){
                                        dataInfo = new UpgradeListBaseInfo();
                                        dataInfo.setDataKind(dataKindArr.getInt(k));
                                        dataInfo.setSize(sizeArr.getInt(k));
                                        dataInfos.add(dataInfo);
                                    }
                                    upgradeListBaseData.setDataInfos(dataInfos);
                                    upGradeListBaseDatas.add(upgradeListBaseData);
                                }
                                getUpgradeListResult.setUpGradeListBaseDatas(upGradeListBaseDatas);

                                List<UpgradeListDiffData> upGradeListDiffDatas = new ArrayList<UpgradeListDiffData>();
                                JSONArray diffDatasArr = regionResultObj.getJSONArray("diff");
                                UpgradeListDiffData diffData;
                                for (int a = 0;a<diffDatasArr.length();a++){
                                    JSONObject diffDatasObj = diffDatasArr.getJSONObject(a);
                                    diffData = new UpgradeListDiffData();
                                    if (diffDatasArr!=null&&diffDatasArr.length()>0){
                                        diffData.setVerCode(diffDatasObj.getString("verCode"));
                                        diffData.setPublicTime(diffDatasObj.getString("publicTime"));
                                        JSONArray dataKindArr = diffDatasObj.getJSONArray("dataKind");
                                        JSONArray sizeArr = diffDatasObj.getJSONArray("size");
                                        JSONArray updateSizeArr = diffDatasObj.getJSONArray("updateSize");
                                        UpgradeListDiffInfo dataInfo ;
                                        List<UpgradeListDiffInfo> dataInfos = new ArrayList<UpgradeListDiffInfo>();
                                        for (int k=0;k<dataKindArr.length();k++){
                                            dataInfo = new UpgradeListDiffInfo();
                                            dataInfo.setDataKind(dataKindArr.getInt(k));
                                            dataInfo.setSize(sizeArr.getInt(k));
                                            dataInfo.setUpdateSize(updateSizeArr.getInt(k));
                                            dataInfos.add(dataInfo);
                                        }
                                        diffData.setDataInfos(dataInfos);
                                    }
                                    upGradeListDiffDatas.add(diffData);
                                }
                                getUpgradeListResult.setUpGradeListDiffDatas(upGradeListDiffDatas);
                                getUpgradeListResults.add(getUpgradeListResult);
                            }
                            mOnGetUpgradeListListener.onGetUpgradeList(getUpgradeListResults);
                        }else {
                            mOnGetUpgradeListListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetUpgradeListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
            case NDS_downloadMutiDiffData:
                if (mOnDownloadMutiDiffDataListener != null) {
                    if(hasDSPTError){
                        mOnDownloadMutiDiffDataListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray regionsArr = jsonObject.getJSONArray("regions");
                            List<DownloadDataInfo> regions = new ArrayList<DownloadDataInfo>();
                            DownloadDataInfo downloadDataInfo;
                            for (int i=0;i<regionsArr.length();i++){
                                JSONObject regionObj = regionsArr.getJSONObject(i);
                                downloadDataInfo = new DownloadDataInfo();
                                downloadDataInfo.setRegionId(regionObj.getString("regionId"));
                                JSONArray dataKindArr = regionObj.getJSONArray("dataKind");
                                List<DataKindInfo> dataKind = new ArrayList<DataKindInfo>();
                                DataKindInfo dataKindInfo ;
                                for (int j=0;j<dataKindArr.length();j++){
                                    JSONObject dataKindObj = dataKindArr.getJSONObject(j);
                                    dataKindInfo = new DataKindInfo();
                                    dataKindInfo.setDataKind(dataKindObj.getInt("id"));
                                    dataKindInfo.setUrl(dataKindObj.getString("url"));
                                    dataKind.add(dataKindInfo);
                                }
                                downloadDataInfo.setDataKind(dataKind);
                                regions.add(downloadDataInfo);
                            }
                            mOnDownloadMutiDiffDataListener.onDownloadMutiDiffData(regions);
                        }else {
                            mOnDownloadMutiDiffDataListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnDownloadMutiDiffDataListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
//            case NDS_getShortestPath:
//                if (mOnGetShortestPathListener != null) {
//                    if(hasDSPTError){
//                        mOnGetShortestPathListener.onFail(errorCode,errorMsg);
//                        return;
//                    }
//                    JSONTokener jsonParser = new JSONTokener(response);
//                    try {
//                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
//                        int respCode = jsonObject.getInt("respCode");
//                        if (respCode == Constants.RESPCODE_SUCCESS) {
//                            List<RegionShortestPathResult> regionShortestPathResults = new ArrayList<RegionShortestPathResult>();
//                            JSONArray regionResultsArr = jsonObject.getJSONArray("regions");
//                            RegionShortestPathResult regionShortestPathResult;
//                            for (int i=0;i<regionResultsArr.length();i++){
//                                JSONObject regionResultObj = regionResultsArr.getJSONObject(i);
//                                regionShortestPathResult = new RegionShortestPathResult();
//                                regionShortestPathResult.setRegionId(regionResultObj.getString("regionId"));
//                                regionShortestPathResult.setCurVer(regionResultObj.getString("curVer"));
//                                List<BaseData> baseDatas = new ArrayList<BaseData>();
//
//                                BaseData baseData ;
//                                JSONArray baseDatasArr = regionResultObj.getJSONArray("base");
//                                for (int j=0;j<baseDatasArr.length();j++){
//                                    JSONObject baseDataObj = baseDatasArr.getJSONObject(j);
//                                    baseData = new BaseData();
//                                    baseData.setCurVerCode(baseDataObj.getString("curVerCode"));
//                                    baseData.setPublicTime(baseDataObj.getString("publicTime"));
//                                    baseData.setDestVerCode(baseDataObj.getString("destVerCode"));
//                                    JSONArray dataKindArr = baseDataObj.getJSONArray("dataKind");
//                                    JSONArray sizeArr = baseDataObj.getJSONArray("size");
//                                    JSONArray updateSizeArr = baseDataObj.getJSONArray("updateSize");
//                                    JSONArray uncompressSizeArr = baseDataObj.getJSONArray("uncompressSize");
//                                    JSONArray md5Arr = baseDataObj.getJSONArray("md5");
//                                    DataInfo dataInfo ;
//                                    List<DataInfo> dataInfos = new ArrayList<DataInfo>();
//                                    for (int k=0;k<dataKindArr.length();k++){
//                                        dataInfo = new DataInfo();
//                                        dataInfo.setDataKind(dataKindArr.getInt(k));
//                                        dataInfo.setMd5(md5Arr.getString(k));
//                                        dataInfo.setSize(sizeArr.getInt(k));
//                                        dataInfo.setUncompressSize(updateSizeArr.getInt(k));
//                                        dataInfo.setUpdateSize(uncompressSizeArr.getInt(k));
//                                        dataInfos.add(dataInfo);
//                                    }
//                                    baseData.setDataInfos(dataInfos);
//                                    baseDatas.add(baseData);
//                                }
//                                regionShortestPathResult.setBaseDatas(baseDatas);
//
//                                List<DiffData> diffDatas = new ArrayList<DiffData>();
//                                JSONArray diffDatasArr = regionResultObj.getJSONArray("diff");
//                                DiffData diffData;
//                                for (int a = 0;a<diffDatasArr.length();a++){
//                                    JSONObject diffDatasObj = diffDatasArr.getJSONObject(a);
//                                    diffData = new DiffData();
//                                    if (diffDatasArr!=null&&diffDatasArr.length()>0){
//                                        diffData.setDestVerCode(diffDatasObj.getString("destVerCode"));
//                                        diffData.setCurVerCode(diffDatasObj.getString("curVerCode"));
//                                        diffData.setPublicTime(diffDatasObj.getString("publicTime"));
//                                        JSONArray dataKindArr = diffDatasObj.getJSONArray("dataKind");
//                                        JSONArray sizeArr = diffDatasObj.getJSONArray("size");
//                                        JSONArray updateSizeArr = diffDatasObj.getJSONArray("updateSize");
//                                        JSONArray uncompressSizeArr = diffDatasObj.getJSONArray("uncompressSize");
//                                        JSONArray md5Arr = diffDatasObj.getJSONArray("md5");
//                                        DataInfo dataInfo ;
//                                        List<DataInfo> dataInfos = new ArrayList<DataInfo>();
//                                        for (int k=0;k<dataKindArr.length();k++){
//                                            dataInfo = new DataInfo();
//                                            dataInfo.setDataKind(dataKindArr.getInt(k));
//                                            dataInfo.setMd5(md5Arr.getString(k));
//                                            dataInfo.setSize(sizeArr.getInt(k));
//                                            dataInfo.setUncompressSize(updateSizeArr.getInt(k));
//                                            dataInfo.setUpdateSize(uncompressSizeArr.getInt(k));
//                                            dataInfos.add(dataInfo);
//                                        }
//                                        diffData.setDataInfos(dataInfos);
//                                    }
//                                    diffDatas.add(diffData);
//                                }
//                                regionShortestPathResult.setDiffDatas(diffDatas);
//                                regionShortestPathResults.add(regionShortestPathResult);
//                            }
//                            mOnGetShortestPathListener.onGetShortestPath(regionShortestPathResults);
//                        }else {
//                            mOnGetShortestPathListener.onFail(respCode,"出错了");
//                        }
//                    } catch (JSONException e) {
//                        e.printStackTrace();
//                        mOnGetShortestPathListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
//                    }
//                }
//                break;
            case NDS_getShortestPath:
                if (mOnGetShortestPathListener != null) {
                    if(hasDSPTError){
                        mOnGetShortestPathListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        GetShortestPathResult getShortestPathResult = new GetShortestPathResult();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<RegionShortestPathResult> regionShortestPathResults = new ArrayList<RegionShortestPathResult>();
                            JSONArray regionResultsArr = jsonObject.getJSONArray("regions");
                            RegionShortestPathResult regionShortestPathResult;
                            for (int i=0;i<regionResultsArr.length();i++){
                                JSONObject regionResultObj = regionResultsArr.getJSONObject(i);
                                regionShortestPathResult = new RegionShortestPathResult();
                                regionShortestPathResult.setRegionId(regionResultObj.getString("regionId"));
//                                List<BaseData> baseDatas = new ArrayList<BaseData>();

                                BaseDataInfo baseDataInfo = null ;
                                JSONArray baseDatasArr = regionResultObj.getJSONArray("base");
                                int baseLength = baseDatasArr.length();
                                for (int j=0;j<baseLength;j++){
                                    JSONObject baseDataObj = baseDatasArr.getJSONObject(j);
                                    baseDataInfo = new BaseDataInfo();
                                    baseDataInfo.setCurVerCode(baseDataObj.getString("curVerCode"));
                                    baseDataInfo.setPublicTime(baseDataObj.getString("publicTime"));
                                    baseDataInfo.setDestVerCode(baseDataObj.getString("destVerCode"));
                                    JSONArray dataKindArr = baseDataObj.getJSONArray("dataKind");
                                    JSONArray sizeArr = baseDataObj.getJSONArray("size");
                                    JSONArray updateSizeArr = baseDataObj.getJSONArray("updateSize");
                                    JSONArray uncompressSizeArr = baseDataObj.getJSONArray("uncompressSize");
                                    JSONArray md5Arr = baseDataObj.getJSONArray("md5");
                                    DataInfo dataInfo ;
                                    List<DataInfo> dataInfos = new ArrayList<DataInfo>();
                                    for (int k=0;k<dataKindArr.length();k++){
                                        dataInfo = new DataInfo();
                                        dataInfo.setDataKind(dataKindArr.getInt(k));
                                        dataInfo.setMd5(md5Arr.getString(k));
                                        dataInfo.setSize(sizeArr.getInt(k));
                                        dataInfo.setUncompressSize(updateSizeArr.getInt(k));
                                        dataInfo.setUpdateSize(uncompressSizeArr.getInt(k));
                                        dataInfos.add(dataInfo);
                                    }
                                    baseDataInfo.setDataInfos(dataInfos);
//                                    baseDatas.add(baseData);
                                }
                                regionShortestPathResult.setBaseDataInfo(baseDataInfo);

                                List<DiffData> diffDatas = new ArrayList<DiffData>();
                                JSONArray diffDatasArr = regionResultObj.getJSONArray("diff");
                                DiffData diffData;
                                for (int a = 0;a<diffDatasArr.length();a++){
                                    JSONObject diffDatasObj = diffDatasArr.getJSONObject(a);
                                    diffData = new DiffData();
                                    if (diffDatasArr!=null&&diffDatasArr.length()>0){
                                        diffData.setDestVerCode(diffDatasObj.getString("destVerCode"));
                                        diffData.setCurVerCode(diffDatasObj.getString("curVerCode"));
                                        diffData.setPublicTime(diffDatasObj.getString("publicTime"));

                                        JSONArray dataKindArr = diffDatasObj.getJSONArray("dataKind");
                                        JSONArray sizeArr = diffDatasObj.getJSONArray("size");
                                        JSONArray updateSizeArr = diffDatasObj.getJSONArray("updateSize");
                                        JSONArray uncompressSizeArr = diffDatasObj.getJSONArray("uncompressSize");
                                        JSONArray md5Arr = diffDatasObj.getJSONArray("md5");
                                        DataInfo dataInfo ;
                                        List<DataInfo> dataInfos = new ArrayList<DataInfo>();
                                        for (int k=0;k<dataKindArr.length();k++){
                                            dataInfo = new DataInfo();
                                            dataInfo.setDataKind(dataKindArr.getInt(k));
                                            dataInfo.setMd5(md5Arr.getString(k));
                                            dataInfo.setSize(sizeArr.getInt(k));
                                            dataInfo.setUncompressSize(updateSizeArr.getInt(k));
                                            dataInfo.setUpdateSize(uncompressSizeArr.getInt(k));
                                            dataInfos.add(dataInfo);
                                        }
                                        diffData.setDataInfos(dataInfos);
                                    }
                                    diffDatas.add(diffData);
                                }
                                regionShortestPathResult.setDiffDatas(diffDatas);
                                regionShortestPathResults.add(regionShortestPathResult);
                            }
                            getShortestPathResult.setRegionShortestPathResults(regionShortestPathResults);
                            List<DataVerInfoCodeResult> dataVerInfoCodeResults = new ArrayList<DataVerInfoCodeResult>();
                            JSONArray codeArr = jsonObject.getJSONArray("code");
                            DataVerInfoCodeResult dataVerInfoCodeResult;
                            for (int m = 0;m<codeArr.length();m++){
                                JSONObject codeObj = codeArr.getJSONObject(m);
                                dataVerInfoCodeResult = new DataVerInfoCodeResult();
                                dataVerInfoCodeResult.setDestVerPublicNum(codeObj.getString("destVerPublicNum"));
                                dataVerInfoCodeResult.setDestVerCensorCode(codeObj.getString("destVerCensorCode"));
                                dataVerInfoCodeResult.setDestVer(codeObj.getString("destVer"));
                                dataVerInfoCodeResults.add(dataVerInfoCodeResult);
                            }
                            getShortestPathResult.setDataVerInfoCodeResults(dataVerInfoCodeResults);
                            mOnGetShortestPathListener.onGetShortestPath(getShortestPathResult);
                        }else {
                            mOnGetShortestPathListener.onFail(respCode,"出错了");
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetShortestPathListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case NDS_downloadMutiBaseData:
                if (mOndownloadMutiBaseDataListener != null) {
                    if(hasDSPTError){
                        mOndownloadMutiBaseDataListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            DownloadMutiBaseDataResult result = new DownloadMutiBaseDataResult();
                            JSONArray dataHeadArr = jsonObject.getJSONArray("dataHead");
                            List<String> dataHeadList = new ArrayList<String>();
                            for (int i=0;i<dataHeadArr.length();i++){
                                String dataHead = dataHeadArr.getString(i);
                                dataHeadList.add(dataHead);
                            }
                            result.setDataHead(dataHeadList);
                            JSONArray regionsArr = jsonObject.getJSONArray("regions");
                            List<DownloadDataInfo> regions = new ArrayList<DownloadDataInfo>();
                            DownloadDataInfo downloadDataInfo;
                            List<DataKindInfo> dataKindList;
                            for (int j=0;j<regionsArr.length();j++){
                                JSONObject regionsObj = regionsArr.getJSONObject(j);
                                JSONArray dataKindArr = regionsObj.getJSONArray("dataKind");
                                downloadDataInfo = new DownloadDataInfo();
                                DataKindInfo dataKindInfo;
                                dataKindList = new ArrayList<DataKindInfo>();
                                for (int k=0;k<dataKindArr.length();k++){
                                    JSONObject dataKindObj = dataKindArr.getJSONObject(k);
                                    dataKindInfo = new DataKindInfo();
                                    dataKindInfo.setDataKind(dataKindObj.getInt("id"));
                                    dataKindInfo.setUrl(dataKindObj.getString("url"));
                                    dataKindList.add(dataKindInfo);
                                }
                                downloadDataInfo.setRegionId(regionsObj.getString("regionId"));
                                downloadDataInfo.setDataKind(dataKindList);
                                regions.add(downloadDataInfo);
                            }
                            result.setRegions(regions);
                            mOndownloadMutiBaseDataListener.ondownloadMutiBaseData(result);
                        }else {
                            mOndownloadMutiBaseDataListener.onFail(respCode,"出错了");
                        }
                    }catch (JSONException e){
                        e.printStackTrace();
                        mOndownloadMutiBaseDataListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }

    }
}
