package com.mxnavi.tspv2.ota;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.ota.model.CheckUpdatesInfo;
import com.mxnavi.tspv2.ota.model.CheckUpdatesResult;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.ota.model.DisclaimerResult;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;
import static com.mxnavi.tspv2.core.MessageBuilder.instance;

/**
 * Created by Administrator on 2016/4/11.
 * desc
 */
public class OtaUpgrade extends BaseTsp {
//    String product_id ="9533";
    // 请求服务类型为OTA升级
     final int thisServiceType = Config.MxNGTPServiceType.MX_OTA.getServiceType();
    //  OTA升级的ServiceData协议版本
     final int serviceDataVer =  Config.MxNGTPServiceType.MX_OTA.getServiceVer();

     // 服务类型为OTA-TBOX及协议版本
    final int tboxServiceType = Config.MxNGTPServiceType.MX_OTA_TBOX.getServiceType();
    final int tboxServiceDataVer = Config.MxNGTPServiceType.MX_OTA_TBOX.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int CHECKUPDATAINFO = 1;//检查OTA升级包
    final int SAVE_SOFTVER = 2;//上传当前软件版本号和设备号
    final int CHECKUPDATAINFOFORAPPP = 3;//检查OTA升级包(川汽项目for APP)
    final int GET_DISCLAIMER = 4; // 导航机免责声明取得

    OnOtaUpgradeListener mOnOtaUpgradeListener;
    OnOtaSaveSoftverListener mOnOtaSaveSoftverListener;
    OnGetDisclaimerListener mOnGetDisclaimerListener;

    /**
     * 检查OTA升级包
     * @param checkUpdatesInfo 请求信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int checkUpdates(CheckUpdatesInfo checkUpdatesInfo,int timeOutSecs){
        return checkUpdates(checkUpdatesInfo,timeOutSecs, Config.DEFAULT_PRIORITY);
    }
    /**
     * 检查OTA升级包(不关注超时和优先级)
     * @param checkUpdatesInfo 请求信息
     * @return 请求ID
     */
    public int checkUpdates(CheckUpdatesInfo checkUpdatesInfo) {
        return checkUpdates(checkUpdatesInfo,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }
    /**
     * 检查OTA升级包
     *
     * @param checkUpdatesInfo 请求信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @param priorityLevel    接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int checkUpdates(CheckUpdatesInfo checkUpdatesInfo,int timeOutSecs, int priorityLevel) {
        if (checkUpdatesInfo == null || checkUpdatesInfo.getProductId() == null ) {
            if (mOnOtaUpgradeListener != null) {
                mOnOtaUpgradeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "check/checkUpdates";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", checkUpdatesInfo.getProductId());
        params.put("curr_softver",Utils.converNullToEmpty(checkUpdatesInfo.getCurrSoftver()));
        params.put("type",checkUpdatesInfo.getType() );
        if (checkUpdatesInfo.getTdeviceNo() != null){
            params.put("tdevice_no", checkUpdatesInfo.getTdeviceNo());
        }
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        listenertypeHashMap.put(eventId, CHECKUPDATAINFO);
        addCallbackMap(eventId, CHECKUPDATAINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }
    /**
     * 上传设备号和设备当前的软件版本号(不关注超时和优先级)
     * @param currentSoftVer 当前软件版本号,规则是xxx-00.00.00.00-xxx
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int saveSoftVerAndDeviceCode(String currentSoftVer,int timeOutSecs){
        return saveSoftVerAndDeviceCode(currentSoftVer,timeOutSecs, Config.DEFAULT_PRIORITY);
    }
    /**
     * 上传设备号和设备当前的软件版本号
     * @param currentSoftVer 当前软件版本号,规则是xxx-00.00.00.00-xxx
     * @return 请求ID
     */
    public int saveSoftVerAndDeviceCode(String currentSoftVer) {
        return saveSoftVerAndDeviceCode(currentSoftVer, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 上传设备号和设备当前的软件版本号
     * @param currentSoftVer 当前软件版本号,规则是xxx-00.00.00.00-xxx
     * @param timeOutSecs    接口超时时间，传0为不关注
     * @param priorityLevel  接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int saveSoftVerAndDeviceCode(String currentSoftVer, int timeOutSecs, int priorityLevel){
        String url = "upload/saveDeviceCode";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("curr_softver",currentSoftVer);
        params.put("device_no",Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        listenertypeHashMap.put(eventId, SAVE_SOFTVER);
        addCallbackMap(eventId, SAVE_SOFTVER);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 检查OTA升级包(川汽项目for APP)
     * @param checkUpdatesInfo 请求信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int checkUpdatesForApp(CheckUpdatesInfo checkUpdatesInfo,String userId,String token,int timeOutSecs){
        return checkUpdatesForApp(checkUpdatesInfo,userId,token,timeOutSecs, Config.DEFAULT_PRIORITY);
    }
    /**
     * 检查OTA升级包(川汽项目for APP)(不关注超时和优先级)
     * @param checkUpdatesInfo 请求信息
     * @return 请求ID
     */
    public int checkUpdatesForApp(CheckUpdatesInfo checkUpdatesInfo,String userId,String token) {
        return checkUpdatesForApp(checkUpdatesInfo,userId,token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }
    /**
     * 检查OTA升级包(川汽项目for APP)
     *
     * @param checkUpdatesInfo 请求信息
     * @param timeOutSecs      接口超时时间，传0为不关注
     * @param priorityLevel    接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int checkUpdatesForApp(CheckUpdatesInfo checkUpdatesInfo,String userId,String token, int timeOutSecs, int priorityLevel) {
        if (checkUpdatesInfo == null) {
            if (mOnOtaUpgradeListener != null) {
                mOnOtaUpgradeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "check/checkUpdates";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("product_id", Config.CUSTOMER_KEY);
        params.put("curr_softver",Utils.converNullToEmpty(checkUpdatesInfo.getCurrSoftver()) );
        params.put("type",checkUpdatesInfo.getType() );
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        listenertypeHashMap.put(eventId, CHECKUPDATAINFOFORAPPP);
        addCallbackMap(eventId, CHECKUPDATAINFOFORAPPP);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 导航机免责声明取得
     * @param version
     * @return
     */
    public int getDisclaimer(String version) {
        return getDisclaimer(version, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 导航机免责声明取得
     * @param version
     * @param timeOutSecs
     * @return
     */
    public int getDisclaimer(String version, int timeOutSecs) {
        return getDisclaimer(version, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 导航机免责声明取得
     * @param version
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getDisclaimer(String version, int timeOutSecs, int priorityLevel) {
        if (version == null ){
            if (mOnGetDisclaimerListener != null ){
                mOnGetDisclaimerListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "navi/disclaimer";
        LinkedHashMap<String, Object> params = new LinkedHashMap<String, Object>();
        params.put("version",version);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, tboxServiceType, tboxServiceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系
        addCallbackMap(eventId, GET_DISCLAIMER);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, OtaUpgrade.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case CHECKUPDATAINFO:
                if (mOnOtaUpgradeListener != null) {
                    mOnOtaUpgradeListener.onTimeOut();
                }
                break;
            case SAVE_SOFTVER:
                if (mOnOtaSaveSoftverListener != null) {
                    mOnOtaSaveSoftverListener.onTimeOut();
                }
                break;
            case CHECKUPDATAINFOFORAPPP:
                if (mOnOtaUpgradeListener != null) {
                    mOnOtaUpgradeListener.onTimeOut();
                }
                break;
            case GET_DISCLAIMER:
                if (mOnGetDisclaimerListener != null){
                    mOnGetDisclaimerListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }
    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case CHECKUPDATAINFOFORAPPP:
            case CHECKUPDATAINFO:
                if (mOnOtaUpgradeListener != null) {
                    if(hasDSPTError){
                        mOnOtaUpgradeListener.onFail(errorCode,errMsg);
                        return;
                    }
                    List<CheckUpdatesResult> list = new ArrayList<CheckUpdatesResult>();
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            int isEnforce = jsonObject.getInt("isEnforce");
                            if (!jsonObject.isNull("upgradeInfos")){
                                JSONArray dataarray = jsonObject.getJSONArray("upgradeInfos");
                                CheckUpdatesResult result;
                                for (int i = 0; i < dataarray.length(); i++) {
                                    JSONObject object = dataarray.getJSONObject(i);
                                    result = new CheckUpdatesResult();
                                    result.setDestVer(object.getInt("verCode"));
                                    result.setVerName(Utils.converNullToEmpty(object.getString("verName")));
                                    result.setPackageName(object.getString("packageName"));
                                    result.setType(object.getInt("type"));
                                    result.setDownloadPath(Utils.converNullToEmpty(object.getString("downloadPath")));
                                    result.setSize(object.getLong("size"));
                                    result.setPublicTime(object.getLong("publicTime"));
                                    result.setDetailInfo(Utils.converNullToEmpty(object.getString("detailInfo")));
                                    list.add(result);

                                }
                            }
                            mOnOtaUpgradeListener.onCheckUpdates(isEnforce==1?true:false,list);//等于true的时候为强制升级，false为不需要强制升级
                        } else {
                            String errorMsg = jsonObject.getString("msg");
                            mOnOtaUpgradeListener.onFail(respCode, errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnOtaUpgradeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }

                }
                break;
            case SAVE_SOFTVER:
                if(mOnOtaSaveSoftverListener!=null){
                    if(hasDSPTError){
                        mOnOtaSaveSoftverListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnOtaSaveSoftverListener.onSaveSoftVerSuccess();
                        } else {
                            String errorMsg = jsonObject.getString("msg");
                            mOnOtaSaveSoftverListener.onFail(respCode, errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnOtaSaveSoftverListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR );
                    }
                }
                break;
            case GET_DISCLAIMER:
                if (mOnGetDisclaimerListener != null ){
                    if (hasDSPTError){
                        mOnGetDisclaimerListener.onFail(errorCode, errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            DisclaimerResult disclaimerResult;
                            if (jsonObject.has("data")) {
                                disclaimerResult = new DisclaimerResult();
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                disclaimerResult.setNotice(dataObject.getString("notice"));
                                disclaimerResult.setVersion(dataObject.getString("version"));
                                mOnGetDisclaimerListener.onGetDisclaimer(requestHandleId, disclaimerResult);
                            }else {
                                mOnGetDisclaimerListener.onFail(respCode, errMsg);
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetDisclaimerListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            default:
                break;
        }
    }

    public interface OnOtaUpgradeListener extends CallBackListener {
        /**
         *
         * @param isEnforce 是否强制升级，true-强制升级，false-不强制升级
         * @param list 升级信息列表
         */
        public void onCheckUpdates(boolean isEnforce ,List<CheckUpdatesResult> list);
    }
    public interface OnOtaSaveSoftverListener extends CallBackListener {
        /**
         * 上传成功
         */
        public void onSaveSoftVerSuccess();
    }

    public interface OnGetDisclaimerListener extends CallBackListener {
        /**
         * 导航机免责声明信息
         * @param requestId
         * @param disclaimerResult
         */
        public void onGetDisclaimer(int requestId, DisclaimerResult disclaimerResult);
    }

    public void setOnOtaUpgradeListener(OnOtaUpgradeListener onOtaUpgradeListener) {
        this.mOnOtaUpgradeListener = onOtaUpgradeListener;
    }
    public void setOnOtaSaveSoftverListener(OnOtaSaveSoftverListener onOtaSaveSoftverListener) {
        this.mOnOtaSaveSoftverListener = onOtaSaveSoftverListener;
    }
    public void setOnGetDisclaimerListener(OnGetDisclaimerListener onGetDisclaimerListener) {
        this.mOnGetDisclaimerListener = onGetDisclaimerListener;
    }
}