package com.mxnavi.tspv2.parklot;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.parklot.model.Cost;
import com.mxnavi.tspv2.parklot.model.ParkingLotDetail;
import com.mxnavi.tspv2.parklot.model.ParkingLotGeometry;
import com.mxnavi.tspv2.parklot.model.ParkingLotInfo;
import com.mxnavi.tspv2.parklot.model.ParkingLotRating;
import com.mxnavi.tspv2.parklot.model.ParkingLotSearchQuery;
import com.mxnavi.tspv2.parklot.model.ParkingLotSearchResult;
import com.mxnavi.tspv2.parklot.model.Price;
import com.mxnavi.tspv2.parklot.model.RealTimeParkLotInfo;
import com.mxnavi.tspv2.parklot.model.Time;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2016/4/29.
 * 动态停车场
 */
public class ParkingLotSearch extends BaseTsp {
    // 请求服务类型为动态停车场
    final int thisServiceType = Config.MxNGTPServiceType.PARKINGLOTS.getServiceType();
    //  动态停车场的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.PARKINGLOTS.getServiceVer();


    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int SEARCHNRARBYPARKING = 1;//3.1周边停车场检索
    final int GETPARKINGINFO = 2;//3.2获得停车场详细信息

    OnSearchParkingLotListener mOnSearchParkingLotListener;
    OnGetParkingLotDetailListener mOnGetParkingLotDetailListener;

    /**
     *3.1周边停车场检索
     * @param parkingLotSearchQuery
     * @return
     */
    public int searchParkingLot(ParkingLotSearchQuery parkingLotSearchQuery){
        return searchParkingLot(parkingLotSearchQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 3.1周边停车场检索
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param parkingLotSearchQuery
     * @return
     */
    public int searchParkingLot(ParkingLotSearchQuery parkingLotSearchQuery,int timeOutSecs){
        return searchParkingLot(parkingLotSearchQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *  3.1周边停车场检索
     * @param parkingLotSearchQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return  请求ID
     */
    public int searchParkingLot(ParkingLotSearchQuery parkingLotSearchQuery, int timeOutSecs, int priorityLevel){
        if(parkingLotSearchQuery == null||parkingLotSearchQuery.getLocation()==null){
            if (mOnSearchParkingLotListener != null) {
                mOnSearchParkingLotListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "search";
        HashMap<String, Object> params = new HashMap<String, Object>();
        if(parkingLotSearchQuery.getKeyword()!=null){
            params.put("keyword", parkingLotSearchQuery.getKeyword());
        }
        params.put("lat",parkingLotSearchQuery.getLocation().getLatitude() );
        params.put("lon", parkingLotSearchQuery.getLocation().getLongitude());
        if( parkingLotSearchQuery.getRadius()>0){
            params.put("radius",  parkingLotSearchQuery.getRadius());
        }else{
            params.put("radius",  1000);
        }
        if( parkingLotSearchQuery.getSort()>0||parkingLotSearchQuery.getSort()<4){
            params.put("sort", parkingLotSearchQuery.getSort());
        }else{
            params.put("sort",  0);
        }
        if( parkingLotSearchQuery.getStartPage()>0){
            params.put("page", parkingLotSearchQuery.getStartPage());
        }else{
            params.put("page", 1);
        }
        if( parkingLotSearchQuery.getLimit()>0) {
            params.put("rows",  parkingLotSearchQuery.getLimit());
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEARCHNRARBYPARKING);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }
    /**
     *3.2获得停车场详细信息
     * @param parkingId 停车场唯一ID
     * @return
     */
    public int getParkingLotDetail(int parkingId){
        return getParkingLotDetail(parkingId,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 3.2获得停车场详细信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param parkingId 停车场唯一ID
     * @return
     */
    public int getParkingLotDetail(int parkingId,int timeOutSecs){
        return getParkingLotDetail(parkingId,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     *  3.2获得停车场详细信息
     * @param parkingId 停车场唯一ID
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return  请求ID
     */
    public int getParkingLotDetail(int parkingId, int timeOutSecs, int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "get_detail";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("id", parkingId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETPARKINGINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    public interface OnSearchParkingLotListener extends CallBackListener{
        public void onSearchParkingLot(int requestId, ParkingLotSearchResult parkingLotSearchResult);
    }
    public interface OnGetParkingLotDetailListener extends CallBackListener{
        public void onGetParkingLotDetail(int requestId, ParkingLotDetail parkingLotDetail);
    }

    public void setOnSearchParkingLotListener(OnSearchParkingLotListener onSearchParkingLotListener) {
        this.mOnSearchParkingLotListener = onSearchParkingLotListener;
    }

    public void setOnGetParkingLotDetailListener(OnGetParkingLotDetailListener onGetParkingLotDetailListener) {
        this.mOnGetParkingLotDetailListener = onGetParkingLotDetailListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, ParkingLotSearch.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    public void cancleRequest(List<Integer> requestId) {
        for (int eventId: requestId) {
            removeCallbackMap(eventId);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHNRARBYPARKING:
                if (mOnSearchParkingLotListener != null) {
                    mOnSearchParkingLotListener.onTimeOut();
                }
                break;
            case GETPARKINGINFO:
                if (mOnGetParkingLotDetailListener != null) {
                    mOnGetParkingLotDetailListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SEARCHNRARBYPARKING:
                if(mOnSearchParkingLotListener != null){
                    if(hasDSPTError){
                        mOnSearchParkingLotListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONObject resObject  = jsonObject.getJSONObject("result");
                            ParkingLotSearchResult parkingLotSearchResult = new ParkingLotSearchResult();
                            parkingLotSearchResult.setTotal(resObject.getInt("total"));
                            JSONArray spacesobj =resObject.getJSONArray("spaces");
                            List<ParkingLotInfo> parkingLots = new ArrayList<ParkingLotInfo>();
                            for(int i = 0; i<spacesobj.length(); i++){
                                JSONObject object = spacesobj.getJSONObject(i);
                                ParkingLotInfo parkingLotInfo = new ParkingLotInfo();
                                parkingLotInfo.setId(object.getInt("id"));
                                parkingLotInfo.setTitle("");
                                LatLng location = new LatLng();
                                location.setLatitude(object.getDouble("lat"));
                                location.setLongitude(object.getDouble("lng"));
                                parkingLotInfo.setLocation(location);
                                parkingLotInfo.setTypeId(object.getInt("typeid"));
                                parkingLotInfo.setParkingSpaceNum(object.getInt("num"));

                                List<String> addresses = new ArrayList<String>();
                                JSONArray addobj =object.getJSONArray("addresses");
                                for(int j = 0; j<addobj.length();j++){
                                    String a =(String)addobj.get(j);
                                    addresses.add(a);
                                }
                                parkingLotInfo.setAddresses(addresses);
                                if(object.has("phone")){
                                    parkingLotInfo.setPhone(object.getString("phone"));
                                }
                                JSONObject priceschemaobj = object.getJSONObject("priceschema");
                                List<Price> priceschema = new ArrayList<Price>();
                                JSONArray pricearray = priceschemaobj.getJSONArray("prices");
                                for(int k = 0; k<pricearray.length();k++){
                                    JSONObject pri =pricearray.getJSONObject(k);
                                    Price price = new Price();
                                    price.setVehicleClass(pri.getInt("class"));
                                    List<Cost> costs = new ArrayList<Cost>();
                                    List<Time> times = new ArrayList<Time>();
                                    JSONArray costarray = pri.getJSONArray("costs");
                                    JSONArray timearray = pri.getJSONArray("times");
                                    for(int l =0;l<costarray.length();l++){
                                        JSONObject costItem =costarray.getJSONObject(l);
                                        Cost cost = new Cost();
                                        cost.setAmount(costItem.getDouble("amount"));
                                        cost.setDurringTimeUnit(costItem.getInt("dur"));
                                        cost.setText(costItem.getString("text"));
                                        costs.add(cost);
                                    }
                                    for(int m =0; m<timearray.length(); m++){
                                        JSONObject timeItem =timearray.getJSONObject(m);
                                        Time time = new Time();
                                        time.setDay(timeItem.getInt("day"));
                                        time.setFromTime(timeItem.getString("from"));
                                        time.setToTime(timeItem.getString("to"));
                                        times.add(time);
                                    }
                                    price.setCosts(costs);
                                    price.setTimes(times);
                                    priceschema.add(price);
                                }
                                parkingLotInfo.setPriceschema(priceschema);
                                if(object.has("rt")){
                                    RealTimeParkLotInfo realTimeParkLotInfo = new RealTimeParkLotInfo();
                                    JSONObject rtobj = object.getJSONObject("rt");
                                    realTimeParkLotInfo.setTotal(rtobj.getInt("total"));
                                    realTimeParkLotInfo.setProbetime(rtobj.getString("probetime"));
                                    realTimeParkLotInfo.setIndicator(rtobj.getInt("indicator"));
                                    parkingLotInfo.setRealTimeParkLotInfo(realTimeParkLotInfo);
                                }
                                parkingLots.add(parkingLotInfo);
                                }
                            parkingLotSearchResult.setParkingLots(parkingLots);
                            mOnSearchParkingLotListener.onSearchParkingLot(requestHandleId, parkingLotSearchResult);
                        }else{
                            mOnSearchParkingLotListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchParkingLotListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETPARKINGINFO:
                if(mOnGetParkingLotDetailListener != null){
                    if(hasDSPTError){
                        mOnGetParkingLotDetailListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            ParkingLotDetail parkingLotDetail = new ParkingLotDetail();
                            JSONObject resObject  = jsonObject.getJSONObject("result");
                            JSONArray spacesarray = resObject.getJSONArray("spaces");
                            for(int b = 0; b<spacesarray.length();b++){
                                JSONObject spacesobj =spacesarray.getJSONObject(b);
                                parkingLotDetail.setId(spacesobj.getInt("id"));
                                parkingLotDetail.setTitle(spacesobj.getString("title"));
                                LatLng location = new LatLng();
                                location.setLatitude(spacesobj.getDouble("lat"));
                                location.setLongitude(spacesobj.getDouble("lng"));
                                parkingLotDetail.setLocation(location);
                                parkingLotDetail.setTypeId(spacesobj.getInt("typeid"));
                                parkingLotDetail.setParkingSpaceNum(spacesobj.getInt("num"));
                                List<String> addresses = new ArrayList<String>();
                                JSONArray addressesobj =spacesobj.getJSONArray("addresses");
                                for(int j = 0; j<addressesobj.length();j++){
                                    String a =(String)addressesobj.get(j);
                                    addresses.add(a);
                                }
                                parkingLotDetail.setAddresses(addresses);
                                if(spacesobj.has("phone")){
                                    parkingLotDetail.setPhone(spacesobj.getString("phone"));
                                }
                                if (spacesobj.has("info")) {
                                    parkingLotDetail.setInfo(spacesobj.getString("info"));
                                }
                                parkingLotDetail.setFeatures(spacesobj.getInt("features"));
                                parkingLotDetail.setCashType(spacesobj.getInt("ctype"));
                                parkingLotDetail.setPayType(spacesobj.getInt("ptype"));
                                parkingLotDetail.setSurface(spacesobj.getInt("surface"));
                                parkingLotDetail.setLimitations(spacesobj.getInt("grestr"));
                                parkingLotDetail.setMaxHeight(spacesobj.getInt("height"));
                                List<Time> openingTime = new ArrayList<Time>();
                                JSONArray timearray = spacesobj.getJSONArray("openinghours");
                                for(int n =0; n<timearray.length(); n++){
                                    JSONObject timeItem =timearray.getJSONObject(n);
                                    Time time = new Time();
                                    time.setDay(timeItem.getInt("days"));
                                    time.setFromTime(timeItem.getString("from"));
                                    time.setToTime(timeItem.getString("to"));
                                    openingTime.add(time);
                                }
                                parkingLotDetail.setOpeningTime(openingTime);
                                if (spacesobj.has("rating")) {
                                    ParkingLotRating rating = new ParkingLotRating();
                                    JSONObject ratingObject = spacesobj.getJSONObject("rating");
                                    rating.setScore(ratingObject.getDouble("score"));
                                    rating.setNumReviews(ratingObject.getInt("numreviews"));
                                    rating.setNumVotes(ratingObject.getInt("numvotes"));
                                    parkingLotDetail.setRating(rating);
                                }
                                JSONObject priceschemaobj = spacesobj.getJSONObject("priceschema");
                                List<Price> priceschema2 = new ArrayList<Price>();
                                JSONArray pricearray = priceschemaobj.getJSONArray("prices");
                                for(int k = 0; k<pricearray.length();k++){
                                    JSONObject pri =pricearray.getJSONObject(k);
                                    Price price2 = new Price();
                                    price2.setVehicleClass(pri.getInt("class"));
                                    List<Cost> costs = new ArrayList<Cost>();
                                    List<Time> times = new ArrayList<Time>();
                                    JSONArray costarray2 = pri.getJSONArray("costs");
                                    JSONArray timearray2 = pri.getJSONArray("times");
                                    for(int l =0;l<costarray2.length();l++){
                                        JSONObject costItem =costarray2.getJSONObject(l);
                                        Cost cost = new Cost();
                                        cost.setAmount(costItem.getDouble("amount"));
                                        cost.setDurringTimeUnit(costItem.getInt("dur"));
                                        cost.setText(costItem.getString("text"));
                                        costs.add(cost);
                                    }
                                    for(int m =0; m<timearray2.length(); m++){
                                        JSONObject timeItem = timearray2.getJSONObject(m);
                                        Time time = new Time();
                                        time.setDay(timeItem.getInt("day"));
                                        time.setFromTime(timeItem.getString("from"));
                                        time.setToTime(timeItem.getString("to"));
                                        times.add(time);
                                    }
                                    price2.setCosts(costs);
                                    price2.setTimes(times);
                                    priceschema2.add(price2);
                                }
                                parkingLotDetail.setPriceschema(priceschema2);
                                if(spacesobj.has("geometry")){
                                    List<ParkingLotGeometry> geometry = new ArrayList<ParkingLotGeometry>();
                                    JSONArray geometryarray =  spacesobj.getJSONArray("geometry");
                                    for(int a = 0; a <geometryarray.length();a++ ){
                                        ParkingLotGeometry parkingLotGeometry = new ParkingLotGeometry();
                                        JSONObject Lotitem =geometryarray.getJSONObject(a);
                                        parkingLotGeometry.setType(Lotitem.getInt("type"));
                                        List<LatLng> points = new ArrayList<LatLng>();
                                        String LatLng = Lotitem.getString("value");
                                        String[] LatLngarray =  LatLng.split(" ");
                                        for(int j = 0;j<LatLngarray.length;j++){
                                            String[]  LatLngitem = LatLngarray[j].split(",");
                                            LatLng latLog = new LatLng();
                                            latLog.setLongitude(Double.parseDouble(LatLngitem[1]));
                                            latLog.setLatitude(Double.parseDouble(LatLngitem[0]));
                                            points.add(latLog);
                                        }
                                        parkingLotGeometry.setPoints(points);
                                    }
                                }
                                if(spacesobj.has("rt")){
                                    RealTimeParkLotInfo realTimeParkLotDetail = new RealTimeParkLotInfo();
                                    JSONObject rtobj = spacesobj.getJSONObject("rt");
                                    realTimeParkLotDetail.setTotal(rtobj.getInt("total"));
                                    realTimeParkLotDetail.setProbetime(rtobj.getString("probetime"));
                                    realTimeParkLotDetail.setIndicator(rtobj.getInt("indicator"));
                                    parkingLotDetail.setRealTimeParkLotDetail(realTimeParkLotDetail);
                                }
                            }
                            mOnGetParkingLotDetailListener.onGetParkingLotDetail(requestHandleId, parkingLotDetail);
                        }else{
                            mOnGetParkingLotDetailListener.onFail(respCode,"");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetParkingLotDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;

        }

    }
}
