package com.mxnavi.tspv2.search;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.search.model.AddrInfo;
import com.mxnavi.tspv2.search.model.DistInfo;
import com.mxnavi.tspv2.search.model.GetLocationInfoQuery;
import com.mxnavi.tspv2.search.model.ServiceStationQuery;
import com.mxnavi.tspv2.search.model.ServiceStationResultInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/6/8.
 */
public class SearchService extends BaseTsp {
    //请求服务类型为在线检索服务(高德)
    final int thisServiceType = Config.MxNGTPServiceType.MX_SEARCH_GD.getServiceType();
    //在线检索服务(高德)的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.MX_SEARCH_GD.getServiceVer() ;
    //请求服务类型为行政区详情获取
    final int cityDetail_ServiceType = Config.MxNGTPServiceType.MX_SEARCH_GD_CITYDETAIL.getServiceType();
    //行政区详情获取的ServiceData协议版本
    final int cityDetail_serviceDataVer = Config.MxNGTPServiceType.MX_SEARCH_GD_CITYDETAIL.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int SearchServiceStation = 1;//升级网点查询
    final int GetLocationInfo = 2;//获取行政区信息
    final int CityDetail =3;//行政区详情获取

    /**
     * 升级网点查询
     * @param serviceStationQuery
     * @return
     */
    public int searchServiceStation(ServiceStationQuery serviceStationQuery){
        return searchServiceStation(serviceStationQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     *升级网点查询
     * @param serviceStationQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int searchServiceStation(ServiceStationQuery serviceStationQuery,int timeOutSecs){
        return searchServiceStation(serviceStationQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *升级网点查询
     * @param serviceStationQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int searchServiceStation(ServiceStationQuery serviceStationQuery,int timeOutSecs, int priorityLevel){
        if( serviceStationQuery== null||serviceStationQuery.getLatLng()==null){
            if (mOnServiceStationListener != null) {
                mOnServiceStationListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "poi/SearchServiceStation";
        JSONObject param = new JSONObject();
        try {
            param.put("product_id",Config.CUSTOMER_KEY);
            param.put("dist_lon",Utils.converDoubleCoordinate2MX(serviceStationQuery.getLatLng().getLongitude()));
            param.put("dist_lat",Utils.converDoubleCoordinate2MX(serviceStationQuery.getLatLng().getLatitude()));
            if (serviceStationQuery.getDistLevel()!=null){
                param.put("dist_level",serviceStationQuery.getDistLevel());
            }
            param.put("offset",serviceStationQuery.getOffset());
            param.put("limit",serviceStationQuery.getLimit());
        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param);
        params.put("return_type",0);
        params.put("compress",0);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SearchServiceStation);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 获取行政区信息
     * @param getLocationInfoQuery
     * @return
     */
    public int getLocationInfo (GetLocationInfoQuery getLocationInfoQuery){
        return getLocationInfo(getLocationInfoQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     *获取行政区信息
     * @param getLocationInfoQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getLocationInfo (GetLocationInfoQuery getLocationInfoQuery,int timeOutSecs){
        return getLocationInfo(getLocationInfoQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *获取行政区信息
     * @param getLocationInfoQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getLocationInfo (GetLocationInfoQuery getLocationInfoQuery,int timeOutSecs, int priorityLevel){
        if( getLocationInfoQuery== null||(getLocationInfoQuery.getLatLng()==null&&getLocationInfoQuery.getAdminCode()==null)){
            if (mOnGetLocationInfoListener != null) {
                mOnGetLocationInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "poi/GetLocationInfo";
        JSONObject param = new JSONObject();
        try {
            param.put("product_id",Config.CUSTOMER_KEY);
            if (getLocationInfoQuery.getLatLng()!=null){
                param.put("pos_lon",Utils.converDoubleCoordinate2MX(getLocationInfoQuery.getLatLng().getLongitude()));
                param.put("pos_lat",Utils.converDoubleCoordinate2MX(getLocationInfoQuery.getLatLng().getLatitude()));
            }
            if (getLocationInfoQuery.isCalDistance()==true){
                param.put("cal_distance","yes");
            }else {
                param.put("cal_distance","no");
            }
            if (getLocationInfoQuery.getAdminCode()!=null){
                param.put("admin_code",getLocationInfoQuery.getAdminCode());
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param);
        params.put("return_type",0);
        params.put("compress",0);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, GetLocationInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 行政区详情获取
     * @param distName 行政区名
     * @param latLng 当前位置经纬度
     * @param onCityDetailListener 回调监听
     * @return 请求ID，0:参数错误
     */
    public int cityDetail(String distName,LatLng latLng,OnCityDetailListener onCityDetailListener){
        return cityDetail(distName,latLng,onCityDetailListener,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 行政区详情获取
     * @param distName 行政区名
     * @param latLng 当前位置经纬度
     * @param onCityDetailListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int cityDetail(String distName,LatLng latLng,OnCityDetailListener onCityDetailListener,int timeOutSecs){
        return cityDetail(distName,latLng,onCityDetailListener,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 行政区详情获取
     * @param distName 行政区名
     * @param latLng 当前位置经纬度
     * @param onCityDetailListener 回调监听
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int cityDetail(String distName,LatLng latLng,OnCityDetailListener onCityDetailListener,int timeOutSecs, int priorityLevel){
        if (distName==null||latLng==null){
            if (onCityDetailListener!=null){
                onCityDetailListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        this.mOnCityDetailListener = onCityDetailListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "citydetail";
        JSONObject param = new JSONObject();
        try {
            param.put("name",distName);
            param.put("pos_lat",Utils.converDoubleCoordinate2MX(latLng.getLatitude()));
            param.put("pos_lon",Utils.converDoubleCoordinate2MX(latLng.getLongitude()));
        } catch (JSONException e) {
            e.printStackTrace();
        }
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("param",param);
        String requestData = Utils.makeUrl(url, params);
        Utils.loge("requestData = "+requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, cityDetail_ServiceType, cityDetail_serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, CityDetail);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    public interface OnServiceStationListener extends CallBackListener{
        public void onServiceStationListener(int requestHandleId, List<ServiceStationResultInfo> serviceStationResultInfos);
    }

    public interface OnGetLocationInfoListener extends CallBackListener{
        public void onGetLocationInfoListener(int requestHandleId, AddrInfo addrInfo);
    }

    public interface OnCityDetailListener extends CallBackListener{
        public void onCityDetail(int requestHandleId, DistInfo distInfo);
    }

    OnServiceStationListener mOnServiceStationListener;
    OnGetLocationInfoListener mOnGetLocationInfoListener;
    OnCityDetailListener mOnCityDetailListener;

    public void setOnServiceStationListener(OnServiceStationListener onServiceStationListener){
        this.mOnServiceStationListener = onServiceStationListener;
    }
    public void setOnGetLocationInfoListener(OnGetLocationInfoListener onGetLocationInfoListener){
        this.mOnGetLocationInfoListener = onGetLocationInfoListener;
    }

    protected void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, SearchService.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }


    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }


    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SearchServiceStation:
                if (mOnServiceStationListener != null) {
                    mOnServiceStationListener.onTimeOut();
                }
                break;
            case GetLocationInfo:
                if (mOnGetLocationInfoListener != null) {
                    mOnGetLocationInfoListener.onTimeOut();
                }
                break;
            case CityDetail:
                if (mOnCityDetailListener!=null){
                    mOnGetLocationInfoListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge(response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SearchServiceStation:
                if (mOnServiceStationListener != null) {
                    if(hasDSPTError){
                        mOnServiceStationListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    try {
                        List<ServiceStationResultInfo> list = new ArrayList<ServiceStationResultInfo>();
                        ServiceStationResultInfo serviceStationResultInfo ;
                        JSONTokener jsonParser = new JSONTokener(response);
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            if(jsonObject.has("data")){
                                JSONArray  dataarray = jsonObject.getJSONArray("data");
                                for (int j = 0; j < dataarray.length(); j++) {
                                    JSONObject dataObj = dataarray.getJSONObject(j);
                                    serviceStationResultInfo= new ServiceStationResultInfo();
                                    if(dataObj.has("addr")){
                                        String addr = dataObj.getString("addr");
                                        serviceStationResultInfo.setAddr(addr);
                                    }
                                    if(dataObj.has("adminCode")){
                                        serviceStationResultInfo.setAdminCode(dataObj.getString("adminCode"));
                                    }
                                    if(dataObj.has("city")){
                                        String city = dataObj.getString("city");
                                        serviceStationResultInfo.setCityName(city);
                                    }
                                    if(dataObj.has("desc")){
                                        String desc = dataObj.getString("desc");
                                        serviceStationResultInfo.setDesc(desc);
                                    }
                                    if(dataObj.has("dist")){
                                        String dist = dataObj.getString("dist");
                                        serviceStationResultInfo.setDistName(dist);
                                    }
                                    if(dataObj.has("id")){
                                        int id = dataObj.getInt("id");
                                        serviceStationResultInfo.setId(id);
                                    }
                                    if(dataObj.has("img")){
                                        String img = dataObj.getString("img");
                                        serviceStationResultInfo.setImg(img);
                                    }
                                    if(dataObj.has("mCode")){
                                        String mCode = dataObj.getString("mCode");
                                        serviceStationResultInfo.setmCode(mCode);
                                    }
                                    if(dataObj.has("name")){
                                        String name = dataObj.getString("name");
                                        serviceStationResultInfo.setName(name);
                                    }
                                    if(dataObj.has("prov")){
                                        String prov = dataObj.getString("prov");
                                        serviceStationResultInfo.setProvName(prov);
                                    }
                                    if(dataObj.has("tel")){
                                        String tel = dataObj.getString("tel");
                                        serviceStationResultInfo.setTel(tel);
                                    }
                                    list.add(serviceStationResultInfo);
                                }
                            }
                            mOnServiceStationListener.onServiceStationListener(requestHandleId, list);
                        }else{
                            mOnServiceStationListener.onFail(respCode,"出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnServiceStationListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GetLocationInfo:
                if (mOnGetLocationInfoListener != null) {
                    if(hasDSPTError){
                        mOnGetLocationInfoListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    try {
                        JSONTokener jsonParser = new JSONTokener(response);
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            JSONObject addrInfoObject = jsonObject.getJSONObject("addrInfo");
                            AddrInfo addrInfo = new AddrInfo();
                            if(addrInfoObject.has("adminCode")){
                                addrInfo.setAdminCode(addrInfoObject.getString("adminCode"));
                            }
                            if(addrInfoObject.has("minDistance")){
                                if (!"".equals(addrInfoObject.getString("minDistance"))){
                                    addrInfo.setMinDistance(Double.parseDouble(addrInfoObject.getString("minDistance")));
                                }
                            }
                            if(addrInfoObject.has("province")){
                                addrInfo.setProvince(addrInfoObject.getString("province"));
                            }
                            if(addrInfoObject.has("provCode")){
                                addrInfo.setProvCode(addrInfoObject.getString("provCode"));
                            }
                            if(addrInfoObject.has("city")){
                                addrInfo.setCity(addrInfoObject.getString("city"));
                            }
                            if(addrInfoObject.has("cityCode")){
                                addrInfo.setCityCode(addrInfoObject.getString("cityCode"));
                            }
                            if(addrInfoObject.has("dist")){
                                addrInfo.setDist(addrInfoObject.getString("dist"));
                            }
                            if(addrInfoObject.has("dist_lat")){
                                String lat = addrInfoObject.getString("dist_lat");
                                String lon = addrInfoObject.getString("dist_lon");
                                LatLng location = new LatLng();
                                if(!"".equals(lat)){
                                    location.setLatitude(Utils.converMX2DoubleCoordinate(Integer.parseInt(lat)));
                                }
                                if (!"".equals(lon)){
                                    location.setLongitude(Utils.converMX2DoubleCoordinate(Integer.parseInt(lon)));
                                }
                                addrInfo.setLocation(location);
                            }
                            mOnGetLocationInfoListener.onGetLocationInfoListener(requestHandleId, addrInfo);
                        }else{
                            mOnGetLocationInfoListener.onFail(respCode,"出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnGetLocationInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case CityDetail:
                if (mOnCityDetailListener!=null){
                    if(hasDSPTError){
                        mOnCityDetailListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode ==  Constants.RESPCODE_SUCCESS) {
                            if (jsonObject.has("distInfo")){
                                JSONObject distInfoObj = jsonObject.getJSONObject("distInfo");
                                DistInfo distInfo = new DistInfo();
                                if (distInfoObj.has("adminCode")){
                                    distInfo.setAdminCode(distInfoObj.getString("adminCode"));
                                }
                                if (distInfoObj.has("provinceName")){
                                    distInfo.setProvinceName(distInfoObj.getString("provinceName"));
                                }
                                if (distInfoObj.has("cityName")){
                                    distInfo.setCityName(distInfoObj.getString("cityName"));
                                }
                                if (distInfoObj.has("districtName")){
                                    distInfo.setDistrictName(distInfoObj.getString("districtName"));
                                }
                                if (distInfoObj.has("province_name_en")){
                                    distInfo.setProvinceNameEn(distInfoObj.getString("province_name_en"));
                                }
                                if (distInfoObj.has("city_name_en")){
                                    distInfo.setCityNameEn(distInfoObj.getString("city_name_en"));
                                }
                                if (distInfoObj.has("dist_name_en")){
                                    distInfo.setDistNameEn(distInfoObj.getString("dist_name_en"));
                                }
                                if (distInfoObj.has("province_all_letters")){
                                    distInfo.setProvinceAllLetters(distInfoObj.getString("province_all_letters"));
                                }
                                if (distInfoObj.has("city_all_letters")){
                                    distInfo.setCityAllLetters(distInfoObj.getString("city_all_letters"));
                                }
                                if (distInfoObj.has("district_all_letters")){
                                    distInfo.setDistrictAllLetters(distInfoObj.getString("district_all_letters"));
                                }
                                if (distInfoObj.has("province_first_letters")){
                                    distInfo.setProvinceFirstLetters(distInfoObj.getString("province_first_letters"));
                                }
                                if (distInfoObj.has("city_first_letters")){
                                    distInfo.setCityFirstLetters(distInfoObj.getString("city_first_letters"));
                                }
                                if (distInfoObj.has("district_first_letters")){
                                    distInfo.setDistrictFirstLetters(distInfoObj.getString("district_first_letters"));
                                }
                                if (distInfoObj.has("provCode")){
                                    distInfo.setProvCode(distInfoObj.getString("provCode"));
                                }
                                if (distInfoObj.has("cityCode")){
                                    distInfo.setCityCode(distInfoObj.getString("cityCode"));
                                }

                                if (distInfoObj.has("dist_lon")&&distInfoObj.has("dist_lat")){
                                    double distLat = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("dist_lat")));
                                    double distLon = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("dist_lon")));
                                    distInfo.setDistLatLng(new LatLng(distLat,distLon));
                                }

                                if (distInfoObj.has("left_bottom_lon")&&distInfoObj.has("left_bottom_lat")){
                                    double leftBottomLat = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("left_bottom_lat")));
                                    double leftBottomLon = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("left_bottom_lon")));
                                    distInfo.setLeftBottomLatLng(new LatLng(leftBottomLat ,leftBottomLon));
                                }

                                if (distInfoObj.has("right_top_lon")&&distInfoObj.has("right_top_lat")){
                                    double rightTopLat = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("right_top_lat")));
                                    double rightTopLon = Utils.converMX2DoubleCoordinate(Integer.parseInt(distInfoObj.getString("right_top_lon")));
                                    distInfo.setRightTopLatLng(new LatLng(rightTopLat,rightTopLon));
                                }
                                mOnCityDetailListener.onCityDetail(requestHandleId, distInfo);
                            }

                        }else {
                            if (jsonObject.has("msg")){
                                mOnCityDetailListener.onFail(respCode,jsonObject.getString("msg"));
                            }else {
                                mOnCityDetailListener.onFail(respCode,"查询失败");
                            }

                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnCityDetailListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            default:
                break;
        }
    }
}
