package com.mxnavi.tspv2.sendtocar;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.core.Notification;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.notification.model.MessageInfo;
import com.mxnavi.tspv2.notification.model.WeChatUserInfo;
import com.mxnavi.tspv2.sendtocar.model.ConfirmReceivedQuery;
import com.mxnavi.tspv2.sendtocar.model.ContentInfo;
import com.mxnavi.tspv2.sendtocar.model.ContentListInfo;
import com.mxnavi.tspv2.sendtocar.model.HistoryQuery;
import com.mxnavi.tspv2.sendtocar.model.HistoryResult;
import com.mxnavi.tspv2.sendtocar.model.MxBindOrUnbindInfo;
import com.mxnavi.tspv2.sendtocar.model.PushMessageInfo;
import com.mxnavi.tspv2.sendtocar.model.SearchDeviceListQuery;
import com.mxnavi.tspv2.sendtocar.model.SearchDeviceListResult;
import com.mxnavi.tspv2.sendtocar.model.SearchUserListResult;
import com.mxnavi.tspv2.sendtocar.model.SendMsgToCarInfo;
import com.mxnavi.tspv2.sendtocar.model.SendToVehicleQuery;
import com.mxnavi.tspv2.sendtocar.model.SendToVehicleResult;
import com.mxnavi.tspv2.sendtocar.model.WeChatDeviceInfo;
import com.mxnavi.tspv2.sendtocar.model.WeChatUserInfoResult;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2016/4/15.
 * 微信发送到车模块
 */
public class SendToCar extends BaseTsp {
    private  String vin = Config.VIN;
    private  String product_id = String.valueOf(Config.CUSTOMER_KEY);
    // 请求服务类型为发送到车
    final int thisServiceType = Config.MxNGTPServiceType.SEND_TO_CAR.getServiceType();
    //  发送到车的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.SEND_TO_CAR.getServiceVer();

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    final int STCGETWECHATUSERINFO = 1;//获取公众号用户信息 (公众号/C&I)
    final int STCGRTMSEEAGE = 2;//获取发送到车机的数据(腾讯、公众号)
    final int USERCONFIRMRECEIVEDMSG= 3;//已收到的数据的确认(腾讯、公众号)
    final int SEARCHDEVICELIST = 4;//	查询导航（公众号/C&I）
    final int MXBINDORUNBIND = 5;//绑定/解绑导航
    final int SEARCHUSERLIST = 6;//批量取得用户信息（公众号/C&I）
    final int PUSHMESSAGE = 7;//发送数据（公众号/C&I）
    final int SENDMSGTOCAR = 8;//发送数据(app)
    final int USER_CONFIRM_RECEIVED_DATA = 9; // 已收到的数据的确认(车机端)
    final int GET_DATA_SEND_TO_VEHICLE = 10; // 获取发送到车机的数据(车机端)
    final int QUERY_VEHICLE_HISTORY_MSG  = 11; // 查询车机历史信息 (车机端/app)

    /**
     * 获取公众号用户信息
     * @param weChatUserInfoQuery
     * @return
     */
    public int getWeChatUserInfo(WeChatDeviceInfo weChatUserInfoQuery){
        return getWeChatUserInfo(weChatUserInfoQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取公众号用户信息
     * @param weChatUserInfoQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getWeChatUserInfo(WeChatDeviceInfo weChatUserInfoQuery,int timeOutSecs){
        return getWeChatUserInfo( weChatUserInfoQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取公众号用户信息
     * @param weChatUserInfoQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getWeChatUserInfo(WeChatDeviceInfo weChatUserInfoQuery,int timeOutSecs,int priorityLevel){
        if( weChatUserInfoQuery== null||weChatUserInfoQuery.getWeChatOpenId()==null){
            if (mOnSendToCarListener != null) {
                mOnSendToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/getMxUserInfoRealTime";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", vin);
        params.put("product_id", product_id);
        params.put("open_id", weChatUserInfoQuery.getWeChatOpenId());
        String requestData = Utils.makeUrl(url, params);
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, STCGETWECHATUSERINFO);
            RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 确认已收到信息
     * @param weChatCarId 腾讯定义的车辆ID
     * @param messageInfoList
     * @return
     */
    public int confirmReceivedMsg(String weChatCarId,List<MessageInfo> messageInfoList){

        return confirmReceivedMsg( weChatCarId,messageInfoList,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 确认已收到信息
     * @param weChatCarId 腾讯定义的车辆ID
     * @param messageInfoList
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int confirmReceivedMsg(String weChatCarId,List<MessageInfo> messageInfoList,int timeOutSecs){

        return confirmReceivedMsg( weChatCarId, messageInfoList,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 确认已收到信息
     * @param weChatCarId 腾讯定义的车辆ID
     * @param messageInfoList
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int confirmReceivedMsg(String weChatCarId,List<MessageInfo> messageInfoList,int timeOutSecs,int priorityLevel){
        if( weChatCarId== null||messageInfoList== null){
            if (mOnSendToCarListener != null) {
                mOnSendToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/ConfirmInfoToVehicle";
        JSONObject params = new JSONObject();
        try{
            params.put("vin", vin);
            params.put("wecar_id",  weChatCarId);
            params.put("product_id", product_id);
            JSONObject object;
            JSONArray array = new JSONArray();
            int size = messageInfoList.size();
            for (int i = 0; i < size; i++) {
                object = new JSONObject();
                MessageInfo info = messageInfoList.get(i);
                object.put("weixin_openid",info.getWeChatOpenId());
                object.put("msg_type",info.getMsgType());
                object.put("msg_subtype",info.getMsgSubtype());
                object.put("msg_id",info.getMsgId());
                array.put(object);
            }
            params.put("content_list",array);
            String requestData = url + "?param="+params.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, USERCONFIRMRECEIVEDMSG);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
        }catch (Exception e) {
            e.printStackTrace();
        }
        return eventId;
    }
    /**
     * 获取发送到车机的数据(腾讯、公众号)
     * @param weChatDeviceInfo
     * @return
     */
    public int getMessages(WeChatDeviceInfo weChatDeviceInfo){
        return getMessages(weChatDeviceInfo,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取发送到车机的数据(腾讯、公众号)
     * @param weChatDeviceInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int getMessages(WeChatDeviceInfo weChatDeviceInfo,int timeOutSecs){
        return getMessages( weChatDeviceInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }
    /**
     * 获取发送到车机的数据(腾讯、公众号)
     * @param weChatDeviceInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int getMessages(WeChatDeviceInfo weChatDeviceInfo,int timeOutSecs,int priorityLevel){
        if( weChatDeviceInfo== null||weChatDeviceInfo.getWeChatCarId()== null||weChatDeviceInfo.getVin()==null||weChatDeviceInfo.getProductId()==null){
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/NotifyInfoToVehicle";
        JSONObject params = new JSONObject();
        try{
            params.put("vin", weChatDeviceInfo.getVin());
            params.put("product_id", weChatDeviceInfo.getProductId());
            params.put("wecar_id",  weChatDeviceInfo.getWeChatCarId());
            params.put("weixin_openid",  weChatDeviceInfo.getWeChatOpenId());
            String requestData = url + "?param="+params.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, STCGRTMSEEAGE);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
            RequestDispathUtil.submitRequest(requestMessage);

        }catch (Exception e) {
            e.printStackTrace();
        }
        return eventId;
    }

    /**
     *查询导航（公众号/C&I）
     * @param searchDeviceListQuery
     * @return
     */
    public int searchDeviceList(SearchDeviceListQuery searchDeviceListQuery){
        return searchDeviceList(searchDeviceListQuery,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     *查询导航（公众号/C&I）
     * @param searchDeviceListQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int searchDeviceList(SearchDeviceListQuery searchDeviceListQuery,int timeOutSecs){
        return searchDeviceList(searchDeviceListQuery,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *查询导航（公众号/C&I）
     * @param searchDeviceListQuery
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int searchDeviceList(SearchDeviceListQuery searchDeviceListQuery,int timeOutSecs,int priorityLevel){
        if (searchDeviceListQuery==null||searchDeviceListQuery.getAppId()==null||(searchDeviceListQuery.getOpenId()==null&&searchDeviceListQuery.getUserId()==null)){
            if (mOnSearchDeviceListListener!=null){
                mOnSearchDeviceListListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/searchDeviceList";
        JSONObject param = new JSONObject();
        try {
            param.put("appId",searchDeviceListQuery.getAppId());
            if (searchDeviceListQuery.getOpenId()!=null){
                param.put("openId",searchDeviceListQuery.getOpenId());
            }
            if (searchDeviceListQuery.getUserId()!=null){
                param.put("userId",searchDeviceListQuery.getUserId());
            }
            String requestData = url + "?param="+param.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, SEARCHDEVICELIST);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return eventId;
    }

    /**
     *绑定/解绑导航（公众号/C&I/车机端）
     * @param mxBindOrUnbindInfo
     * @return
     */
    public int mxBindOrUnbind (MxBindOrUnbindInfo mxBindOrUnbindInfo){
        return mxBindOrUnbind(mxBindOrUnbindInfo,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     *绑定/解绑导航（公众号/C&I/车机端）
     * @param mxBindOrUnbindInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int mxBindOrUnbind (MxBindOrUnbindInfo mxBindOrUnbindInfo,int timeOutSecs){
        return mxBindOrUnbind(mxBindOrUnbindInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *绑定/解绑导航（公众号/C&I/车机端）
     * @param mxBindOrUnbindInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int mxBindOrUnbind (MxBindOrUnbindInfo mxBindOrUnbindInfo,int timeOutSecs,int priorityLevel){
        if (mxBindOrUnbindInfo ==null|| mxBindOrUnbindInfo.getVin()==null|| mxBindOrUnbindInfo.getProductId()==null|| mxBindOrUnbindInfo.getUserId()==null|| mxBindOrUnbindInfo.getAppId()==null|| mxBindOrUnbindInfo.getMsgSubtype()==null){
            if (mOnMxBindOrUnbindListener!=null){
                mOnMxBindOrUnbindListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/mxBindOrUnbind";
        JSONObject param = new JSONObject();
        try {
            param.put("vin", mxBindOrUnbindInfo.getVin());
            param.put("productId", mxBindOrUnbindInfo.getProductId());
            if (mxBindOrUnbindInfo.getDeviceName()!=null){
                param.put("deviceName", mxBindOrUnbindInfo.getDeviceName());
            }
            if (mxBindOrUnbindInfo.getOpenId()!=null){
                param.put("openId", mxBindOrUnbindInfo.getOpenId());
            }
            param.put("userId", mxBindOrUnbindInfo.getUserId());
            param.put("appId", mxBindOrUnbindInfo.getAppId());
            param.put("msgSubtype", mxBindOrUnbindInfo.getMsgSubtype());
            param.put("msgType","wecar_event");
            param.put("createTime", String.valueOf(System.currentTimeMillis()));
            String requestData = url + "?param="+param.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, MXBINDORUNBIND);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return eventId;
    }

    /**
     * 批量取得用户信息（公众号/C&I）
     * @return
     */
    public int searchUserList(){
        return searchUserList(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 批量取得用户信息（公众号/C&I）
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int searchUserList(int timeOutSecs){
        return searchUserList(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 批量取得用户信息（公众号/C&I）
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int searchUserList(int timeOutSecs,int priorityLevel){
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/searchUserList";
        String requestData = url + "?vin="+vin+"&productId="+product_id;
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEARCHUSERLIST);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 发送数据（公众号/C&I）
     * @param pushMessageInfo
     * @return
     */
    public int pushMessage(PushMessageInfo pushMessageInfo){
        return pushMessage(pushMessageInfo,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送数据（公众号/C&I）
     * @param pushMessageInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int pushMessage(PushMessageInfo pushMessageInfo,int timeOutSecs){
        return pushMessage(pushMessageInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送数据（公众号/C&I）
     * @param pushMessageInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int pushMessage(PushMessageInfo pushMessageInfo,int timeOutSecs,int priorityLevel){
        if (pushMessageInfo ==null|| pushMessageInfo.getVin()==null|| pushMessageInfo.getProductId()==null|| pushMessageInfo.getUserId()==null|| pushMessageInfo.getWeixinOpenId()==null||
                pushMessageInfo.getWecarId()==null|| pushMessageInfo.getContent()==null){
            if (mOnPushMessageListener!=null){
                mOnPushMessageListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "mx_wechat/sendtocar/PushMessage";
        JSONObject param = new JSONObject();
        JSONObject contentParam = new JSONObject();
        try {
            contentParam.put("name", pushMessageInfo.getContent().getName());
            contentParam.put("address", pushMessageInfo.getContent().getAddress());
            contentParam.put("lat", pushMessageInfo.getContent().getLatLng().getLatitude());
            contentParam.put("lng", pushMessageInfo.getContent().getLatLng().getLongitude());
            contentParam.put("city", pushMessageInfo.getContent().getCityName());
            contentParam.put("subscribe_time", pushMessageInfo.getContent().getSubscribeTime());
            contentParam.put("openid", pushMessageInfo.getContent().getOpenId());
            contentParam.put("nickname", pushMessageInfo.getContent().getNickName());
            contentParam.put("sex", pushMessageInfo.getContent().getSex());
            contentParam.put("language", pushMessageInfo.getContent().getLanguage());
            contentParam.put("province", pushMessageInfo.getContent().getProvName());
            contentParam.put("country", pushMessageInfo.getContent().getCountryName());
            contentParam.put("headimgurl", pushMessageInfo.getContent().getHeadimgurl());
            contentParam.put("groupid", pushMessageInfo.getContent().getGroupId());

            param.put("vin", pushMessageInfo.getVin());
            param.put("product_id", pushMessageInfo.getProductId());
            param.put("source","mx_svracct");
            param.put("user_id", pushMessageInfo.getUserId());
            param.put("weixin_openid", pushMessageInfo.getWeixinOpenId());
            param.put("msg_type","wecar_app");
            param.put("msg_subtype","poi");
            param.put("wecar_id", pushMessageInfo.getWecarId());
            param.put("create_time",String.valueOf(System.currentTimeMillis()));
            param.put("msg_id", pushMessageInfo.getMsgId());
            param.put("content", contentParam);
            String requestData = url + "?param="+param.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, PUSHMESSAGE);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return eventId;
    }

    /**
     * 发送数据(app)
     * @param sendMsgToCarInfo
     * @param token
     * @return
     */
    public int sendMsgToCar(SendMsgToCarInfo sendMsgToCarInfo, String token){
        return sendMsgToCar(sendMsgToCarInfo,token,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送数据(app)
     * @param sendMsgToCarInfo
     * @param token
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int sendMsgToCar(SendMsgToCarInfo sendMsgToCarInfo, String token, int timeOutSecs){
        return sendMsgToCar(sendMsgToCarInfo,token,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 发送数据(app)
     * @param sendMsgToCarInfo
     * @param token
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int sendMsgToCar(SendMsgToCarInfo sendMsgToCarInfo, String token, int timeOutSecs,int priorityLevel){
        if (sendMsgToCarInfo == null
                || sendMsgToCarInfo.getVin() == null
                || sendMsgToCarInfo.getProductId() == null
                || sendMsgToCarInfo.getSource() == null
                || sendMsgToCarInfo.getUserId() == null
                || sendMsgToCarInfo.getAppId() == null
                || sendMsgToCarInfo.getMsgType() == null
                || sendMsgToCarInfo.getMsgId() == null
                || Utils.strIsEmpty(token)){
            if (mOnSendMsgToCarListener!=null){
                mOnSendMsgToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/pushMessage";
        JSONObject param = new JSONObject();
        JSONObject contentParam = new JSONObject();
        try {
        	if (sendMsgToCarInfo.getMsgType().equals("poi")) {
        		if (sendMsgToCarInfo.getPoiContent() == null) {
        			mOnSendMsgToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                }
                contentParam.put("name", sendMsgToCarInfo.getPoiContent().getName());
                contentParam.put("address", sendMsgToCarInfo.getPoiContent().getAddress());
                if (sendMsgToCarInfo.getPoiContent().getLatLng() != null) {
                	contentParam.put("lat", sendMsgToCarInfo.getPoiContent().getLatLng().getLatitude());
                    contentParam.put("lng", sendMsgToCarInfo.getPoiContent().getLatLng().getLongitude());
                }
                if (sendMsgToCarInfo.getPoiContent().getNickName() != null){
                    contentParam.put("nickName", sendMsgToCarInfo.getPoiContent().getNickName());
                }
                if (sendMsgToCarInfo.getPoiContent().getHeadImgUrl() != null){
                    contentParam.put("headImgUrl", sendMsgToCarInfo.getPoiContent().getHeadImgUrl());
                }
                if (sendMsgToCarInfo.getPoiContent().getMsg() != null) {
                    contentParam.put("msg", sendMsgToCarInfo.getPoiContent().getMsg());
                }
        	}
        	if (sendMsgToCarInfo.getMsgType().equals("demo")) {
        		if (sendMsgToCarInfo.getMsgContent() == null) {
        			mOnSendMsgToCarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                }
                contentParam.put("type", sendMsgToCarInfo.getMsgContent().getType());
                contentParam.put("msg", sendMsgToCarInfo.getMsgContent().getMsg());
        	}
        	
            param.put("vin", sendMsgToCarInfo.getVin());
            param.put("productId", sendMsgToCarInfo.getProductId());
            param.put("source",sendMsgToCarInfo.getSource());
            param.put("userId", sendMsgToCarInfo.getUserId());
            param.put("appId", sendMsgToCarInfo.getAppId());
            param.put("msgType",sendMsgToCarInfo.getMsgType());
            param.put("msgId",sendMsgToCarInfo.getMsgId());
            param.put("content",contentParam);
            String requestData = url + "?"+param.toString();
            MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                    sendMsgToCarInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                    MessageBuilder.ENCODINGTYPE_JSON_UTF8, NON_SUB_SERVICE_TYPE);
            PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
            // 添加对应关系
            addCallbackMap(eventId, SENDMSGTOCAR);
            RequestDispathUtil.submitRequest(requestMessage);
            requestMessage.timeOutSeconds = timeOutSecs;
            requestMessage.priority = priorityLevel;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return eventId;
    }

    /**
     * 已收到的数据的确认(车机端)
     * @param confirmReceivedQuery
     * @return
     */
    public int confirmReceivedData(ConfirmReceivedQuery confirmReceivedQuery) {
        return confirmReceivedData(confirmReceivedQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 已收到的数据的确认(车机端)
     * @param confirmReceivedQuery
     * @param timeOutSecs
     * @return
     */
    public int confirmReceivedData(ConfirmReceivedQuery confirmReceivedQuery, int timeOutSecs) {
        return confirmReceivedData(confirmReceivedQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 已收到的数据的确认(车机端)
     * @param confirmReceivedQuery
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int confirmReceivedData(ConfirmReceivedQuery confirmReceivedQuery, int timeOutSecs, int priorityLevel) {
        if ( confirmReceivedQuery == null || confirmReceivedQuery.getProductId() == null
                ||confirmReceivedQuery.getVin() == null || confirmReceivedQuery.getIds() == null) {
            if (mOnConfirmReceivedDataListener != null) {
                mOnConfirmReceivedDataListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/confirmNotify";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin",confirmReceivedQuery.getVin());
        params.put("productId",confirmReceivedQuery.getProductId());
        params.put("ids",confirmReceivedQuery.getIds());
        params.put("msgType","poi");
        String requestData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData,
                thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USER_CONFIRM_RECEIVED_DATA);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 获取发送到车机的数据(车机端)
     * @param sendToVehicleQuery
     * @return
     */
    public int getDataSendToVehicle(SendToVehicleQuery sendToVehicleQuery){
        return getDataSendToVehicle(sendToVehicleQuery,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取发送到车机的数据(车机端)
     * @param sendToVehicleQuery
     * @param timeOutSecs
     * @return
     */
    public int getDataSendToVehicle(SendToVehicleQuery sendToVehicleQuery, int timeOutSecs){
        return getDataSendToVehicle(sendToVehicleQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取发送到车机的数据(车机端)
     * @param sendToVehicleQuery
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getDataSendToVehicle(SendToVehicleQuery sendToVehicleQuery, int timeOutSecs, int priorityLevel){
        if (sendToVehicleQuery == null || sendToVehicleQuery.getVin() == null || sendToVehicleQuery.getProductId() == null ){
            if (mOnGetDataSendToVehicleListener != null ){
                mOnConfirmReceivedDataListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/getNotify";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin",sendToVehicleQuery.getVin());
        params.put("productId",sendToVehicleQuery.getProductId());
        params.put("userId",sendToVehicleQuery.getUserId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData,
                thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_DATA_SEND_TO_VEHICLE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询车机历史信息 (车机端/app)
     * @param historyQuery
     * @return
     */
    public int getVehicleHistoryMsg(HistoryQuery historyQuery){
        return getVehicleHistoryMsg(historyQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车机历史信息 (车机端/app)
     * @param historyQuery
     * @param timeOutSecs
     * @return
     */
    public int getVehicleHistoryMsg(HistoryQuery historyQuery, int timeOutSecs){
        return getVehicleHistoryMsg(historyQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询车机历史信息 (车机端/app)
     * @param historyQuery
     * @param timeOutTecs
     * @param priorityLevel
     * @return
     */
    public int getVehicleHistoryMsg(HistoryQuery historyQuery, int timeOutTecs, int priorityLevel){
        if (historyQuery == null || historyQuery.getVin() == null || historyQuery.getProductId() == null
                || historyQuery.getLimit() == null ){
            if (mOnQueryVehicleHistoryListener != null ){
                mOnQueryVehicleHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "v2/getHistory";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("vin", historyQuery.getVin());
        params.put("productId", historyQuery.getProductId());
        params.put("limit", historyQuery.getLimit());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData,
                thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, QUERY_VEHICLE_HISTORY_MSG);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutTecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    OnSendToCarListener mOnSendToCarListener;
    OnSearchDeviceListListener mOnSearchDeviceListListener;
    OnMxBindOrUnbindListener mOnMxBindOrUnbindListener;
    OnSearchUserListListener mOnSearchUserListListener;
    OnPushMessageListener mOnPushMessageListener;
    OnSendMsgToCarListener mOnSendMsgToCarListener;
    OnConfirmReceivedDataListener mOnConfirmReceivedDataListener;
    OnGetDataSendToVehicleListener mOnGetDataSendToVehicleListener;
    OnQueryVehicleHistoryListener mOnQueryVehicleHistoryListener;

    public interface OnSendToCarListener extends CallBackListener{
        public void onGetWeChatUserInfo(WeChatUserInfoResult weChatUserInfoResult);
        public void onConfirmReceivedMsgSuccess();
    }

    public interface OnSearchDeviceListListener  extends CallBackListener{
        public void onSearchDeviceListListener(List<SearchDeviceListResult> list);
    }

    public interface OnMxBindOrUnbindListener extends CallBackListener{
        public void onMxBindOrUnbindListener();
    }

    public interface OnSearchUserListListener extends CallBackListener{
        public void onSearchUserListListener(List<SearchUserListResult> list);
    }

    public interface OnPushMessageListener extends CallBackListener{
        public void onPushMessageListener();
    }

    public interface OnSendMsgToCarListener extends CallBackListener{
        public void onSendMsgToCarListener();
    }

    public interface OnConfirmReceivedDataListener extends CallBackListener{
        public void onConfirmReceiveDataSuccess();
    }

    public interface OnGetDataSendToVehicleListener extends CallBackListener{
        public void onGetDataSendToVehicle(SendToVehicleResult sendToVehicleResult);
    }

    public interface OnQueryVehicleHistoryListener extends CallBackListener{
        public void onQueryVehicleHistory(List<HistoryResult> resultList);
    }

    public void setOnSendToCarListener(OnSendToCarListener mOnSendToCarListener){
        this.mOnSendToCarListener = mOnSendToCarListener;
    }

    public void setOnSearchDeviceListListener(OnSearchDeviceListListener onSearchDeviceListListener){
        this.mOnSearchDeviceListListener = onSearchDeviceListListener;
    }
    public void setOnMxBindOrUnbindListener(OnMxBindOrUnbindListener onMxBindOrUnbindListener){
        this.mOnMxBindOrUnbindListener = onMxBindOrUnbindListener;
    }

    public void setOnSearchUserListListener(OnSearchUserListListener onSearchUserListListener){
        this.mOnSearchUserListListener = onSearchUserListListener;
    }

    public void setOnPushMessageListener(OnPushMessageListener onPushMessageListener){
        this.mOnPushMessageListener = onPushMessageListener;
    }

    public void setOnSendMsgToCarListener(OnSendMsgToCarListener onSendMsgToCarListener){
        this.mOnSendMsgToCarListener = onSendMsgToCarListener;
    }

    public void setOnConfirmReceivedDataListener(OnConfirmReceivedDataListener onConfirmReceivedDataListener){
        this.mOnConfirmReceivedDataListener = onConfirmReceivedDataListener;
    }

    public void setOnGetDataSendToVehicleListener(OnGetDataSendToVehicleListener onGetDataSendToVehicleListener){
        this.mOnGetDataSendToVehicleListener = onGetDataSendToVehicleListener;
    }

    public void setOnQueryVehicleHistoryListener(OnQueryVehicleHistoryListener onQueryVehicleHistoryListener){
        this.mOnQueryVehicleHistoryListener = onQueryVehicleHistoryListener;
    }

//    /**
//     * 发送通知
//     * @param notice
//     */
//    public static void receiveNotice(String notice){
//            // 解析完通知内容回调APP通知接口
//            List<MessagePoiItem> Poilist = new ArrayList<MessagePoiItem>();
//            List<MessageBindItem> bindlist = new ArrayList<MessageBindItem>();
//            JSONTokener jsonParser = new JSONTokener(notice);
//            try {
//                JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
//                int respCode = jsonObject.getInt("respCode");
//                if (respCode == Constants.RESPCODE_SUCCESS) {
//                    String weChatCarId = Utils.converNullToEmpty(jsonObject.getString("wecarId"));
//                    JSONArray array = jsonObject.getJSONArray("contentList");
//                    MessagePoiItem poiresult;
//                    MessageBindItem bindresult;
//                    for (int i = 0; i < array.length(); i++) {
//                        JSONObject object = array.getJSONObject(i);
//                        JSONObject content =object.getJSONObject("content");
//                        String msgSubtype = Utils.converNullToEmpty(object.getString("msgSubtype"));
//                        if("poi".equals(msgSubtype)){
//                            poiresult = new MessagePoiItem();
//                            MessagePoiContent poicontent = new MessagePoiContent();
//                            if(content.has("source")){
//                                poicontent.setSource(content.getString("source"));
//                            }
//                            if(content.has("name")){
//                                poicontent.setName(content.getString("name"));
//                            }
//                            if(content.has("address")){
//                                poicontent.setAddress(content.getString("address"));
//                            }
//                            LatLng latLng =new LatLng();
//                            if(content.has("lat")){
//                                latLng.setLatitude(content.getDouble("lat"));
//                            }
//                            if(content.has("lng")){
//                                latLng.setLongitude(content.getDouble("lng"));
//                            }
//                            poicontent.setLatLng(latLng);
//                            LatLng maxLatLng =new LatLng();
//                            if(content.has("mxLat")){
//                                maxLatLng.setLatitude(content.getDouble("mxLat"));
//                            }
//                            if(content.has("mxLng")){
//                                maxLatLng.setLongitude(content.getDouble("mxLng"));
//                            }
//                            poicontent.setMaxLatLng(maxLatLng);
//                            if(content.has("phone")){
//                                poicontent.setPhone(content.getString("phone"));
//                            }
//                            WeChatUserInfo weChatUserInfo = new WeChatUserInfo();
//                            if(content.has("sex")){
//                                weChatUserInfo.setSex(content.getInt("sex"));
//                            }
//                            if(content.has("region")){
//                                weChatUserInfo.setRegion(content.getString("region"));
//                            }
//                            if(content.has("province")){
//                                weChatUserInfo.setProvince(content.getString("province"));
//                            }
//                            if(content.has("language")){
//                                weChatUserInfo.setLanguage(content.getString("language"));
//                            }
//                            if(content.has("nickname")){
//                                weChatUserInfo.setNickNname(content.getString("nickname"));
//                            }
//                            if(content.has("city")){
//                                weChatUserInfo.setCity(content.getString("city"));
//                            }
//                            if(content.has("country")){
//                                weChatUserInfo.setCountry(content.getString("country"));
//                            }
//                            if(content.has("headimgurl")){
//                                weChatUserInfo.setHeadmgurl(content.getString("headimgurl"));
//                            }
//                            poicontent.setWeChatUserInfo(weChatUserInfo);
//                            if(content.has("precision")){
//                                poicontent.setPrecision(content.getString("precision"));
//                            }
//                            if(content.has("similar")){
//                                poicontent.setSimilar(content.getInt("similar"));
//                            }
//                            if(content.has("openid")){
//                                poicontent.setOpenId(Utils.converNullToEmpty(content.getString("openid")));
//                            }
//                            if(content.has("groupid")){
//                                poicontent.setGroupId(Utils.converNullToEmpty(content.getString("groupid")));
//                            }
//                            poiresult.setContent(poicontent);
//                            poiresult.setCreateTime(object.getLong("createTime"));
//                            MessageInfo msginfo = new MessageInfo();
//                            if(object.has("weixinOpenid")){
//                                msginfo.setWeChatOpenId(Utils.converNullToEmpty(object.getString("weixinOpenid")));
//                            }
//                            if(object.has("msgType")){
//                                msginfo.setMsgType(Utils.converNullToEmpty(object.getString("msgType")));
//                            }
//                            if(object.has("msgSubtype")){
//                                msginfo.setMsgSubtype(Utils.converNullToEmpty(object.getString("msgSubtype")));
//                            }
//                            if(object.has("msgId")){
//                                msginfo.setMsgId(Utils.converNullToEmpty(object.getString("msgId")));
//                            }
//                            poiresult.setMessageInfo(msginfo);
//                            Poilist.add(poiresult);
//                        }else {
//                            bindresult = new MessageBindItem();
//                            MessageBindContent bindContent = new  MessageBindContent();
//                            bindContent.setBindStatus(content.getString("bindStatus"));
//                            bindContent.setUserId(content.getString("userId"));
//                            bindresult.setContent(bindContent);
//                            bindresult.setCreateTime(object.getLong("createTime"));
//                            MessageInfo msginfo = new MessageInfo();
//                            if(object.has("weixinOpenid")){
//                                msginfo.setWeChatOpenId(Utils.converNullToEmpty(object.getString("weixinOpenid")));
//                            }
//                            if(object.has("msgType")){
//                                msginfo.setMsgType(Utils.converNullToEmpty(object.getString("msgType")));
//                            }
//                            if(object.has("msgSubtype")){
//                                msginfo.setMsgSubtype(Utils.converNullToEmpty(object.getString("msgSubtype")));
//                            }
//                            if(object.has("msgId")){
//                                msginfo.setMsgId(Utils.converNullToEmpty(object.getString("msgId")));
//                            }
//                            bindresult.setMessageInfo(msginfo);
//                            bindlist.add(bindresult);
//                        }
//
//                    }
//                    if(TSP.mNotificationListener!=null){
//                        TSP.mNotificationListener.onReceiveSendToCarNotice(weChatCarId,Poilist,bindlist);
//                    }
//
//                }else{
//                    String errorMsg = jsonObject.getString("errmsg");
//
//
//                }
//            } catch (Exception e) {
//                e.printStackTrace();
//
//            }
//
//
//
//    }

    private  void addCallbackMap(int eventId,int type){
        super.addCallbackMap(eventId, SendToCar.this);

        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }
    @Override
    protected void removeCallbackMap(int eventId){
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    @Deprecated
    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg){
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case STCGETWECHATUSERINFO:
                if (mOnSendToCarListener != null) {
                    if(hasDSPTError){
                        mOnSendToCarListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            WeChatUserInfoResult weChatUserInfoResult = new  WeChatUserInfoResult();
                            weChatUserInfoResult.setDeviceNo(jsonObject.getString("vin"));
                            weChatUserInfoResult.setWeChatCarId(jsonObject.getString("wecarId"));
                            weChatUserInfoResult.setWeChatOppenid(Utils.converNullToEmpty(jsonObject.getString("weixinOpenid")));
                            if(jsonObject.has("wechatUserInfo") )
                            {   JSONObject weChatUserInfoResultobj =jsonObject.getJSONObject("wechatUserInfo");
                                WeChatUserInfo weChatUserInfo = new WeChatUserInfo();
                                weChatUserInfo.setCity(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("country")));
                                weChatUserInfo.setCountry(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("city")));
                                weChatUserInfo.setHeadmgurl(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("headmgurl")));
                                weChatUserInfo.setLanguage(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("language")));
                                weChatUserInfo.setNickNname(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("nickname")));
                                weChatUserInfo.setProvince(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("province")));
                                weChatUserInfo.setRegion(Utils.converNullToEmpty(weChatUserInfoResultobj.getString("region")));
                                weChatUserInfo.setSex(weChatUserInfoResultobj.getInt("sex"));
                                weChatUserInfoResult.setWeChatUserInfo(weChatUserInfo);
                            }
                            mOnSendToCarListener.onGetWeChatUserInfo(weChatUserInfoResult);
                        }else{
                            if (jsonObject.has("errmsg")){
                                String errorMsg = jsonObject.getString("errmsg");
                                mOnSendToCarListener.onFail(respCode, errorMsg);
                            }else{
                                String errorMsg = "出错了";
                                mOnSendToCarListener.onFail(respCode, errorMsg);
                            }


                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSendToCarListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }

                }
                break;
            case STCGRTMSEEAGE:
            	if(hasDSPTError){
            		Utils.loge("STCGRTMSEEAGE errorcode = "+errorCode+"errMsg = "+errMsg);
            	}else if(response!=null&&response.length()>0){
            		Notification.noticeSendToCar(response);
            	}
                
                break;
            case USERCONFIRMRECEIVEDMSG:
                if (mOnSendToCarListener != null) {
                    if(hasDSPTError){
                        mOnSendToCarListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnSendToCarListener.onConfirmReceivedMsgSuccess();
                        }else{
                            if (jsonObject.has("errmsg")){
                                String errorMsg = jsonObject.getString("errmsg");
                                mOnSendToCarListener.onFail(respCode, errorMsg);
                            }else{
                                String errorMsg = "出错了";
                                mOnSendToCarListener.onFail(respCode, errorMsg);
                            }
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSendToCarListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }

                }
                break;
            case SEARCHDEVICELIST:
                if (mOnSearchDeviceListListener!=null){
                    if(hasDSPTError){
                        mOnSearchDeviceListListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        List<SearchDeviceListResult> list = new ArrayList<SearchDeviceListResult>();
                        SearchDeviceListResult searchDeviceListResult;
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray results = jsonObject.getJSONArray("data");
                            for (int i=0;i<results.length();i++){
                                JSONObject result=results.getJSONObject(i);
                                searchDeviceListResult = new SearchDeviceListResult();
                                searchDeviceListResult.setVin(result.getString("vin"));
                                searchDeviceListResult.setWecarID(result.getString("wecarID"));
                                searchDeviceListResult.setWechatOpenID(result.getString("wechatOpenID"));
                                searchDeviceListResult.setUserID(result.getString("userID"));
                                searchDeviceListResult.setRegistTime(result.getString("registTime"));
                                searchDeviceListResult.setQrCode(result.getString("qrCode"));
                                searchDeviceListResult.setAppId(result.getString("appId"));
                                searchDeviceListResult.setProductID(result.getString("productID"));
                                searchDeviceListResult.setDeviceName(result.getString("deviceName"));
                                list.add(searchDeviceListResult);
                            }
                            mOnSearchDeviceListListener.onSearchDeviceListListener(list);
                        }else {
                            mOnSearchDeviceListListener.onFail(respCode, "出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchDeviceListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case MXBINDORUNBIND:
                if(mOnMxBindOrUnbindListener!=null){
                    if(hasDSPTError){
                        mOnMxBindOrUnbindListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnMxBindOrUnbindListener.onMxBindOrUnbindListener();
                        }else {
                            mOnMxBindOrUnbindListener.onFail(respCode, "出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnMxBindOrUnbindListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SEARCHUSERLIST:
                if (mOnSearchUserListListener!=null){
                    if(hasDSPTError){
                        mOnSearchUserListListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        List<SearchUserListResult> list = new ArrayList<SearchUserListResult>();
                        SearchUserListResult result;
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray userInfoList = jsonObject.getJSONArray("user_info_list");
                            for (int i=0;i<userInfoList.length();i++){
                                JSONObject userInfo = userInfoList.getJSONObject(i);
                                result = new SearchUserListResult();
                                result.setAppId(userInfo.getString("appId"));
                                if (userInfo.has("groupid")){
                                    result.setGroupId(userInfo.getString("groupid"));
                                }
                                if (userInfo.has("remark")){
                                    result.setRemark(userInfo.getString("remark"));
                                }
                                if (userInfo.has("unionid")){
                                    result.setUnionId(userInfo.getString("unionid"));
                                }
                                if (userInfo.has("subscribe_time")){
                                    result.setSubScribeTime(userInfo.getLong("subscribe_time"));
                                }
                                if (userInfo.has("headimgurl")){
                                    result.setHeadImgUrl(userInfo.getString("headimgurl"));
                                }
                                if (userInfo.has("language")){
                                    result.setLanguage(userInfo.getString("language"));
                                }
                                if (userInfo.has("province")){
                                    result.setProvinceName(userInfo.getString("province"));
                                }
                                if (userInfo.has("country")){
                                    result.setCityName(userInfo.getString("country"));
                                }
                                if (userInfo.has("city")){
                                    result.setCityName(userInfo.getString("city"));
                                }
                                if (userInfo.has("sex")){
                                    result.setSex(userInfo.getInt("sex"));
                                }
                                if (userInfo.has("nickname")){
                                    result.setNickName(userInfo.getString("nickname"));
                                }
                                if (userInfo.has("openid")){
                                    result.setOpenId(userInfo.getString("openid"));
                                }
                                if (userInfo.has("subscribe")){
                                    result.setSubscribe(userInfo.getInt("subscribe"));
                                }
                                list.add(result);
                            }
                            mOnSearchUserListListener.onSearchUserListListener(list);
                        }else {
                            mOnSearchUserListListener.onFail(respCode, "出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSearchUserListListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case PUSHMESSAGE:
                if (mOnPushMessageListener!=null){
                    if(hasDSPTError){
                        mOnPushMessageListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("errcode");
                        if (respCode == Constants.PUSHMESSAGE_SUCCESS) {
                            mOnPushMessageListener.onPushMessageListener();
                        }else {
                            mOnPushMessageListener.onFail(respCode, jsonObject.getString("errmsg"));
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnPushMessageListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SENDMSGTOCAR:
                if (mOnSendMsgToCarListener!=null){
                    if(hasDSPTError){
                        mOnSendMsgToCarListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnSendMsgToCarListener.onSendMsgToCarListener();
                        }else {
                            mOnSendMsgToCarListener.onFail(respCode, "出错了");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnSendMsgToCarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case USER_CONFIRM_RECEIVED_DATA:
                if (mOnConfirmReceivedDataListener != null ){
                    if (hasDSPTError) {
                        mOnConfirmReceivedDataListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnConfirmReceivedDataListener.onConfirmReceiveDataSuccess();
                        }else {
                            mOnConfirmReceivedDataListener.onFail(respCode, errMsg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnConfirmReceivedDataListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            case GET_DATA_SEND_TO_VEHICLE:
                if (mOnGetDataSendToVehicleListener != null ){
                    if (hasDSPTError){
                        mOnGetDataSendToVehicleListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        SendToVehicleResult sendToVehicleResult;
                        List<ContentListInfo> contentListInfoList;
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            sendToVehicleResult = new SendToVehicleResult();
                            sendToVehicleResult.setVin(jsonObject.getString("productId"));
                            sendToVehicleResult.setProductId(jsonObject.getString("vin"));
                            JSONArray jsonArray = jsonObject.getJSONArray("contentList");
                            contentListInfoList = new ArrayList<ContentListInfo>();
                            for (int i = 0; i<jsonArray.length(); i++){
                                JSONObject object = jsonArray.getJSONObject(i);
                                ContentListInfo contentListInfo = new ContentListInfo();
                                ContentInfo contentInfo = new ContentInfo();
                                contentListInfo.setMsgType(object.getString("msgType"));
                                contentListInfo.setCreateTime(object.getString("createTime"));
                                contentListInfo.setId(object.getString("id"));
                                JSONObject contentObject = object.getJSONObject("content");
                                LatLng latLng = new LatLng();
                                latLng.setLatitude(contentObject.getDouble("lat"));
                                latLng.setLongitude(contentObject.getDouble("lng"));
                                contentInfo.setLatLng(latLng);
                                contentInfo.setName(contentObject.getString("name"));
                                contentInfo.setAddress(contentObject.getString("address"));
                                contentInfo.setNickName(contentObject.getString("nickName"));
                                contentInfo.setHeadImgUrl(contentObject.getString("headImgUrl"));
                                contentInfo.setMsg(contentObject.getString("msg"));
                                contentInfo.setTypeCode(contentObject.getString("typeCode"));
                                contentListInfo.setContent(contentInfo);
                                contentListInfoList.add(contentListInfo);
                            }
                            sendToVehicleResult.setContentList(contentListInfoList);
                            mOnGetDataSendToVehicleListener.onGetDataSendToVehicle(sendToVehicleResult);
                        }else {
                            mOnGetDataSendToVehicleListener.onFail(respCode,errMsg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnGetDataSendToVehicleListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case QUERY_VEHICLE_HISTORY_MSG:
                if (mOnQueryVehicleHistoryListener != null){
                    if (hasDSPTError){
                        mOnQueryVehicleHistoryListener.onFail(errorCode,errMsg);
                        return;
                    }
                    List<HistoryResult> resultList;
                    HistoryResult historyResult;
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject)jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            resultList = new ArrayList<HistoryResult>();
                            JSONArray jsonArray = jsonObject.getJSONArray("contentList");
                            historyResult = new HistoryResult();
                            for (int i = 0; i<jsonArray.length(); i++){
                                JSONObject object = jsonArray.getJSONObject(i);
                                historyResult.setMsgType(object.getString("msgType"));
                                historyResult.setId(object.getString("id"));
                                historyResult.setCreateTime(object.getString("createTime"));
                                historyResult.setDeleteFlag(object.getInt("deleteFlag"));
                                JSONObject contentObject = object.getJSONObject("content");
                                ContentInfo contentInfo = new ContentInfo();
                                LatLng latLng = new LatLng();
                                latLng.setLongitude(contentObject.getDouble("lng"));
                                latLng.setLatitude(contentObject.getDouble("lat"));
                                contentInfo.setLatLng(latLng);
                                contentInfo.setName(contentObject.getString("name"));
                                contentInfo.setAddress(contentObject.getString("address"));
                                contentInfo.setNickName(contentObject.getString("nickName"));
                                contentInfo.setHeadImgUrl(contentObject.getString("headImgUrl"));
                                contentInfo.setMsg(contentObject.getString("msg"));
                                historyResult.setContent(contentInfo);
                                resultList.add(historyResult);
                            }
                            mOnQueryVehicleHistoryListener.onQueryVehicleHistory(resultList);
                        }else {
                            mOnQueryVehicleHistoryListener.onFail(respCode,errMsg);
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                        mOnQueryVehicleHistoryListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,
                                Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            default:
                break;
        }
    }

    protected void callTimeOut(int requestHandleId){
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case STCGETWECHATUSERINFO:
                if (mOnSendToCarListener != null) {
                    mOnSendToCarListener.onTimeOut();
                }
                break;
            case STCGRTMSEEAGE:
                break;
            case USERCONFIRMRECEIVEDMSG:
                if (mOnSendToCarListener != null) {
                    mOnSendToCarListener.onTimeOut();
                }
                break;
            case SEARCHDEVICELIST:
                if (mOnSearchDeviceListListener != null) {
                    mOnSearchDeviceListListener.onTimeOut();
                }
                break;
            case MXBINDORUNBIND:
                if (mOnMxBindOrUnbindListener != null) {
                    mOnMxBindOrUnbindListener.onTimeOut();
                }
                break;
            case SEARCHUSERLIST:
                if (mOnSearchUserListListener!=null){
                    mOnSearchUserListListener.onTimeOut();
                }
                break;
            case PUSHMESSAGE:
                if (mOnPushMessageListener != null) {
                    mOnPushMessageListener.onTimeOut();
                }
                break;
            case SENDMSGTOCAR:
                if (mOnSendMsgToCarListener != null) {
                    mOnSendMsgToCarListener.onTimeOut();
                }
                break;
            case USER_CONFIRM_RECEIVED_DATA:
                if (mOnConfirmReceivedDataListener != null ){
                    mOnConfirmReceivedDataListener.onTimeOut();
                }
                break;
            case GET_DATA_SEND_TO_VEHICLE:
                if (mOnGetDataSendToVehicleListener != null ){
                    mOnGetDataSendToVehicleListener.onTimeOut();
                }
                break;
            case QUERY_VEHICLE_HISTORY_MSG:
                if (mOnQueryVehicleHistoryListener != null){
                    mOnQueryVehicleHistoryListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }
}
