package com.mxnavi.tspv2.serviceintegrator;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.serviceintegrator.model.BindDeviceInfo;
import com.mxnavi.tspv2.serviceintegrator.model.DeviceUserInfo;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by lenovo on 2016/6/6.
 */
public class ServiceIntegrator extends BaseTsp {

    //请求服务类型为服务集成
    final int thisServiceType = Config.MxNGTPServiceType.MX_SI.getServiceType();
    //服务集成的ServiceData协议版本
    final int serviceDataVer =Config.MxNGTPServiceType.MX_SI.getServiceVer() ;

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();

    final int SI_findDeviceUser = 1;//查询导航集合
    final int SI_bindDeviceUser = 2;//绑定导航
    final int SI_unbindDeviceUser = 3;//解绑导航
    final int SI_obtainWechatQrCode = 4;//车机获取微信公众号二维码
    /**
     * 车机/美小助查询导航集合
     * @param userId 用户标识
     * @param token 用户登录标识
     * @return
     */
    public int findDeviceUser(String userId,String token){
        return findDeviceUser(userId,token,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     *车机/美小助查询导航集合
     * @param userId 用户标识
     * @param token 用户登录标识
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return
     */
    public int findDeviceUser(String userId,String token,int timeOutSecs){
        return findDeviceUser(userId,token,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     *车机/美小助查询导航集合
     * @param userId 用户标识
     * @param token 用户登录标识
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return
     */
    public int findDeviceUser(String userId,String token,int timeOutSecs,int priorityLevel){
        if(Utils.strIsEmpty(userId)||Utils.strIsEmpty(token)){
            if (mOnFindDeviceUserListener != null) {
                mOnFindDeviceUserListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "device/findDeviceUser";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId",userId);
        params.put("token",token);
        String requestData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SI_findDeviceUser);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 车机/美小助绑定导航
     * @param deviceName 设备名称
     * @param deviceUserInfo
     * @return 请求ID，0:参数错误
     */
    public int bindDeviceUser(String deviceName, DeviceUserInfo deviceUserInfo){
        return bindDeviceUser(deviceName,deviceUserInfo,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机/美小助绑定导航
     * @param deviceName 设备名称
     * @param deviceUserInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int bindDeviceUser(String deviceName, DeviceUserInfo deviceUserInfo,int timeOutSecs){
        return bindDeviceUser(deviceName,deviceUserInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机/美小助绑定导航
     * @param deviceName 设备名称
     * @param deviceUserInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int bindDeviceUser(String deviceName, DeviceUserInfo deviceUserInfo,int timeOutSecs,int priorityLevel){
        if(deviceUserInfo == null|| deviceUserInfo.getUserId()==null ||deviceUserInfo.getProductId()==null ||deviceUserInfo.getDeviceNo()==null ||deviceUserInfo.getToken()==null){
            if (mOnBindDeviceUserListener != null) {
                mOnBindDeviceUserListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "device/bindDeviceUser";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("deviceName",Utils.converNullToEmpty(deviceName));
        params.put("userId",deviceUserInfo.getUserId());
        params.put("token",deviceUserInfo.getToken());
        params.put("productId",deviceUserInfo.getProductId());
        params.put("deviceNo", deviceUserInfo.getDeviceNo());
        String requestData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SI_bindDeviceUser);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 车机/美小助解绑导航
     * @param deviceUserInfo
     * @return 请求ID，0:参数错误
     */
    public int unbindDeviceUser(DeviceUserInfo deviceUserInfo){
        return unbindDeviceUser(deviceUserInfo,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机/美小助解绑导航
     * @param deviceUserInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int unbindDeviceUser(DeviceUserInfo deviceUserInfo,int timeOutSecs){
        return unbindDeviceUser(deviceUserInfo,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机/美小助解绑导航
     * @param deviceUserInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int unbindDeviceUser(DeviceUserInfo deviceUserInfo,int timeOutSecs,int priorityLevel){
        if(deviceUserInfo == null|| deviceUserInfo.getUserId()==null ||deviceUserInfo.getProductId()==null ||deviceUserInfo.getDeviceNo()==null ||deviceUserInfo.getToken()==null){
            if (mOnUnbindDeviceUserListener != null) {
                mOnUnbindDeviceUserListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "device/unbindDeviceUser";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId",deviceUserInfo.getUserId());
        params.put("token",deviceUserInfo.getToken());
        params.put("productId",deviceUserInfo.getProductId());
        params.put("deviceNo", deviceUserInfo.getDeviceNo());
        String requestData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SI_unbindDeviceUser);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 车机获取微信公众号二维码
     * @param onObtainWechatQrCodeListener 回调接口
     * @return 请求ID，0:参数错误
     */
    public int obtainWechatQrCode(OnObtainWechatQrCodeListener onObtainWechatQrCodeListener){
        return obtainWechatQrCode(onObtainWechatQrCodeListener,Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机获取微信公众号二维码
     * @param onObtainWechatQrCodeListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int obtainWechatQrCode(OnObtainWechatQrCodeListener onObtainWechatQrCodeListener,int timeOutSecs){
        return obtainWechatQrCode(onObtainWechatQrCodeListener,timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 车机获取微信公众号二维码
     * @param onObtainWechatQrCodeListener 回调接口
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int obtainWechatQrCode(OnObtainWechatQrCodeListener onObtainWechatQrCodeListener,int timeOutSecs,int priorityLevel){
        this.mOnObtainWechatQrCodeListener = onObtainWechatQrCodeListener;
        int eventId = RequestDispathUtil.createEventId();
        String url = "device/obtainWechatQrCode";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId",Config.CUSTOMER_KEY);
        params.put("deviceNo",Config.VIN);
        String requestData = Utils.makeUrl(url,params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系;
        addCallbackMap(eventId, SI_obtainWechatQrCode);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }


    protected void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, ServiceIntegrator.this);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SI_findDeviceUser:
                if (mOnFindDeviceUserListener != null) {
                    mOnFindDeviceUserListener.onTimeOut();
                }
                break;
            case SI_bindDeviceUser:
                if (mOnBindDeviceUserListener != null) {
                    mOnBindDeviceUserListener.onTimeOut();
                }
                break;
            case SI_unbindDeviceUser:
                if (mOnUnbindDeviceUserListener != null) {
                    mOnUnbindDeviceUserListener.onTimeOut();
                }
                break;
            case SI_obtainWechatQrCode:
                if (mOnObtainWechatQrCodeListener != null) {
                    mOnObtainWechatQrCodeListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SI_findDeviceUser://查询导航集合
                if (mOnFindDeviceUserListener != null) {
                    if(hasDSPTError){
                        mOnFindDeviceUserListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        List<BindDeviceInfo> bindDeviceInfos = new ArrayList<BindDeviceInfo>();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONArray bindDeviceInfosArray = jsonObject.getJSONArray("rows");
                            for (int i = 0; i < bindDeviceInfosArray.length(); i++) {
                                JSONObject bindDeviceInfoObject = bindDeviceInfosArray.getJSONObject(i);
                                BindDeviceInfo bindDeviceInfo = new BindDeviceInfo();
                                bindDeviceInfo.setDeviceName(bindDeviceInfoObject.getString("deviceName"));
                                bindDeviceInfo.setDeviceNo(bindDeviceInfoObject.getString("deviceNo"));
                                bindDeviceInfo.setProductId(bindDeviceInfoObject.getString("productId"));
                                bindDeviceInfo.setAppId(bindDeviceInfoObject.getString("appId"));
                                bindDeviceInfos.add(bindDeviceInfo);
                            }
                            mOnFindDeviceUserListener.onFindDeviceUser(bindDeviceInfos);
                        }else {
                            mOnFindDeviceUserListener.onFail(respCode,"出错了！");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnFindDeviceUserListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SI_bindDeviceUser://绑定导航
                if (mOnBindDeviceUserListener != null) {
                    if(hasDSPTError){
                        mOnBindDeviceUserListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnBindDeviceUserListener.onBindDeviceUser();
                        }else {
                            mOnBindDeviceUserListener.onFail(respCode,"出错了!");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnBindDeviceUserListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SI_unbindDeviceUser://解绑导航
                if (mOnUnbindDeviceUserListener != null) {
                    if(hasDSPTError){
                        mOnUnbindDeviceUserListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mOnUnbindDeviceUserListener.onunbindDeviceUser();
                        }else {
                            mOnUnbindDeviceUserListener.onFail(respCode,"出错了!");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnUnbindDeviceUserListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SI_obtainWechatQrCode://车机获取微信公众号二维码
                if (mOnObtainWechatQrCodeListener != null) {
                    if(hasDSPTError){
                        mOnObtainWechatQrCodeListener.onFail(errorCode,errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String ticketURL = jsonObject.getString("ticketURL");
                            int expireSeconds = jsonObject.getInt("expireSeconds");
                            mOnObtainWechatQrCodeListener.onObtainWechatQrCode(ticketURL,expireSeconds);
                        }else {
                            mOnObtainWechatQrCodeListener.onFail(respCode,"出错了!");
                        }
                    }catch (Exception e){
                        e.printStackTrace();
                        mOnObtainWechatQrCodeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }


    public interface OnFindDeviceUserListener extends CallBackListener{
        public void onFindDeviceUser(List<BindDeviceInfo> BindDeviceInfos);
    }
    public interface OnBindDeviceUserListener extends CallBackListener{
        public void onBindDeviceUser();
    }
    public interface OnUnbindDeviceUserListener extends CallBackListener{
        public void onunbindDeviceUser();
    }

    public interface OnObtainWechatQrCodeListener extends CallBackListener{
        /**
         * 回调方法
         * @param ticketURL 二维码的URL
         * @param expireSeconds 有效期
         */
        public void onObtainWechatQrCode(String ticketURL,int expireSeconds);
    }

    OnFindDeviceUserListener mOnFindDeviceUserListener;
    OnBindDeviceUserListener mOnBindDeviceUserListener;
    OnUnbindDeviceUserListener mOnUnbindDeviceUserListener;
    OnObtainWechatQrCodeListener mOnObtainWechatQrCodeListener;

    public void setOnFindDeviceUserListener(OnFindDeviceUserListener onFindDeviceUserListener) {
        this.mOnFindDeviceUserListener = onFindDeviceUserListener;
    }
    public void setOnBindDeviceUserListener(OnBindDeviceUserListener onBindDeviceUserListener) {
        this.mOnBindDeviceUserListener = onBindDeviceUserListener;
    }
    public void setOnUnbindDeviceUserListener(OnUnbindDeviceUserListener onUnbindDeviceUserListener) {
        this.mOnUnbindDeviceUserListener = onUnbindDeviceUserListener;
    }
    public void setOnObtainWechatQrCodeListener(OnObtainWechatQrCodeListener onObtainWechatQrCodeListener) {
        this.mOnObtainWechatQrCodeListener = onObtainWechatQrCodeListener;
    }

}
