package com.mxnavi.tspv2.trafficrestriction;

import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.model.LatLng;
import com.mxnavi.tspv2.trafficrestriction.model.RestrictCityInfo;
import com.mxnavi.tspv2.trafficrestriction.model.RestrictInfo;
import com.mxnavi.tspv2.trafficrestriction.model.SearchRestrictionResult;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by Administrator on 2016/5/4.
 * 限行检索服务
 */
public class TrafficRestrictionSearch extends BaseTsp {

    // 请求服务类型为限行
    final int thisServiceType = Config.MxNGTPServiceType.MX_VEHICLE_RESTRICTION.getServiceType();
    // 注册此对象为限行业务信息接收类
    final int serviceDataVer = Config.MxNGTPServiceType.MX_VEHICLE_RESTRICTION.getServiceVer();


    private Map<Integer,Integer> listenertypeHashMap = new HashMap<Integer,Integer>();
    final int SearchTraffic_Restrict = 1;
    final int SearchCity_Restrict = 2;

    OnSearchTrafficRestrictListener mOnSearchTrafficRestrictListener;
    OnSearchCityRestrictListener mOnSearchCityRestrictListener;

    /**
     * 请求查询当前车所在地区限行信息
     * @param plateNo 车牌号
     * @param cityLocation 位置信息经纬度
     * @return 请求ID
     */
    public int searchTrafficRestrict(String plateNo,LatLng cityLocation){
        return searchTrafficRestrict(plateNo,cityLocation,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询当前车所在地区限行信息
     * @param plateNo 车牌号
     * @param cityLocation 位置信息经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int searchTrafficRestrict(String plateNo,LatLng cityLocation,int timeOutSecs){
        return searchTrafficRestrict(plateNo,cityLocation,timeOutSecs, Config.DEFAULT_PRIORITY);
    }


    /**
     * 请求查询当前车所在地区限行信息
     * @param plateNo 车牌号
     * @param cityLocation 位置信息经纬度
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int searchTrafficRestrict(String plateNo,LatLng cityLocation,int timeOutSecs,int priorityLevel){
        if(cityLocation==null){
            if (mOnSearchTrafficRestrictListener != null) {
                mOnSearchTrafficRestrictListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        String url = "searchLocationRestrict";

        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("lon",cityLocation.getLongitude());
        params.put("lat", cityLocation.getLatitude());
        if (plateNo != null){
            params.put("plate",plateNo);
        }

        String requestData = Utils.makeUrl(url, params);

        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, SearchTraffic_Restrict);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    /**
     * 请求查询全国限行城市信息
     * @return 请求ID
     */
    public int searchCityRestrict(){
        return searchCityRestrict(Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询全国限行城市信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID
     */
    public int searchCityRestrict(int timeOutSecs){
        return searchCityRestrict(timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 请求查询全国限行城市信息
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID
     */
    public int searchCityRestrict(int timeOutSecs,int priorityLevel){
        String requestData = "searchCityList";
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage =  MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData, thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        // 添加对应关系

        addCallbackMap(eventId, SearchCity_Restrict);
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    @Override
    protected void onCallBack(int requestHandleId, String response,boolean hasDSPTError,int errorCode,String errMsg){
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type){
            case SearchTraffic_Restrict:
                if(mOnSearchTrafficRestrictListener != null) {
                    if(hasDSPTError){
                        mOnSearchTrafficRestrictListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    SearchRestrictionResult result = new SearchRestrictionResult();
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");

                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String areaCode = jsonObject.getString("areaCode");
                            String areaName = jsonObject.getString("areaName");
                            String matchFlag = jsonObject.getString("matchFlag");
//                            if(jsonObject.getString("matchFlag").equals("匹配")){
//                                matchFlag ="true";
//                            }else if(jsonObject.getString("matchFlag").equals("不匹配")){
//                                matchFlag ="false";
//                            }else if(jsonObject.getString("matchFlag").equals("")||jsonObject.getString("matchFlag").equals("")){
//                                matchFlag ="false";
//                            }


                            if(jsonObject.has("weekRestrict")){
                                JSONArray weekRestrictDataArray = jsonObject.getJSONArray("weekRestrict");
                                List<RestrictInfo> weekRestrictList = new ArrayList<RestrictInfo>();
                                RestrictInfo restrictInfo;
                                for (int i = 0; i < weekRestrictDataArray.length(); i++) {
                                    restrictInfo = new RestrictInfo();
                                    JSONObject object = weekRestrictDataArray.getJSONObject(i);
                                    restrictInfo.setDate(object.getString("date"));
                                    restrictInfo.setTime(object.getString("time"));
                                    restrictInfo.setRule(object.getString("rule"));
                                    restrictInfo.setArea(object.getString("area"));
                                    restrictInfo.setMsg(object.getString("msg"));
                                    weekRestrictList.add(restrictInfo);
                                }
                                result.setWeekRestrict(weekRestrictList);
                            }

                            if(jsonObject.has("numberRestrict")){
                                JSONArray numberRestrictDataArray = jsonObject.getJSONArray("numberRestrict");
                                List<RestrictInfo> numberRestrictList = new ArrayList<RestrictInfo>();
                                RestrictInfo restrictInfo;
                                for (int i = 0; i < numberRestrictDataArray.length(); i++) {
                                    restrictInfo = new RestrictInfo();
                                    JSONObject object = numberRestrictDataArray.getJSONObject(i);
                                    restrictInfo.setDate(object.getString("date"));
                                    restrictInfo.setTime(object.getString("time"));
                                    restrictInfo.setRule(object.getString("rule"));
                                    restrictInfo.setArea(object.getString("area"));
                                    restrictInfo.setMsg(object.getString("msg"));
                                    numberRestrictList.add(restrictInfo);
                                }
                                result.setWeekRestrict(numberRestrictList);
                            }

                            if(jsonObject.has("dateRestrict")){
                                JSONArray dateRestrictDataArray = jsonObject.getJSONArray("dateRestrict");
                                List<RestrictInfo> dateRestrictList = new ArrayList<RestrictInfo>();
                                RestrictInfo restrictInfo;
                                for (int i = 0; i < dateRestrictDataArray.length(); i++) {
                                    restrictInfo = new RestrictInfo();
                                    JSONObject object = dateRestrictDataArray.getJSONObject(i);
                                    restrictInfo.setDate(object.getString("date"));
                                    restrictInfo.setTime(object.getString("time"));
                                    restrictInfo.setRule(object.getString("rule"));
                                    restrictInfo.setArea(object.getString("area"));
                                    restrictInfo.setMsg(object.getString("msg"));
                                    dateRestrictList.add(restrictInfo);
                                }
                                result.setDateRestrict(dateRestrictList);
                            }

                            if(jsonObject.has("otherRestrict")){
                                JSONArray otherRestrictDataArray = jsonObject.getJSONArray("otherRestrict");
                                List<RestrictInfo> otherRestrictList = new ArrayList<RestrictInfo>();
                                RestrictInfo restrictInfo;
                                for (int i = 0; i < otherRestrictDataArray.length(); i++) {
                                    restrictInfo = new RestrictInfo();
                                    JSONObject object = otherRestrictDataArray.getJSONObject(i);
                                    restrictInfo.setDate(object.getString("date"));
                                    restrictInfo.setTime(object.getString("time"));
                                    restrictInfo.setRule(object.getString("rule"));
                                    restrictInfo.setArea(object.getString("area"));
                                    restrictInfo.setMsg(object.getString("msg"));
                                    otherRestrictList.add(restrictInfo);
                                }
                                result.setOtherRestrict(otherRestrictList);
                            }

                            result.setAreaCode(areaCode);
                            result.setAreaName(areaName);
                            result.setMatchFlag(matchFlag);

                            mOnSearchTrafficRestrictListener.onSearchTrafficRestrict(result);
                        } else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnSearchTrafficRestrictListener.onFail(respCode, errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSearchTrafficRestrictListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case SearchCity_Restrict:
                if(mOnSearchCityRestrictListener != null){
                    if(hasDSPTError){
                        mOnSearchCityRestrictListener.onFail(errorCode,errMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        List<RestrictCityInfo> restrictCityInfoList = new ArrayList<RestrictCityInfo>();
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            RestrictCityInfo restrictCityInfo;
                            JSONArray resultDataArray = jsonObject.getJSONArray("result");
                            for (int i = 0; i < resultDataArray.length(); i++){
                                restrictCityInfo = new RestrictCityInfo();
                                JSONObject object = resultDataArray.getJSONObject(i);
                                restrictCityInfo.setCityCode(object.getString("areaCode"));
                                restrictCityInfo.setCityName(object.getString("areaName"));
                                LatLng cityLocation = new LatLng();
                                cityLocation.setLongitude(object.getDouble("lon"));
                                cityLocation.setLatitude(object.getDouble("lat"));
                                restrictCityInfo.setCityLocation(cityLocation);
                                restrictCityInfoList.add(restrictCityInfo);
                            }
                            mOnSearchCityRestrictListener.onSearchCityRestrict(restrictCityInfoList);
                        }else{
                            String errorMsg = "";
                            if(jsonObject.has("msg")){
                                errorMsg = jsonObject.getString("msg");
                            }
                            mOnSearchCityRestrictListener.onFail(respCode,errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSearchCityRestrictListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {

        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case SearchTraffic_Restrict:
                if (mOnSearchTrafficRestrictListener != null) {
                    mOnSearchTrafficRestrictListener.onTimeOut();
                }
                break;
            case SearchCity_Restrict:
                if (mOnSearchCityRestrictListener != null) {
                    mOnSearchCityRestrictListener.onTimeOut();
                }
                break;
        }
    }

    @Override
    protected void removeCallbackMap(int eventId){
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.remove(eventId);
        }
    }

    public interface OnSearchTrafficRestrictListener extends CallBackListener{
        public  void onSearchTrafficRestrict(SearchRestrictionResult searchRestrictionResult);
    }
    public interface OnSearchCityRestrictListener extends CallBackListener{
        /**
         * 返回限行城市
         * @param list 全国限行城市列表
         */
        public  void onSearchCityRestrict(List<RestrictCityInfo> list);
    }



    public void setOnSearchTrafficRestrictListener(OnSearchTrafficRestrictListener onSearchTrafficRestrictListener) {
        mOnSearchTrafficRestrictListener = onSearchTrafficRestrictListener;
    }

    public void setOnSearchCityRestrictListener(OnSearchCityRestrictListener onSearchCityRestrictListener) {
        mOnSearchCityRestrictListener = onSearchCityRestrictListener;
    }

    private  void addCallbackMap(int eventId,int type){
        super.addCallbackMap(eventId, TrafficRestrictionSearch.this);
        synchronized (listenertypeHashMap){
            listenertypeHashMap.put(eventId, type);
        }
    }
}
