package com.mxnavi.tspv2.tsp;

import android.content.Context;

import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Connection;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.ResponseDispathUtil;
import com.mxnavi.tspv2.core.TimeOut;
import com.mxnavi.tspv2.dns.DnsService;
import com.mxnavi.tspv2.model.InitializeInfo;
import com.mxnavi.tspv2.core.Notification;
import com.mxnavi.tspv2.notification.NotificationListener;
import com.mxnavi.tspv2.utils.Utils;

/**
 * Created by Administrator on 2016/3/16.
 */
public class TSP {

//    static ThreadPool businessThreadPool = ThreadPool.getInstance();

//    public static PriorityBlockingQueue<PrioritizedRequestMessage> queue =new PriorityBlockingQueue<PrioritizedRequestMessage>();
    static public Context context;
    private TSP() {}
    /**
     * 重新设置初始化信息
     * @param customerKey 产品线ID
     * @param secretKey 秘钥
     * @param vin 设备ID
     * @param deviceType  0：默认未知；1：美行车机；2：手机；3：美行后视镜；4：PC端
     */
    public static void reSetInitializeInfo(int customerKey,String secretKey,String vin,int deviceType)  {
    	   Config.CUSTOMER_KEY = customerKey;
           Config.DEVICE_TYPE = deviceType;
           Config.SECRETKEY = secretKey;
           Config.VIN = vin;
           MessageBuilder.instance().reSetInitializeInfo(customerKey, secretKey, vin);

    }
    /**
     * 初始化并启动TSP的SDK
     * @param initializeInfo 初始化信息
     *  @param mcontext 安卓context
     */
    public static void start(InitializeInfo initializeInfo,Context mcontext)  {

//        MessageBuilder.instance().setInitializeInfo(initializeInfo);
        if(mcontext==null||initializeInfo==null|| Utils.strIsEmpty(initializeInfo.getSecretKey())||Utils.strIsEmpty(initializeInfo.getVin())
                ||initializeInfo.getCustomerKey()==0||initializeInfo.getPositionListener()==null||Utils.strIsEmpty(initializeInfo.getTsDataPath())
                ||(Utils.strIsEmpty(initializeInfo.getTsConfigPath())&&Utils.strIsEmpty(initializeInfo.getTsConfigAssetsPath()))){
            throw new IllegalArgumentException("Tsp sdk Initialized arguments are not complete");
        }

        context = mcontext.getApplicationContext();
        MessageBuilder.instance().setInitializeInfo(initializeInfo);
        Config.CUSTOMER_KEY = initializeInfo.getCustomerKey();
        Config.DEVICE_TYPE = initializeInfo.getDeviceType();
        Config.SECRETKEY = initializeInfo.getSecretKey();
        Config.VIN = initializeInfo.getVin();
        Config.APPID = initializeInfo.getAppId();
//
        if(!Utils.strIsEmpty(initializeInfo.getTsConfigPath())){
            Connection.getInstance().start(initializeInfo.getTsConfigPath(),null);
        }else{
            Connection.getInstance().start(initializeInfo.getTsConfigAssetsPath(),context.getAssets());
        }

        Notification.initNotificationListener();
        
        if (tspInitializedListener != null) {
            tspInitializedListener.onTSPInitializeSuccess();
        }
    }
    /**
     * 停止 TSP SDK
     */
    public static void stop(){
        Connection.getInstance().stop();
        RequestDispathUtil.clearStorage();
        TimeOut.stopCheckTimeOut();
        DnsService.getInstance().stopDnsService();
//        NativeInitialize.uninitialize();
//        return	NativeInitialize.stop();
    }

    public static void noticeNetWorkState(boolean isConnected){
        Connection.getInstance().noticeNetWorkState(isConnected);
    }

    /**
     * 设置通知回调接口
     * @param notificationListener
     */
    public static void setNotificationListener(NotificationListener notificationListener){
//        mNotificationListener = notificationListener;
        Notification.initNotificationListener();
        Notification.setNotificationListener(notificationListener);
//        SendToCar.setNotificationListener(notificationListener);
    }
    /**
     * 设置DSPT服务器http协议的域名和端口号,
     * @param url 如："http://mx-t10-http-test-v1.mxnavi.com:8102"
     */
    public static void setDSPTHTTPUrl(String url){
    	if(url!=null&&url.length()>0){
    		Config.HTTP_URL  = url+Config.HTTP_PATH;
    	}
    	
    }
    public static void setAppPushKey(String pushKey){
        MessageBuilder.PUSH_KEY = pushKey;
    }


    public interface UserTokenExpireListener{
        /**
         * 用户token失效
         */
        public void onUserTokenExpire();

        /**
         * 无效的用户（用户不存在或者被注销）
         */
        public void onUserInvalid();

        /**
         * 用户被踢出登陆
         */
        public void onUserKickedOut();
    }

    /**
     * 设置用户token失效
     * @param userTokenExpireListener
     */
    public static void setUserTokenExpireListener(UserTokenExpireListener userTokenExpireListener) {
        ResponseDispathUtil.setUserTokenExpireListener(userTokenExpireListener);
    }
    
    /**
     * VERSION：false - 测试版，true = 正式版
     */
    public static boolean isRelease() {
    	boolean isRelease = true;
    	if (Config.TCPHOST.equals("mx-tsp2-dspt-tcp-sy.mxnavi.com") 
    			|| Config.TCPHOST.equals("mx-t10-longlived-inner-test.mxnavi.com")) {
    		isRelease = false;
    	}
    	
    	if (Config.TCPHOST.equals("mx-tsp2-dspt-tcp-ca.mxnavi.com") 
    			|| Config.TCPHOST.equals("mx-t10-longlived-v1.mxnavi.com")) {
    		isRelease = true;
    	}
    	return isRelease;
    }
    
    private static TSPInitializedListener tspInitializedListener;

    public interface TSPInitializedListener {
        public void onTSPInitializeSuccess();
    }

    public static void setTSPInitializedListener(TSPInitializedListener tspInitializedListener) {
        TSP.tspInitializedListener = tspInitializedListener;
    }

    //TSPSDK-长连接登录成功回调通知
    public static TSPLoginListener tspLoginListener;
    public interface TSPLoginListener {
        void onTSPLoginSuccess();
    }
    public static void setTSPLoginListener(TSPLoginListener tspLoginListener) {
        TSP.tspLoginListener = tspLoginListener;
    }
}
