package com.mxnavi.tspv2.userauth;


import com.google.gson.Gson;
import com.mxnavi.tspv2.core.BaseTsp;
import com.mxnavi.tspv2.core.Config;
import com.mxnavi.tspv2.core.Constants;
import com.mxnavi.tspv2.core.MessageBuilder;
import com.mxnavi.tspv2.core.PrioritizedRequestMessage;
import com.mxnavi.tspv2.core.RequestDispathUtil;
import com.mxnavi.tspv2.core.common.CallBackListener;
import com.mxnavi.tspv2.core.message.MxNGTPMessage;
import com.mxnavi.tspv2.http.AsyncHttpResponseHandler;
import com.mxnavi.tspv2.http.HttpRequestUtil;
import com.mxnavi.tspv2.http.RequestParams;
import com.mxnavi.tspv2.userauth.model.AuthenticationInfo;
import com.mxnavi.tspv2.userauth.model.ECallContactInfo;
import com.mxnavi.tspv2.userauth.model.IndividualUserInfo;
import com.mxnavi.tspv2.userauth.model.ListGroup;
import com.mxnavi.tspv2.userauth.model.ListGroupInfo;
import com.mxnavi.tspv2.userauth.model.LoginData;
import com.mxnavi.tspv2.userauth.model.LoginDataInfo;
import com.mxnavi.tspv2.userauth.model.LoginInfo;
import com.mxnavi.tspv2.userauth.model.Param;
import com.mxnavi.tspv2.userauth.model.ScanCodeLoginInfo;
import com.mxnavi.tspv2.userauth.model.SendPhoneValidNumInfo;
import com.mxnavi.tspv2.userauth.model.UpdatePinPasswordInfo;
import com.mxnavi.tspv2.userauth.model.UpdatePwdInfo;
import com.mxnavi.tspv2.userauth.model.UploadFileInfo;
import com.mxnavi.tspv2.userauth.model.UserInfoResult;
import com.mxnavi.tspv2.userauth.model.UserLoginQuery;
import com.mxnavi.tspv2.userauth.model.ValidNumInfo;
import com.mxnavi.tspv2.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.json.JSONTokener;

import java.io.File;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import static com.mxnavi.tspv2.core.MessageBuilder.NON_SUB_SERVICE_TYPE;

/**
 * Created by Administrator on 2018/1/30.
 */

public class UserAuth extends BaseTsp {

    // 请求服务类型为用户鉴权
    final int thisServiceType = Config.MxNGTPServiceType.USER_AUTH.getServiceType();
    // 用户鉴权的ServiceData协议版本
    final int serviceDataVer = Config.MxNGTPServiceType.USER_AUTH.getServiceVer();

    // 请求服务类型为集成服务鉴权及协议版本
    final int monitorServiceType = Config.MxNGTPServiceType.MONITOR_INTEGRATOR.getServiceType();
    final int monitorServiceDataVer = Config.MxNGTPServiceType.MONITOR_INTEGRATOR.getServiceVer();

    public UserAuth() {
    }

    private Map<Integer, Integer> listenertypeHashMap = new HashMap<Integer, Integer>();
    private Map<Integer, UploadFileInfo> uploadFileInfoHashMap = new HashMap<Integer, UploadFileInfo>();
    private Map<Integer, String> uploadFileUrlHashMap = new HashMap<Integer, String>();


    final int USERSENDPHONEVALIDNUM = 1;//获取验证码
    final int VERIFY_PHONE_VALID_NUM = 2;//手机验证码验证
    final int REGISTERBYPHONE = 3;//添加个人用户
    final int UPLOAD_PHOTO = 4;//上传图片
    final int UPDATEAUTHENTICATIONINFO = 5;//修改个人用户认证信息
    final int UPDATE_USER_NICKNAME = 6;//修改昵称
    final int UPDATE_USER_ECALLCONTACT = 7;//修改紧急联系人信息
    final int APP_LOGOUT = 8;//APP退出登录
    final int UPDATE_USER_AVATAR = 9;//上传头像
    final int REFRESH_TOKEN = 10;//刷新token
    final int VERIFY_TOKEN = 11;//验证token
    final int UPDATEINDIVIDUALUSERPHONE = 12;//修改个人用户手机号
    final int UPDATEPASSWORDBYPHONE = 13;//手机验证码找回密码
    final int USERLOGINBYACCOUNT = 14;//APP密码登陆
    final int LOGIN_BY_VALID_NUM = 15;//APP验证码登陆
    final int GETUSERINFO = 16;//查询个人用户信息
//    final int CREATEUPLOADPATH = 17;//创建上传路径
    final int USERAUTHORITYLOGINBYCODE  = 18;//扫码登陆
    final int USERLOGINBYQRCODE = 19;//扫码登陆轮询
    final int USERUPDATAPWD = 20;//使用旧密码修改密码
    final int SETUPPINPASSWORD = 21;//PIN码首次设置
    final int UPDATEPINPASSWORD = 22;//旧PIN码修改新PIN码修改
    final int GET_QRLOGIN_CODE = 23;//获取二维码登录生成二维码所需code
    final int GET_LIST_GROUP = 24;//查询集团信息
    final int USER_LOGIN_BY_CODE = 25; // 扫码登陆(新)
    final int SEARCH_GROUPINFO_BY_ID = 26; // 根据用户ID查询绑定的集团信息（app使用）

    /**
     * 获取手机验证码
     * @param phoneValidNumInfo
     * @return 请求ID，0:参数错误
     */
    public int sendPhoneValidNum(SendPhoneValidNumInfo phoneValidNumInfo) {
        return sendPhoneValidNum(phoneValidNumInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取手机验证码
     * @param phoneValidNumInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @return 请求ID，0:参数错误
     */
    public int sendPhoneValidNum(SendPhoneValidNumInfo phoneValidNumInfo, int timeOutSecs) {
        return sendPhoneValidNum(phoneValidNumInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取手机验证码
     * @param phoneValidNumInfo
     * @param timeOutSecs 接口超时时间，传0为不关注
     * @param priorityLevel 接口优先级 0：低优先级，1：中优先级，2：高优先级
     * @return 请求ID，0:参数错误
     */
    public int sendPhoneValidNum(SendPhoneValidNumInfo phoneValidNumInfo, int timeOutSecs, int priorityLevel) {
        if (phoneValidNumInfo == null
                || phoneValidNumInfo.getCheckType() == null
                || phoneValidNumInfo.getModelType() == null
                || phoneValidNumInfo.getPhone() == null
                || phoneValidNumInfo.getTypeId() == null) {
            if (sendPhoneValidNumSuccessListener != null) {
                sendPhoneValidNumSuccessListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/phoneSms/sendPhoneSms";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", phoneValidNumInfo.getPhone());
        params.put("modelType", phoneValidNumInfo.getModelType());
        params.put("checkType", phoneValidNumInfo.getCheckType());
        params.put("typeId", phoneValidNumInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USERSENDPHONEVALIDNUM);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 验证手机验证码
     * @param validNumInfo
     * @return 请求ID
     */
    public int verifyPhoneValidNum(ValidNumInfo validNumInfo) {
        return verifyPhoneValidNum(validNumInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 验证手机验证码
     * @param validNumInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int verifyPhoneValidNum(ValidNumInfo validNumInfo, int timeOutSecs) {
        return verifyPhoneValidNum(validNumInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 验证手机验证码
     * @param validNumInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int verifyPhoneValidNum(ValidNumInfo validNumInfo, int timeOutSecs, int priorityLevel) {
        if (validNumInfo == null
                || validNumInfo.getPhone() == null
                || validNumInfo.getValidateCode() == null
                || validNumInfo.getValidSessionId() == null) {
            if (mVerifyValidNumListener != null) {
                mVerifyValidNumListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/phoneSms/verifyPhoneValidCode";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", validNumInfo.getPhone());
        params.put("validateCode", validNumInfo.getValidateCode());
        params.put("validSessionId", validNumInfo.getValidSessionId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, VERIFY_PHONE_VALID_NUM);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 添加个人用户
     * @param individualUserInfo
     * @return 请求ID
     */
    public int registerByPhone(IndividualUserInfo individualUserInfo) {
        return registerByPhone(individualUserInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 添加个人用户
     * @param individualUserInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int registerByPhone(IndividualUserInfo individualUserInfo, int timeOutSecs) {
        return registerByPhone(individualUserInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 添加个人用户
     * @param individualUserInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int registerByPhone(IndividualUserInfo individualUserInfo, int timeOutSecs, int priorityLevel) {
        if (individualUserInfo == null
                || individualUserInfo.getPhone() == null
                || individualUserInfo.getValidateCode() == null
                || individualUserInfo.getValidSessionId() == null
                || individualUserInfo.getPassword() == null
                || individualUserInfo.getTypeId() ==null) {
            if (mRegisterByPhoneListener != null) {
                mRegisterByPhoneListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/individualUser/savePhoneUserApp";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", individualUserInfo.getPhone());
        params.put("validateCode", individualUserInfo.getValidateCode());
        params.put("validSessionId", individualUserInfo.getValidSessionId());
        params.put("password", Utils.md5(individualUserInfo.getPassword()));
        if (individualUserInfo.getNickName() != null){
            params.put("nickName", individualUserInfo.getNickName());
        }
        if (individualUserInfo.getCreateUserId() != null){
            params.put("createUserId", individualUserInfo.getCreateUserId());
        }
        params.put("typeId", individualUserInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, REGISTERBYPHONE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 上传图片
     * @param uploadFileInfo
     * @param uploadFileListener
     * @return 请求ID
     */
    public int uploadPhoto(UploadFileInfo uploadFileInfo, UploadFileListener uploadFileListener) {
        return uploadPhoto(uploadFileInfo, uploadFileListener, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 上传图片
     * @param uploadFileInfo
     * @param uploadFileListener
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int uploadPhoto(UploadFileInfo uploadFileInfo, UploadFileListener uploadFileListener,int timeOutSecs) {
        return uploadPhoto(uploadFileInfo,uploadFileListener, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 上传图片
     * @param uploadFileInfo
     * @param uploadFileListener
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int uploadPhoto(UploadFileInfo uploadFileInfo, UploadFileListener uploadFileListener,int timeOutSecs, int priorityLevel) {
        mUploadFileListener = uploadFileListener;
        if (uploadFileInfo==null||Utils.strIsEmpty(uploadFileInfo.getFilePath())
                || Utils.strIsEmpty(uploadFileInfo.getUserId())
                || Utils.strIsEmpty(uploadFileInfo.getToken())|| Utils.strIsEmpty(uploadFileInfo.getFileName())) {
            if (uploadFileListener != null) {
                uploadFileListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/auth/upload/createUploadPath";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("fileName", uploadFileInfo.getFileName());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                uploadFileInfo.getUserId(),uploadFileInfo.getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPLOAD_PHOTO);
        uploadFileInfoHashMap.put(eventId,uploadFileInfo);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改个人用户认证信息
     * @param authenticationInfo
     * @return 请求ID
     */
    public int updateAuthenticationInfo(AuthenticationInfo authenticationInfo, String token) {
        return updateAuthenticationInfo(authenticationInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户认证信息
     * @param authenticationInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateAuthenticationInfo(AuthenticationInfo authenticationInfo, String token, int timeOutSecs) {
        return updateAuthenticationInfo(authenticationInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户认证信息
     * @param authenticationInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateAuthenticationInfo(AuthenticationInfo authenticationInfo, String token, int timeOutSecs, int priorityLevel) {
        if (authenticationInfo == null
                || authenticationInfo.getAuthState() == null
                || authenticationInfo.getBackPhotoUrl() == null
                || authenticationInfo.getCertificateCode() == null
                || authenticationInfo.getCertificatePhotoUrl() == null
                || authenticationInfo.getCertificateType() == null
                || authenticationInfo.getFrontPhotoUrl() == null
                || authenticationInfo.getUserFullname() == null
                || authenticationInfo.getUserId() == null
                || Utils.strIsEmpty(token)) {
            if (mUpdateAuthenticationInfoListener != null) {
                mUpdateAuthenticationInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/individualUser/updateAuthenticationInfo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", authenticationInfo.getUserId());
        params.put("userFullname", authenticationInfo.getUserFullname());
        params.put("certificateType", authenticationInfo.getCertificateType());
        params.put("certificateCode", authenticationInfo.getCertificateCode());
        params.put("frontPhotoUrl", authenticationInfo.getFrontPhotoUrl());
        params.put("backPhotoUrl", authenticationInfo.getBackPhotoUrl());
        params.put("certificatePhotoUrl", authenticationInfo.getCertificatePhotoUrl());
        params.put("authState", authenticationInfo.getAuthState());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                authenticationInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEAUTHENTICATIONINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改昵称
     * @param userId 用户ID
     * @param nickName 昵称
     * @return 请求ID
     */
    public int updateUserNickName(String userId, String nickName, String token) {
        return updateUserNickName(userId, nickName, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改昵称
     * @param userId 用户ID
     * @param nickName 昵称
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateUserNickName(String userId, String nickName, String token, int timeOutSecs) {
        return updateUserNickName(userId, nickName, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改昵称
     * @param userId 用户ID
     * @param nickName 昵称
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateUserNickName(String userId, String nickName, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)) {
            if (mUpdateUserNickNameListener != null) {
                mUpdateUserNickNameListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/updateUserNickName";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        if (!Utils.strIsEmpty(nickName)){
            params.put("nickName", nickName);
        }
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATE_USER_NICKNAME);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改紧急联系人信息
     * @param eCallContactInfo
     * @return 请求ID
     */
    public int updateUserECallContact(ECallContactInfo eCallContactInfo,  String token) {
        return updateUserECallContact(eCallContactInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改紧急联系人信息
     * @param eCallContactInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateUserECallContact(ECallContactInfo eCallContactInfo, String token, int timeOutSecs) {
        return updateUserECallContact(eCallContactInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改紧急联系人信息
     * @param eCallContactInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateUserECallContact(ECallContactInfo eCallContactInfo, String token, int timeOutSecs, int priorityLevel) {
        if (eCallContactInfo == null
                || eCallContactInfo.getUserId() == null
                || eCallContactInfo.geteCallContactName() == null
                || eCallContactInfo.geteCallContactPhone() == null
                || eCallContactInfo.geteCallContactRelationship() == null
                || Utils.strIsEmpty(token)) {
            if (mUpdateUserECallContactListener != null) {
                mUpdateUserECallContactListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/updateUserECallContact";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", eCallContactInfo.getUserId());
        params.put("eCallContactName", eCallContactInfo.geteCallContactName());
        params.put("eCallContactPhone", eCallContactInfo.geteCallContactPhone());
        params.put("eCallContactRelationship", eCallContactInfo.geteCallContactRelationship());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                eCallContactInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATE_USER_ECALLCONTACT);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 退出登录
     * @param userId
     * @param token
     * @return 请求ID
     */
    public int logout(String userId,  String token) {
        return logout(userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 退出登录
     * @param userId
     * @param token
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int logout(String userId,  String token, int timeOutSecs) {
        return logout(userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 退出登录
     * @param userId
     * @param token
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int logout(String userId, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)) {
            if (mLogoutListener != null) {
                mLogoutListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/appLogout";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("appId", Config.APPID);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, APP_LOGOUT);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 上传头像
     * @param uploadFileInfo
     * @return 请求ID
     */
    public int updateUserAvatar(UploadFileInfo uploadFileInfo) {
        return updateUserAvatar(uploadFileInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 上传头像
     * @param uploadFileInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateUserAvatar(UploadFileInfo uploadFileInfo, int timeOutSecs) {
        return updateUserAvatar(uploadFileInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 上传头像
     * @param uploadFileInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateUserAvatar(final UploadFileInfo uploadFileInfo,final int timeOutSecs, final int priorityLevel) {
        if (uploadFileInfo == null
                || uploadFileInfo.getUserId() == null
                || Utils.strIsEmpty(uploadFileInfo.getToken())|| Utils.strIsEmpty(uploadFileInfo.getFilePath())) {
            if (mUpdateUserAvatarListener != null) {
                mUpdateUserAvatarListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        UploadFileListener uploadFileListener = new UploadFileListener() {
            @Override
            public void onUploadSuccess(String fileUrl) {
                int eventId = RequestDispathUtil.createEventId();
                String url = "/user/userInfo/updateUserAvatar";
                HashMap<String, Object> params = new HashMap<String, Object>();
                params.put("userId", uploadFileInfo.getUserId());
                params.put("avatar", fileUrl);
                String requestData = Utils.makeUrl(url, params);
                Utils.loge(requestData);
                MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                        uploadFileInfo.getUserId(), uploadFileInfo.getToken(), requestData, thisServiceType, serviceDataVer,
                        MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
                PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
                // 添加对应关系
                addCallbackMap(eventId, UPDATE_USER_AVATAR);
                uploadFileUrlHashMap.put(eventId,fileUrl);
                requestMessage.timeOutSeconds = timeOutSecs;
                requestMessage.priority = priorityLevel;
                RequestDispathUtil.submitRequest(requestMessage);

            }

            @Override
            public void onUploadProgress(float progress) {
                if (mUpdateUserAvatarListener != null) {
                    mUpdateUserAvatarListener.onUploadProgress(progress);
                }
            }

            @Override
            public void onTimeOut() {
                if (mUpdateUserAvatarListener != null) {
                    mUpdateUserAvatarListener.onTimeOut();
                }
            }

            @Override
            public void onFail(int errorCode, String errorMsg) {
                if (mUpdateUserAvatarListener != null) {
                    mUpdateUserAvatarListener.onFail(errorCode,errorMsg);
                }
            }
        };
       return uploadPhoto(uploadFileInfo, uploadFileListener,timeOutSecs,priorityLevel);
    }


    /**
     * 刷新token
     * @param userId
     * @param refreshToken 登录刷新票据，token失效后可用于刷新
     * @return 请求ID
     */
    public int refreshToken(String userId, String refreshToken) {
        return refreshToken(userId, refreshToken, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 刷新token
     * @param userId
     * @param refreshToken 登录刷新票据，token失效后可用于刷新
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int refreshToken(String userId, String refreshToken, int timeOutSecs) {
        return refreshToken(userId, refreshToken, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 刷新token
     * @param userId
     * @param refreshToken 登录刷新票据，token失效后可用于刷新
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int refreshToken(String userId, String refreshToken, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(refreshToken)) {
            if (mRefreshTokenListener != null) {
                mRefreshTokenListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/refreshToken";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("appId", Config.APPID);
        params.put("refreshToken", refreshToken);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, REFRESH_TOKEN);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 验证token
     * @param userId
     * @param token 用户登录标识
     * @return 请求ID
     */
    public int verifyToken(String userId, String token) {
        return verifyToken(userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 验证token
     * @param userId
     * @param token 用户登录标识
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int verifyToken(String userId, String token, int timeOutSecs) {
        return verifyToken(userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 验证token
     * @param userId
     * @param token 用户登录标识
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int verifyToken(String userId, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)){
            if (mVerifyTokenListener != null) {
                mVerifyTokenListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/verifyToken";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("appId", Config.APPID);
        params.put("token", token);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, VERIFY_TOKEN);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 修改个人用户手机号
     * @param validNumInfo
     * @param userId 用户ID
     * @return 请求ID
     */
    public int updateIndividualUserPhone(ValidNumInfo validNumInfo, String userId, String token) {
        return updateIndividualUserPhone(validNumInfo, userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户手机号
     * @param validNumInfo
     * @param userId 用户ID
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updateIndividualUserPhone(ValidNumInfo validNumInfo, String userId, String token, int timeOutSecs) {
        return updateIndividualUserPhone(validNumInfo, userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 修改个人用户手机号
     * @param validNumInfo
     * @param userId 用户ID
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updateIndividualUserPhone(ValidNumInfo validNumInfo, String userId, String token, int timeOutSecs, int priorityLevel) {
        if (validNumInfo == null
                || validNumInfo.getPhone() == null
                || validNumInfo.getValidateCode() == null
                || validNumInfo.getValidSessionId() == null
                || validNumInfo.getTypeId() == null
                || Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)) {
            if (mUpdateIndividualUserPhoneListener != null) {
                mUpdateIndividualUserPhoneListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/individualUser/updateAppUserPhone";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", validNumInfo.getPhone());
        params.put("validSessionId", validNumInfo.getValidSessionId());
        params.put("validateCode", validNumInfo.getValidateCode());
        params.put("typeId", validNumInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEINDIVIDUALUSERPHONE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 手机验证码找回密码
     * @param validNumInfo
     * @param password 新密码
     * @return 请求ID
     */
    public int updatePasswordByPhone(ValidNumInfo validNumInfo, String password) {
        return updatePasswordByPhone(validNumInfo, password, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 手机验证码找回密码
     * @param validNumInfo
     * @param password 新密码
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updatePasswordByPhone(ValidNumInfo validNumInfo, String password, int timeOutSecs) {
        return updatePasswordByPhone(validNumInfo, password, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 手机验证码找回密码
     * @param validNumInfo
     * @param password 新密码
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updatePasswordByPhone(ValidNumInfo validNumInfo, String password, int timeOutSecs, int priorityLevel) {
        if (validNumInfo == null
                || validNumInfo.getPhone() == null
                || validNumInfo.getValidateCode() == null
                || validNumInfo.getValidSessionId() == null
                || validNumInfo.getTypeId() == null
                || Utils.strIsEmpty(password)) {
            if (mUpdatePasswordByPhonePhoneListener != null) {
                mUpdatePasswordByPhonePhoneListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/updatePasswordByPhone";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("password", Utils.md5(password));
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", validNumInfo.getPhone());
        params.put("validSessionId", validNumInfo.getValidSessionId());
        params.put("validateCode", validNumInfo.getValidateCode());
        params.put("typeId", validNumInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEPASSWORDBYPHONE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * APP密码登陆
     * @param loginInfo
     * @return 请求ID
     */
    public int loginByAccount(LoginInfo loginInfo) {
        return loginByAccount(loginInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP密码登陆
     * @param loginInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int loginByAccount(LoginInfo loginInfo, int timeOutSecs) {
        return loginByAccount(loginInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP密码登陆
     * @param loginInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int loginByAccount(LoginInfo loginInfo, int timeOutSecs, int priorityLevel) {
        if (loginInfo == null
                || loginInfo.getPhone() == null
                || loginInfo.getPassword() == null
                || loginInfo.getTypeId() == null) {
            if (mLoginListener != null) {
                mLoginListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/appPhonePasswordLogin";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("phone", loginInfo.getPhone());
        params.put("password", Utils.md5(loginInfo.getPassword()));
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("appId", Config.APPID);
        params.put("typeId", loginInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        Utils.loge(requestData);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USERLOGINBYACCOUNT);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * APP验证码登陆
     * @param validNumInfo
     * @return 请求ID
     */
    public int loginByValidNum(ValidNumInfo validNumInfo){
        return loginByValidNum(validNumInfo, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP验证码登陆
     * @param validNumInfo
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int loginByValidNum(ValidNumInfo validNumInfo, int timeOutSecs){
        return loginByValidNum(validNumInfo, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * APP验证码登陆
     * @param validNumInfo
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int loginByValidNum(ValidNumInfo validNumInfo, int timeOutSecs, int priorityLevel){
        if (validNumInfo == null
                || validNumInfo.getPhone() == null
                || validNumInfo.getValidateCode() == null
                || validNumInfo.getValidSessionId() == null
                || validNumInfo.getTypeId() == null) {
            if (mLoginListener != null) {
                mLoginListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/appPhoneSmsLogin";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("appId", Config.APPID);
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("phone", validNumInfo.getPhone());
        params.put("validSessionId", validNumInfo.getValidSessionId());
        params.put("validateCode", validNumInfo.getValidateCode());
        params.put("typeId", validNumInfo.getTypeId());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, LOGIN_BY_VALID_NUM);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询个人用户信息
     * @param userId
     * @return 请求ID
     */
    public int getUserInfo(String userId, String token){
        return getUserInfo(userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询个人用户信息
     * @param userId
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int getUserInfo(String userId, String token, int timeOutSecs){
        return getUserInfo(userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询个人用户信息
     * @param userId
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int getUserInfo(String userId, String token, int timeOutSecs, int priorityLevel){
        if (Utils.strIsEmpty(userId) || Utils.strIsEmpty(token)) {
            if (mGetUserInfoListener != null) {
                mGetUserInfoListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/getUserInfo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GETUSERINFO);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }


    /**
     * 扫码登陆
     * @param scanCodeLoginInfo
     * @param userId
     * @return 请求ID
     */
    public int authorityLoginByCode(ScanCodeLoginInfo scanCodeLoginInfo, String userId){
        return authorityLoginByCode(scanCodeLoginInfo, userId, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆
     * @param scanCodeLoginInfo
     * @param userId
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int authorityLoginByCode(ScanCodeLoginInfo scanCodeLoginInfo, String userId, int timeOutSecs){
        return authorityLoginByCode(scanCodeLoginInfo, userId, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆
     * @param scanCodeLoginInfo
     * @param userId
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int authorityLoginByCode(ScanCodeLoginInfo scanCodeLoginInfo, String userId, int timeOutSecs, int priorityLevel){
        if (scanCodeLoginInfo == null
                || scanCodeLoginInfo.getAppId() == null
                || scanCodeLoginInfo.getProductId() == null
                || scanCodeLoginInfo.getQrLoginCode() == null
                || Utils.strIsEmpty(userId)) {
            if (mAuthorityLoginByCodeListener != null) {
                mAuthorityLoginByCodeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/scanCodeLogin";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("appId", scanCodeLoginInfo.getAppId());
        params.put("productId", scanCodeLoginInfo.getProductId());
        params.put("uniqueId", scanCodeLoginInfo.getQrLoginCode());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USERAUTHORITYLOGINBYCODE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 扫码登陆轮询
     * @return 请求ID
     */
    public int loginByQrCode(String loginCode,LoginByQrcodeListener loginByQrcodeListener){
        return loginByQrCode(loginCode,loginByQrcodeListener,Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆轮询
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int loginByQrCode(String loginCode,LoginByQrcodeListener loginByQrcodeListener,int timeOutSecs){
        return loginByQrCode(loginCode,loginByQrcodeListener,timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆轮询
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int loginByQrCode(String loginCode,LoginByQrcodeListener loginByQrcodeListener,int timeOutSecs, int priorityLevel) {
    	  mLoginByQrcodeListener = loginByQrcodeListener;
    	if (Utils.strIsEmpty(loginCode)) {
            if (mLoginByQrcodeListener != null) {
            	mLoginByQrcodeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }

        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/login/getScanCodeLoginInfo";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("appId", Config.APPID);
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("uniqueId", loginCode);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USERLOGINBYQRCODE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 使用旧密码修改密码
     * @param updatePwdInfo
     * @param token
     * @return 请求ID
     */
    public int updatePwd(UpdatePwdInfo updatePwdInfo, String token){
        return updatePwd(updatePwdInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 使用旧密码修改密码
     * @param updatePwdInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updatePwd(UpdatePwdInfo updatePwdInfo, String token, int timeOutSecs){
        return updatePwd(updatePwdInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 使用旧密码修改密码
     * @param updatePwdInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updatePwd(UpdatePwdInfo updatePwdInfo, String token, int timeOutSecs, int priorityLevel) {
        if (updatePwdInfo == null
                || updatePwdInfo.getNewPassword() == null
                || updatePwdInfo.getOldPassword() == null
                || updatePwdInfo.getUserId() == null
                || Utils.strIsEmpty(token)) {
            if (mUpdatePwdListener != null) {
                mUpdatePwdListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/updatePassword";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", updatePwdInfo.getUserId());
        params.put("oldPassword", Utils.md5(updatePwdInfo.getOldPassword()));
        params.put("newPassword", Utils.md5(updatePwdInfo.getNewPassword()));
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                updatePwdInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USERUPDATAPWD);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * PIN码首次设置
     * @param pinPassword PIN密码
     * @param userId 用户ID
     * @param token
     * @return 请求ID
     */
    public int setUpPinPassword(String pinPassword, String userId, String token){
        return setUpPinPassword(pinPassword, userId, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * PIN码首次设置
     * @param pinPassword PIN密码
     * @param userId 用户ID
     * @param token
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int setUpPinPassword(String pinPassword, String userId, String token, int timeOutSecs){
        return setUpPinPassword(pinPassword, userId, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * PIN码首次设置
     * @param pinPassword PIN密码
     * @param userId 用户ID
     * @param token
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int setUpPinPassword(String pinPassword, String userId, String token, int timeOutSecs, int priorityLevel) {
        if (Utils.strIsEmpty(pinPassword)
                || Utils.strIsEmpty(userId)
                || Utils.strIsEmpty(token)) {
            if (mSetUpPinPasswordListener != null) {
                mSetUpPinPasswordListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/setUpPinPassword";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("pinPassword", Utils.md5(pinPassword));
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                userId, token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SETUPPINPASSWORD);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 旧PIN码修改新PIN码修改
     * @param updatePinPasswordInfo
     * @param token
     * @return 请求ID
     */
    public int updatePinPassword(UpdatePinPasswordInfo updatePinPasswordInfo, String token){
        return updatePinPassword(updatePinPasswordInfo, token, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 旧PIN码修改新PIN码修改
     * @param updatePinPasswordInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int updatePinPassword(UpdatePinPasswordInfo updatePinPasswordInfo, String token, int timeOutSecs){
        return updatePinPassword(updatePinPasswordInfo, token, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 旧PIN码修改新PIN码修改
     * @param updatePinPasswordInfo
     * @param token
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int updatePinPassword(UpdatePinPasswordInfo updatePinPasswordInfo, String token, int timeOutSecs, int priorityLevel) {
        if (updatePinPasswordInfo ==null
                || updatePinPasswordInfo.getNewPinPassword() ==null
                || updatePinPasswordInfo.getOldPinPassword() ==null
                | updatePinPasswordInfo.getNewPinPassword() ==null
                || Utils.strIsEmpty(token)) {
            if (mUpdatePinPasswordListener != null) {
                mUpdatePinPasswordListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/user/userInfo/updatePinPassword";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", updatePinPasswordInfo.getUserId());
        params.put("oldPinPassword", Utils.md5(updatePinPasswordInfo.getOldPinPassword()));
        params.put("newPinPassword", Utils.md5(updatePinPasswordInfo.getNewPinPassword()));
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                updatePinPasswordInfo.getUserId(), token, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, UPDATEPINPASSWORD);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 获取二维码登录生成二维码所需code
     * @return
     */
    public int getQrLoginCode(){
        return getQrLoginCode(Config.DEFAULT_TIMEOUT_SECS,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取二维码登录生成二维码所需code
     * @param timeOutSecs
     * @return
     */
    public int getQrLoginCode(int timeOutSecs){
        return getQrLoginCode(timeOutSecs,Config.DEFAULT_PRIORITY);
    }

    /**
     * 获取二维码登录生成二维码所需code
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getQrLoginCode(int timeOutSecs, int priorityLevel){

        String url = "/user/login/getLoginCode";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", Config.CUSTOMER_KEY);
        params.put("deviceNo", Config.VIN);
        String requestData = Utils.makeUrl(url, params);
        int eventId = RequestDispathUtil.createEventId();
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgBeforeLoginByPhone(eventId, requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_QRLOGIN_CODE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 查询集团信息
     * @param listGroup
     * @return 请求ID
     */
    public int getListGroup(ListGroup listGroup){
        return getListGroup(listGroup, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询集团信息
     * @param listGroup
     * @param timeOutSecs 超时时间
     * @return 请求ID
     */
    public int getListGroup(ListGroup listGroup, int timeOutSecs){
        return getListGroup(listGroup, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 查询集团信息
     * @param listGroup
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return 请求ID
     */
    public int getListGroup(ListGroup listGroup, int timeOutSecs, int priorityLevel){
        if (listGroup == null
                || Utils.strIsEmpty(listGroup.getUserType())
                || Utils.strIsEmpty(listGroup.getUserId())
                || Utils.strIsEmpty(listGroup.getToken())) {
            if (mGetListGroupListener != null) {
                mGetListGroupListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "/auth/group/listGroup";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("productId", listGroup.getProductId());
        params.put("groupId", listGroup.getGroupId());
        params.put("groupName", listGroup.getGroupName());
        params.put("contactName", listGroup.getContactName());
        params.put("contactPhone", listGroup.getContactPhone());
        params.put("userType", listGroup.getUserType());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsgByPhone(eventId,
                listGroup.getUserId(), listGroup.getToken(), requestData, thisServiceType, serviceDataVer,
                MessageBuilder.ENCODINGTYPE_PLAIN_TEXT, NON_SUB_SERVICE_TYPE);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, GET_LIST_GROUP);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 扫码登陆（新）
     * @param userLoginQuery
     * @return
     */
    public int getUserLoginByCode(UserLoginQuery userLoginQuery){
        return getUserLoginByCode(userLoginQuery, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆（新）
     * @param userLoginQuery
     * @param timeOutSecs
     * @return
     */
    public int getUserLoginByCode(UserLoginQuery userLoginQuery, int timeOutSecs){
        return getUserLoginByCode(userLoginQuery, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 扫码登陆（新）
     * @param userLoginQuery
     * @param timeOutSecs
     * @param priorityLevel
     * @return
     */
    public int getUserLoginByCode(UserLoginQuery userLoginQuery, int timeOutSecs, int priorityLevel){
        if (userLoginQuery == null || userLoginQuery.getUserType() == null || userLoginQuery.getUserId() == null
                || userLoginQuery.getAppId() == null || userLoginQuery.getProductId() == null
                || userLoginQuery.getDeviceNo() == null || userLoginQuery.getCode() == null ){
            if (mOnGetUserLoginCodeListener != null ){
                mOnGetUserLoginCodeListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "userVehicle/user/scanCodeLogin";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userType",userLoginQuery.getUserType());
        params.put("userId",userLoginQuery.getUserId());
        params.put("appId",userLoginQuery.getAppId());
        params.put("productId",userLoginQuery.getProductId());
        params.put("deviceNo",userLoginQuery.getDeviceNo());
        params.put("code",userLoginQuery.getCode());
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId,
                requestData, monitorServiceType, monitorServiceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, USER_LOGIN_BY_CODE);
        RequestDispathUtil.submitRequest(requestMessage);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        return eventId;
    }

    /**
     * 根据用户ID查询绑定的集团信息（app使用）
     * @param userId 用户ID
     * @param userType 登陆用户的用户类型
     * @return
     */
    public int searchGroupInfoById(String userId, String userType) {
        return searchGroupInfoById(userId, userType, Config.DEFAULT_TIMEOUT_SECS, Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据用户ID查询绑定的集团信息（app使用）
     * @param userId 用户ID
     * @param userType 登陆用户的用户类型
     * @param timeOutSecs 超时时间
     * @return
     */
    public int searchGroupInfoById(String userId, String userType, int timeOutSecs) {
        return searchGroupInfoById(userId, userType, timeOutSecs, Config.DEFAULT_PRIORITY);
    }

    /**
     * 根据用户ID查询绑定的集团信息（app使用）
     * @param userId 用户ID
     * @param userType 登陆用户的用户类型
     * @param timeOutSecs 超时时间
     * @param priorityLevel 优先级
     * @return
     */
    public int searchGroupInfoById(String userId, String userType, int timeOutSecs, int priorityLevel) {
        if (userId == null || userType == null ){
            if (mOnSearchGroupInfoByIdListener != null ){
                mOnSearchGroupInfoByIdListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT, Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
            }
            return 0;
        }
        int eventId = RequestDispathUtil.createEventId();
        String url = "auth/group/listUserGroup";
        HashMap<String, Object> params = new HashMap<String, Object>();
        params.put("userId", userId);
        params.put("userType", userType);
        String requestData = Utils.makeUrl(url, params);
        MxNGTPMessage.MxMessage businessMessage = MessageBuilder.instance().makeRequestBusinessMsg(eventId, requestData,
                thisServiceType, serviceDataVer);
        PrioritizedRequestMessage requestMessage = new PrioritizedRequestMessage(businessMessage);
        // 添加对应关系
        addCallbackMap(eventId, SEARCH_GROUPINFO_BY_ID);
        requestMessage.timeOutSeconds = timeOutSecs;
        requestMessage.priority = priorityLevel;
        RequestDispathUtil.submitRequest(requestMessage);
        return eventId;
    }

    public static interface GetQrLoginCodeListener extends CallBackListener {
        public void onGetQrLoginCode(String code, String expireTime);
    }
    public interface SendPhoneValidNumSuccessListener extends CallBackListener {
        public void onSendPhoneValidNumSuccess(String validSessionId);
    }
    public interface VerifyValidNumListener extends CallBackListener {
        public void onVerifyValidNum();
    }
    public interface RegisterByPhoneListener extends CallBackListener {
        public void onRegisterByPhone();
    }
//    public interface UploadPhotoListener extends CallBackListener {
//        public void onUploadPhoto(String url);
//        public void onUploadPhotoProgress(float percent);
//    }
    public interface UpdateAuthenticationInfoListener extends CallBackListener {
        public void onUpdateAuthenticationInfo();
    }
    public interface UpdateUserNickNameListener extends CallBackListener {
        public void onUpdateUserNickName();
    }
    public interface UpdateUserECallContactListener extends CallBackListener {
        public void onUpdateUserECallContact();
    }
    public interface LogoutListener extends CallBackListener {
        public void onLogout();
    }

    public interface RefreshTokenListener extends CallBackListener {
        public void onRefreshToken(LoginDataInfo loginInfo);
    }
    public interface VerifyTokenListener extends CallBackListener {
        public void onVerifyToken();
    }
    public interface UpdateIndividualUserPhoneListener extends CallBackListener {
        public void onUpdateIndividualUserPhone();
    }
    public interface UpdatePasswordByPhonePhoneListener extends CallBackListener {
        public void onUpdatePasswordByPhone();
    }
    public interface LoginListener extends CallBackListener {
        public void onLoginSuccess(LoginData loginData);
    }
    public interface LoginByQrcodeListener extends CallBackListener {
        /**
         * 登录成功
         * @param loginData
         */
        public void onLoginSuccess(LoginData loginData);

        /**
         * 未登录成功
         * @param errorCode  -100: 失败，未知错误
                            -102:缺失参数
                            -108:用户不存在
                            -131:未扫码登陆
                            -132: 登陆code超时
         * @param errorMsg
         */
        public void onFail(int errorCode, String errorMsg);
    }
    public interface GetUserInfoListener extends CallBackListener {
        public void onGetUserInfo(UserInfoResult userInfoResult);
    }

    public interface AuthorityLoginByCodeListener extends CallBackListener {
        public void onAuthorityLoginByCode();

        /**
         * 授权登录失败
         * @param errorCode -100: 失败，未知错误
                            -102: 缺失参数
                            -108: 用户不存在
                            -110: 无法登陆该应用
                            -132: 登陆code超时
         *
         * @param errorMsg
         */
        public void onFail(int errorCode, String errorMsg);
    }
    public interface UpdatePwdListener extends CallBackListener {
        public void onUpdatePwdSuccess();
    }
    public interface SetUpPinPasswordListener extends CallBackListener {
        public void onSetUpPinPassword();
    }
    public interface UpdatePinPasswordListener extends CallBackListener {
        public void onUpdatePinPassword();
    }
    public interface UploadFileListener extends CallBackListener {
        public void onUploadSuccess(String fileUrl);
        public void onUploadProgress(float progress);
    }
    public interface UpdateUserAvatarListener extends UploadFileListener {
    }

    public interface GetListGroupListener extends CallBackListener {
        public void onGetListGroup(List<ListGroupInfo> listGroupInfo);
    }

    public interface OnGetUserLoginCodeListener extends CallBackListener{
        public void onGetUserLoginCode();
    }

    public interface OnSearchGroupInfoByIdListener extends CallBackListener {
        public void onSearchGroupInfoById(List<ListGroupInfo> listGroupInfo);
    }

    SendPhoneValidNumSuccessListener sendPhoneValidNumSuccessListener;//获取验证码
    VerifyValidNumListener mVerifyValidNumListener;//手机验证码验证
    RegisterByPhoneListener mRegisterByPhoneListener;//添加个人用户
//    UploadPhotoListener mUploadPhotoListener;//上传图片
    UpdateAuthenticationInfoListener mUpdateAuthenticationInfoListener;//修改个人用户认证信息
    UpdateUserNickNameListener mUpdateUserNickNameListener;//修改昵称
    UpdateUserECallContactListener mUpdateUserECallContactListener;//修改紧急联系人信息
    LogoutListener mLogoutListener;//APP退出登录
    UpdateUserAvatarListener mUpdateUserAvatarListener;//上传头像
    RefreshTokenListener mRefreshTokenListener;//刷新token
    VerifyTokenListener mVerifyTokenListener;//验证token
    UpdateIndividualUserPhoneListener mUpdateIndividualUserPhoneListener;//修改个人用户手机号
    UpdatePasswordByPhonePhoneListener mUpdatePasswordByPhonePhoneListener;//手机验证码找回密码
    LoginListener mLoginListener;//APP密码登陆
    GetUserInfoListener mGetUserInfoListener;//查询个人用户信息
    AuthorityLoginByCodeListener mAuthorityLoginByCodeListener;//扫码登陆
    UpdatePwdListener mUpdatePwdListener; //使用旧密码修改密码
    SetUpPinPasswordListener mSetUpPinPasswordListener;//PIN码首次设置
    UpdatePinPasswordListener mUpdatePinPasswordListener;//旧PIN码修改新PIN码修改
    UploadFileListener mUploadFileListener;//上传文件
    GetQrLoginCodeListener mGetQrLoginCodeListener;
    LoginByQrcodeListener mLoginByQrcodeListener;// 二维码登录轮询回调
    GetListGroupListener mGetListGroupListener; //查询集团信息回调
    OnGetUserLoginCodeListener mOnGetUserLoginCodeListener; // 扫码登陆(新)
    OnSearchGroupInfoByIdListener mOnSearchGroupInfoByIdListener; // 根据用户ID查询绑定的集团信息（app使用）

    public void setSendPhoneValidNumSuccessListener(SendPhoneValidNumSuccessListener sendPhoneValidNumSuccessListener) {
        this.sendPhoneValidNumSuccessListener = sendPhoneValidNumSuccessListener;
    }
    public void setVerifyValidNumListener(VerifyValidNumListener verifyValidNumListener) {
        mVerifyValidNumListener = verifyValidNumListener;
    }
    public void setRegisterByPhoneListener(RegisterByPhoneListener registerByPhoneListener) {
        mRegisterByPhoneListener = registerByPhoneListener;
    }
//    public void setUploadPhotoListener(UploadPhotoListener uploadPhotoListener) {
//        mUploadPhotoListener = uploadPhotoListener;
//    }
    public void setUpdateAuthenticationInfoListener(UpdateAuthenticationInfoListener updateAuthenticationInfoListener) {
        mUpdateAuthenticationInfoListener = updateAuthenticationInfoListener;
    }
    public void setUpdateUserNickNameListener(UpdateUserNickNameListener updateUserNickNameListener) {
        mUpdateUserNickNameListener = updateUserNickNameListener;
    }
    public void setUpdateUserECallContactListener(UpdateUserECallContactListener updateUserNickNameListener) {
        mUpdateUserECallContactListener = updateUserNickNameListener;
    }
    public void setLogoutListener(LogoutListener appLogoutListener) {
        mLogoutListener = appLogoutListener;
    }
    public void setUpdateUserAvatarListener(UpdateUserAvatarListener updateUserAvatarListener) {
        mUpdateUserAvatarListener = updateUserAvatarListener;
    }
    public void setRefreshTokenListener(RefreshTokenListener refreshTokenListener) {
        mRefreshTokenListener = refreshTokenListener;
    }
    public void setVerifyTokenListener(VerifyTokenListener verifyTokenListener) {
        mVerifyTokenListener = verifyTokenListener;
    }
    public void setUpdateIndividualUserPhoneListener(UpdateIndividualUserPhoneListener updateIndividualUserPhoneListener) {
        mUpdateIndividualUserPhoneListener = updateIndividualUserPhoneListener;
    }
    public void setUpdatePasswordByPhonePhoneListener(UpdatePasswordByPhonePhoneListener updatePasswordByPhonePhoneListener) {
        mUpdatePasswordByPhonePhoneListener = updatePasswordByPhonePhoneListener;
    }
    public void setLoginListener(LoginListener loginListener) {
        mLoginListener = loginListener;
    }
    public void setGetUserInfoListener(GetUserInfoListener getUserInfoListener) {
        mGetUserInfoListener = getUserInfoListener;
    }

    public void setAuthorityLoginByCodeListener(AuthorityLoginByCodeListener authorityLoginByCodeListener) {
        mAuthorityLoginByCodeListener = authorityLoginByCodeListener;
    }
    public void setUpdatePwdListener(UpdatePwdListener updatePwdListener) {
        mUpdatePwdListener = updatePwdListener;
    }
    public void setPinPasswordListener(SetUpPinPasswordListener setUpPinPasswordListener) {
        mSetUpPinPasswordListener = setUpPinPasswordListener;
    }
    public void setUpdatePinPasswordListener(UpdatePinPasswordListener updatePinPasswordListener) {
        mUpdatePinPasswordListener = updatePinPasswordListener;
    }
    public void setGetQrLoginCodeListener(GetQrLoginCodeListener getQrLoginCodeListener) {
        mGetQrLoginCodeListener = getQrLoginCodeListener;
    }

    public void setGetListGroupListener(GetListGroupListener mGetListGroupListener) {
        this.mGetListGroupListener = mGetListGroupListener;
    }

    public void setOnGetUserLoginCodeListener(OnGetUserLoginCodeListener mOnGetUserLoginCodeListener){
        this.mOnGetUserLoginCodeListener = mOnGetUserLoginCodeListener;
    }

    public void setOnSearchGroupInfoByIdListener(OnSearchGroupInfoByIdListener mOnSearchGroupInfoByIdListener) {
        this.mOnSearchGroupInfoByIdListener = mOnSearchGroupInfoByIdListener;
    }

    private void addCallbackMap(int eventId, int type) {
        super.addCallbackMap(eventId, UserAuth.this);

        synchronized (listenertypeHashMap) {
            listenertypeHashMap.put(eventId, type);
        }
    }

    @Override
    protected void removeCallbackMap(int eventId) {
        super.removeCallbackMap(eventId);
        synchronized (listenertypeHashMap) {
            listenertypeHashMap.remove(eventId);
            uploadFileInfoHashMap.remove(eventId);
            uploadFileUrlHashMap.remove(eventId);
        }
    }

    @Override
    protected void callTimeOut(int requestHandleId) {
        int type = listenertypeHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case USERSENDPHONEVALIDNUM://1获取验证码
                if (sendPhoneValidNumSuccessListener != null) {
                    sendPhoneValidNumSuccessListener.onTimeOut();
                }
                break;
            case VERIFY_PHONE_VALID_NUM:
                if (mVerifyValidNumListener != null) {
                    mVerifyValidNumListener.onTimeOut();
                }
                break;
            case REGISTERBYPHONE:
                if (mRegisterByPhoneListener != null) {
                    mRegisterByPhoneListener.onTimeOut();
                }
                break;
            case UPLOAD_PHOTO:
                if (mUploadFileListener != null) {
                    mUploadFileListener.onTimeOut();
                }
                break;
            case UPDATEAUTHENTICATIONINFO:
                if (mUpdateAuthenticationInfoListener != null) {
                    mUpdateAuthenticationInfoListener.onTimeOut();
                }
                break;
            case UPDATE_USER_NICKNAME:
                if (mUpdateUserNickNameListener != null) {
                    mUpdateUserNickNameListener.onTimeOut();
                }
                break;
            case UPDATE_USER_ECALLCONTACT:
                if (mUpdateUserECallContactListener != null) {
                    mUpdateUserECallContactListener.onTimeOut();
                }
                break;
            case APP_LOGOUT:
                if (mLogoutListener != null) {
                    mLogoutListener.onTimeOut();
                }
                break;
            case UPDATE_USER_AVATAR:
                if (mUpdateUserAvatarListener != null) {
                    mUpdateUserAvatarListener.onTimeOut();
                }
                break;
            case REFRESH_TOKEN:
                if (mRefreshTokenListener != null) {
                    mRefreshTokenListener.onTimeOut();
                }
                break;
            case VERIFY_TOKEN:
                if (mVerifyTokenListener != null) {
                    mVerifyTokenListener.onTimeOut();
                }
                break;
            case UPDATEINDIVIDUALUSERPHONE:
                if (mUpdateIndividualUserPhoneListener != null) {
                    mUpdateIndividualUserPhoneListener.onTimeOut();
                }
                break;
            case UPDATEPASSWORDBYPHONE:
                if (mUpdatePasswordByPhonePhoneListener != null) {
                    mUpdatePasswordByPhonePhoneListener.onTimeOut();
                }
                break;
            case USERLOGINBYACCOUNT:
                if (mLoginListener != null) {
                    mLoginListener.onTimeOut();
                }
                break;
            case LOGIN_BY_VALID_NUM:
                if (mLoginListener != null) {
                    mLoginListener.onTimeOut();
                }
                break;
            case GETUSERINFO:
                if (mGetUserInfoListener != null) {
                    mGetUserInfoListener.onTimeOut();
                }
                break;

            case USERAUTHORITYLOGINBYCODE:
                if (mAuthorityLoginByCodeListener != null) {
                    mAuthorityLoginByCodeListener.onTimeOut();
                }
                break;
            case USERLOGINBYQRCODE:
                if (mLoginByQrcodeListener != null) {
                    mLoginByQrcodeListener.onTimeOut();
                }
                break;
            case USERUPDATAPWD:
                if (mUpdatePwdListener != null) {
                    mUpdatePwdListener.onTimeOut();
                }
                break;
            case SETUPPINPASSWORD:
                if (mSetUpPinPasswordListener != null) {
                    mSetUpPinPasswordListener.onTimeOut();
                }
                break;
            case UPDATEPINPASSWORD:
                if (mUpdatePinPasswordListener != null) {
                    mUpdatePinPasswordListener.onTimeOut();
                }
                break;
            case GET_QRLOGIN_CODE:
                if (mGetQrLoginCodeListener != null) {
                    mGetQrLoginCodeListener.onTimeOut();
                }
                break;
            case GET_LIST_GROUP:
                if (mGetListGroupListener != null) {
                    mGetListGroupListener.onTimeOut();
                }
                break;
            case USER_LOGIN_BY_CODE:
                if (mOnGetUserLoginCodeListener != null ){
                    mOnGetUserLoginCodeListener.onTimeOut();
                }
                break;
            case SEARCH_GROUPINFO_BY_ID:
                if (mOnSearchGroupInfoByIdListener != null ){
                    mOnSearchGroupInfoByIdListener.onTimeOut();
                }
                break;
            default:
                break;
        }
    }

    @Override
    protected void onCallBack(int requestHandleId, String response, boolean hasDSPTError, int errorCode, String errorMsg) {
        Utils.loge("response=" + response);
        int type = listenertypeHashMap.get(requestHandleId);
        UploadFileInfo uploadFileInfo = uploadFileInfoHashMap.get(requestHandleId);
        String url = uploadFileUrlHashMap.get(requestHandleId);
        removeCallbackMap(requestHandleId);
        switch (type) {
            case USERSENDPHONEVALIDNUM://1获取验证码
                if (sendPhoneValidNumSuccessListener != null) {
                    if (hasDSPTError) {
                        sendPhoneValidNumSuccessListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            JSONObject dataObject = jsonObject.getJSONObject("data");
                            String validSessionId = dataObject.getString("validSessionId");
                            sendPhoneValidNumSuccessListener.onSendPhoneValidNumSuccess(validSessionId);
                        } else {
                            sendPhoneValidNumSuccessListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        sendPhoneValidNumSuccessListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case VERIFY_PHONE_VALID_NUM:
                if (mVerifyValidNumListener != null) {
                    if (hasDSPTError) {
                        mVerifyValidNumListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mVerifyValidNumListener.onVerifyValidNum();
                        } else {
                            mVerifyValidNumListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mVerifyValidNumListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case REGISTERBYPHONE:
                if (mRegisterByPhoneListener != null) {
                    if (hasDSPTError) {
                        mRegisterByPhoneListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mRegisterByPhoneListener.onRegisterByPhone();
                        } else {
                            mRegisterByPhoneListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mRegisterByPhoneListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPLOAD_PHOTO:
                if (mUploadFileListener != null) {
                    if (hasDSPTError) {
                        mUploadFileListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            if (jsonObject.has("data")) {
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                String uploadUrl = null;
                                if (dataObject.has("uploadUrl")) {
                                    uploadUrl = dataObject.getString("uploadUrl");
                                }
//                                if (dataObject.has("filename")) {
//                                    uploadPathInfo.setFilename(dataObject.getString("filename"));
//                                }
                                String domainPath = "";
                                if (dataObject.has("domainPath")){
                                	domainPath = dataObject.getString("domainPath");
                                }
                                List<Param> list = new ArrayList<Param>();
                                if (dataObject.has("params")) {
                                    JSONObject paramsObject = dataObject.getJSONObject("params");


                                    Iterator<String> it = paramsObject.keys();
                                    Param param;
                                    while (it.hasNext()){
                                        String key = it.next();
                                        String value = paramsObject.getString(key);
                                        param = new Param();
                                        param.setName(key);
                                        param.setValue(value);
                                        list.add(param);
                                    }
                                }
                                uploadFile(uploadUrl,domainPath,list,uploadFileInfo.getFilePath(),mUploadFileListener);
                            }else{
                                mUploadFileListener.onFail(Constants
                                                .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                        , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                                );

                            }
                        } else {
                            mUploadFileListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUploadFileListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }


                }
                break;
            case UPDATEAUTHENTICATIONINFO:
                if (mUpdateAuthenticationInfoListener != null) {
                    if (hasDSPTError) {
                        mUpdateAuthenticationInfoListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateAuthenticationInfoListener.onUpdateAuthenticationInfo();
                        } else {
                            mUpdateAuthenticationInfoListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdateAuthenticationInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATE_USER_NICKNAME:
                if (mUpdateUserNickNameListener != null) {
                    if (hasDSPTError) {
                        mUpdateUserNickNameListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateUserNickNameListener.onUpdateUserNickName();
                        } else {
                            mUpdateUserNickNameListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdateUserNickNameListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATE_USER_ECALLCONTACT:
                if (mUpdateUserECallContactListener != null) {
                    if (hasDSPTError) {
                        mUpdateUserECallContactListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateUserECallContactListener.onUpdateUserECallContact();
                        } else {
                            mUpdateUserECallContactListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdateUserECallContactListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case APP_LOGOUT:
                if (mLogoutListener != null) {
                    if (hasDSPTError) {
                        mLogoutListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mLogoutListener.onLogout();
                        } else {
                            mLogoutListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mLogoutListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATE_USER_AVATAR:
                if (mUpdateUserAvatarListener != null) {
                    if (hasDSPTError) {
                        mUpdateUserAvatarListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {

                            mUpdateUserAvatarListener.onUploadSuccess(url);
                        } else {
                            mUpdateUserAvatarListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdateUserAvatarListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case REFRESH_TOKEN:
                if (mRefreshTokenListener != null) {
                    if (hasDSPTError) {
                        mRefreshTokenListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            LoginDataInfo loginDataInfo = new LoginDataInfo();
                            if (jsonObject.has("data")) {
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                if (dataObject.has("token")) {
                                    loginDataInfo.setToken(dataObject.getString("token"));
                                }
                                if (dataObject.has("refreshToken")) {
                                    loginDataInfo.setRefreshToken(dataObject.getString("refreshToken"));
                                }
                            }
                            mRefreshTokenListener.onRefreshToken(loginDataInfo);
                        } else {
                            mRefreshTokenListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mRefreshTokenListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case VERIFY_TOKEN:
                if (mVerifyTokenListener != null) {
                    if (hasDSPTError) {
                        mVerifyTokenListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mVerifyTokenListener.onVerifyToken();
                        } else {
                            mVerifyTokenListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mVerifyTokenListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATEINDIVIDUALUSERPHONE:
                if (mUpdateIndividualUserPhoneListener != null) {
                    if (hasDSPTError) {
                        mUpdateIndividualUserPhoneListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdateIndividualUserPhoneListener.onUpdateIndividualUserPhone();
                        } else {
                            mUpdateIndividualUserPhoneListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdateIndividualUserPhoneListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case UPDATEPASSWORDBYPHONE:
                if (mUpdatePasswordByPhonePhoneListener != null) {
                    if (hasDSPTError) {
                        mUpdatePasswordByPhonePhoneListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdatePasswordByPhonePhoneListener.onUpdatePasswordByPhone();
                        } else {
                            mUpdatePasswordByPhonePhoneListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdatePasswordByPhonePhoneListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case USERLOGINBYQRCODE:
                if (mLoginByQrcodeListener != null) {
                    if (hasDSPTError) {
                        mLoginByQrcodeListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            LoginData loginData = decodeLoginData(jsonObject);
                            mLoginByQrcodeListener.onLoginSuccess(loginData);
                        } else {
                            mLoginByQrcodeListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mLoginByQrcodeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case LOGIN_BY_VALID_NUM:
            case USERLOGINBYACCOUNT:
                if (mLoginListener != null) {
                    if (hasDSPTError) {
                        mLoginListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            LoginData loginData = decodeLoginData(jsonObject);
                            mLoginListener.onLoginSuccess(loginData);
                        } else {
                            mLoginListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mLoginListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case GETUSERINFO:
                if (mGetUserInfoListener != null) {
                    if (hasDSPTError) {
                        mGetUserInfoListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            UserInfoResult userInfoResult = new UserInfoResult();
                            if (jsonObject.has("data")) {
                                JSONObject dataObject = jsonObject.getJSONObject("data");
                                if (dataObject.has("userId")) {
                                    userInfoResult.setUserId(dataObject.getString("userId"));
                                }
                                if (dataObject.has("authState")) {
                                    userInfoResult.setAuthState(dataObject.getString("authState"));
                                }
                                if (dataObject.has("backPhotoUrl")) {
                                    userInfoResult.setBackPhotoUrl(dataObject.getString("backPhotoUrl"));
                                }
                                if (dataObject.has("certificateCode")) {
                                    userInfoResult.setCertificateCode(dataObject.getString("certificateCode"));
                                }
                                if (dataObject.has("certificatePhotoUrl")) {
                                    userInfoResult.setCertificatePhotoUrl(dataObject.getString("certificatePhotoUrl"));
                                }
                                if (dataObject.has("certificateType")) {
                                    userInfoResult.setCertificateType(dataObject.getString("certificateType"));
                                }
                                if (dataObject.has("frontPhotoUrl")) {
                                    userInfoResult.setFrontPhotoUrl(dataObject.getString("frontPhotoUrl"));
                                }
                                if (dataObject.has("userFullname")) {
                                    userInfoResult.setUserFullname(dataObject.getString("userFullname"));
                                }
                                if (dataObject.has("userName")) {
                                    userInfoResult.setUserName(dataObject.getString("userName"));
                                }
                                if (dataObject.has("productId")) {
                                    userInfoResult.setProductId(dataObject.getString("productId"));
                                }
                                if (dataObject.has("typeId")) {
                                    userInfoResult.setTypeId(dataObject.getString("typeId"));
                                }
                                if (dataObject.has("groupId")) {
                                    userInfoResult.setGroupId(dataObject.getString("groupId"));
                                }
                                if (dataObject.has("fleetId")) {
                                    userInfoResult.setFleetId(dataObject.getString("fleetId"));
                                }
                                if (dataObject.has("nickName")) {
                                    userInfoResult.setNickName(dataObject.getString("nickName"));
                                }
                                if (dataObject.has("avatar")) {
                                    userInfoResult.setAvatar(dataObject.getString("avatar"));
                                }
                                if (dataObject.has("phone")) {
                                    userInfoResult.setPhone(dataObject.getString("phone"));
                                }
                                if (dataObject.has("mail")) {
                                    userInfoResult.setMail(dataObject.getString("mail"));
                                }
                                if (dataObject.has("mailActivation")) {
                                    userInfoResult.setMailActivation(dataObject.getString("mailActivation"));
                                }
                                if (dataObject.has("password")) {
                                    userInfoResult.setPassword(dataObject.getString("password"));
                                }
                                if (dataObject.has("salt")) {
                                    userInfoResult.setSalt(dataObject.getString("salt"));
                                }
                                if (dataObject.has("gender")) {
                                    userInfoResult.setGender(dataObject.getString("gender"));
                                }
                                if (dataObject.has("address")) {
                                    userInfoResult.setAddress(dataObject.getString("address"));
                                }
                                if (dataObject.has("companyName")) {
                                    userInfoResult.setCompanyName(dataObject.getString("companyName"));
                                }
                                if (dataObject.has("departmentName")) {
                                    userInfoResult.setDepartmentName(dataObject.getString("departmentName"));
                                }
                                if (dataObject.has("jobName")) {
                                    userInfoResult.setJobName(dataObject.getString("jobName"));
                                }
                                if (dataObject.has("reason")) {
                                    userInfoResult.setReason(dataObject.getString("reason"));
                                }
                                if (dataObject.has("eCallContactName")) {
                                    userInfoResult.seteCallContactName(dataObject.getString("eCallContactName"));
                                }
                                if (dataObject.has("eCallContactPhone")) {
                                    userInfoResult.seteCallContactPhone(dataObject.getString("eCallContactPhone"));
                                }
                                if (dataObject.has("eCallContactRelationship")) {
                                    userInfoResult.seteCallContactRelationship(dataObject.getString("eCallContactRelationship"));
                                }
                                if (dataObject.has("pinPassword")) {
                                    userInfoResult.setPinPassword(dataObject.getString("pinPassword"));
                                }
                                if (dataObject.has("pinSalt")) {
                                    userInfoResult.setPinSalt(dataObject.getString("pinSalt"));
                                }
                            }
                            mGetUserInfoListener.onGetUserInfo(userInfoResult);
                        } else {
                            mGetUserInfoListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mGetUserInfoListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR, Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;

            case USERAUTHORITYLOGINBYCODE://扫码登陆
                if (mAuthorityLoginByCodeListener != null) {
                    if (hasDSPTError) {
                        mAuthorityLoginByCodeListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mAuthorityLoginByCodeListener.onAuthorityLoginByCode();
                        } else {
                            mAuthorityLoginByCodeListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mAuthorityLoginByCodeListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case USERUPDATAPWD://修改密码
                if (mUpdatePwdListener != null) {
                    if (hasDSPTError) {
                        mUpdatePwdListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdatePwdListener.onUpdatePwdSuccess();
                        } else {
                            mUpdatePwdListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdatePwdListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case SETUPPINPASSWORD://PIN码首次设置
                if (mSetUpPinPasswordListener != null) {
                    if (hasDSPTError) {
                        mSetUpPinPasswordListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mSetUpPinPasswordListener.onSetUpPinPassword();
                        } else {
                            mSetUpPinPasswordListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mSetUpPinPasswordListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case UPDATEPINPASSWORD://旧PIN码修改新PIN码修改
                if (mUpdatePinPasswordListener != null) {
                    if (hasDSPTError) {
                        mUpdatePinPasswordListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            mUpdatePinPasswordListener.onUpdatePinPassword();
                        } else {
                            mUpdatePinPasswordListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mUpdatePinPasswordListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case GET_QRLOGIN_CODE:
                if (mGetQrLoginCodeListener != null) {
                    if (hasDSPTError) {
                        mGetQrLoginCodeListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            String code = jsonObject.getJSONObject("data").getString("loginCode");
                            String expireTime = jsonObject.getJSONObject("data").getString("expireTime");
                            mGetQrLoginCodeListener.onGetQrLoginCode(code, expireTime);
                        } else {
                            mGetQrLoginCodeListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mGetQrLoginCodeListener.onFail(Constants.ERROR_CODE_RESPONSEMSG_PARSEERROR,
                                Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR
                        );
                    }
                }
                break;
            case GET_LIST_GROUP:
                if (mGetListGroupListener != null) {
                    if (hasDSPTError) {
                        mGetListGroupListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<ListGroupInfo> listGroupInfoList = decodeListGroupInfo(jsonObject);
                            mGetListGroupListener.onGetListGroup(listGroupInfoList);
                        } else {
                            mGetListGroupListener.onFail(respCode, "出错了");
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mGetListGroupListener.onFail(Constants
                                        .ERROR_CODE_RESPONSEMSG_PARSEERROR
                                , Constants.ERROR_MESSAGE_RESPONSEMSG_PARSEERROR);
                    }
                }
                break;
            case USER_LOGIN_BY_CODE:
                if (mOnGetUserLoginCodeListener != null ){
                    if (hasDSPTError){
                        mOnGetUserLoginCodeListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonTokener.nextValue();
                        int respCode = jsonObject.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS){
                            mOnGetUserLoginCodeListener.onGetUserLoginCode();
                        }else {
                            switch (respCode){
                                case -100:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "失败，未知错误");
                                    break;
                                case -258102:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "缺失参数");
                                    break;
                                case -258201:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "用户不存在");
                                    break;
                                case -258202:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "用户未实名");
                                    break;
                                case -258203:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "未被授权或者不在授权期限内");
                                    break;
                                case -258204:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "授权码无效或者过期");
                                    break;
                                case -258205:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "车辆未绑定或绑定未审核通过");
                                    break;
                                case -258206:
                                    mOnGetUserLoginCodeListener.onFail(respCode, "用户无法登陆该应用");
                                    break;
                                default:
                                    break;
                            }
                        }
                    } catch (JSONException e) {
                        e.printStackTrace();
                    }
                }
                break;
            case SEARCH_GROUPINFO_BY_ID:
                if (mOnSearchGroupInfoByIdListener != null){
                    if (hasDSPTError){
                        mOnSearchGroupInfoByIdListener.onFail(errorCode, errorMsg);
                        return;
                    }
                    JSONTokener jsonTokener = new JSONTokener(response);
                    try {
                        JSONObject object = (JSONObject) jsonTokener.nextValue();
                        int respCode = object.getInt("respCode");
                        if (respCode == Constants.RESPCODE_SUCCESS) {
                            List<ListGroupInfo> listGroupInfoList = decodeListGroupInfo(object);
                            mOnSearchGroupInfoByIdListener.onSearchGroupInfoById(listGroupInfoList);
                        }else {
                            mOnSearchGroupInfoByIdListener.onFail(respCode, errorMsg);
                        }
                    } catch (Exception e) {
                        e.printStackTrace();
                        mOnSearchGroupInfoByIdListener.onFail(Constants.ERROR_CODE_INVALID_ARGUMENT,
                                Constants.ERROR_MESSAGE_INVALID_ARGUMENT);
                    }
                }
                break;
            default:
                break;
        }
    }
    private void uploadFile(final String url, final String domain,List<Param> paramList, String filePath,
                            final UploadFileListener uploadFileListener){

        File file = new File(filePath);
        if (file.exists() && file.length() > 0) {
            RequestParams requestParams = new RequestParams();
            for(Param param: paramList){
                requestParams.put(param.getName(),param.getValue());
            }
            requestParams.put("file",file);
            HttpRequestUtil.doPost(url, requestParams, new AsyncHttpResponseHandler() {
                @Override
                public void onSuccess(int i, byte[] bytes) {
                    if(uploadFileListener!=null){
                        uploadFileListener.onUploadProgress(100);
                    }
                    Utils.loge("uploadFileSuccess");
                    String response = new String(bytes);
                    Utils.loge("userauth uploadfile response =  "+response);
                    JSONTokener jsonParser = new JSONTokener(response);
                    try {
                        JSONObject jsonObject = (JSONObject) jsonParser.nextValue();
                        String mimetype = jsonObject.getString("mimetype");
                        String secondurl = jsonObject.getString("url");

                        try {
                            mimetype = URLEncoder.encode(mimetype,"UTF-8");
                        }catch (UnsupportedEncodingException e){
                            e.printStackTrace();
                        }
                        if (uploadFileListener != null) {
                        	String downloadurl = domain+secondurl;
                        	try {
                        	  downloadurl = URLEncoder.encode(downloadurl,"UTF-8");
	                        }catch (UnsupportedEncodingException e){
	                            e.printStackTrace();
	                        }

                        	  Utils.loge("userauth uploadfile downloadurlnew =  "+downloadurl);
                        	// 又拍云返回的secondurl不带域名，所以需要拼接上域名
                            uploadFileListener.onUploadSuccess(downloadurl);
                        }
//                        noticeUploadSuccess("code=200&mimetype="+mimetype+"&url="+url,20,Config.DEFAULT_PRIORITY);
                    }catch (JSONException e){
                        e.printStackTrace();
                        if(uploadFileListener!=null){
                            uploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,e.getMessage());
                        }
                    }
                }

                @Override
                public void onFailure(int statusCode, byte[] responseBody, Throwable throwable) {
                    // 上传失败后要做到工作
                    Utils.loge("上传文件失败-statusCode=" + statusCode);
                    if(responseBody!=null&&responseBody.length>0){
                        Utils.loge("上传文件失败-responseBody=" + new String(responseBody));
                    }

                    if(throwable !=null ){
                        Utils.loge("上传文件失败-"+throwable.toString());
                    }
                    if(uploadFileListener!=null){
                        uploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL,"上传失败");
                    }
                }
                @Override
                public void onRequestProgress(long bytesWritten, long contentLength, boolean done){
                    Utils.loge("上传文件 Progress>>>>>"+ bytesWritten + " / " + contentLength);
                    if(uploadFileListener!=null){
                        float percent = getPercent(bytesWritten,contentLength);
                        if(percent<99){// 解决文件写完到输出流，还需要挺长时间返回成功的问题
                            uploadFileListener.onUploadProgress(percent);
                        }

                    }
                }
            },true);

        }else{
            if(uploadFileListener!=null){
                uploadFileListener.onFail(Constants.ERROR_CODE_UPLOADFILE_FAIL, "文件不存在");
            }
        }


    }
    private float getPercent(long hasSendLength, long totalSize){
        int temp = (int)(((float)hasSendLength)/totalSize*10000);
        float percent = (float)temp/100;
        return percent;
    }
    private LoginData decodeLoginData(JSONObject jsonObject)throws Exception{
        LoginData loginData = new LoginData();
        LoginDataInfo loginDataInfo = new LoginDataInfo();
        UserInfoResult userInfoResult = new UserInfoResult();
        if (jsonObject.has("data")) {
            JSONObject dataObject = jsonObject.getJSONObject("data");
            if (dataObject.has("loginInfo")) {
                JSONObject loginInfoObject = dataObject.getJSONObject("loginInfo");
                if (loginInfoObject.has("token")) {
                    loginDataInfo.setToken(loginInfoObject.getString("token"));
                }
                if (loginInfoObject.has("refreshToken")) {
                    loginDataInfo.setRefreshToken(loginInfoObject.getString("refreshToken"));
                }
            }
            loginData.setLoginDataInfo(loginDataInfo);
            if (dataObject.has("userInfo")) {
                JSONObject userObject = dataObject.getJSONObject("userInfo");
                if (userObject.has("userId")) {
                    userInfoResult.setUserId(userObject.getString("userId"));
                }
                if (userObject.has("authState")) {
                    userInfoResult.setAuthState(userObject.getString("authState"));
                }
                if (userObject.has("backPhotoUrl")) {
                    userInfoResult.setBackPhotoUrl(userObject.getString("backPhotoUrl"));
                }
                if (userObject.has("certificateCode")) {
                    userInfoResult.setCertificateCode(userObject.getString("certificateCode"));
                }
                if (userObject.has("certificatePhotoUrl")) {
                    userInfoResult.setCertificatePhotoUrl(userObject.getString("certificatePhotoUrl"));
                }
                if (userObject.has("certificateType")) {
                    userInfoResult.setCertificateType(userObject.getString("certificateType"));
                }
                if (userObject.has("frontPhotoUrl")) {
                    userInfoResult.setFrontPhotoUrl(userObject.getString("frontPhotoUrl"));
                }
                if (userObject.has("userFullname")) {
                    userInfoResult.setUserFullname(userObject.getString("userFullname"));
                }
                if (userObject.has("userName")) {
                    userInfoResult.setUserName(userObject.getString("userName"));
                }
                if (userObject.has("productId")) {
                    userInfoResult.setProductId(userObject.getString("productId"));
                }
                if (userObject.has("typeId")) {
                    userInfoResult.setTypeId(userObject.getString("typeId"));
                }
                if (userObject.has("groupId")) {
                    userInfoResult.setGroupId(userObject.getString("groupId"));
                }
                if (userObject.has("fleetId")) {
                    userInfoResult.setFleetId(userObject.getString("fleetId"));
                }
                if (userObject.has("nickName")) {
                    userInfoResult.setNickName(userObject.getString("nickName"));
                }
                if (userObject.has("avatar")) {
                    userInfoResult.setAvatar(userObject.getString("avatar"));
                }
                if (userObject.has("phone")) {
                    userInfoResult.setPhone(userObject.getString("phone"));
                }
                if (userObject.has("mail")) {
                    userInfoResult.setMail(userObject.getString("mail"));
                }
                if (userObject.has("mailActivation")) {
                    userInfoResult.setMailActivation(userObject.getString("mailActivation"));
                }
                if (userObject.has("password")) {
                    userInfoResult.setPassword(userObject.getString("password"));
                }
                if (userObject.has("salt")) {
                    userInfoResult.setSalt(userObject.getString("salt"));
                }
                if (userObject.has("gender")) {
                    userInfoResult.setGender(userObject.getString("gender"));
                }
                if (userObject.has("address")) {
                    userInfoResult.setAddress(userObject.getString("address"));
                }
                if (userObject.has("companyName")) {
                    userInfoResult.setCompanyName(userObject.getString("companyName"));
                }
                if (userObject.has("departmentName")) {
                    userInfoResult.setDepartmentName(userObject.getString("departmentName"));
                }
                if (userObject.has("jobName")) {
                    userInfoResult.setJobName(userObject.getString("jobName"));
                }
                if (userObject.has("reason")) {
                    userInfoResult.setReason(userObject.getString("reason"));
                }
                if (userObject.has("eCallContactName")) {
                    userInfoResult.seteCallContactName(userObject.getString("eCallContactName"));
                }
                if (userObject.has("eCallContactPhone")) {
                    userInfoResult.seteCallContactPhone(userObject.getString("eCallContactPhone"));
                }
                if (userObject.has("eCallContactRelationship")) {
                    userInfoResult.seteCallContactRelationship(userObject.getString("eCallContactRelationship"));
                }
                if (userObject.has("pinPassword")) {
                    userInfoResult.setPinPassword(userObject.getString("pinPassword"));
                }
                if (userObject.has("pinSalt")) {
                    userInfoResult.setPinSalt(userObject.getString("pinSalt"));
                }
            }
            loginData.setUserInfoResult(userInfoResult);
            return loginData;
        }
        return null;
    }

    private List<ListGroupInfo> decodeListGroupInfo(JSONObject jsonObject)throws Exception{
        Gson gson = new Gson();
        List<ListGroupInfo> listGroupInfoList = new ArrayList<ListGroupInfo>();
        if (jsonObject.has("rows")) {
            JSONArray dataArray = jsonObject.getJSONArray("rows");
            for (int i = 0; i < dataArray.length(); i++) {
                ListGroupInfo listGroupInfo = gson.fromJson(dataArray.get(i).toString(), ListGroupInfo.class);
                listGroupInfoList.add(listGroupInfo);
            }
        }
        return listGroupInfoList;
    }
}
