package com.mxnavi.tspv2.utils;

import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.os.Environment;
import android.os.StatFs;
import android.util.Log;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.UnsupportedEncodingException;
import java.net.InetAddress;
import java.net.InterfaceAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URLEncoder;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.UUID;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import android.util.Base64;

/**
 * Created by zhoumo on 2015/6/11.
 */
public class Utils {
    public static final String TAG = "TSP-SDK";
    private static final boolean Verbose = true;

    public static void logd(String tag, String msg) {
        if (Verbose) {
            Log.d(tag, msg);
        }
    }
    public static void logd(String msg) {
        if (Verbose) {
            Log.d(TAG, msg);
        }
    }
    public static void logv(String tag, String msg) {
        if (Verbose) {
            Log.v(tag, msg);
        }
    }

    public static void loge(String tag, String msg) {
        Log.e(tag, msg);
    }
    public static void loge(String msg) {
        Log.e(TAG, msg);
    }

    public static void logi(String tag, String msg) {
        Log.i(tag, msg);
    }
    public static boolean getNetState( ConnectivityManager mConnectivityManager) {
        NetworkInfo activeInfo = mConnectivityManager.getActiveNetworkInfo();
        boolean wifiConnected;
        boolean mobileConnected;
        if (activeInfo != null && activeInfo.isConnected()) {
            wifiConnected = activeInfo.getType() == ConnectivityManager.TYPE_WIFI;
            mobileConnected = activeInfo.getType() == ConnectivityManager.TYPE_MOBILE;
        } else {
            wifiConnected = false;
            mobileConnected = false;
        }
        return mobileConnected || wifiConnected;
    }

    public static String getUNICODE(String source) {
        StringBuffer sb = new StringBuffer();
        try {
            for (int i = 0; i < source.length(); i++) {
                Character c = Character.toLowerCase(source.charAt(i));
                if (c <= 256) {
                    sb.append(Integer.toHexString(c).toUpperCase() + "00");
                } else {
                    String s = URLEncoder.encode(c.toString(), "UTF-16LE")
                            .replace("%", "");
                    sb.append(s);
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return sb.toString();
    }

    public static String getMD5(String val)  {
        try{
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(val.getBytes());
            byte[] m = md5.digest();//加密
            return getString(m);
        }catch ( NoSuchAlgorithmException e){
        }
        return "";
    }
    public static byte[] getMD5Byte(String val)  {
        byte[] m = new byte[1] ;
        try{
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(val.getBytes());
            m = md5.digest();//加密
            return m;
        }catch ( NoSuchAlgorithmException e){
        }
        return m;
    }
    public static byte[] getMD5Byte(byte[] val)  {
        byte[] m = new byte[1] ;
        try{
            MessageDigest md5 = MessageDigest.getInstance("MD5");
            md5.update(val);
            m = md5.digest();//加密
            return m;
        }catch ( NoSuchAlgorithmException e){
        }
        return m;
    }
    public static String getString(byte[] b){
        StringBuffer sb = new StringBuffer();
        for(int i = 0; i < b.length; i ++){
            sb.append(b[i]);
        }
        return sb.toString();
    }
    private static final char[] DIGITS_LOWER =
            {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    public  static char[] encodeHex(byte[] data) {
        final int l = data.length;
        final char[] out = new char[l << 1];
        // two characters form the hex value.
        for (int i = 0, j = 0; i < l; i++) {
            out[j++] = DIGITS_LOWER[(0xF0 & data[i]) >>> 4];
            out[j++] = DIGITS_LOWER[0x0F & data[i]];
        }
        return out;
    }
    public static byte[] decodeHex(final char[] data) throws Exception {

        final int len = data.length;

        if ((len & 0x01) != 0) {
            throw new Exception("Odd number of characters.");
        }

        final byte[] out = new byte[len >> 1];

        // two characters form the hex value.
        for (int i = 0, j = 0; j < len; i++) {
            int f = toDigit(data[j], j) << 4;
            j++;
            f = f | toDigit(data[j], j);
            j++;
            out[i] = (byte) (f & 0xFF);
        }

        return out;
    }

    private static int toDigit(final char ch, final int index) throws Exception {
        final int digit = Character.digit(ch, 16);
        if (digit == -1) {
            throw new Exception("Illegal hexadecimal character " + ch + " at index " + index);
        }
        return digit;
    }
    public static int converDoubleCoordinate2MX(double x){
        return (int)(x*3600*2560);
    }
    public static double converMX2DoubleCoordinate(int x){
        DecimalFormat df = new DecimalFormat("0.000000");
        return Double.parseDouble(df.format(((double) x) / 3600 / 2560));
    }

    public static String join(Object[] array, char separator) {
        if (array == null) {
            return null;
        }
        int arraySize = array.length;
        int bufSize = (arraySize == 0 ? 0 : ((array[0] == null ? 16 : array[0].toString().length()) + 1) * arraySize);
        StringBuffer buf = new StringBuffer(bufSize);

        for (int i = 0; i < arraySize; i++) {
            if (i > 0) {
                buf.append(separator);
            }
            if (array[i] != null) {
                buf.append(array[i]);
            }
        }
        return buf.toString();
    }

    public static String join(List list, char separator) {
        if (list == null) {
            return null;
        }
        Object[] array = list.toArray();
        return Utils.join(array, separator);
    }


    public  static String makeUrl(String url,HashMap<String, Object> params){
// 添加url参数
        if (params != null) {
            Iterator<String> it = params.keySet().iterator();
            StringBuffer a = null;
            while (it.hasNext()) {
                String key = it.next();
                String value;
                if(params.get(key) == null){
                    value = "";
                }else{
                    value = String.valueOf(params.get(key));
                }
                if (a == null) {
                    a = new StringBuffer();
                    a.append("?");
                } else {
                    a.append("&");
                }
                a.append(key);
                a.append("=");
                a.append(value);
            }
            url += a.toString();
        }
        return url;

    }

    public static String makeRestFulUrlPrefix(String url, HashMap<String, Object> params) {
        if (params != null) {
            Iterator<String> it = params.keySet().iterator();
            StringBuffer a = new StringBuffer();
            while (it.hasNext()) {
                String key = it.next();
                String value;
                if (params.get(key) == null) {
                    value = "0";
                } else {
                    value = String.valueOf(params.get(key));
                }
                a.append("/");
                a.append(value);
            }
            url += a.toString();
        }
        return url;
    }

    public static String makeRestFulUrlSuffix(LinkedHashMap<String, Object> params, String url) {
        if (params != null) {
            Iterator<String> it = params.keySet().iterator();
            StringBuffer a = new StringBuffer();
            while (it.hasNext()) {
                String key = it.next();
                String value;
                if (params.get(key) == null) {
                    value = "0";
                } else {
                    value = String.valueOf(params.get(key));
                }
                a.append("/");
                a.append(value);
            }
            url = a.toString() + "/" + url ;
        }
        return url;
    }

    /**
     *判断字符串是否为空
     * @param str
     * @return 如果为空返回true,否则返回false
     */
    public static boolean strIsEmpty(String str){
        if(str!=null&&str.length()>0){
            return false;
        }
        return true;
    }

    /**
     * 转换为Null的字符串为空字符串
     * @param str
     * @return
     */
    public static String converNullToEmpty(String str){
        if(str == null){
            return "";
        }
        return str;
    }

    /**
     * 转换日期格式
     * @param strTime
     * @return
     */
    public static String formatDateString(String strTime) {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try{
            Date date = formatter.parse(strTime);
            return formatter.format(date);
        }catch (ParseException e){
            e.printStackTrace();
        }
        return "";
    }

    private static final char[] DIGITS = { '0', '1', '2', '3', '4', '5', '6',
            '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };

    /**
     * 对字符串进行MD5加密
     *
     * @param text 明文
     * @return 密文
     */
    public static String md5(String text) {
        MessageDigest msgDigest = null;

        try {
            msgDigest = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalStateException(
                    "System doesn't support MD5 algorithm.");
        }

        try {
            msgDigest.update(text.getBytes("UTF-8"));

        } catch (UnsupportedEncodingException e) {

            throw new IllegalStateException(
                    "System doesn't support your  EncodingException.");

        }

        byte[] bytes = msgDigest.digest();

        String md5Str = new String(encodeHex(bytes));

        return md5Str;
    }

    public static String md5(byte[] bytestr) {
        MessageDigest msgDigest = null;

        try {
            msgDigest = MessageDigest.getInstance("MD5");
        } catch (NoSuchAlgorithmException e) {
            throw new IllegalStateException(
                    "System doesn't support MD5 algorithm.");
        }

        msgDigest.update(bytestr);

        byte[] bytes = msgDigest.digest();

        String md5Str = new String(encodeHex(bytes));

        return md5Str;
    }
    public static boolean externalMemoryAvailable() {
        return android.os.Environment.getExternalStorageState().equals(
                android.os.Environment.MEDIA_MOUNTED);
    }

    /**
     * 获取SDCARD剩余存储空间
     *
     * @return
     */
    public static long getAvailableExternalMemorySize() {
        if (externalMemoryAvailable()) {
            File path = Environment.getExternalStorageDirectory();
            StatFs stat = new StatFs(path.getPath());
            long blockSize = stat.getBlockSize();
            long availableBlocks = stat.getAvailableBlocks();
            return availableBlocks * blockSize;
        } else {
            return -1;
        }
    }
    public static boolean isIP(String str) {
        String num = "(25[0-5]|2[0-4]\\d|[1]\\d{2}|[1-9]?\\d)";
        String regex = "^" + num + "\\." + num + "\\." + num + "\\." + num + "$";
        return match(regex, str);
    }
    private static boolean match(String regex, String str) {
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(str);
        return matcher.matches();
    }
//    public static InetAddress getBroadcastAddress() {
//        try {
//            Enumeration<NetworkInterface> eni = NetworkInterface
//                    .getNetworkInterfaces();
//            while (eni.hasMoreElements()) {
//
//                NetworkInterface networkCard = eni.nextElement();
//                List<InterfaceAddress> ncAddrList = networkCard
//                        .getInterfaceAddresses();
//                Iterator<InterfaceAddress> ncAddrIterator = ncAddrList.iterator();
//                while (ncAddrIterator.hasNext()) {
//                    InterfaceAddress networkCardAddress = ncAddrIterator.next();
//                    InetAddress address = networkCardAddress.getAddress();
//                    if (!address.isLoopbackAddress()) {
//                        String hostAddress = address.getHostAddress();
//                        System.out.println("address        =   " + hostAddress);
//
//                        if (hostAddress.indexOf(":") > 0) {
//                            // case : ipv6
//                            continue;
//                        } else {
////                            if (isValidIpAddress(hostAddress) && !hostAddress.startsWith("10.")) {
//                                String broadcastAddress = networkCardAddress.getBroadcast().getHostAddress();
//
//                                String[] strsArray = broadcastAddress.split(".");
//                                byte[] bytes = new byte[4];
//                                for(int i=0;i<4;i++){
//                                    bytes[i] = Byte.parseByte(strsArray[i]);
//
//                                }
//                                return InetAddress.getByAddress(bytes);
////                            }
//
//                        }
//                    } else {
//                        String loopback = networkCardAddress.getAddress().getHostAddress();
//                        Utils.loge("loopback addr  =   " + loopback +"\n");
//                    }
//                }
//
//
//            }
//        } catch (Exception e) {
//            e.printStackTrace();
//        }
//        return null;
//    }
    public static InetAddress getBroadcastAddress() {
        try {
            Enumeration<NetworkInterface> eni = NetworkInterface
                    .getNetworkInterfaces();
            while (eni.hasMoreElements()) {

                NetworkInterface networkCard = eni.nextElement();
                List<InterfaceAddress> ncAddrList = networkCard
                        .getInterfaceAddresses();
                Iterator<InterfaceAddress> ncAddrIterator = ncAddrList.iterator();
                while (ncAddrIterator.hasNext()) {
                    InterfaceAddress networkCardAddress = ncAddrIterator.next();
                    InetAddress address = networkCardAddress.getAddress();
                    if(address == null){
                    	continue;
                    }
                    if (!address.isLoopbackAddress()) {
                        String hostAddress = address.getHostAddress();
                        if(hostAddress == null){
                        	continue;
                        }
                        if (hostAddress.indexOf(":") > 0|| !isValidIpAddress(hostAddress) ) {
                            // case : ipv6
                            continue;
                        } else {
                            // case : ipv4
//                            String maskAddress = calcMaskByPrefixLength(networkCardAddress.getNetworkPrefixLength());
//                            String subnetAddress = calcSubnetAddress(hostAddress, maskAddress);
                        	if(networkCardAddress.getBroadcast() == null){
                        		continue;
                        	}
                        	
                        	String broadcastAddress = networkCardAddress.getBroadcast().getHostAddress();
                        	if(broadcastAddress== null){
                        		continue;
                        	}

//                            Utils.loge("subnetmask     =   "+ maskAddress);
//                            Utils.loge("subnet         =   "+ subnetAddress);
//                            Utils.loge("broadcast      =   "+ broadcastAddress+"\n");
                            String[] strsArray = broadcastAddress.split("\\.");
                                byte[] bytes = new byte[4];
                                for(int i=0;i<4;i++){
                                    bytes[i] = (byte)Integer.parseInt(strsArray[i]);

                                }              
                                return InetAddress.getByAddress(bytes);

                        }
                    }
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
            Utils.loge(Log.getStackTraceString(e));
        }
 
        return getBroadcastAddressB();
    }

//    public static String calcMaskByPrefixLength(int length) {
//        int mask = -1 << (32 - length);
//        int partsNum = 4;
//        int bitsOfPart = 8;
//        int maskParts[] = new int[partsNum];
//        int selector = 0x000000ff;
//
//        for (int i = 0; i < maskParts.length; i++) {
//            int pos = maskParts.length - 1 - i;
//            maskParts[pos] = (mask >> (i * bitsOfPart)) & selector;
//        }
//
//        String result = "";
//        result = result + maskParts[0];
//        for (int i = 1; i < maskParts.length; i++) {
//            result = result + "." + maskParts[i];
//        }
//        return result;
//    }
//
//    public static String calcSubnetAddress(String ip, String mask) {
//        String result = "";
//        try {
//            // calc sub-net IP
//            InetAddress ipAddress = InetAddress.getByName(ip);
//            InetAddress maskAddress = InetAddress.getByName(mask);
//
//            byte[] ipRaw = ipAddress.getAddress();
//            byte[] maskRaw = maskAddress.getAddress();
//
//            int unsignedByteFilter = 0x000000ff;
//            int[] resultRaw = new int[ipRaw.length];
//            for (int i = 0; i < resultRaw.length; i++) {
//                resultRaw[i] = (ipRaw[i] & maskRaw[i] & unsignedByteFilter);
//            }
//
//            // make result string
//            result = result + resultRaw[0];
//            for (int i = 1; i < resultRaw.length; i++) {
//                result = result + "." + resultRaw[i];
//            }
//        } catch (UnknownHostException e) {
//            e.printStackTrace();
//        }
//
//        return result;
//    }


        public static InetAddress getBroadcastAddressB()  {
        try {
            for (Enumeration<NetworkInterface> en = NetworkInterface.getNetworkInterfaces();
                 en.hasMoreElements(); ) {
                NetworkInterface networkInterface = en.nextElement();
                List<InterfaceAddress> ncAddrList = networkInterface.getInterfaceAddresses();
                for (Enumeration<InetAddress> enumIpAddr = networkInterface.getInetAddresses();
                     enumIpAddr.hasMoreElements(); ) {
                    InetAddress inetAddress = enumIpAddr.nextElement();
                    if (!inetAddress.isLoopbackAddress()) {
                        String address = inetAddress.getHostAddress();

                        if (isValidIpAddress(address) && !address.startsWith("10.")) {
                            byte[]	bytes = inetAddress.getAddress();
                            bytes[3] = (byte)255;
                            return	InetAddress.getByAddress(bytes);
                        }
                    }
                }
            }
        } catch (SocketException ex) {
            ex.printStackTrace();
        }catch(IOException e){
        	e.printStackTrace();
        }

        return null;
    }
    public static boolean isValidIpAddress(String ipaddr) {
        String reg = "(([1-9])|([1-9]\\d)|([1]\\d{2})|([2][0-4]\\d)|([2][5][0-5]))" +
                "(\\.((\\d)|([1-9]\\d)|([1]\\d{2})|([2][0-4]\\d)|([2][5][0-5]))){3}";

        Matcher matcher = Pattern.compile(reg).matcher(ipaddr);
        return matcher.matches();
    }
    /**
     * 解密算法
     * @param
     * @return
     */
    public static String decode(String str) {
        byte[] bytes;
        try
        {
            bytes = Base64.decode(str,Base64.NO_WRAP);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return "";
        }
        byte[] tmp = new byte[bytes.length];
        for (int i = 0; i < tmp.length; i++) {
            tmp[i] = (byte)(bytes[i] ^ 0x13);
        }
        String s = "";
        try {
            s = new String(tmp,"GBK");
        }catch (UnsupportedEncodingException e){
            e.printStackTrace();
        }

        //System.out.println(new String(tmp));
        return s;
    }
    public static String encodeBase64(byte[] bytes){
        return Base64.encodeToString(bytes, Base64.NO_WRAP);
    }
    /**
     * 加密算法
     * @param str
     * @return
     */
    public static String encode(String str){
        byte[] baKeyword = null;
        try {
            byte[] bytes = str.getBytes();
            baKeyword = new byte[bytes.length];
            for(int i=0;i<bytes.length;i++){

                baKeyword[i] = (byte)(0x13 ^ bytes[i]);
            }

        } catch(Exception e) {
            e.printStackTrace();
        }
//        Base64 encoder = new BASE64Encoder();

        return Base64.encodeToString(baKeyword, Base64.NO_WRAP);
        //return new String(Base64.encodeBase64(baKeyword));
    }
    public static String getFileMD5String(File file,MessageDigest messagedigest) throws IOException {
        InputStream fis;
        fis = new FileInputStream(file);
        byte[] buffer = new byte[1024];
        int numRead = 0;
        while ((numRead = fis.read(buffer)) > 0) {
            messagedigest.update(buffer, 0, numRead);
        }
        fis.close();
        return bufferToHex(messagedigest.digest());
    }
//    public static String getFileMD5String(byte[] buffer,MessageDigest messagedigest) throws IOException {
//        messagedigest.update(buffer, 0, buffer.length);
//        return bufferToHex(messagedigest.digest());
//    }
    public static void fileMd5(File file,MessageDigest messagedigest) throws IOException {
        InputStream fis;
        fis = new FileInputStream(file);
        byte[] buffer = new byte[1024];
        int numRead = 0;
        while ((numRead = fis.read(buffer)) > 0) {
            messagedigest.update(buffer, 0, numRead);
        }
        fis.close();
    }
    public  static String bufferToHex(byte bytes[]) {
        return bufferToHex(bytes, 0, bytes.length);
    }
    private static String bufferToHex(byte bytes[], int m, int n) {
        StringBuffer stringbuffer = new StringBuffer(2 * n);
        int k = m + n;
        for (int l = m; l < k; l++) {
            appendHexPair(bytes[l], stringbuffer);
        }
        return stringbuffer.toString();
    }
    private static void appendHexPair(byte bt, StringBuffer stringbuffer) {
        char c0 = DIGITS_LOWER[(bt & 0xf0) >> 4];// 取字节中高 4 位的数字转换
        // 为逻辑右移，将符号位一起右移,此处未发现两种符号有何不同
        char c1 = DIGITS_LOWER[bt & 0xf];// 取字节中低 4 位的数字转换
        stringbuffer.append(c0);
        stringbuffer.append(c1);
    }


    public static  byte[] int2byte(int res) {
        byte[] targets = new byte[4];

        targets[3] = (byte) (res & 0xff);// 最低位
        targets[2] = (byte) ((res >> 8) & 0xff);// 次低位
        targets[1] = (byte) ((res >> 16) & 0xff);// 次高位
        targets[0] = (byte) (res >>> 24);// 最高位,无符号右移。
        return targets;
    }
    public  static int bytes2Int(byte[] bRefArr) {
        int iOutcome = 0;
        byte bLoop;

        for (int i = 0; i < bRefArr.length; i++) {
            bLoop = bRefArr[i];
            iOutcome += (bLoop & 0xFF) << (8 * (bRefArr.length-i-1));
        }
        return iOutcome;
    }

    public static  byte[] concat(byte[] first, byte[] second) {
        byte[] result = Arrays.copyOf(first, first.length + second.length);
        System.arraycopy(second, 0, result, first.length, second.length);
        return result;
    }
    /**
     * 获取UUID
     * @return
     */
    public static String getUUID(){
        return UUID.randomUUID().toString();
    }
    /**
     * 获得sd卡剩余容量，即可用大小
     *
     * @return
     */
    public static long getSDAvailableSize() {
        File path = Environment.getExternalStorageDirectory();
        StatFs stat = new StatFs(path.getPath());
        long blockSize = stat.getBlockSize();
        long availableBlocks = stat.getAvailableBlocks();
        return  blockSize * availableBlocks;
    }

    /**
     * 比较当前日期是否过期
     */
    public static boolean hasExpired(Date current, Date end) {
        return hasExpired(current, end, 0);
    }

    public static boolean hasExpired(Date current, Date end, long expired) {
        if(null == current || null == end)
            return false;
        Calendar cal1 = Calendar.getInstance();
        cal1.setTime(current);
        cal1.set(Calendar.HOUR_OF_DAY, 0);
        cal1.set(Calendar.MINUTE, 0);
        cal1.set(Calendar.SECOND, 0);
        cal1.set(Calendar.MILLISECOND, 0);
        Calendar cal2 = Calendar.getInstance();
        cal2.setTime(end);
        cal2.set(Calendar.HOUR_OF_DAY, 0);
        cal2.set(Calendar.MINUTE, 0);
        cal2.set(Calendar.SECOND, 0);
        cal2.set(Calendar.MILLISECOND, 0);
        long a = cal1.getTime().getTime();
        long b = (cal2.getTime().getTime() + expired);
        return  cal1.getTime().getTime() > (cal2.getTime().getTime() + expired);
    }
}
